@@ -1,7 +1,7 @@
 /** @file
  *  @brief Xapian::Error base class.
  */
-/* Copyright (C) 2007,2008,2011,2013,2014,2015 Olly Betts
+/* Copyright (C) 2007,2008,2011,2013,2014,2015,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,10 +38,10 @@
 
 using namespace std;
 
-Xapian::Error::Error(const std::string &msg_, const std::string &context_,
+Xapian::Error::Error(std::string_view msg_, std::string_view context_,
 		     const char * type_, const char * error_string_)
     : msg(msg_), context(context_), error_string(), type(type_),
-      my_errno(0), already_handled(false)
+      my_errno(0)
 {
     if (error_string_) error_string.assign(error_string_);
 }
@@ -78,7 +78,7 @@ Xapian::Error::get_error_string() const
 	} else {
 	    // POSIX says only that EAI_* constants are "non-zero" - they're
 	    // negative on Linux, but we allow for them being positive.  We
-	    // check they all that the same sign in net/remoteconnection.h.
+	    // check they all that the same sign in net/resolver.h.
 	    if (EAI_FAIL > 0)
 		error_string.assign(gai_strerror(-my_errno));
 	    else
