@@ -1,7 +1,7 @@
-/** @file queryinternal.h
+/** @file
  * @brief Xapian::Query internals
  */
-/* Copyright (C) 2011,2012,2013,2014,2015,2016,2018 Olly Betts
+/* Copyright (C) 2011-2026 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,19 +21,22 @@
 #ifndef XAPIAN_INCLUDED_QUERYINTERNAL_H
 #define XAPIAN_INCLUDED_QUERYINTERNAL_H
 
-#include "postlist.h"
+#include "api/editdistance.h"
 #include "queryvector.h"
+#include "stringutils.h"
 #include "xapian/intrusive_ptr.h"
 #include "xapian/query.h"
 
 /// Default set_size for OP_ELITE_SET:
 const Xapian::termcount DEFAULT_ELITE_SET_SIZE = 10;
 
-class QueryOptimiser;
-
 namespace Xapian {
 namespace Internal {
 
+class PostList;
+struct PostListAndEstimate;
+class QueryOptimiser;
+
 class QueryTerm : public Query::Internal {
     std::string term;
 
@@ -45,18 +48,29 @@ class QueryTerm : public Query::Internal {
     // Construct a "MatchAll" QueryTerm.
     QueryTerm() : term(), wqf(1), pos(0) { }
 
-    QueryTerm(const std::string & term_,
+    QueryTerm(std::string_view term_,
 	      Xapian::termcount wqf_,
 	      Xapian::termpos pos_)
 	: term(term_), wqf(wqf_), pos(pos_) { }
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
     const std::string & get_term() const { return term; }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    termcount get_wqf() const { return wqf; }
 
-    termcount get_length() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION {
+    termpos get_pos() const { return pos; }
+
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
+
+    bool postlist_sub_and_like(AndContext& ctx,
+			       QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
+
+    termcount get_length() const noexcept XAPIAN_PURE_FUNCTION {
 	return wqf;
     }
 
@@ -73,11 +87,13 @@ class QueryPostingSource : public Query::Internal {
   public:
     explicit QueryPostingSource(PostingSource * source_);
 
-    PostingIterator::Internal * postlist(QueryOptimiser *qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     void serialise(std::string & result) const;
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
     std::string get_description() const;
 };
@@ -90,16 +106,23 @@ class QueryScaleWeight : public Query::Internal {
   public:
     QueryScaleWeight(double factor, const Query & subquery_);
 
-    PostingIterator::Internal * postlist(QueryOptimiser *qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    termcount get_length() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION {
+    bool postlist_sub_and_like(AndContext& ctx,
+			       QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
+
+    termcount get_length() const noexcept XAPIAN_PURE_FUNCTION {
 	return subquery.internal->get_length();
     }
 
     void serialise(std::string & result) const;
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
-    size_t get_num_subqueries() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
+    size_t get_num_subqueries() const noexcept XAPIAN_PURE_FUNCTION;
     const Query get_subquery(size_t n) const;
 
     std::string get_description() const;
@@ -123,15 +146,17 @@ class QueryValueRange : public QueryValueBase {
 
   public:
     QueryValueRange(Xapian::valueno slot_,
-		    const std::string &begin_,
-		    const std::string &end_)
+		    std::string_view begin_,
+		    std::string_view end_)
 	: QueryValueBase(slot_), begin(begin_), end(end_) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser *qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     void serialise(std::string & result) const;
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
     std::string get_description() const;
 };
@@ -140,14 +165,16 @@ class QueryValueLE : public QueryValueBase {
     std::string limit;
 
   public:
-    QueryValueLE(Xapian::valueno slot_, const std::string &limit_)
+    QueryValueLE(Xapian::valueno slot_, std::string_view limit_)
 	: QueryValueBase(slot_), limit(limit_) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser *qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     void serialise(std::string & result) const;
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
     std::string get_description() const;
 };
@@ -156,14 +183,16 @@ class QueryValueGE : public QueryValueBase {
     std::string limit;
 
   public:
-    QueryValueGE(Xapian::valueno slot_, const std::string &limit_)
+    QueryValueGE(Xapian::valueno slot_, std::string_view limit_)
 	: QueryValueBase(slot_), limit(limit_) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser *qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     void serialise(std::string & result) const;
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
     std::string get_description() const;
 };
@@ -178,18 +207,36 @@ class QueryBranch : public Query::Internal {
 
     void serialise_(std::string & result, Xapian::termcount parameter = 0) const;
 
-    void do_or_like(OrContext& ctx, QueryOptimiser * qopt, double factor,
-		    Xapian::termcount elite_set_size = 0, size_t first = 0) const;
+    void do_bool_or_like(OrContext& ctx,
+			 QueryOptimiser* qopt,
+			 TermFreqs* termfreqs,
+			 size_t first = 0) const;
 
-    PostList * do_synonym(QueryOptimiser * qopt, double factor) const;
+    /** Process OR-like subqueries.
+     *
+     *  @param keep_zero_weight  By default zero-weight subqueries are kept,
+     *				 but in some situations (such as on the right
+     *				 side of OP_AND_MAYBE when not under
+     *				 OP_SYNONYM) they can be ignored.
+     */
+    void do_or_like(OrContext& ctx, QueryOptimiser* qopt, double factor,
+		    TermFreqs* termfreqs,
+		    Xapian::termcount elite_set_size = 0, size_t first = 0,
+		    bool keep_zero_weight = true) const;
 
-    PostList * do_max(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate do_synonym(QueryOptimiser* qopt,
+				   double factor,
+				   TermFreqs* termfreqs) const;
+
+    PostListAndEstimate do_max(QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
 
     const std::string get_description_helper(const char * op,
 					     Xapian::termcount window = 0) const;
 
   public:
-    termcount get_length() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    termcount get_length() const noexcept XAPIAN_PURE_FUNCTION;
 
     void serialise(std::string & result) const;
 
@@ -197,8 +244,8 @@ class QueryBranch : public Query::Internal {
 
     virtual void add_subquery(const Xapian::Query & subquery) = 0;
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
-    size_t get_num_subqueries() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
+    size_t get_num_subqueries() const noexcept XAPIAN_PURE_FUNCTION;
     const Query get_subquery(size_t n) const;
 
     virtual Query::Internal * done() = 0;
@@ -214,9 +261,14 @@ class QueryAndLike : public QueryBranch {
 
     Query::Internal * done();
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_and_like(AndContext& ctx, QueryOptimiser * qopt, double factor) const;
+    bool postlist_sub_and_like(AndContext& ctx,
+			       QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
 };
 
 class QueryOrLike : public QueryBranch {
@@ -245,9 +297,18 @@ class QueryOr : public QueryOrLike {
   public:
     explicit QueryOr(size_t n_subqueries) : QueryOrLike(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_or_like(OrContext& ctx, QueryOptimiser * qopt, double factor) const;
+    void postlist_sub_or_like(OrContext& ctx, QueryOptimiser* qopt,
+			      double factor,
+			      TermFreqs* termfreqs,
+			      bool keep_zero_weight) const;
+
+    void postlist_sub_bool_or_like(OrContext& ctx,
+				   QueryOptimiser* qopt,
+				   TermFreqs* termfreqs) const;
 
     std::string get_description() const;
 };
@@ -258,11 +319,14 @@ class QueryAndNot : public QueryBranch {
   public:
     explicit QueryAndNot(size_t n_subqueries) : QueryBranch(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_and_like(AndContext& ctx,
+    bool postlist_sub_and_like(AndContext& ctx,
 			       QueryOptimiser* qopt,
-			       double factor) const;
+			       double factor,
+			       TermFreqs* termfreqs) const;
 
     void add_subquery(const Xapian::Query & subquery);
 
@@ -277,9 +341,14 @@ class QueryXor : public QueryOrLike {
   public:
     explicit QueryXor(size_t n_subqueries) : QueryOrLike(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_xor(XorContext& ctx, QueryOptimiser * qopt, double factor) const;
+    void postlist_sub_xor(XorContext& ctx,
+			  QueryOptimiser* qopt,
+			  double factor,
+			  TermFreqs* termfreqs) const;
 
     std::string get_description() const;
 };
@@ -290,11 +359,14 @@ class QueryAndMaybe : public QueryBranch {
   public:
     explicit QueryAndMaybe(size_t n_subqueries) : QueryBranch(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_and_like(AndContext& ctx,
+    bool postlist_sub_and_like(AndContext& ctx,
 			       QueryOptimiser* qopt,
-			       double factor) const;
+			       double factor,
+			       TermFreqs* termfreqs) const;
 
     void add_subquery(const Xapian::Query & subquery);
 
@@ -309,9 +381,14 @@ class QueryFilter : public QueryAndLike {
   public:
     explicit QueryFilter(size_t n_subqueries) : QueryAndLike(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_and_like(AndContext& ctx, QueryOptimiser * qopt, double factor) const;
+    bool postlist_sub_and_like(AndContext& ctx,
+			       QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
 
     std::string get_description() const;
 };
@@ -323,8 +400,9 @@ class QueryWindowed : public QueryAndLike {
     QueryWindowed(size_t n_subqueries, Xapian::termcount window_)
 	: QueryAndLike(n_subqueries), window(window_) { }
 
-    void postlist_windowed(Xapian::Query::op op, AndContext& ctx,
-			   QueryOptimiser * qopt, double factor) const;
+    bool postlist_windowed(Xapian::Query::op op, AndContext& ctx,
+			   QueryOptimiser* qopt, double factor,
+			   TermFreqs* termfreqs) const;
 
   public:
     size_t get_window() const { return window; }
@@ -341,7 +419,10 @@ class QueryNear : public QueryWindowed {
 
     void serialise(std::string & result) const;
 
-    void postlist_sub_and_like(AndContext& ctx, QueryOptimiser * qopt, double factor) const;
+    bool postlist_sub_and_like(AndContext& ctx,
+			       QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
 
     std::string get_description() const;
 };
@@ -355,7 +436,10 @@ class QueryPhrase : public QueryWindowed {
 
     void serialise(std::string & result) const;
 
-    void postlist_sub_and_like(AndContext& ctx, QueryOptimiser * qopt, double factor) const;
+    bool postlist_sub_and_like(AndContext& ctx,
+			       QueryOptimiser* qopt,
+			       double factor,
+			       TermFreqs* termfreqs) const;
 
     std::string get_description() const;
 };
@@ -372,9 +456,15 @@ class QueryEliteSet : public QueryOrLike {
 
     void serialise(std::string & result) const;
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
-    void postlist_sub_or_like(OrContext& ctx, QueryOptimiser * qopt, double factor) const;
+    void postlist_sub_or_like(OrContext& ctx,
+			      QueryOptimiser* qopt,
+			      double factor,
+			      TermFreqs* termfreqs,
+			      bool keep_zero_weight) const;
 
     std::string get_description() const;
 };
@@ -385,7 +475,9 @@ class QuerySynonym : public QueryOrLike {
   public:
     explicit QuerySynonym(size_t n_subqueries) : QueryOrLike(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     Query::Internal * done();
 
@@ -398,7 +490,9 @@ class QueryMax : public QueryOrLike {
   public:
     explicit QueryMax(size_t n_subqueries) : QueryOrLike(n_subqueries) { }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     std::string get_description() const;
 };
@@ -408,30 +502,66 @@ class QueryWildcard : public Query::Internal {
 
     Xapian::termcount max_expansion;
 
-    int max_type;
+    int flags;
 
     Query::op combiner;
 
-    Xapian::Query::op get_op() const;
+    /** Fixed head and tail lengths, and min/max length term that can match.
+     *
+     *  All in bytes.
+     */
+    size_t head = 0, tail = 0, min_len = 0, max_len = 0;
+
+    // If candidate.size() >= min_len && candidate.size() < min_check_len then
+    // we don't need to actually do the pattern check since the length and
+    // head/tail checks are enough.
+    //
+    // This optimises cases `*` or `*?` or `?*` (which covers a lot of common
+    // cases) and also a pattern where there's no `*` and a single `?` where
+    // we only need to check the pattern when candidate.size() > min_len.
+    //
+    // Note that we can't handle cases like `*??` here since `?` matches a
+    // single UTF-8 character, which can be more than one byte.
+    size_t min_check_len = size_t(-1);
+
+    std::string prefix, suffix;
+
+    bool test_wildcard_(const std::string& candidate, size_t o, size_t p,
+			size_t i) const;
 
   public:
-    QueryWildcard(const std::string &pattern_,
+    QueryWildcard(std::string_view pattern_,
 		  Xapian::termcount max_expansion_,
-		  int max_type_,
-		  Query::op combiner_)
-	: pattern(pattern_),
-	  max_expansion(max_expansion_),
-	  max_type(max_type_),
-	  combiner(combiner_)
-    { }
+		  int flags_,
+		  Query::op combiner_);
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    /// Perform wildcard test on candidate known to match prefix.
+    bool test_prefix_known(const std::string& candidate) const;
 
-    const std::string & get_pattern() const { return pattern; }
+    /// Perform full wildcard test on candidate.
+    bool test(const std::string& candidate) const {
+	return startswith(candidate, prefix) && test_prefix_known(candidate);
+    }
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
-    termcount get_length() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    std::string get_pattern() const { return pattern; }
+
+    Xapian::termcount get_max_expansion() const { return max_expansion; }
+
+    int get_just_flags() const {
+	return flags &~ Xapian::Query::WILDCARD_LIMIT_MASK_;
+    }
+
+    int get_max_type() const {
+	return flags & Xapian::Query::WILDCARD_LIMIT_MASK_;
+    }
+
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
+
+    termcount get_length() const noexcept XAPIAN_PURE_FUNCTION;
 
     void serialise(std::string & result) const;
 
@@ -448,10 +578,99 @@ class QueryWildcard : public Query::Internal {
 	}
 	return new QueryWildcard(pattern,
 				 max_expansion,
-				 max_type,
+				 flags,
 				 new_op);
     }
 
+    /// Return the fixed prefix from the wildcard pattern.
+    std::string get_fixed_prefix() const { return prefix; }
+
+    std::string get_description() const;
+};
+
+class QueryEditDistance : public Query::Internal {
+    std::string pattern;
+
+    Xapian::termcount max_expansion;
+
+    int flags;
+
+    Query::op combiner;
+
+    EditDistanceCalculator edcalc;
+
+    unsigned edit_distance;
+
+    size_t fixed_prefix_len;
+
+  public:
+    QueryEditDistance(std::string_view pattern_,
+		      Xapian::termcount max_expansion_,
+		      int flags_,
+		      Query::op combiner_,
+		      unsigned edit_distance_ = 2,
+		      size_t fixed_prefix_len_ = 0)
+	: pattern(pattern_),
+	  max_expansion(max_expansion_),
+	  flags(flags_),
+	  combiner(combiner_),
+	  edcalc(pattern),
+	  edit_distance(edit_distance_),
+	  fixed_prefix_len(fixed_prefix_len_) { }
+
+    /** Perform edit distance test.
+     *
+     *  @return edit_distance + 1, or 0 for a non-match.
+     */
+    int test(const std::string& candidate) const;
+
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
+
+    std::string get_pattern() const { return pattern; }
+
+    size_t get_fixed_prefix_len() const { return fixed_prefix_len; }
+
+    Xapian::termcount get_max_expansion() const { return max_expansion; }
+
+    int get_just_flags() const {
+	return flags &~ Xapian::Query::WILDCARD_LIMIT_MASK_;
+    }
+
+    int get_max_type() const {
+	return flags & Xapian::Query::WILDCARD_LIMIT_MASK_;
+    }
+
+    unsigned get_threshold() const {
+	return edit_distance;
+    }
+
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
+
+    termcount get_length() const noexcept XAPIAN_PURE_FUNCTION;
+
+    void serialise(std::string & result) const;
+
+    /** Change the combining operator.
+     *
+     *  If there's only one reference to this object we change in-place
+     *  and return a pointer to the existing object; otherwise we create and
+     *  return a new QueryEditDistance object.
+     */
+    QueryEditDistance* change_combiner(Xapian::Query::op new_op) {
+	if (_refs == 1) {
+	    combiner = new_op;
+	    return this;
+	}
+	return new QueryEditDistance(pattern,
+				     max_expansion,
+				     flags,
+				     new_op,
+				     edit_distance,
+				     fixed_prefix_len);
+    }
+
     std::string get_description() const;
 };
 
@@ -459,9 +678,11 @@ class QueryInvalid : public Query::Internal {
   public:
     QueryInvalid() { }
 
-    Xapian::Query::op get_type() const XAPIAN_NOEXCEPT XAPIAN_PURE_FUNCTION;
+    Xapian::Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
-    PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const;
+    PostListAndEstimate postlist(QueryOptimiser* qopt,
+				 double factor,
+				 TermFreqs* termfreqs) const;
 
     void serialise(std::string & result) const;
 
