@@ -1,7 +1,7 @@
 /** @file
  * @brief Class for looking up user subclasses during unserialisation.
  */
-/* Copyright (C) 2006,2007,2008,2009,2010,2016 Olly Betts
+/* Copyright (C) 2006,2007,2008,2009,2010,2016,2024 Olly Betts
  * Copyright (C) 2006,2007,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -31,6 +31,7 @@
 #include "xapian/weight.h"
 
 #include "debuglog.h"
+#include "stringutils.h"
 
 #include <algorithm>
 #include <map>
@@ -38,20 +39,25 @@
 
 using namespace std;
 
+using Xapian::Internal::opt_intrusive_ptr;
+
 class Xapian::Registry::Internal : public Xapian::Internal::intrusive_base {
     friend class Xapian::Registry;
 
     /// Registered weighting schemes.
-    std::map<std::string, Xapian::Weight *> wtschemes;
+    map<string, Xapian::Weight*, std::less<>> wtschemes;
 
     /// Registered external posting sources.
-    std::map<std::string, Xapian::PostingSource *> postingsources;
+    map<string, Xapian::PostingSource*, std::less<>> postingsources;
 
     /// Registered match spies.
-    std::map<std::string, Xapian::MatchSpy *> matchspies;
+    map<string, Xapian::MatchSpy*, std::less<>> matchspies;
 
     /// Registered lat-long metrics.
-    std::map<std::string, Xapian::LatLongMetric *> lat_long_metrics;
+    map<string, Xapian::LatLongMetric*, std::less<>> lat_long_metrics;
+
+    /// Registered KeyMaker subclasses.
+    map<string, opt_intrusive_ptr<Xapian::KeyMaker>, std::less<>> key_makers;
 
     /// Add the standard subclasses provided in the API.
     void add_defaults();
@@ -73,17 +79,36 @@ class Xapian::Registry::Internal : public Xapian::Internal::intrusive_base {
     ~Internal();
 };
 
+/// Register an optionally ref-counted object.
 template<class T>
 static inline void
-register_object(map<string, T*> & registry, const T & obj)
+register_object(map<string, opt_intrusive_ptr<T>, std::less<>>& registry,
+		T* obj_)
+{
+    opt_intrusive_ptr<T> obj(obj_);
+
+    string name = obj->name();
+    if (rare(name.empty())) {
+	throw Xapian::InvalidOperationError("Unable to register object - "
+					    "name() method returned empty "
+					    "string");
+    }
+
+    auto r = registry.insert(make_pair(name, static_cast<T*>(NULL)));
+    r.first->second = std::move(obj);
+}
+
+/// Register an object that requires cloning.
+template<class T>
+static inline void
+register_object(map<string, T*, std::less<>>& registry, const T& obj)
 {
     string name = obj.name();
     if (rare(name.empty())) {
 	throw Xapian::InvalidOperationError("Unable to register object - name() method returned empty string");
     }
 
-    pair<typename map<string, T *>::iterator, bool> r;
-    r = registry.insert(make_pair(name, static_cast<T*>(NULL)));
+    auto r = registry.insert(make_pair(name, static_cast<T*>(NULL)));
     if (!r.second) {
 	// Existing element with this key, so replace the pointer with NULL
 	// and delete the existing pointer.
@@ -106,11 +131,25 @@ register_object(map<string, T*> & registry, const T & obj)
     r.first->second = clone;
 }
 
+/// Look up an optionally ref-counted object.
 template<class T>
-static inline const T *
-lookup_object(map<string, T*> registry, const string & name)
+static inline const T*
+lookup_object(map<string, opt_intrusive_ptr<T>, std::less<>> registry,
+	      string_view name)
 {
-    typename map<string, T*>::const_iterator i = registry.find(name);
+    auto i = registry.find(name);
+    if (i == registry.end()) {
+	return NULL;
+    }
+    return i->second.get();
+}
+
+/// Look up an object that requires cloning.
+template<class T>
+static inline const T*
+lookup_object(map<string, T*, std::less<>> registry, string_view name)
+{
+    auto i = registry.find(name);
     if (i == registry.end()) {
 	return NULL;
     }
@@ -146,8 +185,6 @@ Registry::Internal::add_defaults()
     wtschemes[weighting_scheme->name()] = weighting_scheme;
     weighting_scheme = new Xapian::CoordWeight;
     wtschemes[weighting_scheme->name()] = weighting_scheme;
-    weighting_scheme = new Xapian::TradWeight;
-    wtschemes[weighting_scheme->name()] = weighting_scheme;
     weighting_scheme = new Xapian::TfIdfWeight;
     wtschemes[weighting_scheme->name()] = weighting_scheme;
     weighting_scheme = new Xapian::InL2Weight;
@@ -164,7 +201,15 @@ Registry::Internal::add_defaults()
     wtschemes[weighting_scheme->name()] = weighting_scheme;
     weighting_scheme = new Xapian::DPHWeight;
     wtschemes[weighting_scheme->name()] = weighting_scheme;
-    weighting_scheme = new Xapian::LMWeight;
+    weighting_scheme = new Xapian::LMJMWeight;
+    wtschemes[weighting_scheme->name()] = weighting_scheme;
+    weighting_scheme = new Xapian::LMDirichletWeight;
+    wtschemes[weighting_scheme->name()] = weighting_scheme;
+    weighting_scheme = new Xapian::LMAbsDiscountWeight;
+    wtschemes[weighting_scheme->name()] = weighting_scheme;
+    weighting_scheme = new Xapian::LM2StageWeight;
+    wtschemes[weighting_scheme->name()] = weighting_scheme;
+    weighting_scheme = new Xapian::DiceCoeffWeight;
     wtschemes[weighting_scheme->name()] = weighting_scheme;
 
     Xapian::PostingSource * source;
@@ -188,41 +233,41 @@ Registry::Internal::add_defaults()
     Xapian::LatLongMetric * metric;
     metric = new Xapian::GreatCircleMetric();
     lat_long_metrics[metric->name()] = metric;
+
+    Xapian::KeyMaker* keymaker;
+    keymaker = new Xapian::MultiValueKeyMaker();
+    key_makers[keymaker->name()] = keymaker->release();
 }
 
 void
 Registry::Internal::clear_weighting_schemes()
 {
-    map<string, Xapian::Weight*>::const_iterator i;
-    for (i = wtschemes.begin(); i != wtschemes.end(); ++i) {
-	delete i->second;
+    for (auto&& i : wtschemes) {
+	delete i.second;
     }
 }
 
 void
 Registry::Internal::clear_posting_sources()
 {
-    map<string, Xapian::PostingSource *>::const_iterator i;
-    for (i = postingsources.begin(); i != postingsources.end(); ++i) {
-	delete i->second;
+    for (auto&& i : postingsources) {
+	delete i.second;
     }
 }
 
 void
 Registry::Internal::clear_match_spies()
 {
-    map<string, Xapian::MatchSpy *>::const_iterator i;
-    for (i = matchspies.begin(); i != matchspies.end(); ++i) {
-	delete i->second;
+    for (auto&& i : matchspies) {
+	delete i.second;
     }
 }
 
 void
 Registry::Internal::clear_lat_long_metrics()
 {
-    map<string, Xapian::LatLongMetric *>::const_iterator i;
-    for (i = lat_long_metrics.begin(); i != lat_long_metrics.end(); ++i) {
-	delete i->second;
+    for (auto&& i : lat_long_metrics) {
+	delete i.second;
     }
 }
 
@@ -269,7 +314,7 @@ Registry::register_weighting_scheme(const Xapian::Weight &wt)
 }
 
 const Xapian::Weight *
-Registry::get_weighting_scheme(const string & name) const
+Registry::get_weighting_scheme(string_view name) const
 {
     LOGCALL(API, const Xapian::Weight *, "Xapian::Registry::get_weighting_scheme", name);
     RETURN(lookup_object(internal->wtschemes, name));
@@ -283,7 +328,7 @@ Registry::register_posting_source(const Xapian::PostingSource &source)
 }
 
 const Xapian::PostingSource *
-Registry::get_posting_source(const string & name) const
+Registry::get_posting_source(string_view name) const
 {
     LOGCALL(API, const Xapian::PostingSource *, "Xapian::Registry::get_posting_source", name);
     RETURN(lookup_object(internal->postingsources, name));
@@ -297,7 +342,7 @@ Registry::register_match_spy(const Xapian::MatchSpy &spy)
 }
 
 const Xapian::MatchSpy *
-Registry::get_match_spy(const string & name) const
+Registry::get_match_spy(string_view name) const
 {
     LOGCALL(API, const Xapian::MatchSpy *, "Xapian::Registry::get_match_spy", name);
     RETURN(lookup_object(internal->matchspies, name));
@@ -311,10 +356,24 @@ Registry::register_lat_long_metric(const Xapian::LatLongMetric &metric)
 }
 
 const Xapian::LatLongMetric *
-Registry::get_lat_long_metric(const string & name) const
+Registry::get_lat_long_metric(string_view name) const
 {
     LOGCALL(API, const Xapian::LatLongMetric *, "Xapian::Registry::get_lat_long_metric", name);
     RETURN(lookup_object(internal->lat_long_metrics, name));
 }
 
+void
+Registry::register_key_maker(Xapian::KeyMaker* keymaker)
+{
+    LOGCALL_VOID(API, "Xapian::Registry::register_key_maker", keymaker->name());
+    register_object(internal->key_makers, keymaker);
+}
+
+const Xapian::KeyMaker*
+Registry::get_key_maker(std::string_view name) const
+{
+    LOGCALL(API, const Xapian::KeyMaker*, "Xapian::Registry::get_key_maker", name);
+    RETURN(lookup_object(internal->key_makers, name));
+}
+
 }
