@@ -1,4 +1,4 @@
-/** @file termlist.cc
+/** @file
  * @brief Abstract base class for termlists.
  */
 /* Copyright (C) 2007,2010,2013 Olly Betts
@@ -37,9 +37,9 @@ TermIterator::Internal::accumulate_stats(Xapian::Internal::ExpandStats &) const
     Assert(false);
 }
 
-// Default implementation for when the positions aren't in vector<termpos>.
-const std::vector<Xapian::termpos> *
-TermIterator::Internal::get_vector_termpos() const
+// Default implementation for when the positions aren't in VecCOW<termpos>.
+const Xapian::VecCOW<Xapian::termpos> *
+TermIterator::Internal::get_vec_termpos() const
 {
     return NULL;
 }
