@@ -1,5 +1,5 @@
 /** @file
- * @brief Standard ValueRangeProcessor and RangeProcessor subclasses
+ * @brief Standard RangeProcessor subclass implementations
  */
 /* Copyright (C) 2007-2026 Olly Betts
  *
@@ -23,7 +23,11 @@
 #include <xapian/queryparser.h>
 
 #include <cerrno>
-#include <cstdlib> // For atoi().
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+# include <charconv>
+#else
+# include <cstdlib> // For strtod().
+#endif
 
 #include <string>
 #include "stringutils.h"
@@ -32,49 +36,6 @@ using namespace std;
 
 namespace Xapian {
 
-Xapian::valueno
-StringValueRangeProcessor::operator()(string &begin, string &end)
-{
-    if (str.size()) {
-	if (prefix && begin.empty()) {
-	    // Handle empty start and prefix on end, e.g.: ..$20
-	    if (!startswith(end, str)) {
-		return Xapian::BAD_VALUENO;
-	    }
-	    end.erase(0, str.size());
-	} else if (!prefix && end.empty()) {
-	    // Handle empty end and suffix on start, e.g.: 20kg..
-	    if (!endswith(begin, str)) {
-		return Xapian::BAD_VALUENO;
-	    }
-	    begin.resize(begin.size() - str.size());
-	} else if (prefix) {
-	    // If there's a prefix, require it on the start of the range.
-	    if (!startswith(begin, str)) {
-		// Prefix not given.
-		return Xapian::BAD_VALUENO;
-	    }
-	    begin.erase(0, str.size());
-	    // But it's optional on the end of the range, e.g. $10..50
-	    if (startswith(end, str)) {
-		end.erase(0, str.size());
-	    }
-	} else {
-	    // If there's a suffix, require it on the end of the range.
-	    if (!endswith(end, str)) {
-		// Suffix not given.
-		return Xapian::BAD_VALUENO;
-	    }
-	    end.resize(end.size() - str.size());
-	    // But it's optional on the start of the range, e.g. 10..50kg
-	    if (endswith(begin, str)) {
-		begin.resize(begin.size() - str.size());
-	    }
-	}
-    }
-    return valno;
-}
-
 static bool
 decode_xxy(const string & s, int & x1, int &x2, int &y)
 {
@@ -83,22 +44,28 @@ decode_xxy(const string & s, int & x1, int &x2, int &y)
 	return true;
     }
     if (s.size() < 5 || s.size() > 10) return false;
-    size_t i = s.find_first_not_of("0123456789");
-    if (i < 1 || i > 2 || !(s[i] == '/' || s[i] == '-' || s[i] == '.'))
-	return false;
-    size_t j = s.find_first_not_of("0123456789", i + 1);
-    if (j - (i + 1) < 1 || j - (i + 1) > 2 ||
-	!(s[j] == '/' || s[j] == '-' || s[j] == '.'))
-	return false;
-    if (s.size() - j > 4 + 1) return false;
-    if (s.find_first_not_of("0123456789", j + 1) != string::npos)
-	return false;
-    x1 = atoi(s.c_str());
+    const char* p = s.c_str();
+    if (!C_isdigit(*p)) return false;
+    x1 = *p++ - '0';
+    if (C_isdigit(*p)) {
+	x1 = x1 * 10 + (*p++ - '0');
+    }
     if (x1 < 1 || x1 > 31) return false;
-    x2 = atoi(s.c_str() + i + 1);
+    char sep = *p++;
+    if (sep != '/' && sep != '-' && sep != '.') return false;
+    if (!C_isdigit(*p)) return false;
+    x2 = *p++ - '0';
+    if (C_isdigit(*p)) {
+	x2 = x2 * 10 + (*p++ - '0');
+    }
     if (x2 < 1 || x2 > 31) return false;
-    y = atoi(s.c_str() + j + 1);
-    return true;
+    if (*p++ != sep) return false;
+    if (s.size() - (p - s.c_str()) > 4) return false;
+    y = *p++ - '0';
+    while (C_isdigit(*p)) {
+	y = y * 10 + (*p++ - '0');
+    }
+    return size_t(p - s.c_str()) == s.size();
 }
 
 // We just use this to decide if an ambiguous aa/bb/cc date could be a
@@ -152,123 +119,6 @@ format_yyyymmdd(char * p, int y, int m, int d)
     format_int_fixed_width(p + 6, d, 2);
 }
 
-Xapian::valueno
-DateValueRangeProcessor::operator()(string &begin, string &end)
-{
-    if (StringValueRangeProcessor::operator()(begin, end) == BAD_VALUENO)
-	return BAD_VALUENO;
-
-    if ((begin.size() == 8 || begin.size() == 0) &&
-	(end.size() == 8 || end.size() == 0) &&
-	begin.find_first_not_of("0123456789") == string::npos &&
-	end.find_first_not_of("0123456789") == string::npos) {
-	// YYYYMMDD
-	return valno;
-    }
-    if ((begin.size() == 10 || begin.size() == 0) &&
-	(end.size() == 10 || end.size() == 0)) {
-	if ((begin.empty() || is_yyyy_mm_dd(begin)) &&
-	    (end.empty() || is_yyyy_mm_dd(end))) {
-	    // YYYY-MM-DD
-	    if (!begin.empty()) {
-		begin.erase(7, 1);
-		begin.erase(4, 1);
-	    }
-	    if (!end.empty()) {
-		end.erase(7, 1);
-		end.erase(4, 1);
-	    }
-	    return valno;
-	}
-    }
-
-    int b_d, b_m, b_y;
-    int e_d, e_m, e_y;
-    if (!decode_xxy(begin, b_d, b_m, b_y) || !decode_xxy(end, e_d, e_m, e_y))
-	return Xapian::BAD_VALUENO;
-
-    // Check that the month and day are within range.  Also assume "start" <=
-    // "end" to help decide ambiguous cases.
-    if (!prefer_mdy && vet_dm(b_d, b_m) && vet_dm(e_d, e_m) &&
-	(b_y != e_y || b_m < e_m || (b_m == e_m && b_d <= e_d))) {
-	// OK.
-    } else if (vet_dm(b_m, b_d) && vet_dm(e_m, e_d) &&
-	(b_y != e_y || b_d < e_d || (b_d == e_d && b_m <= e_m))) {
-	swap(b_m, b_d);
-	swap(e_m, e_d);
-    } else if (prefer_mdy && vet_dm(b_d, b_m) && vet_dm(e_d, e_m) &&
-	       (b_y != e_y || b_m < e_m || (b_m == e_m && b_d <= e_d))) {
-	// OK.
-    } else {
-	return Xapian::BAD_VALUENO;
-    }
-
-    char buf[8];
-    if (!begin.empty()) {
-	if (b_y < 100) {
-	    b_y += 1900;
-	    if (b_y < epoch_year) b_y += 100;
-	}
-	format_yyyymmdd(buf, b_y, b_m, b_d);
-	begin.assign(buf, 8);
-    }
-    if (!end.empty()) {
-	if (e_y < 100) {
-	    e_y += 1900;
-	    if (e_y < epoch_year) e_y += 100;
-	}
-	format_yyyymmdd(buf, e_y, e_m, e_d);
-	end.assign(buf, 8);
-    }
-    return valno;
-}
-
-Xapian::valueno
-NumberValueRangeProcessor::operator()(string &begin, string &end)
-{
-    if (StringValueRangeProcessor::operator()(begin, end) == BAD_VALUENO)
-	return BAD_VALUENO;
-
-    // Parse the numbers to floating point.
-    double beginnum;
-
-    if (!begin.empty()) {
-	errno = 0;
-	const char * startptr = begin.c_str();
-	char * endptr;
-	beginnum = strtod(startptr, &endptr);
-	if (endptr != startptr + begin.size())
-	    // Invalid characters in string
-	    return Xapian::BAD_VALUENO;
-	if (errno)
-	    // Overflow or underflow
-	    return Xapian::BAD_VALUENO;
-    } else {
-	// Silence GCC warning.
-	beginnum = 0.0;
-    }
-
-    if (!end.empty()) {
-	errno = 0;
-	const char * startptr = end.c_str();
-	char * endptr;
-	double endnum = strtod(startptr, &endptr);
-	if (endptr != startptr + end.size())
-	    // Invalid characters in string
-	    return Xapian::BAD_VALUENO;
-	if (errno)
-	    // Overflow or underflow
-	    return Xapian::BAD_VALUENO;
-	end.assign(Xapian::sortable_serialise(endnum));
-    }
-
-    if (!begin.empty()) {
-	begin.assign(Xapian::sortable_serialise(beginnum));
-    }
-
-    return valno;
-}
-
 Xapian::Query
 RangeProcessor::check_range(const string& b, const string& e)
 {
@@ -415,6 +265,15 @@ NumberRangeProcessor::operator()(const string& b, const string& e)
     double num_b, num_e;
 
     if (!b.empty()) {
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+	const char* startptr = b.data();
+	const char* endptr = startptr + b.size();
+	const auto& r = from_chars(startptr, endptr, num_b);
+	if (r.ec != std::errc() || r.ptr != endptr) {
+	    // Invalid characters in string || overflow or underflow.
+	    goto not_our_range;
+	}
+#else
 	errno = 0;
 	const char * startptr = b.c_str();
 	char * endptr;
@@ -423,12 +282,22 @@ NumberRangeProcessor::operator()(const string& b, const string& e)
 	    // Invalid characters in string || overflow or underflow.
 	    goto not_our_range;
 	}
+#endif
     } else {
 	// Silence GCC warning.
 	num_b = 0.0;
     }
 
     if (!e.empty()) {
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+	const char* startptr = e.data();
+	const char* endptr = startptr + e.size();
+	const auto& r = from_chars(startptr, endptr, num_e);
+	if (r.ec != std::errc() || r.ptr != endptr) {
+	    // Invalid characters in string || overflow or underflow.
+	    goto not_our_range;
+	}
+#else
 	errno = 0;
 	const char * startptr = e.c_str();
 	char * endptr;
@@ -437,6 +306,7 @@ NumberRangeProcessor::operator()(const string& b, const string& e)
 	    // Invalid characters in string || overflow or underflow.
 	    goto not_our_range;
 	}
+#endif
     } else {
 	// Silence GCC warning.
 	num_e = 0.0;
@@ -450,4 +320,129 @@ not_our_range:
     return Xapian::Query(Xapian::Query::OP_INVALID);
 }
 
+static const char byte_units[4][2] = {
+    "B", "K", "M", "G"
+};
+
+// Return factor for byte unit
+// if string is a valid byte unit
+// else return -1
+static double
+check_byte_unit(const string &s) {
+    double factor = 1;
+    for (int i = 0; i < 4; ++i) {
+	if (endswith(s, byte_units[i])) {
+	    return factor;
+	}
+	factor *= 1024;
+    }
+
+    return -1;
+}
+
+Xapian::Query
+UnitRangeProcessor::operator()(const string& b, const string& e)
+{
+    // Parse the numbers to floating point.
+    double num_b, num_e;
+
+    // True if b has unit, e.g. 20K..
+    bool b_has_unit = false;
+
+    if (!b.empty()) {
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+	const char* startptr = b.data();
+	const char* endptr = startptr + b.size();
+	const auto& r = from_chars(startptr, endptr, num_b);
+	if (r.ec != std::errc()) {
+	    // Invalid characters in string || overflow or underflow.
+	    goto not_our_range;
+	}
+	endptr = r.ptr;
+#else
+	errno = 0;
+	const char * startptr = b.c_str();
+	char * endptr;
+	num_b = strtod(startptr, &endptr);
+
+	if (errno) {
+	    // overflow or underflow
+	    goto not_our_range;
+	}
+#endif
+
+	// For lower range having a unit, e.g. 100K..
+	if (endptr == startptr + b.size() - 1) {
+	    double factor_b = check_byte_unit(b);
+	    if (factor_b == -1) {
+		// Not a valid byte unit
+		goto not_our_range;
+	    }
+	    b_has_unit = true;
+	    num_b *= factor_b;
+	}
+    } else {
+	// Silence GCC warning.
+	num_b = 0.0;
+    }
+
+    if (!e.empty()) {
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+	const char* startptr = e.data();
+	const char* endptr = startptr + e.size();
+	const auto& r = from_chars(startptr, endptr, num_e);
+	if (r.ec != std::errc()) {
+	    // Invalid characters in string || overflow or underflow.
+	    goto not_our_range;
+	}
+	endptr = r.ptr;
+#else
+	errno = 0;
+	const char * startptr = e.c_str();
+	char * endptr;
+	num_e = strtod(startptr, &endptr);
+
+	if (errno) {
+	    // overflow or underflow
+	    goto not_our_range;
+	}
+#endif
+
+	// For upper range having a unit, e.g. ..100K
+	if (endptr == startptr + e.size() - 1) {
+	    double factor_e = check_byte_unit(e);
+	    if (factor_e == -1) {
+		// Not a valid byte unit
+		goto not_our_range;
+	    }
+	    num_e *= factor_e;
+
+	    // When lower range is not empty and
+	    // only upper range unit, e.g. 20..100K
+	    if (!b.empty() && !b_has_unit) {
+		num_b *= factor_e;
+	    }
+	} else {
+	    // When lower range has no unit
+	    goto not_our_range;
+	}
+    } else {
+	// Silence GCC warning.
+	num_e = 0.0;
+
+	// Fail case when lower range
+	// has no unit, e.g. 200..
+	if (!b.empty() && !b_has_unit) {
+	    goto not_our_range;
+	}
+    }
+
+    return RangeProcessor::operator()(
+	    b.empty() ? b : Xapian::sortable_serialise(num_b),
+	    e.empty() ? e : Xapian::sortable_serialise(num_e));
+
+not_our_range:
+    return Xapian::Query(Xapian::Query::OP_INVALID);
+}
+
 }
