@@ -1,7 +1,7 @@
 /** @file
  * @brief A vector-like container of terms which can be iterated.
  */
-/* Copyright (C) 2011,2012,2017 Olly Betts
+/* Copyright (C) 2011,2012,2017,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,7 @@
 
 #include "xapian/types.h"
 
-#include "net/length.h"
+#include "pack.h"
 #include "termlist.h"
 
 /** This class stores a list of terms.
@@ -45,9 +45,6 @@ class VectorTermList : public TermList {
     /// The number of terms in the list.
     Xapian::termcount num_terms;
 
-    /// The current term.
-    std::string current_term;
-
   public:
     template<typename I>
     VectorTermList(I begin, I end) : num_terms(0)
@@ -58,7 +55,7 @@ class VectorTermList : public TermList {
 	    ++num_terms;
 	    const std::string & s = *i;
 	    total_size += s.size() + 1;
-	    if (s.size() >= 255) {
+	    if (s.size() >= 128) {
 		// Not a common case, so just assume the worst case rather than
 		// trying to carefully calculate the exact size.
 		total_size += 5;
@@ -68,9 +65,7 @@ class VectorTermList : public TermList {
 
 	// Now encode all the terms into data.
 	for (I i = begin; i != end; ++i) {
-	    const std::string & s = *i;
-	    data += encode_length(s.size());
-	    data += s;
+	    pack_string(data, *i);
 	}
 
 	p = data.data();
@@ -78,21 +73,17 @@ class VectorTermList : public TermList {
 
     Xapian::termcount get_approx_size() const;
 
-    std::string get_termname() const;
-
     Xapian::termcount get_wdf() const;
 
     Xapian::doccount get_termfreq() const;
 
     TermList * next();
 
-    TermList * skip_to(const std::string &);
-
-    bool at_end() const;
+    TermList* skip_to(std::string_view);
 
     Xapian::termcount positionlist_count() const;
 
-    Xapian::PositionIterator positionlist_begin() const;
+    PositionList* positionlist_begin() const;
 };
 
 #endif // XAPIAN_INCLUDED_VECTORTERMLIST_H
