@@ -1,7 +1,7 @@
 /** @file
  * @brief BACKEND_* constants
  */
-/* Copyright (C) 2015 Olly Betts
+/* Copyright (C) 2015,2017,2018 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,12 @@
 #define XAPIAN_INCLUDED_BACKENDS_H
 
 enum {
+    BACKEND_OLD = -2,
     BACKEND_UNKNOWN = -1,
     BACKEND_REMOTE = 0,
     BACKEND_INMEMORY = 1,
-    BACKEND_CHERT = 2,
-    BACKEND_GLASS = 3,
+    BACKEND_GLASS = 2,
+    BACKEND_HONEY = 3,
     BACKEND_MAX_
 };
 
@@ -35,8 +36,8 @@ inline const char * backend_name(int code) {
     const char * p =
 	"remote\0\0\0"
 	"inmemory\0"
-	"chert\0\0\0\0"
 	"glass\0\0\0\0"
+	"honey\0\0\0\0"
 	"?";
     return p + code * 9;
 }
