@@ -1,7 +1,7 @@
 /** @file
  * @brief Iterate all document ids when they form a contiguous range.
  */
-/* Copyright (C) 2007,2008,2009,2011 Olly Betts
+/* Copyright (C) 2007,2008,2009,2011,2017 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,45 +27,19 @@
 #include "omassert.h"
 #include "str.h"
 
-#include "xapian/error.h"
-
 using namespace std;
 
-Xapian::doccount
-ContiguousAllDocsPostList::get_termfreq() const
-{
-    return doccount;
-}
-
 Xapian::docid
 ContiguousAllDocsPostList::get_docid() const
 {
     Assert(did != 0);
-    Assert(!at_end());
     return did;
 }
 
-Xapian::termcount
-ContiguousAllDocsPostList::get_doclength() const
-{
-    Assert(did != 0);
-    Assert(!at_end());
-    return db->get_doclength(did);
-}
-
-Xapian::termcount
-ContiguousAllDocsPostList::get_unique_terms() const
-{
-    Assert(did != 0);
-    Assert(!at_end());
-    return db->get_unique_terms(did);
-}
-
 Xapian::termcount
 ContiguousAllDocsPostList::get_wdf() const
 {
     Assert(did != 0);
-    Assert(!at_end());
     return 1;
 }
 
@@ -85,9 +59,9 @@ ContiguousAllDocsPostList::open_position_list() const
 PostList *
 ContiguousAllDocsPostList::next(double)
 {
-    Assert(!at_end());
-    if (did == doccount) {
-	db = NULL;
+    // Docids are contiguous from 1 so termfreq gives the highest docid.
+    if (did == termfreq) {
+	did = 0;
     } else {
 	++did;
     }
@@ -97,10 +71,10 @@ ContiguousAllDocsPostList::next(double)
 PostList *
 ContiguousAllDocsPostList::skip_to(Xapian::docid target, double)
 {
-    Assert(!at_end());
     if (target > did) {
-	if (target > doccount) {
-	    db = NULL;
+	// Docids are contiguous from 1 so termfreq gives the highest docid.
+	if (target > termfreq) {
+	    did = 0;
 	} else {
 	    did = target;
 	}
@@ -111,7 +85,7 @@ ContiguousAllDocsPostList::skip_to(Xapian::docid target, double)
 bool
 ContiguousAllDocsPostList::at_end() const
 {
-    return db.get() == NULL;
+    return did == 0;
 }
 
 Xapian::termcount
@@ -124,7 +98,7 @@ string
 ContiguousAllDocsPostList::get_description() const
 {
     string msg("ContiguousAllDocsPostList(1..");
-    msg += str(doccount);
+    msg += str(termfreq);
     msg += ')';
     return msg;
 }
