@@ -1,7 +1,7 @@
 /** @file
  * @brief Helper functions for database handling
  */
-/* Copyright 2002-2019 Olly Betts
+/* Copyright 2002-2020 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -27,6 +27,7 @@
 #include "backends.h"
 
 #include "glass/glass_defs.h"
+#include "honey/honey_defs.h"
 
 #include "io_utils.h"
 #include "omassert.h"
@@ -41,30 +42,74 @@
 
 using namespace std;
 
+static int
+test_if_single_file_db_(int fd, off_t pos)
+{
+#if defined XAPIAN_HAS_GLASS_BACKEND || \
+    defined XAPIAN_HAS_HONEY_BACKEND
+    char magic_buf[14];
+    // FIXME: Don't duplicate magic check here...
+    if (io_read(fd, magic_buf, 14) == 14 &&
+	lseek(fd, pos, SEEK_SET) == pos &&
+	memcmp(magic_buf, "\x0f\x0dXapian ", 9) == 0) {
+	switch (magic_buf[9]) {
+#ifdef XAPIAN_HAS_GLASS_BACKEND
+	    case 'G':
+		if (memcmp(magic_buf + 10, "lass", 4) == 0) {
+		    return BACKEND_GLASS;
+		}
+		break;
+#endif
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+	    case 'H':
+		if (memcmp(magic_buf + 10, "oney", 4) == 0) {
+		    return BACKEND_HONEY;
+		}
+		break;
+#endif
+	}
+	return BACKEND_UNKNOWN;
+    }
+#else
+    (void)fd;
+#endif
+    return BACKEND_UNKNOWN;
+}
+
+int
+test_if_single_file_db(int fd)
+{
+    off_t pos = lseek(fd, 0, SEEK_CUR);
+    if (pos < 0) {
+	return BACKEND_UNKNOWN;
+    }
+    return test_if_single_file_db_(fd, pos);
+}
+
 int
 test_if_single_file_db(const struct stat& sb,
 		       const string& path,
 		       int* fd_ptr)
 {
     Assert(fd_ptr);
-#if defined XAPIAN_HAS_GLASS_BACKEND
+#if defined XAPIAN_HAS_GLASS_BACKEND || \
+    defined XAPIAN_HAS_HONEY_BACKEND
     if (!S_ISREG(sb.st_mode)) return BACKEND_UNKNOWN;
-    // Look at the size as a clue - if it's less than GLASS_MIN_BLOCKSIZE then
-    // it's not a single-file glass database.  For a larger file, we peek at
-    // the start of the file to determine what it is.
-    if (sb.st_size < GLASS_MIN_BLOCKSIZE)
+    // Look at the size as a clue - if it's less than both GLASS_MIN_BLOCKSIZE
+    // and HONEY_MIN_DB_SIZE then it's not a single-file glass or honey
+    // database.  For a larger file, we peek at the start of the file to
+    // determine what it is.
+    if (sb.st_size < min(GLASS_MIN_BLOCKSIZE, HONEY_MIN_DB_SIZE))
 	return BACKEND_UNKNOWN;
     int fd = posixy_open(path.c_str(), O_RDONLY|O_BINARY);
     if (fd != -1) {
-	char magic_buf[14];
-	// FIXME: Don't duplicate magic check here...
-	if (io_read(fd, magic_buf, 14) == 14 &&
-	    lseek(fd, 0, SEEK_SET) == 0 &&
-	    memcmp(magic_buf, "\x0f\x0dXapian Glass", 14) == 0) {
+	int result = test_if_single_file_db_(fd, off_t{0});
+	if (result != BACKEND_UNKNOWN) {
 	    *fd_ptr = fd;
-	    return BACKEND_GLASS;
+	} else {
+	    ::close(fd);
 	}
-	::close(fd);
+	return result;
     }
 #else
     (void)sb;
