@@ -33,9 +33,6 @@
 #if defined XAPIAN_HAS_GLASS_BACKEND && defined XAPIAN_HAS_REMOTE_BACKEND
 # include "glass/glass_databasereplicator.h"
 #endif
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-# include "chert/chert_databasereplicator.h"
-#endif
 
 using namespace std;
 
@@ -50,16 +47,6 @@ DatabaseReplicator::open(const string & path)
 {
     LOGCALL_STATIC(DB, DatabaseReplicator *, "DatabaseReplicator::open", path);
 
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-    if (file_exists(path + "/iamchert")) {
-# ifdef XAPIAN_HAS_REMOTE_BACKEND
-	return new ChertDatabaseReplicator(path);
-# else
-	throw FeatureUnavailableError("Replication disabled");
-# endif
-    }
-#endif
-
 #ifdef XAPIAN_HAS_GLASS_BACKEND
     if (file_exists(path + "/iamglass")) {
 # ifdef XAPIAN_HAS_REMOTE_BACKEND
@@ -70,6 +57,12 @@ DatabaseReplicator::open(const string & path)
     }
 #endif
 
+    // FIXME: Replication of honey databases.
+
+    if (file_exists(path + "/iamchert")) {
+	throw FeatureUnavailableError("Chert backend no longer supported");
+    }
+
     if (file_exists(path + "/iamflint")) {
 	throw FeatureUnavailableError("Flint backend no longer supported");
     }
