@@ -1,8 +1,7 @@
 /** @file
  * @brief Check the consistency of a database or table.
  */
-/* Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2019 Olly Betts
+/* Copyright 2007-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,16 +25,19 @@
 #include "xapian/constants.h"
 #include "xapian/error.h"
 
+// We always need GLASS_TABLE_EXTENSION.
+#include "glass/glass_defs.h"
 #ifdef XAPIAN_HAS_GLASS_BACKEND
 #include "glass/glass_changes.h"
 #include "glass/glass_dbcheck.h"
 #include "glass/glass_version.h"
 #endif
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-#include "chert/chert_database.h"
-#include "chert/chert_dbcheck.h"
-#include "chert/chert_types.h"
-#include "chert/chert_version.h"
+
+// We always need HONEY_TABLE_EXTENSION.
+#include "honey/honey_defs.h"
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+#include "honey/honey_dbcheck.h"
+#include "honey/honey_version.h"
 #endif
 
 #include "backends.h"
@@ -46,6 +48,7 @@
 
 #include <ostream>
 #include <stdexcept>
+#include <string_view>
 
 using namespace std;
 
@@ -62,6 +65,19 @@ static const struct { char name[9]; } glass_tables[] = {
 };
 #endif
 
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+// Tables to check for a honey database.  Note: it's important to check
+// termlist before postlist so that we can cross-check the document lengths.
+static const struct { char name[9]; } honey_tables[] = {
+    { "docdata" },
+    { "termlist" },
+    { "postlist" },
+    { "position" },
+    { "spelling" },
+    { "synonym" }
+};
+#endif
+
 [[noreturn]]
 static void
 throw_no_db_to_check()
@@ -74,7 +90,7 @@ throw_no_db_to_check()
 // It's hard to see how to efficiently.  We do cross-check doclens, but that
 // "only" requires (4 * last_docid()) bytes.
 
-#if defined XAPIAN_HAS_CHERT_BACKEND || defined XAPIAN_HAS_GLASS_BACKEND
+#if defined XAPIAN_HAS_GLASS_BACKEND
 static void
 reserve_doclens(vector<Xapian::termcount>& doclens, Xapian::docid last_docid,
 		ostream * out)
@@ -106,98 +122,12 @@ reserve_doclens(vector<Xapian::termcount>& doclens, Xapian::docid last_docid,
 #endif
 
 static size_t
-check_db_dir(const string & path, int opts, std::ostream *out)
+check_db_dir(string_view path, int opts, std::ostream *out)
 {
     struct stat sb;
-    if (stat((path + "/iamchert").c_str(), &sb) == 0) {
-#ifndef XAPIAN_HAS_CHERT_BACKEND
-	(void)opts;
-	(void)out;
-	throw Xapian::FeatureUnavailableError("Chert database support isn't enabled");
-#else
-	// Check a whole chert database directory.
-	vector<Xapian::termcount> doclens;
-	size_t errors = 0;
-
-	// If we can't read the doccount or last docid, set them to their
-	// maximum values to suppress errors.
-	Xapian::doccount doccount = Xapian::doccount(-1);
-	Xapian::docid db_last_docid = CHERT_MAX_DOCID;
-
-	chert_revision_number_t rev = 0;
-	chert_revision_number_t * rev_ptr = &rev;
-	try {
-	    // Open at the lower level so we can get the revision number.
-	    ChertDatabase db(path);
-	    doccount = db.get_doccount();
-	    db_last_docid = db.get_lastdocid();
-	    reserve_doclens(doclens, db_last_docid, out);
-	    rev = db.get_revision_number();
-	} catch (const Xapian::Error & e) {
-	    // Ignore so we can check a database too broken to open.
-	    if (out)
-		*out << "Database couldn't be opened for reading: "
-		     << e.get_description()
-		     << "\nContinuing check anyway" << endl;
-	    ++errors;
-	}
-
-	size_t pre_table_check_errors = errors;
-
-	// Check all the btrees.
-	//
-	// Note: it's important to check "termlist" before "postlist" so
-	// that we can cross-check the document lengths; also we check
-	// "record" first as that's the last committed, so has the most
-	// reliable rootblock revision in DBCHECK_FIX mode.
-	static const struct { char name[9]; } tables[] = {
-	    { "record" },
-	    { "termlist" },
-	    { "postlist" },
-	    { "position" },
-	    { "spelling" },
-	    { "synonym" }
-	};
-	for (auto t : tables) {
-	    const char * name = t.name;
-	    if (out)
-		*out << name << ":\n";
-	    if (strcmp(name, "record") != 0 && strcmp(name, "postlist") != 0) {
-		// Other tables are created lazily, so may not exist.
-		string table(path);
-		table += '/';
-		table += name;
-		table += ".DB";
-		if (!file_exists(table)) {
-		    if (out) {
-			if (strcmp(name, "termlist") == 0) {
-			    *out << "Not present.\n";
-			} else {
-			    *out << "Lazily created, and not yet used.\n";
-			}
-			*out << endl;
-		    }
-		    continue;
-		}
-	    }
-	    errors += check_chert_table(name, path, rev_ptr, opts, doclens,
-					doccount, db_last_docid, out);
-	}
-
-	if (errors == pre_table_check_errors && (opts & Xapian::DBCHECK_FIX)) {
-	    // Check the version file is OK and if not, recreate it.
-	    ChertVersion iam(path);
-	    try {
-		iam.read_and_check();
-	    } catch (const Xapian::DatabaseError &) {
-		iam.create();
-	    }
-	}
-	return errors;
-#endif
-    }
-
-    if (stat((path + "/iamglass").c_str(), &sb) == 0) {
+    string filename{path};
+    filename += "/iamglass";
+    if (stat(filename.c_str(), &sb) == 0) {
 #ifndef XAPIAN_HAS_GLASS_BACKEND
 	(void)opts;
 	(void)out;
@@ -210,7 +140,7 @@ check_db_dir(const string & path, int opts, std::ostream *out)
 	try {
 	    // Check if the database can actually be opened.
 	    Xapian::Database db(path);
-	} catch (const Xapian::Error & e) {
+	} catch (const Xapian::Error& e) {
 	    // Continue - we can still usefully look at how it is broken.
 	    if (out)
 		*out << "Database couldn't be opened for reading: "
@@ -221,12 +151,12 @@ check_db_dir(const string & path, int opts, std::ostream *out)
 
 	GlassVersion version_file(path);
 	version_file.read();
-	for (glass_revision_number_t r = version_file.get_revision(); r != 0; --r) {
-	    string changes_file = path;
-	    changes_file += "/changes";
-	    changes_file += str(r);
-	    if (file_exists(changes_file))
-		GlassChanges::check(changes_file);
+	for (auto r = version_file.get_revision(); r != 0; --r) {
+	    filename.resize(path.size());
+	    filename += "/changes";
+	    filename += str(r);
+	    if (file_exists(filename))
+		GlassChanges::check(filename);
 	}
 
 	Xapian::docid doccount = version_file.get_doccount();
@@ -248,17 +178,86 @@ check_db_dir(const string & path, int opts, std::ostream *out)
 #endif
     }
 
-    if (stat((path + "/iamflint").c_str(), &sb) == 0) {
+    filename.resize(path.size());
+    filename += "/iamhoney";
+    if (stat(filename.c_str(), &sb) == 0) {
+#ifndef XAPIAN_HAS_HONEY_BACKEND
+	(void)opts;
+	(void)out;
+	auto msg = "Honey database support isn't enabled";
+	throw Xapian::FeatureUnavailableError(msg);
+#else
+	// Check a whole honey database directory.
+	vector<Xapian::termcount> doclens;
+	size_t errors = 0;
+
+	try {
+	    // Check if the database can actually be opened.
+	    Xapian::Database db(path);
+	} catch (const Xapian::Error& e) {
+	    // Continue - we can still usefully look at how it is broken.
+	    if (out)
+		*out << "Database couldn't be opened for reading: "
+		     << e.get_description()
+		     << "\nContinuing check anyway" << endl;
+	    ++errors;
+	}
+
+	HoneyVersion version_file(path);
+	version_file.read();
+#if 0 // FIXME: Honey replication not yet implemented.
+	for (auto r = version_file.get_revision(); r != 0; --r) {
+	    string changes_file = path;
+	    changes_file += "/changes";
+	    changes_file += str(r);
+	    if (file_exists(changes_file))
+		HoneyChanges::check(changes_file);
+	}
+#endif
+
+	Xapian::docid doccount = version_file.get_doccount();
+	Xapian::docid db_last_docid = version_file.get_last_docid();
+	if (db_last_docid < doccount) {
+	    if (out)
+		*out << "last_docid = " << db_last_docid << " < doccount = "
+		     << doccount << endl;
+	    ++errors;
+	}
+	reserve_doclens(doclens, db_last_docid, out);
+
+	// Check all the tables.
+	for (auto t : honey_tables) {
+	    errors += check_honey_table(t.name, path, version_file, opts,
+					doclens, out);
+	}
+	return errors;
+#endif
+    }
+
+    filename.resize(path.size());
+    filename += "/iamchert";
+    if (stat(filename.c_str(), &sb) == 0) {
+	// Chert is no longer supported as of Xapian 2.0.0.
+	throw Xapian::FeatureUnavailableError("Chert database support was removed in Xapian 2.0.0");
+    }
+
+    filename.resize(path.size());
+    filename += "/iamflint";
+    if (stat(filename.c_str(), &sb) == 0) {
 	// Flint is no longer supported as of Xapian 1.3.0.
 	throw Xapian::FeatureUnavailableError("Flint database support was removed in Xapian 1.3.0");
     }
 
-    if (stat((path + "/iambrass").c_str(), &sb) == 0) {
+    filename.resize(path.size());
+    filename += "/iambrass";
+    if (stat(filename.c_str(), &sb) == 0) {
 	// Brass was renamed to glass as of Xapian 1.3.2.
 	throw Xapian::FeatureUnavailableError("Brass database support was removed in Xapian 1.3.2");
     }
 
-    if (stat((path + "/record_DB").c_str(), &sb) == 0) {
+    filename.resize(path.size());
+    filename += "/record_DB";
+    if (stat(filename.c_str(), &sb) == 0) {
 	// Quartz is no longer supported as of Xapian 1.1.0.
 	throw Xapian::FeatureUnavailableError("Quartz database support was removed in Xapian 1.1.0");
     }
@@ -275,7 +274,7 @@ check_db_dir(const string & path, int opts, std::ostream *out)
  *  @param backend	Backend type (a BACKEND_XXX constant)
  */
 static size_t
-check_db_table(const string& filename, int opts, std::ostream* out, int backend)
+check_db_table(string_view filename, int opts, std::ostream* out, int backend)
 {
     size_t p = filename.find_last_of(DIR_SEPS);
     // If we found a directory separator, advance p to the next character.  If
@@ -292,7 +291,7 @@ check_db_table(const string& filename, int opts, std::ostream* out, int backend)
 	tablename += C_tolower(ch);
     }
 
-#if defined XAPIAN_HAS_CHERT_BACKEND || defined XAPIAN_HAS_GLASS_BACKEND
+#if defined XAPIAN_HAS_GLASS_BACKEND
     vector<Xapian::termcount> doclens;
 #else
     (void)opts;
@@ -312,7 +311,19 @@ check_db_table(const string& filename, int opts, std::ostream* out, int backend)
 #endif
       }
 
-      case BACKEND_CHERT:
+      case BACKEND_HONEY: {
+#ifndef XAPIAN_HAS_HONEY_BACKEND
+	auto msg = "Honey database support isn't enabled";
+	throw Xapian::FeatureUnavailableError(msg);
+#else
+	HoneyVersion version_file(dir);
+	version_file.read();
+	return check_honey_table(tablename.c_str(), dir, version_file, opts,
+				 doclens, out);
+#endif
+      }
+
+      case BACKEND_OLD:
 	break;
 
       default:
@@ -320,9 +331,13 @@ check_db_table(const string& filename, int opts, std::ostream* out, int backend)
 	break;
     }
 
-    // Flint and brass also used the extension ".DB", so check that we
-    // haven't been passed a single table in a flint or brass database.
+    // Chert, flint and brass all used the extension ".DB", so check which
+    // to give an appropriate error.
     struct stat sb;
+    if (stat((dir + "/iamchert").c_str(), &sb) == 0) {
+	// Chert is no longer supported as of Xapian 2.0.0.
+	throw Xapian::FeatureUnavailableError("Chert database support was removed in Xapian 2.0.0");
+    }
     if (stat((dir + "/iamflint").c_str(), &sb) == 0) {
 	// Flint is no longer supported as of Xapian 1.3.0.
 	throw Xapian::FeatureUnavailableError("Flint database support was removed in Xapian 1.3.0");
@@ -331,14 +346,8 @@ check_db_table(const string& filename, int opts, std::ostream* out, int backend)
 	// Brass was renamed to glass as of Xapian 1.3.2.
 	throw Xapian::FeatureUnavailableError("Brass database support was removed in Xapian 1.3.2");
     }
-#ifndef XAPIAN_HAS_CHERT_BACKEND
-    throw Xapian::FeatureUnavailableError("Chert database support isn't enabled");
-#else
-    // Set the doccount and the last docid to their maximum values to suppress
-    // errors.
-    return check_chert_table(tablename.c_str(), dir, NULL, opts, doclens,
-			     Xapian::doccount(-1), CHERT_MAX_DOCID, out);
-#endif
+    // Unaccompanied .DB file.
+    throw Xapian::FeatureUnavailableError("Flint, chert and brass database support have all been removed");
 }
 
 /** Check a single file DB from an fd.
@@ -387,6 +396,18 @@ check_db_fd(int fd, int opts, std::ostream* out, int backend)
 	throw Xapian::FeatureUnavailableError("Glass database support isn't enabled");
 #endif
       }
+      case BACKEND_HONEY:
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+	(void)opts;
+	(void)out;
+	::close(fd);
+	throw Xapian::UnimplementedError("Honey database checking not implemented");
+#else
+	(void)opts;
+	(void)out;
+	::close(fd);
+	throw Xapian::FeatureUnavailableError("Honey database support isn't enabled");
+#endif
       default:
 	Assert(false);
     }
@@ -400,22 +421,22 @@ check_stub(const string& stub_path, int opts, std::ostream* out)
 {
     size_t errors = 0;
     read_stub_file(stub_path,
-		   [&errors, opts, out](const string& path) {
+		   [&errors, opts, out](string_view path) {
 		       errors += Database::check(path, opts, out);
 		   },
-		   [&errors, opts, out](const string& path) {
-		       // FIXME: Doesn't check the database type is chert.
-		       errors += Database::check(path, opts, out);
-		   },
-		   [&errors, opts, out](const string& path) {
+		   [&errors, opts, out](string_view path) {
 		       // FIXME: Doesn't check the database type is glass.
 		       errors += Database::check(path, opts, out);
 		   },
-		   [](const string&, const string&) {
+		   [&errors, opts, out](string_view path) {
+		       // FIXME: Doesn't check the database type is honey.
+		       errors += Database::check(path, opts, out);
+		   },
+		   [](string_view, string_view) {
 		       auto msg = "Remote database checking not implemented";
 		       throw Xapian::UnimplementedError(msg);
 		   },
-		   [](const string&, unsigned) {
+		   [](string_view, unsigned) {
 		       auto msg = "Remote database checking not implemented";
 		       throw Xapian::UnimplementedError(msg);
 		   },
@@ -427,7 +448,10 @@ check_stub(const string& stub_path, int opts, std::ostream* out)
 }
 
 size_t
-Database::check_(const string * path_ptr, int fd, int opts, std::ostream *out)
+Database::check_(const string_view* path_ptr,
+		 int fd,
+		 int opts,
+		 std::ostream *out)
 {
     if (!out) {
 	// If we have nowhere to write output, then disable all the options
@@ -439,31 +463,35 @@ Database::check_(const string * path_ptr, int fd, int opts, std::ostream *out)
 	return check_db_fd(fd, opts, out, BACKEND_UNKNOWN);
     }
 
-    const string & path = *path_ptr;
-    if (path.empty()) throw_no_db_to_check();
+    if (path_ptr->empty()) {
+	throw_no_db_to_check();
+    }
+
+    string filename{*path_ptr};
     struct stat sb;
-    if (stat(path.c_str(), &sb) == 0) {
+    if (stat(filename.c_str(), &sb) == 0) {
 	if (S_ISDIR(sb.st_mode)) {
-	    return check_db_dir(path, opts, out);
+	    return check_db_dir(filename, opts, out);
 	}
 
 	if (S_ISREG(sb.st_mode)) {
-	    int backend = test_if_single_file_db(sb, path, &fd);
+	    int backend = test_if_single_file_db(sb, filename, &fd);
 	    if (backend != BACKEND_UNKNOWN) {
 		return check_db_fd(fd, opts, out, backend);
 	    }
 	    // Could be a single table or a stub database file.  Look at the
 	    // extension to determine the type.
-	    if (endswith(path, ".DB")) {
-		// It could also be flint or brass, but we check for those below.
-		backend = BACKEND_CHERT;
-	    } else if (endswith(path, "." GLASS_TABLE_EXTENSION)) {
+	    if (endswith(filename, ".DB")) {
+		backend = BACKEND_OLD;
+	    } else if (endswith(filename, "." GLASS_TABLE_EXTENSION)) {
 		backend = BACKEND_GLASS;
+	    } else if (endswith(filename, "." HONEY_TABLE_EXTENSION)) {
+		backend = BACKEND_HONEY;
 	    } else {
-		return check_stub(path, opts, out);
+		return check_stub(filename, opts, out);
 	    }
 
-	    return check_db_table(path, opts, out, backend);
+	    return check_db_table(filename, opts, out, backend);
 	}
 
 	throw Xapian::DatabaseOpeningError("Not a regular file or directory");
@@ -473,22 +501,23 @@ Database::check_(const string * path_ptr, int fd, int opts, std::ostream *out)
     // table (perhaps with "." after it), so the user can do xapian-check on
     // "foo/termlist" or "foo/termlist." (which you would get from filename
     // completion with older backends).
-    string filename = path;
     if (endswith(filename, '.')) {
 	filename.resize(filename.size() - 1);
     }
 
     int backend = BACKEND_UNKNOWN;
     if (stat((filename + ".DB").c_str(), &sb) == 0) {
-	// It could also be flint or brass, but we check for those below.
-	backend = BACKEND_CHERT;
+	// Could be chert, flint or brass - we check which below.
+	backend = BACKEND_OLD;
     } else if (stat((filename + "." GLASS_TABLE_EXTENSION).c_str(), &sb) == 0) {
 	backend = BACKEND_GLASS;
+    } else if (stat((filename + "." HONEY_TABLE_EXTENSION).c_str(), &sb) == 0) {
+	backend = BACKEND_HONEY;
     } else {
 	throw_no_db_to_check();
     }
 
-    return check_db_table(path, opts, out, backend);
+    return check_db_table(*path_ptr, opts, out, backend);
 }
 
 }
