@@ -1,7 +1,7 @@
 /** @file
  * @brief Database factories for non-remote databases.
  */
-/* Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011,2012,2013,2014,2015,2016,2017,2019 Olly Betts
+/* Copyright 2002-2024 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -42,15 +42,17 @@
 #ifdef XAPIAN_HAS_GLASS_BACKEND
 # include "glass/glass_database.h"
 #endif
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-# include "chert/chert_database.h"
+#include "glass/glass_defs.h"
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+# include "honey/honey_database.h"
 #endif
+#include "honey/honey_defs.h"
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
 # include "inmemory/inmemory_database.h"
 #endif
 // Even if none of the above get included, we still need a definition of
 // Database::Internal.
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
 
 #include <string>
 
@@ -59,27 +61,27 @@ using namespace std;
 namespace Xapian {
 
 static void
-open_stub(Database& db, const string& file)
+open_stub(Database& db, string_view file)
 {
     read_stub_file(file,
-		   [&db](const string& path) {
+		   [&db](string_view path) {
 		       db.add_database(Database(path));
 		   },
-		   [&db](const string& path) {
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-		       db.add_database(Database(new ChertDatabase(path)));
-#else
-		       (void)path;
-#endif
-		   },
-		   [&db](const string& path) {
+		   [&db](string_view path) {
 #ifdef XAPIAN_HAS_GLASS_BACKEND
 		       db.add_database(Database(new GlassDatabase(path)));
 #else
 		       (void)path;
 #endif
 		   },
-		   [&db](const string& prog, const string& args) {
+		   [&db](string_view path) {
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+		       db.add_database(Database(new HoneyDatabase(path)));
+#else
+		       (void)path;
+#endif
+		   },
+		   [&db](string_view prog, string_view args) {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 		       db.add_database(Remote::open(prog, args));
 #else
@@ -87,7 +89,7 @@ open_stub(Database& db, const string& file)
 		       (void)args;
 #endif
 		   },
-		   [&db](const string& host, unsigned port) {
+		   [&db](string_view host, unsigned port) {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 		       db.add_database(Remote::open(host, port));
 #else
@@ -97,7 +99,7 @@ open_stub(Database& db, const string& file)
 		   },
 		   [&db]() {
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
-		       db.add_database(Database(string(), DB_BACKEND_INMEMORY));
+		       db.add_database(Database(""sv, DB_BACKEND_INMEMORY));
 #endif
 		   });
 
@@ -111,21 +113,21 @@ open_stub(Database& db, const string& file)
 }
 
 static void
-open_stub(WritableDatabase& db, const string& file, int flags)
+open_stub(WritableDatabase& db, string_view file, int flags)
 {
     read_stub_file(file,
-		   [&db, flags](const string& path) {
+		   [&db, flags](string_view path) {
 		       db.add_database(WritableDatabase(path, flags));
 		   },
-		   [&db, &flags](const string& path) {
-		       flags |= DB_BACKEND_CHERT;
-		       db.add_database(WritableDatabase(path, flags));
-		   },
-		   [&db, &flags](const string& path) {
+		   [&db, &flags](string_view path) {
 		       flags |= DB_BACKEND_GLASS;
 		       db.add_database(WritableDatabase(path, flags));
 		   },
-		   [&db, flags](const string& prog, const string& args) {
+		   [](string_view) {
+		       auto msg = "Honey databases don't support writing";
+		       throw Xapian::DatabaseOpeningError(msg);
+		   },
+		   [&db, flags](string_view prog, string_view args) {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 		       db.add_database(Remote::open_writable(prog, args,
 							     0, flags));
@@ -134,7 +136,7 @@ open_stub(WritableDatabase& db, const string& file, int flags)
 		       (void)args;
 #endif
 		   },
-		   [&db, flags](const string& host, unsigned port) {
+		   [&db, flags](string_view host, unsigned port) {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
 		       db.add_database(Remote::open_writable(host, port,
 							     0, 10000, flags));
@@ -144,53 +146,59 @@ open_stub(WritableDatabase& db, const string& file, int flags)
 #endif
 		   },
 		   [&db]() {
-		       db.add_database(WritableDatabase(string(),
+		       db.add_database(WritableDatabase(""sv,
 							DB_BACKEND_INMEMORY));
 		   });
 
-    if (db.internal.empty()) {
-	throw DatabaseOpeningError(file + ": No databases listed");
+    if (db.internal->size() == 0) {
+	throw DatabaseOpeningError(string{file} + ": No databases listed");
     }
 }
 
-Database::Database(const string &path, int flags)
+Database::Database(string_view path, int flags)
+    : Database()
 {
     LOGCALL_CTOR(API, "Database", path|flags);
 
     int type = flags & DB_BACKEND_MASK_;
     switch (type) {
 	case DB_BACKEND_CHERT:
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-	    internal.push_back(new ChertDatabase(path));
-	    return;
-#else
-	    throw FeatureUnavailableError("Chert backend disabled");
-#endif
+	    throw FeatureUnavailableError("Chert backend no longer supported");
 	case DB_BACKEND_GLASS:
 #ifdef XAPIAN_HAS_GLASS_BACKEND
-	    internal.push_back(new GlassDatabase(path));
+	    internal = new GlassDatabase(path);
 	    return;
 #else
 	    throw FeatureUnavailableError("Glass backend disabled");
+#endif
+	case DB_BACKEND_HONEY:
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+	    internal = new HoneyDatabase(path);
+	    return;
+#else
+	    throw FeatureUnavailableError("Honey backend disabled");
 #endif
 	case DB_BACKEND_STUB:
 	    open_stub(*this, path);
 	    return;
 	case DB_BACKEND_INMEMORY:
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
-	    internal.push_back(new InMemoryDatabase());
+	    internal = new InMemoryDatabase();
 	    return;
 #else
 	    throw FeatureUnavailableError("Inmemory backend disabled");
 #endif
     }
 
+    string filename{path};
     struct stat statbuf;
-    if (stat(path.c_str(), &statbuf) == -1) {
+    if (stat(filename.c_str(), &statbuf) == -1) {
 	if (errno == ENOENT) {
-	    throw DatabaseNotFoundError("Couldn't stat '" + path + "'", errno);
+	    throw DatabaseNotFoundError("Couldn't stat '" + filename + "'",
+					errno);
 	} else {
-	    throw DatabaseOpeningError("Couldn't stat '" + path + "'", errno);
+	    throw DatabaseOpeningError("Couldn't stat '" + filename + "'",
+				       errno);
 	}
     }
 
@@ -199,14 +207,22 @@ Database::Database(const string &path, int flags)
 
 	// Initialise to avoid bogus warning from GCC 4.9.2 with -Os.
 	int fd = -1;
-	switch (test_if_single_file_db(statbuf, path, &fd)) {
-	  case BACKEND_GLASS:
+	switch (test_if_single_file_db(statbuf, filename, &fd)) {
+	    case BACKEND_GLASS:
 #ifdef XAPIAN_HAS_GLASS_BACKEND
-	    // Single file glass format.
-	    internal.push_back(new GlassDatabase(fd));
-	    return;
+		// Single file glass format.
+		internal = new GlassDatabase(fd);
+		return;
 #else
-	    throw FeatureUnavailableError("Glass backend disabled");
+		throw FeatureUnavailableError("Glass backend disabled");
+#endif
+	    case BACKEND_HONEY:
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+		// Single file honey format.
+		internal = new HoneyDatabase(fd);
+		return;
+#else
+		throw FeatureUnavailableError("Honey backend disabled");
 #endif
 	}
 
@@ -215,89 +231,133 @@ Database::Database(const string &path, int flags)
     }
 
     if (rare(!S_ISDIR(statbuf.st_mode))) {
-	throw DatabaseOpeningError("Not a regular file or directory: '" + path + "'");
+	throw DatabaseOpeningError("Not a regular file or directory: "
+				   "'" + filename + "'");
     }
 
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-    if (file_exists(path + "/iamchert")) {
-	internal.push_back(new ChertDatabase(path));
+#ifdef XAPIAN_HAS_GLASS_BACKEND
+    filename += "/iamglass";
+    if (file_exists(filename)) {
+	internal = new GlassDatabase(path);
 	return;
     }
 #endif
 
-#ifdef XAPIAN_HAS_GLASS_BACKEND
-    if (file_exists(path + "/iamglass")) {
-	internal.push_back(new GlassDatabase(path));
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+    filename.resize(path.size());
+    filename += "/iamhoney";
+    if (file_exists(filename)) {
+	internal = new HoneyDatabase(path);
 	return;
     }
 #endif
 
     // Check for "stub directories".
-    string stub_file = path;
-    stub_file += "/XAPIANDB";
-    if (usual(file_exists(stub_file))) {
-	open_stub(*this, stub_file);
+    filename.resize(path.size());
+    filename += "/XAPIANDB";
+    if (usual(file_exists(filename))) {
+	open_stub(*this, filename);
 	return;
     }
 
-#ifndef XAPIAN_HAS_CHERT_BACKEND
-    if (file_exists(path + "/iamchert")) {
-	throw FeatureUnavailableError("Chert backend disabled");
-    }
-#endif
 #ifndef XAPIAN_HAS_GLASS_BACKEND
-    if (file_exists(path + "/iamglass")) {
+    filename.resize(path.size());
+    filename += "/iamglass";
+    if (file_exists(filename)) {
 	throw FeatureUnavailableError("Glass backend disabled");
     }
 #endif
-    if (file_exists(path + "/iamflint")) {
+#ifndef XAPIAN_HAS_HONEY_BACKEND
+    filename.resize(path.size());
+    filename += "/iamhoney";
+    if (file_exists(filename)) {
+	throw FeatureUnavailableError("Honey backend disabled");
+    }
+#endif
+    filename.resize(path.size());
+    filename += "/iamchert";
+    if (file_exists(filename)) {
+	throw FeatureUnavailableError("Chert backend no longer supported");
+    }
+    filename.resize(path.size());
+    filename += "/iamflint";
+    if (file_exists(filename)) {
 	throw FeatureUnavailableError("Flint backend no longer supported");
     }
 
     throw DatabaseNotFoundError("Couldn't detect type of database");
 }
 
-Database::Database(int fd, int flags)
+/** Helper factory function.
+ *
+ *  This allows us to initialise Database::internal via the constructor's
+ *  initialiser list, which we want to be able to do as Database::internal
+ *  is an intrusive_ptr_nonnull, so we can't set it to NULL in the initialiser
+ *  list and then fill it in later in the constructor body.
+ */
+static Database::Internal*
+database_factory(int fd, int flags)
 {
-    LOGCALL_CTOR(API, "Database", fd|flags);
     if (rare(fd < 0))
-	throw InvalidArgumentError("fd < 0");
+	throw InvalidArgumentError("fd < 0", EBADF);
 
-#ifdef XAPIAN_HAS_GLASS_BACKEND
+#if defined XAPIAN_HAS_GLASS_BACKEND || defined XAPIAN_HAS_HONEY_BACKEND
     int type = flags & DB_BACKEND_MASK_;
+    if (type == 0) {
+	switch (test_if_single_file_db(fd)) {
+	  case BACKEND_GLASS:
+	    type = DB_BACKEND_GLASS;
+	    break;
+	  case BACKEND_HONEY:
+	    type = DB_BACKEND_HONEY;
+	    break;
+	}
+    }
     switch (type) {
-	case 0: case DB_BACKEND_GLASS:
-	    internal.push_back(new GlassDatabase(fd));
-	    return;
+#ifdef XAPIAN_HAS_GLASS_BACKEND
+	case DB_BACKEND_GLASS:
+	    return new GlassDatabase(fd);
+#endif
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+	case DB_BACKEND_HONEY:
+	    return new HoneyDatabase(fd);
+#endif
     }
-#else
-    (void)flags;
 #endif
 
     (void)::close(fd);
-    throw DatabaseOpeningError("Couldn't detect type of database");
+    throw DatabaseOpeningError("Couldn't detect type of database fd");
 }
 
-#if defined XAPIAN_HAS_CHERT_BACKEND || \
-    defined XAPIAN_HAS_GLASS_BACKEND
+Database::Database(int fd, int flags)
+    : internal(database_factory(fd, flags))
+{
+    LOGCALL_CTOR(API, "Database", fd|flags);
+}
+
+#if defined XAPIAN_HAS_GLASS_BACKEND
 #define HAVE_DISK_BACKEND
 #endif
 
-WritableDatabase::WritableDatabase(const std::string &path, int flags, int block_size)
+WritableDatabase::WritableDatabase(std::string_view path,
+				   int flags,
+				   int block_size)
     : Database()
 {
     LOGCALL_CTOR(API, "WritableDatabase", path|flags|block_size);
-    // Avoid warning if both chert and glass are disabled.
+    // Avoid warning if all disk-based backends are disabled.
     (void)block_size;
+    string filename{path};
     int type = flags & DB_BACKEND_MASK_;
     // Clear the backend bits, so we just pass on other flags to open_stub, etc.
     flags &= ~DB_BACKEND_MASK_;
     if (type == 0) {
 	struct stat statbuf;
-	if (stat(path.c_str(), &statbuf) == -1) {
+	if (stat(filename.c_str(), &statbuf) == -1) {
 	    // ENOENT probably just means that we need to create the directory.
 	    if (errno != ENOENT)
-		throw DatabaseOpeningError("Couldn't stat '" + path + "'", errno);
+		throw DatabaseOpeningError("Couldn't stat '" + filename + "'",
+					   errno);
 	} else {
 	    // File or directory already exists.
 
@@ -308,34 +368,48 @@ WritableDatabase::WritableDatabase(const std::string &path, int flags, int block
 	    }
 
 	    if (rare(!S_ISDIR(statbuf.st_mode))) {
-		throw DatabaseOpeningError("Not a regular file or directory: '" + path + "'");
+		throw DatabaseOpeningError("Not a regular file or directory: "
+					   "'" + filename + "'");
 	    }
 
-	    if (file_exists(path + "/iamchert")) {
-		// Existing chert DB.
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-		type = DB_BACKEND_CHERT;
-#else
-		throw FeatureUnavailableError("Chert backend disabled");
-#endif
-	    } else if (file_exists(path + "/iamglass")) {
+	    filename += "/iamglass";
+	    if (file_exists(filename)) {
 		// Existing glass DB.
 #ifdef XAPIAN_HAS_GLASS_BACKEND
 		type = DB_BACKEND_GLASS;
 #else
 		throw FeatureUnavailableError("Glass backend disabled");
 #endif
-	    } else if (file_exists(path + "/iamflint")) {
+	    }
+
+	    filename.resize(path.size());
+	    filename += "/iamhoney";
+	    if (file_exists(filename)) {
+		// Existing honey DB.
+		throw InvalidOperationError("Honey backend doesn't support "
+					    "updating existing databases");
+	    }
+
+	    filename.resize(path.size());
+	    filename += "/iamchert";
+	    if (file_exists(filename)) {
+		// Existing chert DB.
+		throw FeatureUnavailableError("Chert backend no longer supported");
+	    }
+
+	    filename.resize(path.size());
+	    filename += "/iamflint";
+	    if (file_exists(filename)) {
 		// Existing flint DB.
 		throw FeatureUnavailableError("Flint backend no longer supported");
-	    } else {
-		// Check for "stub directories".
-		string stub_file = path;
-		stub_file += "/XAPIANDB";
-		if (usual(file_exists(stub_file))) {
-		    open_stub(*this, stub_file, flags);
-		    return;
-		}
+	    }
+
+	    // Check for "stub directories".
+	    filename.resize(path.size());
+	    filename += "/XAPIANDB";
+	    if (usual(file_exists(filename))) {
+		open_stub(*this, filename, flags);
+		return;
 	    }
 	}
     }
@@ -349,17 +423,17 @@ WritableDatabase::WritableDatabase(const std::string &path, int flags, int block
 	    // by preference.
 #ifdef XAPIAN_HAS_GLASS_BACKEND
 	case DB_BACKEND_GLASS:
-	    internal.push_back(new GlassWritableDatabase(path, flags, block_size));
+	    internal = new GlassWritableDatabase(path, flags, block_size);
 	    return;
 #endif
-#ifdef XAPIAN_HAS_CHERT_BACKEND
+	case DB_BACKEND_HONEY:
+	    throw InvalidArgumentError("Honey backend doesn't support "
+				       "updating existing databases");
 	case DB_BACKEND_CHERT:
-	    internal.push_back(new ChertWritableDatabase(path, flags, block_size));
-	    return;
-#endif
+	    throw FeatureUnavailableError("Chert backend no longer supported");
 	case DB_BACKEND_INMEMORY:
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
-	    internal.push_back(new InMemoryDatabase());
+	    internal = new InMemoryDatabase();
 	    return;
 #else
 	    throw FeatureUnavailableError("Inmemory backend disabled");
