@@ -63,7 +63,7 @@ using namespace std;
 # endif
 #endif
 
-XAPIAN_NORETURN(static void throw_cannot_test_lock());
+[[noreturn]]
 static void
 throw_cannot_test_lock()
 {
