@@ -1,7 +1,7 @@
 /** @file
  * @brief A termlist containing all terms in a glass database.
  */
-/* Copyright (C) 2005,2007,2008,2009,2010 Olly Betts
+/* Copyright (C) 2005,2007,2008,2009,2010,2017,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,18 +28,23 @@
 #include "pack.h"
 #include "stringutils.h"
 
+#include <string_view>
+
+using namespace std;
+
 void
 GlassAllTermsList::read_termfreq() const
 {
     LOGCALL_VOID(DB, "GlassAllTermsList::read_termfreq", NO_ARGS);
     Assert(!current_term.empty());
-    Assert(!at_end());
+    Assert(cursor);
+    Assert(!cursor->after_end());
 
     // Unpack the termfreq from the tag.
     cursor->read_tag();
     const char *p = cursor->current_tag.data();
     const char *pend = p + cursor->current_tag.size();
-    GlassPostList::read_number_of_entries(&p, pend, &termfreq, NULL);
+    GlassPostList::read_freqs(&p, pend, &termfreq, NULL);
 }
 
 GlassAllTermsList::~GlassAllTermsList()
@@ -48,13 +53,13 @@ GlassAllTermsList::~GlassAllTermsList()
     delete cursor;
 }
 
-string
-GlassAllTermsList::get_termname() const
+Xapian::termcount
+GlassAllTermsList::get_approx_size() const
 {
-    LOGCALL(DB, string, "GlassAllTermsList::get_termname", NO_ARGS);
-    Assert(!current_term.empty());
-    Assert(!at_end());
-    RETURN(current_term);
+    // This is an over-estimate and not entirely proportional between shards,
+    // but we only use this value to build a balanced or-tree, and it'll at
+    // least tend to distinguish large databases from small ones.
+    return database->postlist_table.get_entry_count();
 }
 
 Xapian::doccount
@@ -62,7 +67,8 @@ GlassAllTermsList::get_termfreq() const
 {
     LOGCALL(DB, Xapian::doccount, "GlassAllTermsList::get_termfreq", NO_ARGS);
     Assert(!current_term.empty());
-    Assert(!at_end());
+    Assert(cursor);
+    Assert(!cursor->after_end());
     if (termfreq == 0) read_termfreq();
     RETURN(termfreq);
 }
@@ -71,7 +77,6 @@ TermList *
 GlassAllTermsList::next()
 {
     LOGCALL(DB, TermList *, "GlassAllTermsList::next", NO_ARGS);
-    Assert(!at_end());
     // Set termfreq to 0 to indicate no termfreq has been read for the current
     // term.
     termfreq = 0;
@@ -91,17 +96,19 @@ GlassAllTermsList::next()
 		RETURN(NULL);
 	    }
 	}
+	if (cursor->after_end()) {
+	    RETURN(this);
+	}
 	goto first_time;
     }
 
+    Assert(!cursor->after_end());
     while (true) {
-	cursor->next();
-first_time:
-	if (cursor->after_end()) {
-	    current_term.resize(0);
-	    RETURN(NULL);
+	if (!cursor->next()) {
+	    RETURN(this);
 	}
 
+first_time:
 	// Fast check for terms without any zero bytes.  ~8.4% faster.
 	auto nul = cursor->current_key.find('\0');
 	if (nul == string::npos) {
@@ -126,18 +133,16 @@ first_time:
 
     if (!startswith(current_term, prefix)) {
 	// We've reached the end of the prefixed terms.
-	cursor->to_end();
-	current_term.resize(0);
+	RETURN(this);
     }
 
     RETURN(NULL);
 }
 
-TermList *
-GlassAllTermsList::skip_to(const string &term)
+TermList*
+GlassAllTermsList::skip_to(string_view term)
 {
     LOGCALL(DB, TermList *, "GlassAllTermsList::skip_to", term);
-    Assert(!at_end());
     // Set termfreq to 0 to indicate no termfreq has been read for the current
     // term.
     termfreq = 0;
@@ -146,6 +151,7 @@ GlassAllTermsList::skip_to(const string &term)
 	cursor = database->postlist_table.cursor_get();
 	Assert(cursor); // The postlist table isn't optional.
     }
+    Assert(!cursor->after_end());
 
     string key = pack_glass_postlist_key(term);
     if (cursor->find_entry_ge(key)) {
@@ -154,8 +160,7 @@ GlassAllTermsList::skip_to(const string &term)
 	current_term = term;
     } else {
 	if (cursor->after_end()) {
-	    current_term.resize(0);
-	    RETURN(NULL);
+	    RETURN(this);
 	}
 
 	const char *p = cursor->current_key.data();
@@ -167,16 +172,8 @@ GlassAllTermsList::skip_to(const string &term)
 
     if (!startswith(current_term, prefix)) {
 	// We've reached the end of the prefixed terms.
-	cursor->to_end();
-	current_term.resize(0);
+	RETURN(this);
     }
 
     RETURN(NULL);
 }
-
-bool
-GlassAllTermsList::at_end() const
-{
-    LOGCALL(DB, bool, "GlassAllTermsList::at_end", NO_ARGS);
-    RETURN(cursor && cursor->after_end());
-}
