@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2004,2005,2008,2009,2011,2012,2013,2014,2016 Olly Betts
+ * Copyright 2002,2004,2005,2008,2009,2011,2012,2013,2014,2016,2024 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -29,10 +29,11 @@
 #include "unicode/description_append.h"
 #include "xapian/constants.h"
 
-#include "autoptr.h"
 #include <climits>
 #include <cstring>
+#include <memory>
 #include <ostream>
+#include <string_view>
 
 using namespace Glass;
 using namespace std;
@@ -137,9 +138,9 @@ void GlassTableCheck::report_block(int m, int n, const uint8_t * p) const
     *out << endl;
 }
 
-XAPIAN_NORETURN(static void failure(const char *msg, uint4 n, int c = 0));
+[[noreturn]]
 static void
-failure(const char *msg, uint4 n, int c)
+failure(const char *msg, uint4 n, int c = 0)
 {
     string e = "Block ";
     e += str(n);
@@ -259,18 +260,18 @@ GlassTableCheck::block_check(Glass::Cursor * C_, int j, int opts,
     }
 }
 
-GlassTableCheck *
-GlassTableCheck::check(const char * tablename, const string & path, int fd,
+GlassTableCheck*
+GlassTableCheck::check(const char* tablename, string_view path, int fd,
 		       off_t offset_,
-		       const GlassVersion & version_file, int opts,
-		       ostream *out)
+		       const GlassVersion& version_file, int opts,
+		       ostream* out)
 {
     string filename(path);
     filename += '/';
     filename += tablename;
     filename += '.';
 
-    AutoPtr<GlassTableCheck> B(
+    unique_ptr<GlassTableCheck> B(
 	    fd < 0 ?
 	    new GlassTableCheck(tablename, filename, false, out) :
 	    new GlassTableCheck(tablename, fd, offset_, false, out));
@@ -368,3 +369,7 @@ void GlassTableCheck::report_cursor(int N, const Glass::Cursor * C_) const
 		"n=[" << C_[i].get_n() << "], "
 		"rewrite=" << C_[i].rewrite << endl;
 }
+
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
