@@ -1,7 +1,7 @@
 /** @file
  * @brief Compact a glass database, or merge and compact several.
  */
-/* Copyright (C) 2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2017 Olly Betts
+/* Copyright (C) 2004-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,8 +26,9 @@
 #include "xapian/error.h"
 #include "xapian/types.h"
 
-#include "autoptr.h"
 #include <algorithm>
+#include <iterator>
+#include <memory>
 #include <queue>
 
 #include <cerrno>
@@ -84,10 +85,10 @@ class PostlistCursor : private GlassCursor {
     Xapian::docid firstdid;
     Xapian::termcount tf, cf;
 
-    PostlistCursor(GlassTable *in, Xapian::docid offset_)
+    PostlistCursor(const GlassTable *in, Xapian::docid offset_)
 	: GlassCursor(in), offset(offset_), firstdid(0)
     {
-	find_entry(string());
+	rewind();
 	next();
     }
 
@@ -154,7 +155,7 @@ class PostlistCursor : private GlassCursor {
 		key.erase(tmp - 1);
 	    }
 	}
-	firstdid += offset;
+	UNSIGNED_OVERFLOW_OK(firstdid += offset);
 	return true;
     }
 };
@@ -187,12 +188,12 @@ encode_valuestats(Xapian::doccount freq,
 static void
 merge_postlists(Xapian::Compactor * compactor,
 		GlassTable * out, vector<Xapian::docid>::const_iterator offset,
-		vector<GlassTable*>::const_iterator b,
-		vector<GlassTable*>::const_iterator e)
+		vector<const GlassTable*>::const_iterator b,
+		vector<const GlassTable*>::const_iterator e)
 {
     priority_queue<PostlistCursor *, vector<PostlistCursor *>, PostlistCursorGt> pq;
     for ( ; b != e; ++b, ++offset) {
-	GlassTable *in = *b;
+	const GlassTable *in = *b;
 	if (in->empty()) {
 	    // Skip empty tables.
 	    continue;
@@ -387,8 +388,8 @@ merge_postlists(Xapian::Compactor * compactor,
 }
 
 struct MergeCursor : public GlassCursor {
-    explicit MergeCursor(GlassTable *in) : GlassCursor(in) {
-	find_entry(string());
+    explicit MergeCursor(const GlassTable *in) : GlassCursor(in) {
+	rewind();
 	next();
     }
 };
@@ -404,12 +405,12 @@ struct CursorGt {
 
 static void
 merge_spellings(GlassTable * out,
-		vector<GlassTable*>::const_iterator b,
-		vector<GlassTable*>::const_iterator e)
+		vector<const GlassTable*>::const_iterator b,
+		vector<const GlassTable*>::const_iterator e)
 {
     priority_queue<MergeCursor *, vector<MergeCursor *>, CursorGt> pq;
     for ( ; b != e; ++b) {
-	GlassTable *in = *b;
+	const GlassTable *in = *b;
 	if (!in->empty()) {
 	    pq.push(new MergeCursor(in));
 	}
@@ -513,12 +514,12 @@ merge_spellings(GlassTable * out,
 
 static void
 merge_synonyms(GlassTable * out,
-	       vector<GlassTable*>::const_iterator b,
-	       vector<GlassTable*>::const_iterator e)
+	       vector<const GlassTable*>::const_iterator b,
+	       vector<const GlassTable*>::const_iterator e)
 {
     priority_queue<MergeCursor *, vector<MergeCursor *>, CursorGt> pq;
     for ( ; b != e; ++b) {
-	GlassTable *in = *b;
+	const GlassTable *in = *b;
 	if (!in->empty()) {
 	    pq.push(new MergeCursor(in));
 	}
@@ -561,14 +562,15 @@ merge_synonyms(GlassTable * out,
 	    pq.pop();
 	}
 
-	string lastword;
+	string_view lastword;
 	while (!pqtag.empty()) {
 	    ByteLengthPrefixedStringItor * it = pqtag.top();
 	    pqtag.pop();
-	    if (**it != lastword) {
-		lastword = **it;
-		tag += uint8_t(lastword.size() ^ MAGIC_XOR_VALUE);
-		tag += lastword;
+	    string_view word = **it;
+	    if (word != lastword) {
+		tag += uint8_t(word.size() ^ MAGIC_XOR_VALUE);
+		tag += word;
+		lastword = word;
 	    }
 	    ++*it;
 	    if (!it->at_end()) {
@@ -595,12 +597,12 @@ merge_synonyms(GlassTable * out,
 static void
 multimerge_postlists(Xapian::Compactor * compactor,
 		     GlassTable * out, const char * tmpdir,
-		     vector<GlassTable *> tmp,
+		     vector<const GlassTable *> tmp,
 		     vector<Xapian::docid> off)
 {
     unsigned int c = 0;
     while (tmp.size() > 3) {
-	vector<GlassTable *> tmpout;
+	vector<const GlassTable *> tmpout;
 	tmpout.reserve(tmp.size() / 2);
 	vector<Xapian::docid> newoff;
 	newoff.resize(tmp.size() / 2);
@@ -661,9 +663,9 @@ class PositionCursor : private GlassCursor {
     string key;
     Xapian::docid firstdid;
 
-    PositionCursor(GlassTable *in, Xapian::docid offset_)
+    PositionCursor(const GlassTable *in, Xapian::docid offset_)
 	: GlassCursor(in), offset(offset_), firstdid(0) {
-	find_entry(string());
+	rewind();
 	next();
     }
 
@@ -701,12 +703,12 @@ class PositionCursorGt {
 };
 
 static void
-merge_positions(GlassTable *out, const vector<GlassTable*> & inputs,
+merge_positions(GlassTable *out, const vector<const GlassTable*> & inputs,
 		const vector<Xapian::docid> & offset)
 {
     priority_queue<PositionCursor *, vector<PositionCursor *>, PositionCursorGt> pq;
     for (size_t i = 0; i < inputs.size(); ++i) {
-	GlassTable *in = inputs[i];
+	const GlassTable *in = inputs[i];
 	if (in->empty()) {
 	    // Skip empty tables.
 	    continue;
@@ -728,17 +730,17 @@ merge_positions(GlassTable *out, const vector<GlassTable*> & inputs,
 }
 
 static void
-merge_docid_keyed(GlassTable *out, const vector<GlassTable*> & inputs,
+merge_docid_keyed(GlassTable *out, const vector<const GlassTable*> & inputs,
 		  const vector<Xapian::docid> & offset)
 {
     for (size_t i = 0; i < inputs.size(); ++i) {
 	Xapian::docid off = offset[i];
 
-	GlassTable * in = inputs[i];
+	const GlassTable * in = inputs[i];
 	if (in->empty()) continue;
 
 	GlassCursor cur(in);
-	cur.find_entry(string());
+	cur.rewind();
 
 	string key;
 	while (cur.next()) {
@@ -752,7 +754,7 @@ merge_docid_keyed(GlassTable *out, const vector<GlassTable*> & inputs,
 		    msg += inputs[i]->get_path();
 		    throw Xapian::DatabaseCorruptError(msg);
 		}
-		did += off;
+		UNSIGNED_OVERFLOW_OK(did += off);
 		key.resize(0);
 		pack_uint_preserving_sort(key, did);
 		if (d != e) {
@@ -777,9 +779,9 @@ void
 GlassDatabase::compact(Xapian::Compactor * compactor,
 		       const char * destdir,
 		       int fd,
-		       const vector<Xapian::Database::Internal*> & sources,
+		       const vector<const Xapian::Database::Internal*>& sources,
 		       const vector<Xapian::docid> & offset,
-		       size_t block_size,
+		       unsigned block_size,
 		       Xapian::Compactor::compaction_level compaction,
 		       unsigned flags,
 		       Xapian::docid last_docid)
@@ -802,8 +804,7 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 	{ "spelling",	Glass::SPELLING,	true },
 	{ "synonym",	Glass::SYNONYM,		true }
     };
-    const table_list * tables_end = tables +
-	(sizeof(tables) / sizeof(tables[0]));
+    const table_list* tables_end = std::end(tables);
 
     const int FLAGS = Xapian::DB_DANGEROUS;
 
@@ -826,7 +827,8 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 	}
     }
 
-    if (block_size < 2048 || block_size > 65536 ||
+    if (block_size < GLASS_MIN_BLOCKSIZE ||
+	block_size > GLASS_MAX_BLOCKSIZE ||
 	(block_size & (block_size - 1)) != 0) {
 	block_size = GLASS_DEFAULT_BLOCKSIZE;
     }
@@ -840,7 +842,7 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 	}
     }
 
-    AutoPtr<GlassVersion> version_file_out;
+    unique_ptr<GlassVersion> version_file_out;
     if (single_file) {
 	if (destdir) {
 	    fd = open(destdir, O_RDWR|O_CREAT|O_TRUNC|O_BINARY|O_CLOEXEC, 0666);
@@ -856,7 +858,7 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 
     version_file_out->create(block_size);
     for (size_t i = 0; i != sources.size(); ++i) {
-	GlassDatabase * db = static_cast<GlassDatabase*>(sources[i]);
+	auto db = static_cast<const GlassDatabase*>(sources[i]);
 	version_file_out->merge_stats(db->version_file);
     }
 
@@ -869,7 +871,7 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 
     vector<GlassTable *> tabs;
     tabs.reserve(tables_end - tables);
-    off_t prev_size = block_size;
+    file_size_type prev_size = block_size;
     for (const table_list * t = tables; t < tables_end; ++t) {
 	// The postlist table requires an N-way merge, adjusting the
 	// headers of various blocks.  The spelling and synonym tables also
@@ -897,14 +899,14 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 	// amongst the inputs.
 	bool single_file_in = false;
 
-	off_t in_size = 0;
+	file_size_type in_size = 0;
 
-	vector<GlassTable*> inputs;
+	vector<const GlassTable*> inputs;
 	inputs.reserve(sources.size());
 	size_t inputs_present = 0;
 	for (auto src : sources) {
-	    GlassDatabase * db = static_cast<GlassDatabase*>(src);
-	    GlassTable * table;
+	    auto db = static_cast<const GlassDatabase*>(src);
+	    const GlassTable * table;
 	    switch (t->type) {
 		case Glass::POSTLIST:
 		    table = &(db->postlist_table);
@@ -940,7 +942,7 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 		    ++inputs_present;
 		}
 	    } else {
-		off_t db_size = file_size(table->get_path());
+		auto db_size = file_size(table->get_path());
 		if (errno == 0) {
 		    in_size += db_size / 1024;
 		    output_will_exist = true;
@@ -1028,9 +1030,9 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 	}
 	if (single_file) fl_serialised = root_info->get_free_list();
 
-	off_t out_size = 0;
+	file_size_type out_size = 0;
 	if (!bad_stat && !single_file_in) {
-	    off_t db_size;
+	    file_size_type db_size;
 	    if (single_file) {
 		db_size = file_size(fd);
 	    } else {
@@ -1038,9 +1040,10 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
 	    }
 	    if (errno == 0) {
 		if (single_file) {
-		    off_t old_prev_size = max(prev_size, off_t(block_size));
+		    auto old_prev_size = max(prev_size,
+					     file_size_type(block_size));
 		    prev_size = db_size;
-		    db_size = max(db_size, off_t(block_size));
+		    db_size = max(db_size, file_size_type(block_size));
 		    db_size -= old_prev_size;
 		}
 		out_size = db_size / 1024;
@@ -1086,7 +1089,7 @@ GlassDatabase::compact(Xapian::Compactor * compactor,
     // If compacting to a single file output and all the tables are empty, pad
     // the output so that it isn't mistaken for a stub database when we try to
     // open it.  For this it needs to be a multiple of 2KB in size.
-    if (single_file && prev_size < off_t(block_size)) {
+    if (single_file && prev_size < block_size) {
 #ifdef HAVE_FTRUNCATE
 	if (ftruncate(fd, block_size) < 0) {
 	    throw Xapian::DatabaseError("Failed to set size of output database", errno);
