@@ -2,7 +2,7 @@
  * @brief Interface to Btree cursors
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002,2003,2004,2006,2007,2008,2009,2010,2012,2013,2014,2016 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,8 +20,8 @@
  * USA
  */
 
-#ifndef OM_HGUARD_GLASS_CURSOR_H
-#define OM_HGUARD_GLASS_CURSOR_H
+#ifndef XAPIAN_INCLUDED_GLASS_CURSOR_H
+#define XAPIAN_INCLUDED_GLASS_CURSOR_H
 
 #include "glass_defs.h"
 
@@ -31,6 +31,8 @@
 #include <algorithm>
 #include <cstring>
 #include <string>
+#include <string_view>
+
 using std::string;
 
 #define BLK_UNUSED uint4(-1)
@@ -198,8 +200,7 @@ class GlassCursor {
      *    string key;
      *
      *    // Now do something to each key in the Btree
-     *    BC.find_entry(string()); // must give result true
-     *
+     *    BC.rewind();
      *    while (BC.next()) {
      *        BC.get_key(&key);
      *        do_something_to(key);
@@ -243,6 +244,14 @@ class GlassCursor {
      */
     string current_tag;
 
+    /** Position cursor on the dummy empty key.
+     *
+     *  Calling next() after this moves the cursor to the first entry.
+     */
+    void rewind() {
+	(void)find_entry_ge({});
+    }
+
     /** Read the tag from the table and store it in current_tag.
      *
      *  Some cursor users don't need the tag, so for efficiency we
@@ -315,7 +324,7 @@ class GlassCursor {
      *  @return true if the exact key was found in the table, false
      *          otherwise.
      */
-    bool find_entry_ge(const string &key);
+    bool find_entry_ge(std::string_view key);
 
     /** Set the cursor to be off the end of the table.
      */
@@ -357,4 +366,8 @@ class MutableGlassCursor : public GlassCursor {
     bool del();
 };
 
-#endif /* OM_HGUARD_GLASS_CURSOR_H */
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
+
+#endif /* XAPIAN_INCLUDED_GLASS_CURSOR_H */
