@@ -4,7 +4,7 @@
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2019 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  * Copyright 2009 Richard Boulton
  * Copyright 2009 Kan-Ru Chen
@@ -54,10 +54,10 @@
 #include "filetests.h"
 #include "io_utils.h"
 #include "pack.h"
+#include "parseint.h"
 #include "net/remoteconnection.h"
 #include "api/replication.h"
 #include "replicationprotocol.h"
-#include "net/length.h"
 #include "posixy_wrapper.h"
 #include "str.h"
 #include "stringutils.h"
@@ -67,9 +67,9 @@
 #include <sys/types.h>
 
 #include <algorithm>
-#include "autoptr.h"
 #include <cerrno>
 #include <cstdlib>
+#include <memory>
 #include <string>
 
 using namespace std;
@@ -95,9 +95,12 @@ using Xapian::Internal::intrusive_ptr;
 /* This opens the tables, determining the current and next revision numbers,
  * and stores handles to the tables.
  */
-GlassDatabase::GlassDatabase(const string &glass_dir, int flags,
+GlassDatabase::GlassDatabase(string_view glass_dir, int flags,
 			     unsigned int block_size)
-	: db_dir(glass_dir),
+	: Xapian::Database::Internal(flags == Xapian::DB_READONLY_ ?
+				     TRANSACTION_READONLY :
+				     TRANSACTION_NONE),
+	  db_dir(glass_dir),
 	  readonly(flags == Xapian::DB_READONLY_),
 	  version_file(db_dir),
 	  postlist_table(db_dir, readonly),
@@ -119,8 +122,10 @@ GlassDatabase::GlassDatabase(const string &glass_dir, int flags,
 	return;
     }
 
-    // Block size must in the range 2048..65536, and a power of two.
-    if (block_size < 2048 || block_size > 65536 ||
+    // Block size must in the range GLASS_MIN_BLOCKSIZE..GLASS_MAX_BLOCKSIZE
+    // and a power of two.
+    if (block_size < GLASS_MIN_BLOCKSIZE ||
+	block_size > GLASS_MAX_BLOCKSIZE ||
 	(block_size & (block_size - 1)) != 0) {
 	block_size = GLASS_DEFAULT_BLOCKSIZE;
     }
@@ -161,7 +166,8 @@ GlassDatabase::GlassDatabase(const string &glass_dir, int flags,
 }
 
 GlassDatabase::GlassDatabase(int fd)
-	: db_dir(),
+	: Xapian::Database::Internal(TRANSACTION_READONLY),
+	  db_dir(),
 	  readonly(true),
 	  version_file(fd),
 	  postlist_table(fd, version_file.get_offset(), readonly),
@@ -269,13 +275,6 @@ GlassDatabase::open_tables(int flags)
     return true;
 }
 
-glass_revision_number_t
-GlassDatabase::get_revision_number() const
-{
-    LOGCALL(DB, glass_revision_number_t, "GlassDatabase::get_revision_number", NO_ARGS);
-    RETURN(version_file.get_revision());
-}
-
 glass_revision_number_t
 GlassDatabase::get_next_revision_number() const
 {
@@ -381,7 +380,7 @@ GlassDatabase::request_document(Xapian::docid did) const
 }
 
 void
-GlassDatabase::readahead_for_query(const Xapian::Query &query)
+GlassDatabase::readahead_for_query(const Xapian::Query &query) const
 {
     Xapian::TermIterator t;
     for (t = query.get_unique_terms_begin(); t != Xapian::TermIterator(); ++t) {
@@ -440,10 +439,8 @@ GlassDatabase::send_whole_database(RemoteConnection & conn, double end_time)
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
     // Send the current revision number in the header.
     string buf;
-    string uuid = get_uuid();
-    buf += encode_length(uuid.size());
-    buf += uuid;
-    pack_uint(buf, get_revision_number());
+    pack_string(buf, get_uuid());
+    pack_uint(buf, get_revision());
     conn.send_message(REPL_REPLY_DB_HEADER, buf, end_time);
 
     // Send all the tables.  The tables which we want to be cached best after
@@ -464,7 +461,7 @@ GlassDatabase::send_whole_database(RemoteConnection & conn, double end_time)
 	filepath.replace(db_dir.size() + 1, string::npos, p, len);
 	FD fd(posixy_open(filepath.c_str(), O_RDONLY | O_CLOEXEC));
 	if (fd >= 0) {
-	    conn.send_message(REPL_REPLY_DB_FILENAME, string(p, len), end_time);
+	    conn.send_message(REPL_REPLY_DB_FILENAME, {p, len}, end_time);
 	    conn.send_file(REPL_REPLY_DB_FILEDATA, fd, end_time);
 	}
 	p += len + 1;
@@ -477,7 +474,7 @@ GlassDatabase::send_whole_database(RemoteConnection & conn, double end_time)
 
 void
 GlassDatabase::write_changesets_to_fd(int fd,
-				      const string & revision,
+				      string_view revision,
 				      bool need_whole_db,
 				      ReplicationInfo * info)
 {
@@ -517,7 +514,7 @@ GlassDatabase::write_changesets_to_fd(int fd,
 	    whole_db_copies_left--;
 
 	    // Send the whole database across.
-	    start_rev_num = get_revision_number();
+	    start_rev_num = get_revision();
 	    start_uuid = get_uuid();
 
 	    send_whole_database(conn, 0.0);
@@ -533,7 +530,7 @@ GlassDatabase::write_changesets_to_fd(int fd,
 		// copy is safe to make live.
 
 		string buf;
-		needed_rev_num = get_revision_number();
+		needed_rev_num = get_revision();
 		pack_uint(buf, needed_rev_num);
 		conn.send_message(REPL_REPLY_DB_FOOTER, buf, 0.0);
 		if (info != NULL && start_rev_num == needed_rev_num)
@@ -554,13 +551,13 @@ GlassDatabase::write_changesets_to_fd(int fd,
 	    }
 	} else {
 	    // Check if we've sent all the updates.
-	    if (start_rev_num >= get_revision_number()) {
+	    if (start_rev_num >= get_revision()) {
 		reopen();
 		if (start_uuid != get_uuid()) {
 		    need_whole_db = true;
 		    continue;
 		}
-		if (start_rev_num >= get_revision_number()) {
+		if (start_rev_num >= get_revision()) {
 		    break;
 		}
 	    }
@@ -597,7 +594,7 @@ GlassDatabase::write_changesets_to_fd(int fd,
 	    }
 	}
     }
-    conn.send_message(REPL_REPLY_END_OF_CHANGES, string(), 0.0);
+    conn.send_message(REPL_REPLY_END_OF_CHANGES, {}, 0.0);
 #else
     (void)fd;
     (void)revision;
@@ -628,11 +625,6 @@ GlassDatabase::modifications_failed(glass_revision_number_t new_revision,
 	postlist_table.open(flags, version_file.get_root(Glass::POSTLIST), old_revision);
 
 	value_manager.reset();
-
-	// Increase revision numbers to new revision number plus one,
-	// writing increased numbers to all tables.
-	++new_revision;
-	set_revision_number(flags, new_revision);
     } catch (const Xapian::Error &e) {
 	// We failed to roll-back so close the database to avoid the risk of
 	// database corruption.
@@ -748,8 +740,23 @@ GlassDatabase::get_unique_terms(Xapian::docid did) const
     RETURN(GlassTermList(ptrtothis, did).get_unique_terms());
 }
 
+Xapian::termcount
+GlassDatabase::get_wdfdocmax(Xapian::docid did) const
+{
+    LOGCALL(DB, Xapian::termcount, "GlassDatabase::get_wdfdocmax", did);
+    Assert(did != 0);
+    intrusive_ptr<const GlassDatabase> ptrtothis(this);
+    GlassTermList termlist(ptrtothis, did);
+    Xapian::termcount max_wdf = 0;
+    while (termlist.next() == NULL) {
+	Xapian::termcount current_wdf = termlist.get_wdf();
+	if (current_wdf > max_wdf) max_wdf = current_wdf;
+    }
+    RETURN(max_wdf);
+}
+
 void
-GlassDatabase::get_freqs(const string & term,
+GlassDatabase::get_freqs(string_view term,
 			 Xapian::doccount * termfreq_ptr,
 			 Xapian::termcount * collfreq_ptr) const
 {
@@ -792,7 +799,7 @@ GlassDatabase::get_doclength_upper_bound() const
 }
 
 Xapian::termcount
-GlassDatabase::get_wdf_upper_bound(const string & term) const
+GlassDatabase::get_wdf_upper_bound(string_view term) const
 {
     Assert(!term.empty());
     Xapian::termcount wdfub;
@@ -800,11 +807,19 @@ GlassDatabase::get_wdf_upper_bound(const string & term) const
     return min(wdfub, version_file.get_wdf_upper_bound());
 }
 
+Xapian::termcount
+GlassDatabase::get_unique_terms_lower_bound() const
+{
+    return version_file.get_unique_terms_lower_bound();
+}
+
 bool
-GlassDatabase::term_exists(const string & term) const
+GlassDatabase::term_exists(string_view term) const
 {
     LOGCALL(DB, bool, "GlassDatabase::term_exists", term);
-    Assert(!term.empty());
+    if (term.empty()) {
+	RETURN(get_doccount() != 0);
+    }
     RETURN(postlist_table.term_exists(term));
 }
 
@@ -814,21 +829,38 @@ GlassDatabase::has_positions() const
     return !position_table.empty();
 }
 
-LeafPostList *
-GlassDatabase::open_post_list(const string& term) const
+PostList *
+GlassDatabase::open_post_list(string_view term) const
 {
-    LOGCALL(DB, LeafPostList *, "GlassDatabase::open_post_list", term);
+    LOGCALL(DB, PostList *, "GlassDatabase::open_post_list", term);
+    RETURN(GlassDatabase::open_leaf_post_list(term, false));
+}
+
+LeafPostList*
+GlassDatabase::open_leaf_post_list(string_view term, bool need_read_pos) const
+{
+    LOGCALL(DB, LeafPostList *, "GlassDatabase::open_leaf_post_list", term | need_read_pos);
+    (void)need_read_pos;
     intrusive_ptr<const GlassDatabase> ptrtothis(this);
 
     if (term.empty()) {
+	Assert(!need_read_pos);
 	Xapian::doccount doccount = get_doccount();
+	if (rare(doccount == 0)) {
+	    RETURN(nullptr);
+	}
 	if (version_file.get_last_docid() == doccount) {
-	    RETURN(new ContiguousAllDocsPostList(ptrtothis, doccount));
+	    RETURN(new ContiguousAllDocsPostList(doccount));
 	}
 	RETURN(new GlassAllDocsPostList(ptrtothis, doccount));
     }
 
-    RETURN(new GlassPostList(ptrtothis, term, true));
+    auto pl = new GlassPostList(ptrtothis, term, true);
+    if (pl->get_termfreq() == 0) {
+	delete pl;
+	pl = nullptr;
+    }
+    RETURN(pl);
 }
 
 ValueList *
@@ -850,6 +882,12 @@ GlassDatabase::open_term_list(Xapian::docid did) const
     RETURN(new GlassTermList(ptrtothis, did));
 }
 
+TermList *
+GlassDatabase::open_term_list_direct(Xapian::docid did) const
+{
+    return GlassDatabase::open_term_list(did);
+}
+
 Xapian::Document::Internal *
 GlassDatabase::open_document(Xapian::docid did, bool lazy) const
 {
@@ -865,45 +903,37 @@ GlassDatabase::open_document(Xapian::docid did, bool lazy) const
 }
 
 void
-GlassDatabase::read_position_list(GlassPositionList* pos_list,
+GlassDatabase::read_position_list(GlassRePositionList* pos_list,
 				  Xapian::docid did,
-				  const string& term) const
+				  string_view term) const
 {
     Assert(did != 0);
-    pos_list->read_data(&position_table, did, term);
+    pos_list->read_data(did, term);
 }
 
 Xapian::termcount
-GlassDatabase::positionlist_count(Xapian::docid did, const string& term) const
+GlassDatabase::positionlist_count(Xapian::docid did, string_view term) const
 {
     return position_table.positionlist_count(did, term);
 }
 
 PositionList *
-GlassDatabase::open_position_list(Xapian::docid did, const string & term) const
+GlassDatabase::open_position_list(Xapian::docid did, string_view term) const
 {
     Assert(did != 0);
-
-    AutoPtr<GlassPositionList> poslist(new GlassPositionList);
-    if (!poslist->read_data(&position_table, did, term)) {
-	// As of 1.1.0, we don't check if the did and term exist - we just
-	// return an empty positionlist.  If the user really needs to know,
-	// they can check for themselves.
-    }
-
-    return poslist.release();
+    return position_table.open_position_list(did, term);
 }
 
 TermList *
-GlassDatabase::open_allterms(const string & prefix) const
+GlassDatabase::open_allterms(string_view prefix) const
 {
     LOGCALL(DB, TermList*, "GlassDatabase::open_allterms", prefix);
     RETURN(new GlassAllTermsList(intrusive_ptr<const GlassDatabase>(this),
 				 prefix));
 }
 
-TermList *
-GlassDatabase::open_spelling_termlist(const string & word) const
+TermList*
+GlassDatabase::open_spelling_termlist(string_view word) const
 {
     return spelling_table.open_termlist(word);
 }
@@ -918,19 +948,19 @@ GlassDatabase::open_spelling_wordlist() const
 }
 
 Xapian::doccount
-GlassDatabase::get_spelling_frequency(const string & word) const
+GlassDatabase::get_spelling_frequency(string_view word) const
 {
     return spelling_table.get_word_frequency(word);
 }
 
 TermList *
-GlassDatabase::open_synonym_termlist(const string & term) const
+GlassDatabase::open_synonym_termlist(string_view term) const
 {
     return synonym_table.open_termlist(term);
 }
 
 TermList *
-GlassDatabase::open_synonym_keylist(const string & prefix) const
+GlassDatabase::open_synonym_keylist(string_view prefix) const
 {
     GlassCursor * cursor = synonym_table.cursor_get();
     if (!cursor) return NULL;
@@ -939,7 +969,7 @@ GlassDatabase::open_synonym_keylist(const string & prefix) const
 }
 
 string
-GlassDatabase::get_metadata(const string & key) const
+GlassDatabase::get_metadata(string_view key) const
 {
     LOGCALL(DB, string, "GlassDatabase::get_metadata", key);
     string btree_key("\x00\xc0", 2);
@@ -949,8 +979,8 @@ GlassDatabase::get_metadata(const string & key) const
     RETURN(tag);
 }
 
-TermList *
-GlassDatabase::open_metadata_keylist(const std::string &prefix) const
+TermList*
+GlassDatabase::open_metadata_keylist(std::string_view prefix) const
 {
     LOGCALL(DB, TermList*, "GlassDatabase::open_metadata_keylist", prefix);
     GlassCursor * cursor = postlist_table.cursor_get();
@@ -959,13 +989,11 @@ GlassDatabase::open_metadata_keylist(const std::string &prefix) const
 				     cursor, prefix));
 }
 
-string
-GlassDatabase::get_revision_info() const
+Xapian::rev
+GlassDatabase::get_revision() const
 {
-    LOGCALL(DB, string, "GlassDatabase::get_revision_info", NO_ARGS);
-    string buf;
-    pack_uint(buf, get_revision_number());
-    RETURN(buf);
+    LOGCALL(DB, Xapian::rev, "GlassDatabase::get_revision", NO_ARGS);
+    RETURN(version_file.get_revision());
 }
 
 string
@@ -1010,9 +1038,35 @@ GlassDatabase::locked() const
     return lock.test();
 }
 
+Database::Internal*
+GlassDatabase::update_lock(int flags)
+{
+    if (!postlist_table.is_open())
+	GlassTable::throw_database_closed();
+
+    if (flags == Xapian::DB_READONLY_) return this;
+    // Mask out backend type bits as these don't make sense here, and set the
+    // action to DB_OPEN since we don't want to create or overwrite here.
+    flags &= ~(DB_ACTION_MASK_ | DB_BACKEND_MASK_);
+    flags |= Xapian::DB_OPEN;
+    return new GlassWritableDatabase(db_dir, Xapian::DB_OPEN, flags);
+}
+
+string
+GlassDatabase::get_description() const
+{
+    string desc = "Glass(";
+    if (!readonly) {
+	desc += "writable, ";
+    }
+    desc += db_dir;
+    desc += ')';
+    return desc;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
-GlassWritableDatabase::GlassWritableDatabase(const string &dir, int flags,
+GlassWritableDatabase::GlassWritableDatabase(string_view dir, int flags,
 					     int block_size)
 	: GlassDatabase(dir, flags, block_size),
 	  change_count(0),
@@ -1023,8 +1077,12 @@ GlassWritableDatabase::GlassWritableDatabase(const string &dir, int flags,
     LOGCALL_CTOR(DB, "GlassWritableDatabase", dir | flags | block_size);
 
     const char *p = getenv("XAPIAN_FLUSH_THRESHOLD");
-    if (p)
-	flush_threshold = atoi(p);
+    if (p && *p) {
+	if (!parse_unsigned(p, flush_threshold)) {
+	    throw Xapian::InvalidArgumentError("XAPIAN_FLUSH_THRESHOLD must "
+					       "be a non-negative integer");
+	}
+    }
     if (flush_threshold == 0)
 	flush_threshold = 10000;
 }
@@ -1192,14 +1250,11 @@ GlassWritableDatabase::delete_document(Xapian::docid did)
 
 	version_file.delete_document(termlist.get_doclength());
 
-	termlist.next();
-	while (!termlist.at_end()) {
+	while (termlist.next() == NULL) {
 	    string tname = termlist.get_termname();
 	    inverter.delete_positionlist(did, tname);
 
 	    inverter.remove_posting(did, tname, termlist.get_wdf());
-
-	    termlist.next();
 	}
 
 	// Remove the termlist.
@@ -1272,15 +1327,17 @@ GlassWritableDatabase::replace_document(Xapian::docid did,
 
 	if (!modifying || document.internal->terms_modified()) {
 	    bool pos_modified = !modifying ||
-				document.internal->term_positions_modified();
+				document.internal->positions_modified();
+
+	    // Avoid the cost of throwing and handling an exception if
+	    // there's not already a document with docid did.
 	    intrusive_ptr<const GlassWritableDatabase> ptrtothis(this);
 	    GlassTermList termlist(ptrtothis, did, false);
-	    // We passed false for throw_if_not_present so check at_end()
-	    // before next() to see if the document isn't present at all.
-	    if (termlist.at_end()) {
+	    if (termlist.not_present()) {
 		(void)add_document_(did, document);
 		return;
 	    }
+
 	    Xapian::TermIterator term = document.termlist_begin();
 	    Xapian::termcount old_doclen = termlist.get_doclength();
 	    version_file.delete_document(old_doclen);
@@ -1288,10 +1345,10 @@ GlassWritableDatabase::replace_document(Xapian::docid did,
 
 	    string old_tname, new_tname;
 
-	    termlist.next();
-	    while (!termlist.at_end() || term != document.termlist_end()) {
+	    bool termlist_at_end = (termlist.next() != NULL);
+	    while (!termlist_at_end || term != document.termlist_end()) {
 		int cmp;
-		if (termlist.at_end()) {
+		if (termlist_at_end) {
 		    cmp = 1;
 		    new_tname = *term;
 		} else {
@@ -1311,7 +1368,7 @@ GlassWritableDatabase::replace_document(Xapian::docid did,
 		    inverter.remove_posting(did, old_tname, old_wdf);
 		    if (pos_modified)
 			inverter.delete_positionlist(did, old_tname);
-		    termlist.next();
+		    termlist_at_end = (termlist.next() != NULL);
 		} else if (cmp > 0) {
 		    // Term new_tname as been added.
 		    termcount new_wdf = term.get_wdf();
@@ -1335,7 +1392,7 @@ GlassWritableDatabase::replace_document(Xapian::docid did,
 		    version_file.check_wdf(new_wdf);
 
 		    if (old_wdf != new_wdf) {
-			new_doclen += new_wdf - old_wdf;
+			new_doclen = new_doclen - old_wdf + new_wdf;
 			inverter.update_posting(did, new_tname, old_wdf, new_wdf);
 		    }
 
@@ -1344,7 +1401,7 @@ GlassWritableDatabase::replace_document(Xapian::docid did,
 		    }
 
 		    ++term;
-		    termlist.next();
+		    termlist_at_end = (termlist.next() != NULL);
 		}
 	    }
 	    LOGLINE(DB, "Calculated doclen for replacement document " << did << " as " << new_doclen);
@@ -1421,19 +1478,19 @@ GlassWritableDatabase::get_unique_terms(Xapian::docid did) const
 }
 
 void
-GlassWritableDatabase::get_freqs(const string & term,
+GlassWritableDatabase::get_freqs(string_view term,
 				 Xapian::doccount * termfreq_ptr,
 				 Xapian::termcount * collfreq_ptr) const
 {
     LOGCALL_VOID(DB, "GlassWritableDatabase::get_freqs", term | termfreq_ptr | collfreq_ptr);
     Assert(!term.empty());
     GlassDatabase::get_freqs(term, termfreq_ptr, collfreq_ptr);
-    Xapian::termcount_diff tf_delta, cf_delta;
+    Xapian::termcount tf_delta, cf_delta;
     if (inverter.get_deltas(term, tf_delta, cf_delta)) {
 	if (termfreq_ptr)
-	    *termfreq_ptr += tf_delta;
+	    UNSIGNED_OVERFLOW_OK(*termfreq_ptr += tf_delta);
 	if (collfreq_ptr)
-	    *collfreq_ptr += cf_delta;
+	    UNSIGNED_OVERFLOW_OK(*collfreq_ptr += cf_delta);
     }
 }
 
@@ -1468,11 +1525,14 @@ GlassWritableDatabase::get_value_upper_bound(Xapian::valueno slot) const
 }
 
 bool
-GlassWritableDatabase::term_exists(const string & tname) const
+GlassWritableDatabase::term_exists(string_view term) const
 {
-    LOGCALL(DB, bool, "GlassWritableDatabase::term_exists", tname);
+    LOGCALL(DB, bool, "GlassWritableDatabase::term_exists", term);
+    if (term.empty()) {
+	RETURN(get_doccount() != 0);
+    }
     Xapian::doccount tf;
-    get_freqs(tname, &tf, NULL);
+    get_freqs(term, &tf, NULL);
     RETURN(tf != 0);
 }
 
@@ -1482,16 +1542,29 @@ GlassWritableDatabase::has_positions() const
     return inverter.has_positions(position_table);
 }
 
-LeafPostList *
-GlassWritableDatabase::open_post_list(const string& tname) const
+PostList *
+GlassWritableDatabase::open_post_list(string_view term) const
 {
-    LOGCALL(DB, LeafPostList *, "GlassWritableDatabase::open_post_list", tname);
+    LOGCALL(DB, PostList *, "GlassWritableDatabase::open_post_list", term);
+    RETURN(GlassWritableDatabase::open_leaf_post_list(term, false));
+}
+
+LeafPostList *
+GlassWritableDatabase::open_leaf_post_list(string_view term,
+					   bool need_read_pos) const
+{
+    LOGCALL(DB, LeafPostList *, "GlassWritableDatabase::open_leaf_post_list", term | need_read_pos);
+    (void)need_read_pos;
     intrusive_ptr<const GlassWritableDatabase> ptrtothis(this);
 
-    if (tname.empty()) {
+    if (term.empty()) {
+	Assert(!need_read_pos);
 	Xapian::doccount doccount = get_doccount();
+	if (rare(doccount == 0)) {
+	    RETURN(nullptr);
+	}
 	if (version_file.get_last_docid() == doccount) {
-	    RETURN(new ContiguousAllDocsPostList(ptrtothis, doccount));
+	    RETURN(new ContiguousAllDocsPostList(doccount));
 	}
 	inverter.flush_doclengths(postlist_table);
 	RETURN(new GlassAllDocsPostList(ptrtothis, doccount));
@@ -1499,8 +1572,14 @@ GlassWritableDatabase::open_post_list(const string& tname) const
 
     // Flush any buffered changes for this term's postlist so we can just
     // iterate from the flushed state.
-    inverter.flush_post_list(postlist_table, tname);
-    RETURN(new GlassPostList(ptrtothis, tname, true));
+    inverter.flush_post_list(postlist_table, term);
+
+    auto pl = new GlassPostList(ptrtothis, term, true);
+    if (pl->get_termfreq() == 0) {
+	delete pl;
+	pl = nullptr;
+    }
+    RETURN(pl);
 }
 
 ValueList *
@@ -1515,14 +1594,14 @@ GlassWritableDatabase::open_value_list(Xapian::valueno slot) const
 }
 
 void
-GlassWritableDatabase::read_position_list(GlassPositionList* pos_list,
+GlassWritableDatabase::read_position_list(GlassRePositionList* pos_list,
 					  Xapian::docid did,
-					  const string& term) const
+					  string_view term) const
 {
     Assert(did != 0);
     string data;
     if (inverter.get_positionlist(did, term, data)) {
-	pos_list->read_data(data);
+	pos_list->assign_data(std::move(data));
 	return;
     }
     GlassDatabase::read_position_list(pos_list, did, term);
@@ -1530,7 +1609,7 @@ GlassWritableDatabase::read_position_list(GlassPositionList* pos_list,
 
 Xapian::termcount
 GlassWritableDatabase::positionlist_count(Xapian::docid did,
-					  const string& term) const
+					  string_view term) const
 {
     Assert(did != 0);
     string data;
@@ -1542,27 +1621,21 @@ GlassWritableDatabase::positionlist_count(Xapian::docid did,
     return GlassDatabase::positionlist_count(did, term);
 }
 
-PositionList *
-GlassWritableDatabase::open_position_list(Xapian::docid did, const string & term) const
+PositionList*
+GlassWritableDatabase::open_position_list(Xapian::docid did,
+					  string_view term) const
 {
     Assert(did != 0);
-
-    AutoPtr<GlassPositionList> poslist(new GlassPositionList);
-
     string data;
     if (inverter.get_positionlist(did, term, data)) {
-	poslist->read_data(data);
-    } else if (!poslist->read_data(&position_table, did, term)) {
-	// As of 1.1.0, we don't check if the did and term exist - we just
-	// return an empty positionlist.  If the user really needs to know,
-	// they can check for themselves.
+	if (data.empty()) return nullptr;
+	return new GlassPositionList(std::move(data));
     }
-
-    return poslist.release();
+    return GlassDatabase::open_position_list(did, term);
 }
 
 TermList *
-GlassWritableDatabase::open_allterms(const string & prefix) const
+GlassWritableDatabase::open_allterms(string_view prefix) const
 {
     LOGCALL(DB, TermList*, "GlassWritableDatabase::open_allterms", prefix);
     if (change_count) {
@@ -1591,17 +1664,17 @@ GlassWritableDatabase::cancel()
 }
 
 void
-GlassWritableDatabase::add_spelling(const string & word,
+GlassWritableDatabase::add_spelling(string_view word,
 				    Xapian::termcount freqinc) const
 {
     spelling_table.add_word(word, freqinc);
 }
 
-void
-GlassWritableDatabase::remove_spelling(const string & word,
+Xapian::termcount
+GlassWritableDatabase::remove_spelling(string_view word,
 				       Xapian::termcount freqdec) const
 {
-    spelling_table.remove_word(word, freqdec);
+    return spelling_table.remove_word(word, freqdec);
 }
 
 TermList *
@@ -1612,34 +1685,34 @@ GlassWritableDatabase::open_spelling_wordlist() const
 }
 
 TermList *
-GlassWritableDatabase::open_synonym_keylist(const string & prefix) const
+GlassWritableDatabase::open_synonym_keylist(string_view prefix) const
 {
     synonym_table.merge_changes();
     return GlassDatabase::open_synonym_keylist(prefix);
 }
 
 void
-GlassWritableDatabase::add_synonym(const string & term,
-				   const string & synonym) const
+GlassWritableDatabase::add_synonym(string_view term,
+				   string_view synonym) const
 {
     synonym_table.add_synonym(term, synonym);
 }
 
 void
-GlassWritableDatabase::remove_synonym(const string & term,
-				      const string & synonym) const
+GlassWritableDatabase::remove_synonym(string_view term,
+				      string_view synonym) const
 {
     synonym_table.remove_synonym(term, synonym);
 }
 
 void
-GlassWritableDatabase::clear_synonyms(const string & term) const
+GlassWritableDatabase::clear_synonyms(string_view term) const
 {
     synonym_table.clear_synonyms(term);
 }
 
 void
-GlassWritableDatabase::set_metadata(const string & key, const string & value)
+GlassWritableDatabase::set_metadata(string_view key, string_view value)
 {
     LOGCALL_VOID(DB, "GlassWritableDatabase::set_metadata", key | value);
     string btree_key("\x00\xc0", 2);
@@ -1672,3 +1745,29 @@ GlassWritableDatabase::has_uncommitted_changes() const
 	   spelling_table.is_modified() ||
 	   docdata_table.is_modified();
 }
+
+Database::Internal*
+GlassWritableDatabase::update_lock(int flags)
+{
+    if (!postlist_table.is_open())
+	GlassTable::throw_database_closed();
+
+    if (flags != Xapian::DB_READONLY_) {
+	// Allow changing flags on an open DB.
+	postlist_table.set_flags(flags);
+	position_table.set_flags(flags);
+	termlist_table.set_flags(flags);
+	synonym_table.set_flags(flags);
+	spelling_table.set_flags(flags);
+	docdata_table.set_flags(flags);
+	return this;
+    }
+
+    unique_ptr<Database::Internal> result(new GlassDatabase(db_dir));
+    close();
+    return result.release();
+}
+
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
