@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2019 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -22,11 +22,11 @@
  * USA
  */
 
-#ifndef OM_HGUARD_GLASS_DATABASE_H
-#define OM_HGUARD_GLASS_DATABASE_H
+#ifndef XAPIAN_INCLUDED_GLASS_DATABASE_H
+#define XAPIAN_INCLUDED_GLASS_DATABASE_H
 
 #include "backends/backends.h"
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
 #include "glass_changes.h"
 #include "glass_docdata.h"
 #include "glass_inverter.h"
@@ -41,15 +41,15 @@
 #include "glass_defs.h"
 #include "backends/valuestats.h"
 
-#include "noreturn.h"
-
 #include "xapian/compactor.h"
 #include "xapian/constants.h"
 
 #include <map>
+#include <string_view>
 
 class GlassTermList;
 class GlassAllDocsPostList;
+class HoneyDatabase;
 class RemoteConnection;
 
 /** A backend designed for efficient indexing and retrieval, using
@@ -61,6 +61,9 @@ class GlassDatabase : public Xapian::Database::Internal {
     friend class GlassPostList;
     friend class GlassAllTermsList;
     friend class GlassAllDocsPostList;
+    friend class GlassSpellingWordsList;
+    friend class GlassSynonymTermList;
+    friend class HoneyDatabase; // For compacting to convert.
 
   private:
     /** Directory to store databases in.
@@ -225,7 +228,7 @@ class GlassDatabase : public Xapian::Database::Internal {
      *                    correct value, when the database is being
      *                    created.
      */
-    explicit GlassDatabase(const string& db_dir_,
+    explicit GlassDatabase(std::string_view db_dir_,
 			   int flags = Xapian::DB_READONLY_,
 			   unsigned int block_size = 0u);
 
@@ -238,13 +241,6 @@ class GlassDatabase : public Xapian::Database::Internal {
 	return postlist_table.cursor_get();
     }
 
-    /** Get an object holding the revision number which the tables are
-     *  opened at.
-     *
-     *  @return the current revision number.
-     */
-    glass_revision_number_t get_revision_number() const;
-
     /** Virtual methods of Database::Internal. */
     //@{
     Xapian::doccount get_doccount() const;
@@ -252,54 +248,64 @@ class GlassDatabase : public Xapian::Database::Internal {
     Xapian::totallength get_total_length() const;
     Xapian::termcount get_doclength(Xapian::docid did) const;
     Xapian::termcount get_unique_terms(Xapian::docid did) const;
-    void get_freqs(const string & term,
-		   Xapian::doccount * termfreq_ptr,
-		   Xapian::termcount * collfreq_ptr) const;
+    Xapian::termcount get_wdfdocmax(Xapian::docid did) const;
+    void get_freqs(std::string_view term,
+		   Xapian::doccount* termfreq_ptr,
+		   Xapian::termcount* collfreq_ptr) const;
     Xapian::doccount get_value_freq(Xapian::valueno slot) const;
     std::string get_value_lower_bound(Xapian::valueno slot) const;
     std::string get_value_upper_bound(Xapian::valueno slot) const;
     Xapian::termcount get_doclength_lower_bound() const;
     Xapian::termcount get_doclength_upper_bound() const;
-    Xapian::termcount get_wdf_upper_bound(const string & term) const;
-    bool term_exists(const string & tname) const;
+    Xapian::termcount get_wdf_upper_bound(std::string_view term) const;
+    Xapian::termcount get_unique_terms_lower_bound() const;
+    bool term_exists(std::string_view term) const;
     bool has_positions() const;
 
-    LeafPostList * open_post_list(const string & tname) const;
+    PostList* open_post_list(std::string_view tname) const;
+    LeafPostList* open_leaf_post_list(std::string_view term,
+				      bool need_read_pos) const;
     ValueList * open_value_list(Xapian::valueno slot) const;
     Xapian::Document::Internal* open_document(Xapian::docid did,
 					      bool lazy) const;
 
-    virtual void read_position_list(GlassPositionList* pos_list,
+    virtual void read_position_list(GlassRePositionList* pos_list,
 				    Xapian::docid did,
-				    const string& term) const;
+				    std::string_view term) const;
     virtual Xapian::termcount positionlist_count(Xapian::docid did,
-						 const string& term) const;
+						 std::string_view term) const;
     PositionList* open_position_list(Xapian::docid did,
-				     const string& term) const;
+				     std::string_view term) const;
     TermList * open_term_list(Xapian::docid did) const;
-    TermList * open_allterms(const string & prefix) const;
+    TermList * open_term_list_direct(Xapian::docid did) const;
+    TermList* open_allterms(std::string_view prefix) const;
 
-    TermList * open_spelling_termlist(const string & word) const;
+    TermList* open_spelling_termlist(std::string_view word) const;
     TermList * open_spelling_wordlist() const;
-    Xapian::doccount get_spelling_frequency(const string & word) const;
+    Xapian::doccount get_spelling_frequency(std::string_view word) const;
 
-    TermList * open_synonym_termlist(const string & term) const;
-    TermList * open_synonym_keylist(const string & prefix) const;
+    TermList* open_synonym_termlist(std::string_view term) const;
+    TermList* open_synonym_keylist(std::string_view prefix) const;
 
-    string get_metadata(const string & key) const;
-    TermList * open_metadata_keylist(const std::string &prefix) const;
+    string get_metadata(std::string_view key) const;
+    TermList* open_metadata_keylist(std::string_view prefix) const;
     void write_changesets_to_fd(int fd,
-				const string & start_revision,
+				std::string_view start_revision,
 				bool need_whole_db,
 				Xapian::ReplicationInfo * info);
-    string get_revision_info() const;
+    /** Get the revision number which the tables are opened at.
+     *
+     *  @return the current revision number.
+     */
+    Xapian::rev get_revision() const;
     string get_uuid() const;
 
     void request_document(Xapian::docid /*did*/) const;
-    void readahead_for_query(const Xapian::Query &query);
+    void readahead_for_query(const Xapian::Query &query) const;
     //@}
 
-    XAPIAN_NORETURN(void throw_termlist_table_close_exception() const);
+    [[noreturn]]
+    void throw_termlist_table_close_exception() const;
 
     int get_backend_info(string * path) const {
 	if (path) *path = db_dir;
@@ -316,15 +322,19 @@ class GlassDatabase : public Xapian::Database::Internal {
 
     bool locked() const;
 
+    Xapian::Database::Internal* update_lock(int flags);
+
     static void compact(Xapian::Compactor * compactor,
 			const char * destdir,
 			int fd,
-			const std::vector<Xapian::Database::Internal*>& sources,
+			const std::vector<const Xapian::Database::Internal*>& sources,
 			const std::vector<Xapian::docid> & offset,
-			size_t block_size,
+			unsigned block_size,
 			Xapian::Compactor::compaction_level compaction,
 			unsigned flags,
 			Xapian::docid last_docid);
+
+    std::string get_description() const;
 };
 
 /** A writable glass database.
@@ -332,7 +342,7 @@ class GlassDatabase : public Xapian::Database::Internal {
 class GlassWritableDatabase : public GlassDatabase {
     mutable Inverter inverter;
 
-    mutable map<Xapian::valueno, ValueStats> value_stats;
+    mutable std::map<Xapian::valueno, ValueStats> value_stats;
 
     /** The number of documents added, deleted, or replaced since the last
      *  flush.
@@ -408,7 +418,7 @@ class GlassWritableDatabase : public GlassDatabase {
      *
      *  @param dir directory holding glass tables
      */
-    GlassWritableDatabase(const string &dir, int flags, int block_size);
+    GlassWritableDatabase(std::string_view dir, int flags, int block_size);
 
     ~GlassWritableDatabase();
 
@@ -416,43 +426,51 @@ class GlassWritableDatabase : public GlassDatabase {
     //@{
     Xapian::termcount get_doclength(Xapian::docid did) const;
     Xapian::termcount get_unique_terms(Xapian::docid did) const;
-    void get_freqs(const string & term,
-		   Xapian::doccount * termfreq_ptr,
-		   Xapian::termcount * collfreq_ptr) const;
+    void get_freqs(std::string_view term,
+		   Xapian::doccount* termfreq_ptr,
+		   Xapian::termcount* collfreq_ptr) const;
     Xapian::doccount get_value_freq(Xapian::valueno slot) const;
     std::string get_value_lower_bound(Xapian::valueno slot) const;
     std::string get_value_upper_bound(Xapian::valueno slot) const;
-    bool term_exists(const string & tname) const;
+    bool term_exists(std::string_view term) const;
     bool has_positions() const;
 
-    LeafPostList * open_post_list(const string & tname) const;
+    PostList* open_post_list(std::string_view term) const;
+    LeafPostList* open_leaf_post_list(std::string_view term,
+				      bool need_read_pos) const;
     ValueList * open_value_list(Xapian::valueno slot) const;
 
-    void read_position_list(GlassPositionList* pos_list,
+    void read_position_list(GlassRePositionList* pos_list,
 			    Xapian::docid did,
-			    const string& term) const;
+			    std::string_view term) const;
     Xapian::termcount positionlist_count(Xapian::docid did,
-					 const string& term) const;
+					 std::string_view term) const;
     PositionList* open_position_list(Xapian::docid did,
-				     const string& term) const;
-    TermList * open_allterms(const string & prefix) const;
+				     std::string_view term) const;
+    TermList* open_allterms(std::string_view prefix) const;
 
-    void add_spelling(const string & word, Xapian::termcount freqinc) const;
-    void remove_spelling(const string & word,
-			 Xapian::termcount freqdec) const;
+    void add_spelling(std::string_view word, Xapian::termcount freqinc) const;
+    Xapian::termcount remove_spelling(std::string_view word,
+				      Xapian::termcount freqdec) const;
     TermList * open_spelling_wordlist() const;
 
-    TermList * open_synonym_keylist(const string & prefix) const;
-    void add_synonym(const string & word, const string & synonym) const;
-    void remove_synonym(const string & word, const string & synonym) const;
-    void clear_synonyms(const string & word) const;
+    TermList* open_synonym_keylist(std::string_view prefix) const;
+    void add_synonym(std::string_view word, std::string_view synonym) const;
+    void remove_synonym(std::string_view word, std::string_view synonym) const;
+    void clear_synonyms(std::string_view word) const;
 
-    void set_metadata(const string & key, const string & value);
+    void set_metadata(std::string_view key, std::string_view value);
     void invalidate_doc_object(Xapian::Document::Internal * obj) const;
     //@}
 
     /** Return true if there are uncommitted changes. */
     bool has_uncommitted_changes() const;
+
+    Xapian::Database::Internal* update_lock(int flags);
 };
 
-#endif /* OM_HGUARD_GLASS_DATABASE_H */
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
+
+#endif /* XAPIAN_INCLUDED_GLASS_DATABASE_H */
