@@ -2,7 +2,7 @@
  * @brief Check consistency of a glass table.
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002-2022 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 #include <xapian.h>
 
 #include "filetests.h"
-#include "autoptr.h"
+#include <memory>
 #include <ostream>
 #include <vector>
 
@@ -58,10 +58,10 @@ struct VStats : public ValueStats {
 };
 
 size_t
-check_glass_table(const char * tablename, const string &db_dir, int fd,
+check_glass_table(const char* tablename, string_view db_dir, int fd,
 		  off_t offset_,
-		  const GlassVersion & version_file, int opts,
-		  vector<Xapian::termcount> & doclens, ostream * out)
+		  const GlassVersion& version_file, int opts,
+		  vector<Xapian::termcount>& doclens, ostream* out)
 {
     Xapian::docid db_last_docid = version_file.get_last_docid();
     if (out)
@@ -88,16 +88,16 @@ check_glass_table(const char * tablename, const string &db_dir, int fd,
     }
 
     // Check the btree structure.
-    AutoPtr<GlassTableCheck> table(
+    unique_ptr<GlassTableCheck> table(
 	    GlassTableCheck::check(tablename, db_dir, fd, offset_,
 				   version_file, opts, out));
 
     // Now check the glass structures inside the btree.
-    AutoPtr<GlassCursor> cursor(table->cursor_get());
+    unique_ptr<GlassCursor> cursor(table->cursor_get());
 
     size_t errors = 0;
 
-    cursor->find_entry(string());
+    cursor->rewind();
     cursor->next(); // Skip the empty entry.
 
     if (strcmp(tablename, "postlist") == 0) {
@@ -952,7 +952,7 @@ check_glass_table(const char * tablename, const string &db_dir, int fd,
 		// Special case for single entry position list.
 	    } else {
 		// Skip the header we just read.
-		BitReader rd(data, pos - data.data());
+		BitReader rd(pos, end);
 		Xapian::termpos pos_first = rd.decode(pos_last);
 		Xapian::termpos pos_size = rd.decode(pos_last - pos_first) + 2;
 		rd.decode_interpolative(0, pos_size - 1, pos_first, pos_last);
@@ -997,3 +997,7 @@ check_glass_table(const char * tablename, const string &db_dir, int fd,
 
     return errors;
 }
+
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
