@@ -1,7 +1,7 @@
 /** @file
  * @brief Check a glass table.
  */
-/* Copyright (C) 2008,2009,2012,2013,2014,2016 Olly Betts
+/* Copyright (C) 2008,2009,2012,2013,2014,2016,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,17 +31,17 @@
 
 class GlassVersion;
 
-size_t check_glass_table(const char * tablename, const std::string &db_dir,
+size_t check_glass_table(const char* tablename, std::string_view db_dir,
 			 int fd, off_t offset_,
-			 const GlassVersion & version_file, int opts,
-			 std::vector<Xapian::termcount> & doclens,
-			 std::ostream * out);
+			 const GlassVersion& version_file, int opts,
+			 std::vector<Xapian::termcount>& doclens,
+			 std::ostream* out);
 
 inline size_t
-check_glass_table(const char * tablename, const std::string &db_dir,
-		  const GlassVersion & version_file, int opts,
-		  std::vector<Xapian::termcount> & doclens,
-		  std::ostream * out)
+check_glass_table(const char* tablename, std::string_view db_dir,
+		  const GlassVersion& version_file, int opts,
+		  std::vector<Xapian::termcount>& doclens,
+		  std::ostream* out)
 {
     return check_glass_table(tablename, db_dir, -1, 0, version_file, opts,
 			     doclens, out);
@@ -53,7 +53,7 @@ check_glass_table(const char * tablename, int fd, off_t offset_,
 		  std::vector<Xapian::termcount> & doclens,
 		  std::ostream * out)
 {
-    return check_glass_table(tablename, std::string(), fd, offset_,
+    return check_glass_table(tablename, {}, fd, offset_,
 			     version_file, opts,
 			     doclens, out);
 }
