@@ -1,7 +1,7 @@
 /** @file
  * @brief Glass freelist
  */
-/* Copyright 2014,2015,2016 Olly Betts
+/* Copyright 2014,2015,2016,2020 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,17 +27,10 @@
 #include "xapian/error.h"
 
 #include "omassert.h"
+#include "popcount.h"
 #include "wordaccess.h"
 #include <cstring>
 
-#if !HAVE_DECL___BUILTIN_POPCOUNT
-// Only include <intrin.h> if we have to as it can result in warnings about
-// duplicate declarations of builtin functions under mingw.
-# if HAVE_DECL___POPCNT || HAVE_DECL___POPCNT64
-#  include <intrin.h>
-# endif
-#endif
-
 using namespace std;
 using namespace Glass;
 
@@ -289,15 +282,15 @@ GlassFreeListChecker::count_set_bits(uint4 * p_first_bad_blk) const
 	    uint4 first_bad_blk = i * BITS_PER_ELT;
 	    if (false) {
 #if HAVE_DECL___BUILTIN_CTZ
-	    } else if (sizeof(elt_type) == sizeof(unsigned)) {
+	    } else if constexpr(sizeof(elt_type) == sizeof(unsigned)) {
 		first_bad_blk += __builtin_ctz(elt);
 #endif
 #if HAVE_DECL___BUILTIN_CTZL
-	    } else if (sizeof(elt_type) == sizeof(unsigned long)) {
+	    } else if constexpr(sizeof(elt_type) == sizeof(unsigned long)) {
 		first_bad_blk += __builtin_ctzl(elt);
 #endif
 #if HAVE_DECL___BUILTIN_CTZLL
-	    } else if (sizeof(elt_type) == sizeof(unsigned long long)) {
+	    } else if constexpr(sizeof(elt_type) == sizeof(unsigned long long)) {
 		first_bad_blk += __builtin_ctzll(elt);
 #endif
 	    } else {
@@ -310,31 +303,7 @@ GlassFreeListChecker::count_set_bits(uint4 * p_first_bad_blk) const
 	}
 
 	// Count set bits in elt.
-	if (false) {
-#if HAVE_DECL___BUILTIN_POPCOUNT
-	} else if (sizeof(elt_type) == sizeof(unsigned)) {
-	    c += __builtin_popcount(elt);
-#elif HAVE_DECL___POPCNT
-	} else if (sizeof(elt_type) == sizeof(unsigned)) {
-	    c += static_cast<uint4>(__popcnt(elt));
-#endif
-#if HAVE_DECL___BUILTIN_POPCOUNTL
-	} else if (sizeof(elt_type) == sizeof(unsigned long)) {
-	    c += __builtin_popcountl(elt);
-#endif
-#if HAVE_DECL___BUILTIN_POPCOUNTLL
-	} else if (sizeof(elt_type) == sizeof(unsigned long long)) {
-	    c += __builtin_popcountll(elt);
-#elif HAVE_DECL___POPCNT64
-	} else if (sizeof(elt_type) == sizeof(unsigned long long)) {
-	    c += static_cast<uint4>(__popcnt64(elt));
-#endif
-	} else {
-	    do {
-		++c;
-		elt &= elt - 1;
-	    } while (elt);
-	}
+	add_popcount(c, elt);
     }
     return c;
 }
