@@ -1,7 +1,7 @@
 /** @file
  * @brief A position list in a glass database.
  */
-/* Copyright (C) 2005,2006,2008,2009,2010,2011,2013,2016,2019 Olly Betts
+/* Copyright (C) 2005-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,72 +19,28 @@
  * USA
  */
 
-#ifndef XAPIAN_HGUARD_GLASS_POSITIONLIST_H
-#define XAPIAN_HGUARD_GLASS_POSITIONLIST_H
+#ifndef XAPIAN_INCLUDED_GLASS_POSITIONLIST_H
+#define XAPIAN_INCLUDED_GLASS_POSITIONLIST_H
 
 #include <xapian/types.h>
 
-#include "autoptr.h"
 #include "bitstream.h"
+#include "glass_cursor.h"
 #include "glass_lazytable.h"
 #include "pack.h"
 #include "backends/positionlist.h"
 
 #include <string>
 
-using namespace std;
+/** Base-class for a position list in a glass database. */
+class GlassBasePositionList : public PositionList {
+    /// Copying is not allowed.
+    GlassBasePositionList(const GlassBasePositionList&) = delete;
 
-class GlassPositionListTable : public GlassLazyTable {
-  public:
-    static string make_key(Xapian::docid did, const string & term) {
-	string key;
-	pack_string_preserving_sort(key, term);
-	pack_uint_preserving_sort(key, did);
-	return key;
-    }
+    /// Assignment is not allowed.
+    GlassBasePositionList& operator=(const GlassBasePositionList&) = delete;
 
-    /** Create a new GlassPositionListTable object.
-     *
-     *  This method does not create or open the table on disk - you
-     *  must call the create() or open() methods respectively!
-     *
-     *  @param dbdir		The directory the glass database is stored in.
-     *  @param readonly		true if we're opening read-only, else false.
-     */
-    GlassPositionListTable(const string & dbdir, bool readonly)
-	: GlassLazyTable("position", dbdir + "/position.", readonly) { }
-
-    GlassPositionListTable(int fd, off_t offset_, bool readonly_)
-	: GlassLazyTable("position", fd, offset_, readonly_) { }
-
-    /** Pack a position list into a string.
-     *
-     *  @param s The string to append the position list data to.
-     */
-    void pack(string & s, const std::vector<Xapian::termpos> & vec) const;
-
-    /** Set the position list for term tname in document did.
-     */
-    void set_positionlist(Xapian::docid did, const string & tname,
-			  const string & s) {
-	add(make_key(did, tname), s);
-    }
-
-    /// Delete the position list for term tname in document did.
-    void delete_positionlist(Xapian::docid did, const string & tname) {
-	del(make_key(did, tname));
-    }
-
-    /// Return the number of entries in specified position list data.
-    Xapian::termcount positionlist_count(const string& data) const;
-
-    /// Return the number of entries in specified position list.
-    Xapian::termcount positionlist_count(Xapian::docid did,
-					 const string & term) const;
-};
-
-/** A position list in a glass database. */
-class GlassPositionList : public PositionList {
+  protected:
     /// Interpolative decoder.
     BitReader rd;
 
@@ -97,50 +53,25 @@ class GlassPositionList : public PositionList {
     /// Number of entries.
     Xapian::termcount size;
 
-    /// Cursor for locating multiple entries efficiently.
-    AutoPtr<GlassCursor> cursor;
-
     /// Have we started iterating yet?
     bool have_started;
 
-    /// Copying is not allowed.
-    GlassPositionList(const GlassPositionList &);
-
-    /// Assignment is not allowed.
-    void operator=(const GlassPositionList &);
+    /** Set positional data and start to decode it.
+     *
+     *  @param data	The positional data.  Must stay valid
+     *			while this object is using it.
+     */
+    void set_data(std::string_view data);
 
   public:
     /// Default constructor.
-    GlassPositionList() { }
-
-    /// Construct and initialise with data.
-    explicit
-    GlassPositionList(const string& data) {
-	read_data(data);
-    }
-
-    /// Construct and initialise with data.
-    GlassPositionList(const GlassTable * table, Xapian::docid did,
-		      const string & tname) {
-	(void)read_data(table, did, tname);
-    }
-
-    /** Fill list with data, and move the position to the start.
-     *
-     *  @return true if position data was read.
-     */
-    bool read_data(const string & data);
-
-    /** Fill list with data, and move the position to the start.
-     *
-     *  @return true if position data was read.
-     */
-    bool read_data(const GlassTable * table, Xapian::docid did,
-		   const string & tname);
+    GlassBasePositionList() {}
 
     /// Returns size of position list.
     Xapian::termcount get_approx_size() const;
 
+    Xapian::termpos back() const;
+
     /** Returns current position.
      *
      *  Either next() or skip_to() must have been called before this
@@ -155,4 +86,104 @@ class GlassPositionList : public PositionList {
     bool skip_to(Xapian::termpos termpos);
 };
 
-#endif /* XAPIAN_HGUARD_GLASS_POSITIONLIST_H */
+/** A position list in a glass database. */
+class GlassPositionList : public GlassBasePositionList {
+    /// The encoded positional data being read by rd.
+    std::string pos_data;
+
+    /// Copying is not allowed.
+    GlassPositionList(const GlassPositionList&) = delete;
+
+    /// Assignment is not allowed.
+    GlassPositionList& operator=(const GlassPositionList&) = delete;
+
+  public:
+    /// Construct and initialise with data.
+    explicit
+    GlassPositionList(std::string&& data);
+};
+
+/** A reusable position list in a glass database. */
+class GlassRePositionList : public GlassBasePositionList {
+    /// Cursor for locating multiple entries efficiently.
+    GlassCursor cursor;
+
+    /// Copying is not allowed.
+    GlassRePositionList(const GlassRePositionList&) = delete;
+
+    /// Assignment is not allowed.
+    GlassRePositionList& operator=(const GlassRePositionList&) = delete;
+
+  public:
+    /// Constructor.
+    explicit
+    GlassRePositionList(const GlassTable* table)
+	: cursor(table) {}
+
+    /** Fill list with data, and move the position to the start. */
+    void assign_data(std::string&& data);
+
+    /** Fill list with data, and move the position to the start. */
+    void read_data(Xapian::docid did,
+		   std::string_view term);
+};
+
+class GlassPositionListTable : public GlassLazyTable {
+  public:
+    static std::string make_key(Xapian::docid did, std::string_view term) {
+	std::string key;
+	pack_string_preserving_sort(key, term);
+	pack_uint_preserving_sort(key, did);
+	return key;
+    }
+
+    /** Create a new GlassPositionListTable object.
+     *
+     *  This method does not create or open the table on disk - you
+     *  must call the create() or open() methods respectively!
+     *
+     *  @param dbdir		The directory the glass database is stored in.
+     *  @param readonly		true if we're opening read-only, else false.
+     */
+    GlassPositionListTable(const std::string& dbdir, bool readonly)
+	: GlassLazyTable("position", dbdir + "/position.", readonly) { }
+
+    GlassPositionListTable(int fd, off_t offset_, bool readonly_)
+	: GlassLazyTable("position", fd, offset_, readonly_) { }
+
+    GlassPositionList* open_position_list(Xapian::docid did,
+					  std::string_view term) {
+	std::string pos_data;
+	if (!get_exact_entry(make_key(did, term), pos_data))
+	    return nullptr;
+
+	return new GlassPositionList(std::move(pos_data));
+    }
+
+    /** Pack a position list into a string.
+     *
+     *  @param s The string to append the position list data to.
+     */
+    void pack(std::string& s, const Xapian::VecCOW<Xapian::termpos>& vec) const;
+
+    /** Set the position list for term tname in document did.
+     */
+    void set_positionlist(Xapian::docid did, std::string_view tname,
+			  std::string_view s) {
+	add(make_key(did, tname), s);
+    }
+
+    /// Delete the position list for term tname in document did.
+    void delete_positionlist(Xapian::docid did, std::string_view tname) {
+	del(make_key(did, tname));
+    }
+
+    /// Return the number of entries in specified position list data.
+    Xapian::termcount positionlist_count(std::string_view data) const;
+
+    /// Return the number of entries in specified position list.
+    Xapian::termcount positionlist_count(Xapian::docid did,
+					 std::string_view term) const;
+};
+
+#endif /* XAPIAN_INCLUDED_GLASS_POSITIONLIST_H */
