@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2007,2008,2009,2011,2013,2014,2015 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  * Copyright 2007,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -22,22 +22,21 @@
  * USA
  */
 
-#ifndef OM_HGUARD_GLASS_POSTLIST_H
-#define OM_HGUARD_GLASS_POSTLIST_H
+#ifndef XAPIAN_INCLUDED_GLASS_POSTLIST_H
+#define XAPIAN_INCLUDED_GLASS_POSTLIST_H
 
 #include <xapian/database.h>
 
+#include "backends/leafpostlist.h"
 #include "glass_defs.h"
 #include "glass_inverter.h"
 #include "glass_positionlist.h"
-#include "api/leafpostlist.h"
 #include "omassert.h"
 
-#include "autoptr.h"
+#include <memory>
 #include <map>
 #include <string>
-
-using namespace std;
+#include <string_view>
 
 class GlassCursor;
 class GlassDatabase;
@@ -60,7 +59,7 @@ class GlassPostList : public LeafPostList {
     Xapian::Internal::intrusive_ptr<const GlassDatabase> this_db;
 
     /// The position list object for this posting list.
-    GlassPositionList positionlist;
+    GlassRePositionList* positionlist = NULL;
 
     /// Whether we've started reading the list yet.
     bool have_started;
@@ -72,7 +71,7 @@ class GlassPostList : public LeafPostList {
     bool is_at_end;
 
     /// Cursor pointing to current chunk of postlist.
-    AutoPtr<GlassCursor> cursor;
+    std::unique_ptr<GlassCursor> cursor;
 
     /// The first document id in this chunk.
     Xapian::docid first_did_in_chunk;
@@ -92,9 +91,6 @@ class GlassPostList : public LeafPostList {
     /// The wdf of the current document.
     Xapian::termcount wdf;
 
-    /// The number of entries in the posting list.
-    Xapian::doccount number_of_entries;
-
     /// Upper bound on wdf for this postlist.
     Xapian::termcount wdf_upper_bound;
 
@@ -150,7 +146,7 @@ class GlassPostList : public LeafPostList {
     bool move_forward_in_chunk_to_at_least(Xapian::docid desired_did);
 
     GlassPostList(Xapian::Internal::intrusive_ptr<const GlassDatabase> this_db_,
-		  const string & term,
+		  std::string_view term,
 		  GlassCursor * cursor_);
 
     void init();
@@ -158,13 +154,15 @@ class GlassPostList : public LeafPostList {
   public:
     /// Default constructor.
     GlassPostList(Xapian::Internal::intrusive_ptr<const GlassDatabase> this_db_,
-		  const string & term,
+		  std::string_view term,
 		  bool keep_reference);
 
     /// Destructor.
     ~GlassPostList();
 
-    LeafPostList * open_nearby_postlist(const std::string & term_) const;
+    bool open_nearby_postlist(std::string_view term_,
+			      bool need_read_pos,
+			      LeafPostList*& pl) const;
 
     /** Used for looking up doclens.
      *
@@ -172,21 +170,9 @@ class GlassPostList : public LeafPostList {
      */
     bool jump_to(Xapian::docid desired_did);
 
-    /** Returns number of docs indexed by this term.
-     *
-     *  This is the length of the postlist.
-     */
-    Xapian::doccount get_termfreq() const { return number_of_entries; }
-
     /// Returns the current docid.
     Xapian::docid get_docid() const { Assert(have_started); return did; }
 
-    /// Returns the length of current document.
-    Xapian::termcount get_doclength() const;
-
-    /// Returns the number of unique terms in the current document.
-    Xapian::termcount get_unique_terms() const;
-
     /** Returns the Within Document Frequency of the term in the current
      *  document.
      */
@@ -211,19 +197,21 @@ class GlassPostList : public LeafPostList {
 
     Xapian::termcount get_wdf_upper_bound() const;
 
+    void get_docid_range(Xapian::docid& first, Xapian::docid& last) const;
+
     /// Get a description of the document.
     std::string get_description() const;
 
-    /// Read the number of entries and the collection frequency.
-    static void read_number_of_entries(const char ** posptr,
-				       const char * end,
-				       Xapian::doccount * number_of_entries_ptr,
-				       Xapian::termcount * collection_freq_ptr);
+    /// Read the term frequency and collection frequency.
+    static void read_freqs(const char** posptr,
+			   const char* end,
+			   Xapian::doccount* number_of_entries_ptr,
+			   Xapian::termcount* collection_freq_ptr);
 };
 
 class GlassPostListTable : public GlassTable {
     /// PostList for looking up document lengths.
-    mutable AutoPtr<GlassPostList> doclen_pl;
+    mutable std::unique_ptr<GlassPostList> doclen_pl;
 
   public:
     /** Create a new table object.
@@ -238,7 +226,7 @@ class GlassPostListTable : public GlassTable {
      *  @param readonly_      - whether to open the table for read only
      *                          access.
      */
-    GlassPostListTable(const string & path_, bool readonly_)
+    GlassPostListTable(const std::string& path_, bool readonly_)
 	: GlassTable("postlist", path_ + "/postlist.", readonly_),
 	  doclen_pl()
     { }
@@ -255,29 +243,29 @@ class GlassPostListTable : public GlassTable {
     }
 
     /// Merge changes for a term.
-    void merge_changes(const string& term,
+    void merge_changes(std::string_view term,
 		       const Inverter::PostingChanges& changes);
 
     /// Merge document length changes.
-    void merge_doclen_changes(const map<Xapian::docid,
+    void merge_doclen_changes(const std::map<Xapian::docid,
 					Xapian::termcount>& doclens);
 
-    Xapian::docid get_chunk(const string& tname,
+    Xapian::docid get_chunk(std::string_view tname,
 			    Xapian::docid did, bool adding,
 			    Glass::PostlistChunkReader** from,
 			    Glass::PostlistChunkWriter** to);
 
     /// Compose a key from a termname and docid.
-    static string make_key(const string& term, Xapian::docid did) {
+    static std::string make_key(std::string_view term, Xapian::docid did) {
 	return pack_glass_postlist_key(term, did);
     }
 
     /// Compose a key from a termname.
-    static string make_key(const string & term) {
+    static std::string make_key(std::string_view term) {
 	return pack_glass_postlist_key(term);
     }
 
-    bool term_exists(const string & term) const {
+    bool term_exists(std::string_view term) const {
 	return key_exists(make_key(term));
     }
 
@@ -292,10 +280,10 @@ class GlassPostListTable : public GlassTable {
      *			of @a term in the database (or NULL not to
      *			return)
      */
-    void get_freqs(const std::string & term,
-		   Xapian::doccount * termfreq_ptr,
-		   Xapian::termcount * collfreq_ptr,
-		   Xapian::termcount * wdfub_ptr = NULL) const;
+    void get_freqs(std::string_view term,
+		   Xapian::doccount* termfreq_ptr,
+		   Xapian::termcount* collfreq_ptr,
+		   Xapian::termcount* wdfub_ptr = NULL) const;
 
     /** Returns the length of document @a did. */
     Xapian::termcount get_doclength(Xapian::docid did,
@@ -308,5 +296,8 @@ class GlassPostListTable : public GlassTable {
     void get_used_docid_range(Xapian::docid & first,
 			      Xapian::docid & last) const;
 };
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
 
-#endif /* OM_HGUARD_GLASS_POSTLIST_H */
+#endif /* XAPIAN_INCLUDED_GLASS_POSTLIST_H */
