@@ -1,7 +1,7 @@
 /** @file
  * @brief Spelling correction data for a glass database.
  */
-/* Copyright (C) 2004,2005,2006,2007,2008,2009,2010,2011,2015 Olly Betts
+/* Copyright (C) 2004-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,9 @@
 #include <xapian/types.h>
 
 #include "expand/expandweight.h"
+#include "expand/termlistmerger.h"
 #include "glass_spelling.h"
 #include "omassert.h"
-#include "expand/ortermlist.h"
 #include "pack.h"
 
 #include "../prefix_compressed_strings.h"
@@ -37,6 +37,7 @@
 #include <vector>
 #include <set>
 #include <string>
+#include <string_view>
 
 using namespace Glass;
 using namespace std;
@@ -92,10 +93,9 @@ GlassSpellingTable::merge_changes()
     }
     termlist_deltas.clear();
 
-    map<string, Xapian::termcount>::const_iterator j;
-    for (j = wordfreq_changes.begin(); j != wordfreq_changes.end(); ++j) {
-	string key = "W" + j->first;
-	Xapian::termcount wordfreq = j->second;
+    for (auto&& j : wordfreq_changes) {
+	string key = "W" + j.first;
+	Xapian::termcount wordfreq = j.second;
 	if (wordfreq) {
 	    string tag;
 	    pack_uint_last(tag, wordfreq);
@@ -110,7 +110,7 @@ GlassSpellingTable::merge_changes()
 }
 
 void
-GlassSpellingTable::toggle_fragment(fragment frag, const string & word)
+GlassSpellingTable::toggle_fragment(fragment frag, string_view word)
 {
     auto i = termlist_deltas.find(frag);
     if (i == termlist_deltas.end()) {
@@ -118,7 +118,7 @@ GlassSpellingTable::toggle_fragment(fragment frag, const string & word)
     }
     // The commonest case is that we're adding lots of words, so try insert
     // first and if that reports that the word already exists, remove it.
-    auto res = i->second.insert(word);
+    auto res = i->second.emplace(word);
     if (!res.second) {
 	// word is already in the set, so remove it.
 	i->second.erase(res.first);
@@ -126,11 +126,11 @@ GlassSpellingTable::toggle_fragment(fragment frag, const string & word)
 }
 
 void
-GlassSpellingTable::add_word(const string & word, Xapian::termcount freqinc)
+GlassSpellingTable::add_word(string_view word, Xapian::termcount freqinc)
 {
     if (word.size() <= 1) return;
 
-    map<string, Xapian::termcount>::iterator i = wordfreq_changes.find(word);
+    auto i = wordfreq_changes.find(word);
     if (i != wordfreq_changes.end()) {
 	// Word "word" already exists and has been modified.
 	if (i->second) {
@@ -141,7 +141,7 @@ GlassSpellingTable::add_word(const string & word, Xapian::termcount freqinc)
 	// we need to execute the code below to re-add trigrams for it.
 	i->second = freqinc;
     } else {
-	string key = "W" + word;
+	string key = "W"s.append(word);
 	string data;
 	if (get_exact_entry(key, data)) {
 	    // Word "word" already exists, so increment its count.
@@ -150,41 +150,42 @@ GlassSpellingTable::add_word(const string & word, Xapian::termcount freqinc)
 	    if (!unpack_uint_last(&p, p + data.size(), &freq) || freq == 0) {
 		throw Xapian::DatabaseCorruptError("Bad spelling word freq");
 	    }
-	    wordfreq_changes[word] = freq + freqinc;
+	    wordfreq_changes.emplace(word, freq + freqinc);
 	    return;
 	}
-	wordfreq_changes[word] = freqinc;
+	wordfreq_changes.emplace(word, freqinc);
     }
 
     // Add trigrams for word.
     toggle_word(word);
 }
 
-void
-GlassSpellingTable::remove_word(const string & word, Xapian::termcount freqdec)
+Xapian::termcount
+GlassSpellingTable::remove_word(string_view word, Xapian::termcount freqdec)
 {
-    if (word.size() <= 1) return;
+    if (word.size() <= 1) return freqdec;
 
-    map<string, Xapian::termcount>::iterator i = wordfreq_changes.find(word);
+    auto i = wordfreq_changes.find(word);
     if (i != wordfreq_changes.end()) {
 	if (i->second == 0) {
 	    // Word has already been deleted.
-	    return;
+	    return freqdec;
 	}
 	// Word "word" exists and has been modified.
 	if (freqdec < i->second) {
 	    i->second -= freqdec;
-	    return;
+	    return 0;
 	}
+	freqdec -= i->second;
 
 	// Mark word as deleted.
 	i->second = 0;
     } else {
-	string key = "W" + word;
+	string key = "W"s.append(word);
 	string data;
 	if (!get_exact_entry(key, data)) {
 	    // This word doesn't exist.
-	    return;
+	    return freqdec;
 	}
 
 	Xapian::termcount freq;
@@ -193,19 +194,23 @@ GlassSpellingTable::remove_word(const string & word, Xapian::termcount freqdec)
 	    throw Xapian::DatabaseCorruptError("Bad spelling word freq");
 	}
 	if (freqdec < freq) {
-	    wordfreq_changes[word] = freq - freqdec;
-	    return;
+	    wordfreq_changes.emplace(word, freq - freqdec);
+	    return 0;
 	}
+	freqdec -= freq;
+
 	// Mark word as deleted.
-	wordfreq_changes[word] = 0;
+	wordfreq_changes.emplace(word, 0);
     }
 
     // Remove trigrams for word.
     toggle_word(word);
+
+    return freqdec;
 }
 
 void
-GlassSpellingTable::toggle_word(const string & word)
+GlassSpellingTable::toggle_word(string_view word)
 {
     fragment buf;
     // Head:
@@ -254,8 +259,8 @@ struct TermListGreaterApproxSize {
     }
 };
 
-TermList *
-GlassSpellingTable::open_termlist(const string & word)
+TermList*
+GlassSpellingTable::open_termlist(string_view word)
 {
     // This should have been handled by Database::get_spelling_suggestion().
     AssertRel(word.size(),>,1);
@@ -264,9 +269,7 @@ GlassSpellingTable::open_termlist(const string & word)
     // won't be switched live.
     if (!wordfreq_changes.empty()) merge_changes();
 
-    // Build a priority queue of TermList objects which returns those of
-    // greatest approximate size first.
-    priority_queue<TermList*, vector<TermList*>, TermListGreaterApproxSize> pq;
+    vector<TermList*> termlists;
     try {
 	string data;
 	fragment buf;
@@ -276,14 +279,14 @@ GlassSpellingTable::open_termlist(const string & word)
 	buf[1] = word[0];
 	buf[2] = word[1];
 	if (get_exact_entry(string(buf), data))
-	    pq.push(new GlassSpellingTermList(data));
+	    termlists.push_back(new GlassSpellingTermList(data));
 
 	// Tail:
 	buf[0] = 'T';
 	buf[1] = word[word.size() - 2];
 	buf[2] = word[word.size() - 1];
 	if (get_exact_entry(string(buf), data))
-	    pq.push(new GlassSpellingTermList(data));
+	    termlists.push_back(new GlassSpellingTermList(data));
 
 	if (word.size() <= 4) {
 	    // We also generate 'bookends' for two, three, and four character
@@ -295,7 +298,7 @@ GlassSpellingTable::open_termlist(const string & word)
 	    buf[1] = word[0];
 	    buf[3] = '\0';
 	    if (get_exact_entry(string(buf), data))
-		pq.push(new GlassSpellingTermList(data));
+		termlists.push_back(new GlassSpellingTermList(data));
 	}
 	if (word.size() > 2) {
 	    // Middles:
@@ -303,7 +306,7 @@ GlassSpellingTable::open_termlist(const string & word)
 	    for (size_t start = 0; start <= word.size() - 3; ++start) {
 		memcpy(buf.data + 1, word.data() + start, 3);
 		if (get_exact_entry(string(buf), data))
-		    pq.push(new GlassSpellingTermList(data));
+		    termlists.push_back(new GlassSpellingTermList(data));
 	    }
 
 	    if (word.size() == 3) {
@@ -314,13 +317,13 @@ GlassSpellingTable::open_termlist(const string & word)
 		buf[1] = word[1];
 		buf[2] = word[0];
 		if (get_exact_entry(string(buf), data))
-		    pq.push(new GlassSpellingTermList(data));
+		    termlists.push_back(new GlassSpellingTermList(data));
 		// ABC -> ACB
 		buf[1] = word[0];
 		buf[2] = word[2];
 		buf[3] = word[1];
 		if (get_exact_entry(string(buf), data))
-		    pq.push(new GlassSpellingTermList(data));
+		    termlists.push_back(new GlassSpellingTermList(data));
 	    }
 	} else {
 	    Assert(word.size() == 2);
@@ -332,55 +335,33 @@ GlassSpellingTable::open_termlist(const string & word)
 	    buf[1] = word[1];
 	    buf[2] = word[0];
 	    if (get_exact_entry(string(buf), data))
-		pq.push(new GlassSpellingTermList(data));
+		termlists.push_back(new GlassSpellingTermList(data));
 	    buf[0] = 'T';
 	    if (get_exact_entry(string(buf), data))
-		pq.push(new GlassSpellingTermList(data));
+		termlists.push_back(new GlassSpellingTermList(data));
 	}
 
-	if (pq.empty()) return NULL;
-
-	// Build up an OrTermList tree by combine leaves and/or branches in
-	// pairs.  The tree is balanced by the approximated sizes of the leaf
-	// GlassSpellingTermList objects - the way the tree is built are very
-	// similar to how an optimal Huffman code is often constructed.
-	//
-	// Balancing the tree like this should tend to minimise the amount of
-	// work done.
-	while (pq.size() > 1) {
-	    // Build the tree such that left is always >= right so that
-	    // OrTermList can rely on this when trying to minimise work.
-	    TermList * termlist = pq.top();
-	    pq.pop();
-
-	    termlist = new OrTermList(pq.top(), termlist);
-	    pq.pop();
-	    pq.push(termlist);
-	}
-
-	return pq.top();
+	return make_termlist_merger(termlists);
     } catch (...) {
 	// Make sure we delete all the TermList objects to avoid leaking
 	// memory.
-	while (!pq.empty()) {
-	    delete pq.top();
-	    pq.pop();
+	for (auto& t : termlists) {
+	    delete t;
 	}
 	throw;
     }
 }
 
 Xapian::doccount
-GlassSpellingTable::get_word_frequency(const string & word) const
+GlassSpellingTable::get_word_frequency(string_view word) const
 {
-    map<string, Xapian::termcount>::const_iterator i;
-    i = wordfreq_changes.find(word);
+    auto i = wordfreq_changes.find(word);
     if (i != wordfreq_changes.end()) {
 	// Modified frequency for word:
 	return i->second;
     }
 
-    string key = "W" + word;
+    string key = "W"s.append(word);
     string data;
     if (get_exact_entry(key, data)) {
 	// Word "word" already exists.
@@ -405,12 +386,6 @@ GlassSpellingTermList::get_approx_size() const
     return data.size();
 }
 
-std::string
-GlassSpellingTermList::get_termname() const
-{
-    return current_term;
-}
-
 Xapian::termcount
 GlassSpellingTermList::get_wdf() const
 {
@@ -427,9 +402,7 @@ TermList *
 GlassSpellingTermList::next()
 {
     if (p == data.size()) {
-	p = 0;
-	data.resize(0);
-	return NULL;
+	return this;
     }
     if (!current_term.empty()) {
 	current_term.resize(uint8_t(data[p++]) ^ MAGIC_XOR_VALUE);
@@ -443,28 +416,23 @@ GlassSpellingTermList::next()
     return NULL;
 }
 
-TermList *
-GlassSpellingTermList::skip_to(const string & term)
+TermList*
+GlassSpellingTermList::skip_to(string_view term)
 {
-    while (!data.empty() && current_term < term) {
-	(void)GlassSpellingTermList::next();
+    while (current_term < term) {
+	if (GlassSpellingTermList::next())
+	    return this;
     }
     return NULL;
 }
 
-bool
-GlassSpellingTermList::at_end() const
-{
-    return data.empty();
-}
-
 Xapian::termcount
 GlassSpellingTermList::positionlist_count() const
 {
     throw Xapian::UnimplementedError("GlassSpellingTermList::positionlist_count() not implemented");
 }
 
-Xapian::PositionIterator
+PositionList*
 GlassSpellingTermList::positionlist_begin() const
 {
     throw Xapian::UnimplementedError("GlassSpellingTermList::positionlist_begin() not implemented");
