@@ -1,7 +1,7 @@
 /** @file
  * @brief Synonym data for a glass database.
  */
-/* Copyright (C) 2004,2005,2006,2007,2008,2009,2011 Olly Betts
+/* Copyright (C) 2004,2005,2006,2007,2008,2009,2011,2017,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,14 @@
 #include "xapian/error.h"
 
 #include "glass_cursor.h"
+#include "glass_database.h"
 #include "debuglog.h"
 #include "stringutils.h"
 #include "api/vectortermlist.h"
 
 #include <set>
 #include <string>
+#include <string_view>
 #include <vector>
 
 using namespace std;
@@ -48,14 +50,10 @@ GlassSynonymTable::merge_changes()
 	del(last_term);
     } else {
 	string tag;
-
-	set<string>::const_iterator i;
-	for (i = last_synonyms.begin(); i != last_synonyms.end(); ++i) {
-	    const string & synonym = *i;
+	for (const auto& synonym : last_synonyms) {
 	    tag += uint8_t(synonym.size() ^ MAGIC_XOR_VALUE);
 	    tag += synonym;
 	}
-
 	add(last_term, tag);
 	last_synonyms.clear();
     }
@@ -63,7 +61,7 @@ GlassSynonymTable::merge_changes()
 }
 
 void
-GlassSynonymTable::add_synonym(const string & term, const string & synonym)
+GlassSynonymTable::add_synonym(string_view term, string_view synonym)
 {
     if (last_term != term) {
 	merge_changes();
@@ -85,11 +83,11 @@ GlassSynonymTable::add_synonym(const string & term, const string & synonym)
 	}
     }
 
-    last_synonyms.insert(synonym);
+    last_synonyms.emplace(synonym);
 }
 
 void
-GlassSynonymTable::remove_synonym(const string & term, const string & synonym)
+GlassSynonymTable::remove_synonym(string_view term, string_view synonym)
 {
     if (last_term != term) {
 	merge_changes();
@@ -105,17 +103,21 @@ GlassSynonymTable::remove_synonym(const string & term, const string & synonym)
 		    (len = uint8_t(*p) ^ MAGIC_XOR_VALUE) >= size_t(end - p))
 		    throw Xapian::DatabaseCorruptError("Bad synonym data");
 		++p;
-		last_synonyms.insert(string(p, len));
+		last_synonyms.emplace(p, len);
 		p += len;
 	    }
 	}
     }
 
+#ifdef __cpp_lib_associative_heterogeneous_erasure // C++23
     last_synonyms.erase(synonym);
+#else
+    last_synonyms.erase(string(synonym));
+#endif
 }
 
 void
-GlassSynonymTable::clear_synonyms(const string & term)
+GlassSynonymTable::clear_synonyms(string_view term)
 {
     // We don't actually ever need to merge_changes() here, but it's quite
     // likely that someone might clear_synonyms() and then add_synonym() for
@@ -130,8 +132,8 @@ GlassSynonymTable::clear_synonyms(const string & term)
     }
 }
 
-TermList *
-GlassSynonymTable::open_termlist(const string & term)
+TermList*
+GlassSynonymTable::open_termlist(string_view term)
 {
     vector<string> synonyms;
 
@@ -139,9 +141,8 @@ GlassSynonymTable::open_termlist(const string & term)
 	if (last_synonyms.empty()) return NULL;
 
 	synonyms.reserve(last_synonyms.size());
-	set<string>::const_iterator i;
-	for (i = last_synonyms.begin(); i != last_synonyms.end(); ++i) {
-	    synonyms.push_back(*i);
+	for (const auto& i : last_synonyms) {
+	    synonyms.push_back(i);
 	}
     } else {
 	string tag;
@@ -171,14 +172,12 @@ GlassSynonymTermList::~GlassSynonymTermList()
     delete cursor;
 }
 
-string
-GlassSynonymTermList::get_termname() const
+Xapian::termcount
+GlassSynonymTermList::get_approx_size() const
 {
-    LOGCALL(DB, string, "GlassSynonymTermList::get_termname", NO_ARGS);
-    Assert(cursor);
-    Assert(!cursor->current_key.empty());
-    Assert(!at_end());
-    RETURN(cursor->current_key);
+    // This is an over-estimate, but we only use this value to build a balanced
+    // or-tree, and it'll do a decent enough job for that.
+    return database->synonym_table.get_entry_count();
 }
 
 Xapian::doccount
@@ -191,37 +190,34 @@ TermList *
 GlassSynonymTermList::next()
 {
     LOGCALL(DB, TermList *, "GlassSynonymTermList::next", NO_ARGS);
-    Assert(!at_end());
+    Assert(!cursor->after_end());
 
-    cursor->next();
-    if (!cursor->after_end() && !startswith(cursor->current_key, prefix)) {
+    if (!cursor->next() || !startswith(cursor->current_key, prefix)) {
 	// We've reached the end of the prefixed terms.
-	cursor->to_end();
+	RETURN(this);
     }
+    current_term = cursor->current_key;
 
     RETURN(NULL);
 }
 
-TermList *
-GlassSynonymTermList::skip_to(const string &tname)
+TermList*
+GlassSynonymTermList::skip_to(string_view tname)
 {
     LOGCALL(DB, TermList *, "GlassSynonymTermList::skip_to", tname);
-    Assert(!at_end());
+    Assert(!cursor->after_end());
 
-    if (!cursor->find_entry_ge(tname)) {
+    if (cursor->find_entry_ge(tname)) {
+	// Exact match.
+	current_term = tname;
+    } else {
 	// The exact term we asked for isn't there, so check if the next
 	// term after it also has the right prefix.
-	if (!cursor->after_end() && !startswith(cursor->current_key, prefix)) {
+	if (cursor->after_end() || !startswith(cursor->current_key, prefix)) {
 	    // We've reached the end of the prefixed terms.
-	    cursor->to_end();
+	    RETURN(this);
 	}
+	current_term = cursor->current_key;
     }
     RETURN(NULL);
 }
-
-bool
-GlassSynonymTermList::at_end() const
-{
-    LOGCALL(DB, bool, "GlassSynonymTermList::at_end", NO_ARGS);
-    RETURN(cursor->after_end());
-}
