@@ -53,6 +53,7 @@
 
 #include <algorithm>  // for std::min()
 #include <string>
+#include <string_view>
 
 #include "xapian/constants.h"
 
@@ -293,9 +294,9 @@ GlassTable::write_block(uint4 n, const uint8_t * p, bool appending) const
 */
 
 void
-GlassTable::set_overwritten() const
+GlassTable::throw_overwritten() const
 {
-    LOGCALL_VOID(DB, "GlassTable::set_overwritten", NO_ARGS);
+    LOGCALL_VOID(DB, "GlassTable::throw_overwritten", NO_ARGS);
     // If we're writable, there shouldn't be another writer who could cause
     // overwritten to be flagged, so that's a DatabaseCorruptError.
     if (writable)
@@ -340,8 +341,7 @@ GlassTable::block_to_cursor(Glass::Cursor * C_, int j, uint4 n) const
     if (j < level) {
 	/* unsigned comparison */
 	if (rare(REVISION(p) > REVISION(C_[j + 1].get_p()))) {
-	    set_overwritten();
-	    return;
+	    throw_overwritten();
 	}
     }
 
@@ -1256,7 +1256,7 @@ The bracketed parts are left blank. The key is filled in with key_len bytes and
 K set accordingly. c is set to 1.
 */
 
-void GlassTable::form_key(const string & key) const
+void GlassTable::form_key(string_view key) const
 {
     LOGCALL_VOID(DB, "GlassTable::form_key", key);
     kt.form_key(key);
@@ -1286,7 +1286,7 @@ void GlassTable::form_key(const string & key) const
 */
 
 void
-GlassTable::add(const string& key, const string& tag, bool already_compressed)
+GlassTable::add(string_view key, string_view tag, bool already_compressed)
 {
     LOGCALL_VOID(DB, "GlassTable::add", key | tag | already_compressed);
     Assert(writable);
@@ -1406,7 +1406,7 @@ GlassTable::add(const string& key, const string& tag, bool already_compressed)
 */
 
 bool
-GlassTable::del(const string &key)
+GlassTable::del(string_view key)
 {
     LOGCALL(DB, bool, "GlassTable::del", key);
     Assert(writable);
@@ -1442,7 +1442,7 @@ GlassTable::del(const string &key)
 }
 
 bool
-GlassTable::readahead_key(const string &key) const
+GlassTable::readahead_key(string_view key) const
 {
     LOGCALL(DB, bool, "GlassTable::readahead_key", key);
     Assert(!key.empty());
@@ -1485,7 +1485,7 @@ GlassTable::readahead_key(const string &key) const
 }
 
 bool
-GlassTable::get_exact_entry(const string &key, string & tag) const
+GlassTable::get_exact_entry(string_view key, string& tag) const
 {
     LOGCALL(DB, bool, "GlassTable::get_exact_entry", key | tag);
     Assert(!key.empty());
@@ -1508,7 +1508,7 @@ GlassTable::get_exact_entry(const string &key, string & tag) const
 }
 
 bool
-GlassTable::key_exists(const string &key) const
+GlassTable::key_exists(string_view key) const
 {
     LOGCALL(DB, bool, "GlassTable::key_exists", key);
     Assert(!key.empty());
@@ -1662,7 +1662,7 @@ GlassTable::read_root()
 	/* using a root block stored on disk */
 	block_to_cursor(C, level, root);
 
-	if (REVISION(C[level].get_p()) > revision_number) set_overwritten();
+	if (REVISION(C[level].get_p()) > revision_number) throw_overwritten();
 	/* although this is unlikely */
     }
 }
@@ -1707,7 +1707,7 @@ GlassTable::do_open_to_write(const RootInfo * root_info,
     seq_count = SEQ_START_POINT;
 }
 
-GlassTable::GlassTable(const char * tablename_, const string & path_,
+GlassTable::GlassTable(const char* tablename_, string_view path_,
 		       bool readonly_, bool lazy_)
 	: tablename(tablename_),
 	  revision_number(0),
@@ -1795,7 +1795,7 @@ GlassTable::create_and_open(int flags_, const RootInfo & root_info)
     close();
 
     unsigned int block_size_ = root_info.get_blocksize();
-    Assert(block_size_ >= 2048);
+    Assert(block_size_ >= GLASS_MIN_BLOCKSIZE);
     Assert(block_size_ <= BYTE_PAIR_RANGE);
     // Must be a power of two.
     Assert((block_size_ & (block_size_ - 1)) == 0);
@@ -2077,8 +2077,7 @@ GlassTable::prev_for_sequential(Glass::Cursor * C_, int /*dummy*/) const
 		C_[0].set_n(n);
 	    }
 	    if (REVISION(p) > revision_number + writable) {
-		set_overwritten();
-		RETURN(false);
+		throw_overwritten();
 	    }
 	    if (GET_LEVEL(p) == 0) break;
 	}
@@ -2135,8 +2134,7 @@ GlassTable::next_for_sequential(Glass::Cursor * C_, int /*dummy*/) const
 		p = q;
 	    }
 	    if (REVISION(p) > revision_number + writable) {
-		set_overwritten();
-		RETURN(false);
+		throw_overwritten();
 	    }
 	    if (GET_LEVEL(p) == 0) break;
 	}
@@ -2208,3 +2206,7 @@ GlassTable::throw_database_closed()
 {
     throw Xapian::DatabaseClosedError("Database has been closed");
 }
+
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
