@@ -21,8 +21,8 @@
  * USA
  */
 
-#ifndef OM_HGUARD_GLASS_TABLE_H
-#define OM_HGUARD_GLASS_TABLE_H
+#ifndef XAPIAN_INCLUDED_GLASS_TABLE_H
+#define XAPIAN_INCLUDED_GLASS_TABLE_H
 
 #include <xapian/constants.h>
 #include <xapian/error.h>
@@ -32,7 +32,6 @@
 #include "glass_defs.h"
 
 #include "io_utils.h"
-#include "noreturn.h"
 #include "omassert.h"
 #include "str.h"
 #include "stringutils.h"
@@ -42,6 +41,7 @@
 
 #include <algorithm>
 #include <string>
+#include <string_view>
 
 namespace Glass {
 
@@ -115,7 +115,10 @@ inline int DIR_END(const uint8_t * b) { return unaligned_read2(b + 9); }
 const int DIR_START = 11;
 
 inline void SET_REVISION(uint8_t * b, uint4 rev) { aligned_write4(b, rev); }
-inline void SET_LEVEL(uint8_t * b, int x) { AssertRel(x,<,256); b[4] = x; }
+inline void SET_LEVEL(uint8_t* b, int x) {
+    AssertRel(x,<,256);
+    b[4] = uint8_t(x);
+}
 inline void SET_MAX_FREE(uint8_t * b, int x) { unaligned_write2(b + 5, x); }
 inline void SET_TOTAL_FREE(uint8_t * b, int x) { unaligned_write2(b + 7, x); }
 inline void SET_DIR_END(uint8_t * b, int x) { unaligned_write2(b + 9, x); }
@@ -223,7 +226,7 @@ class LeafItem_wr : public LeafItem_base<uint8_t *> {
     void set_key_len(int x) {
 	AssertRel(x, >=, 0);
 	AssertRel(x, <=, GLASS_BTREE_MAX_KEY_LEN);
-	p[I2] = x;
+	p[I2] = uint8_t(x);
     }
     void setI(int x) { unaligned_write2(p, x); }
     static void setX(uint8_t * q, int c, int x) { unaligned_write2(q + c, x); }
@@ -244,8 +247,8 @@ class LeafItem_wr : public LeafItem_base<uint8_t *> {
 	if (rare(I &~ ITEM_SIZE_MASK)) throw Xapian::DatabaseError("item too large!");
 	setI(I);
     }
-    void form_key(const std::string & key_) {
-	std::string::size_type key_len = key_.length();
+    void form_key(std::string_view key_) {
+	auto key_len = key_.length();
 	if (key_len > GLASS_BTREE_MAX_KEY_LEN) {
 	    // We check term length when a term is added to a document but
 	    // glass doubles zero bytes, so this can still happen for terms
@@ -258,12 +261,13 @@ class LeafItem_wr : public LeafItem_base<uint8_t *> {
 	}
 
 	set_key_len(key_len);
-	std::memmove(p + I2 + K1, key_.data(), key_len);
 	*p |= I_FIRST_BIT;
+	if (key_len)
+	    std::memmove(p + I2 + K1, key_.data(), key_len);
     }
     // FIXME passing cd here is icky
     void set_tag(int cd, const char *start, int len, bool compressed, int i, int m) {
-	std::memmove(p + cd, start, len);
+	if (len) std::memmove(p + cd, start, len);
 	set_size(cd + len);
 	if (compressed) *p |= I_COMPRESSED_BIT;
 	if (i == m) *p |= I_LAST_BIT;
@@ -346,7 +350,7 @@ class BItem_wr : public BItem_base<uint8_t *> {
     void set_key_len(int x) {
 	AssertRel(x, >=, 0);
 	AssertRel(x, <, GLASS_BTREE_MAX_KEY_LEN);
-	p[BYTES_PER_BLOCK_NUMBER] = x;
+	p[BYTES_PER_BLOCK_NUMBER] = uint8_t(x);
     }
     static void setX(uint8_t * q, int c, int x) { unaligned_write2(q + c, x); }
   public:
@@ -463,7 +467,7 @@ class GlassTable {
      *  @param lazy		If true, don't create the table until it's
      *			needed.
      */
-    GlassTable(const char * tablename_, const std::string & path_,
+    GlassTable(const char* tablename_, std::string_view path_,
 	       bool readonly_, bool lazy = false);
 
     GlassTable(const char * tablename_, int fd, off_t offset_,
@@ -483,7 +487,7 @@ class GlassTable {
      */
     void close(bool permanent = false);
 
-    bool readahead_key(const string &key) const;
+    bool readahead_key(std::string_view key) const;
 
     /** Determine whether the btree exists on disk.
      */
@@ -563,7 +567,7 @@ class GlassTable {
      *  @return true if key is found in table,
      *          false if key is not found in table.
      */
-    bool get_exact_entry(const std::string & key, std::string & tag) const;
+    bool get_exact_entry(std::string_view key, std::string& tag) const;
 
     /** Check if a key exists in the Btree.
      *
@@ -576,7 +580,7 @@ class GlassTable {
      *  @return true if key is found in table,
      *          false if key is not found in table.
      */
-    bool key_exists(const std::string &key) const;
+    bool key_exists(std::string_view key) const;
 
     /** Read the tag value for the key pointed to by cursor C_.
      *
@@ -606,8 +610,8 @@ class GlassTable {
      *		for example because it is being opaquely copied
      *		(default: false).
      */
-    void add(const std::string& key,
-	     const std::string& tag,
+    void add(std::string_view key,
+	     std::string_view tag,
 	     bool already_compressed = false);
 
     /** Delete an entry from the table.
@@ -625,10 +629,12 @@ class GlassTable {
      *
      *  @return true if an entry was removed; false if it did not exist.
      */
-    bool del(const std::string &key);
+    bool del(std::string_view key);
 
     int get_flags() const { return flags; }
 
+    void set_flags(int new_flags) { flags = new_flags; }
+
     /** Create a new empty btree structure on disk and open it at the
      *  initial revision.
      *
@@ -727,7 +733,8 @@ class GlassTable {
     }
 
     /// Throw an exception indicating that the database is closed.
-    XAPIAN_NORETURN(static void throw_database_closed());
+    [[noreturn]]
+    static void throw_database_closed();
 
     string get_path() const {
 	return name + GLASS_TABLE_EXTENSION;
@@ -739,7 +746,8 @@ class GlassTable {
     void read_block(uint4 n, uint8_t *p) const;
     void write_block(uint4 n, const uint8_t *p,
 		     bool appending = false) const;
-    XAPIAN_NORETURN(void set_overwritten() const);
+    [[noreturn]]
+    void throw_overwritten() const;
     void block_to_cursor(Glass::Cursor *C_, int j, uint4 n) const;
     void alter();
     void compact(uint8_t *p);
@@ -756,7 +764,7 @@ class GlassTable {
     int add_kt(bool found);
     void read_root();
     void split_root(uint4 split_n);
-    void form_key(const std::string & key) const;
+    void form_key(std::string_view key) const;
 
     /// The name of the table (used when writing changesets).
     const char * tablename;
@@ -990,4 +998,8 @@ compare(BItem a, BItem b)
 
 }
 
-#endif /* OM_HGUARD_GLASS_TABLE_H */
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
+
+#endif /* XAPIAN_INCLUDED_GLASS_TABLE_H */
