@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2006,2007,2008,2009,2010,2014,2019 Olly Betts
+ * Copyright 2002,2003,2004,2006,2007,2008,2009,2010,2014,2019,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -112,18 +112,11 @@ void
 GlassTermList::accumulate_stats(Xapian::Internal::ExpandStats & stats) const
 {
     LOGCALL_VOID(DB, "GlassTermList::accumulate_stats", stats);
-    Assert(!at_end());
+    Assert(pos != NULL);
     stats.accumulate(shard_index,
 		     current_wdf, doclen, get_termfreq(), db->get_doccount());
 }
 
-string
-GlassTermList::get_termname() const
-{
-    LOGCALL(DB, string, "GlassTermList::get_termname", NO_ARGS);
-    RETURN(current_term);
-}
-
 Xapian::termcount
 GlassTermList::get_wdf() const
 {
@@ -144,10 +137,9 @@ TermList *
 GlassTermList::next()
 {
     LOGCALL(DB, TermList *, "GlassTermList::next", NO_ARGS);
-    Assert(!at_end());
+    Assert(pos != NULL);
     if (pos == end) {
-	pos = NULL;
-	RETURN(NULL);
+	RETURN(this);
     }
 
     // Reset to 0 to indicate that the termfreq needs to be read.
@@ -186,23 +178,17 @@ GlassTermList::next()
     RETURN(NULL);
 }
 
-TermList *
-GlassTermList::skip_to(const string & term)
+TermList*
+GlassTermList::skip_to(string_view term)
 {
     LOGCALL(API, TermList *, "GlassTermList::skip_to", term);
-    while (pos != NULL && current_term < term) {
-	(void)GlassTermList::next();
+    while (current_term < term) {
+	if (GlassTermList::next())
+	    RETURN(this);
     }
     RETURN(NULL);
 }
 
-bool
-GlassTermList::at_end() const
-{
-    LOGCALL(DB, bool, "GlassTermList::at_end", NO_ARGS);
-    RETURN(pos == NULL);
-}
-
 Xapian::termcount
 GlassTermList::positionlist_count() const
 {
@@ -210,9 +196,13 @@ GlassTermList::positionlist_count() const
     RETURN(db->positionlist_count(did, current_term));
 }
 
-Xapian::PositionIterator
+PositionList*
 GlassTermList::positionlist_begin() const
 {
-    LOGCALL(DB, Xapian::PositionIterator, "GlassTermList::positionlist_begin", NO_ARGS);
-    RETURN(Xapian::PositionIterator(db->open_position_list(did, current_term)));
+    LOGCALL(DB, PositionList*, "GlassTermList::positionlist_begin", NO_ARGS);
+    RETURN(db->open_position_list(did, current_term));
 }
+
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
