@@ -1,7 +1,7 @@
 /** @file
  * @brief A TermList in a glass database.
  */
-/* Copyright (C) 2007,2008,2009,2010,2011 Olly Betts
+/* Copyright (C) 2007,2008,2009,2010,2011,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -69,9 +69,6 @@ class GlassTermList : public TermList {
     /// Pointer to the end of the encoded tag value.
     const char *end;
 
-    /// The termname at the current position.
-    std::string current_term;
-
     /// The wdf for the term at the current position.
     Xapian::termcount current_wdf;
 
@@ -96,6 +93,14 @@ class GlassTermList : public TermList {
     GlassTermList(Xapian::Internal::intrusive_ptr<const GlassDatabase> db_,
 		  Xapian::docid did_, bool throw_if_not_present = true);
 
+    /** Check if the term isn't present.
+     *
+     *  If you call the constructor with throw_if_not_present=false then you
+     *  need to call this method to check if the term is present before you
+     *  call other methods.
+     */
+    bool not_present() const { return pos == NULL; }
+
     /** Return the length of this document.
      *
      *  This is a non-virtual method, used by GlassDatabase.
@@ -117,9 +122,6 @@ class GlassTermList : public TermList {
     /// Collate weighting information for the current term.
     void accumulate_stats(Xapian::Internal::ExpandStats & stats) const;
 
-    /// Return the termname at the current position.
-    std::string get_termname() const;
-
     /// Return the wdf for the term at the current position.
     Xapian::termcount get_wdf() const;
 
@@ -142,16 +144,13 @@ class GlassTermList : public TermList {
      */
     TermList * next();
 
-    TermList * skip_to(const std::string & term);
-
-    /// Return true if the current position is past the last term in this list.
-    bool at_end() const;
+    TermList* skip_to(std::string_view term);
 
     /// Return the length of the position list for the current position.
     Xapian::termcount positionlist_count() const;
 
     /// Return a PositionIterator for the current position.
-    Xapian::PositionIterator positionlist_begin() const;
+    PositionList* positionlist_begin() const;
 };
 
 #endif // XAPIAN_INCLUDED_GLASS_TERMLIST_H
