@@ -27,14 +27,14 @@
 #include "glass_postlist.h"
 #include "glass_termlist.h"
 #include "debuglog.h"
-#include "backends/document.h"
+#include "backends/documentinternal.h"
 #include "pack.h"
 
 #include "xapian/error.h"
 #include "xapian/valueiterator.h"
 
 #include <algorithm>
-#include "autoptr.h"
+#include <memory>
 
 using namespace Glass;
 using namespace std;
@@ -156,9 +156,9 @@ GlassValueManager::get_chunk_containing_did(Xapian::valueno slot,
 					    string &chunk) const
 {
     LOGCALL(DB, Xapian::docid, "GlassValueManager::get_chunk_containing_did", slot | did | chunk);
-    if (!cursor.get())
+    if (!cursor)
 	cursor.reset(postlist_table->cursor_get());
-    if (!cursor.get()) RETURN(0);
+    if (!cursor) RETURN(0);
 
     bool exact = cursor->find_entry(make_valuechunk_key(slot, did));
     if (!exact) {
@@ -270,7 +270,7 @@ class ValueUpdater {
 	    last_allowed_did = GLASS_MAX_DOCID;
 	    Assert(tag.empty());
 	    new_first_did = 0;
-	    AutoPtr<GlassCursor> cursor(table->cursor_get());
+	    unique_ptr<GlassCursor> cursor(table->cursor_get());
 	    if (cursor->find_entry(make_valuechunk_key(slot, did))) {
 		// We found an exact match, so the first docid is the one
 		// we looked for.
@@ -387,7 +387,8 @@ GlassValueManager::add_document(Xapian::docid did, const Xapian::Document &doc,
 
 	add_value(did, slot, value);
 	if (termlist_table->is_open()) {
-	    pack_uint(slots_used, slot - prev_slot - 1);
+	    // prev_slot starts as Xapian::valueno(-1).
+	    pack_uint(slots_used, slot - UNSIGNED_OVERFLOW_OK(prev_slot + 1));
 	    prev_slot = slot;
 	}
 	++it;
@@ -421,7 +422,8 @@ GlassValueManager::delete_document(Xapian::docid did,
 	if (!unpack_uint(&p, end, &slot)) {
 	    throw Xapian::DatabaseCorruptError("Value slot encoding corrupt");
 	}
-	slot += prev_slot + 1;
+	// prev_slot starts as Xapian::valueno(-1).
+	slot += UNSIGNED_OVERFLOW_OK(prev_slot + 1);
 	prev_slot = slot;
 
 	std::pair<map<Xapian::valueno, ValueStats>::iterator, bool> i;
@@ -462,7 +464,7 @@ GlassValueManager::replace_document(Xapian::docid did,
 	// document 4 from another will unnecessarily trigger this, but forcing
 	// the values to be read is fairly harmless, and this is unlikely to be
 	// a common case.
-	doc.internal->need_values();
+	doc.internal->ensure_values_fetched();
     }
     delete_document(did, value_stats);
     add_document(did, doc, value_stats);
@@ -518,7 +520,8 @@ GlassValueManager::get_all_values(map<Xapian::valueno, string> & values,
 	if (!unpack_uint(&p, end, &slot)) {
 	    throw Xapian::DatabaseCorruptError("Value slot encoding corrupt");
 	}
-	slot += prev_slot + 1;
+	// prev_slot starts as Xapian::valueno(-1).
+	slot += UNSIGNED_OVERFLOW_OK(prev_slot + 1);
 	prev_slot = slot;
 	values.insert(make_pair(slot, get_value(did, slot)));
     }
