@@ -28,11 +28,11 @@
 #include "xapian/error.h"
 #include "xapian/types.h"
 
-#include "autoptr.h"
 #include <map>
+#include <memory>
 #include <string>
 
-class GlassCursor;
+#include "glass_cursor.h"
 
 namespace Glass {
 
@@ -93,7 +93,7 @@ class GlassValueManager {
 
     std::map<Xapian::valueno, std::map<Xapian::docid, std::string>> changes;
 
-    mutable AutoPtr<GlassCursor> cursor;
+    mutable std::unique_ptr<GlassCursor> cursor;
 
     void add_value(Xapian::docid did, Xapian::valueno slot,
 		   const std::string & val);
