@@ -25,6 +25,7 @@
 
 #include "debuglog.h"
 #include "fd.h"
+#include "glass_defs.h"
 #include "io_utils.h"
 #include "omassert.h"
 #include "pack.h"
@@ -294,7 +295,26 @@ GlassVersion::write(glass_revision_number_t new_rev, int flags)
 	else
 	    tmpfile += "/v.tmp";
 
-	fd = posixy_open(tmpfile.c_str(), O_CREAT|O_TRUNC|O_WRONLY|O_BINARY, 0666);
+#ifdef __EMSCRIPTEN__
+	// Emscripten < 1.39.10 fails to create a file if O_TRUNC is specified
+	// and the filename is the previous name of a renamed file (which it
+	// will be the second time we write out the version file for a DB):
+	//
+	// https://github.com/emscripten-core/emscripten/issues/8187
+	//
+	// We avoid triggering this bug by not using O_TRUNC and instead
+	// truncating once the file is opened.
+	fd = posixy_open(tmpfile.c_str(),
+			 O_CREAT|O_WRONLY|O_BINARY,
+			 0666);
+	if (fd >= 0)
+	    ftruncate(fd, 0);
+#else
+	fd = posixy_open(tmpfile.c_str(),
+			 O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,
+			 0666);
+#endif
+
 	if (rare(fd < 0))
 	    throw Xapian::DatabaseOpeningError("Couldn't write new rev file: " + tmpfile,
 					       errno);
@@ -396,7 +416,7 @@ static const uint4 compress_min_tab[] = {
 void
 GlassVersion::create(unsigned blocksize)
 {
-    AssertRel(blocksize,>=,2048);
+    AssertRel(blocksize,>=,GLASS_MIN_BLOCKSIZE);
     uuid.generate();
     for (unsigned table_no = 0; table_no < Glass::MAX_; ++table_no) {
 	root[table_no].init(blocksize, compress_min_tab[table_no]);
@@ -408,7 +428,7 @@ namespace Glass {
 void
 RootInfo::init(unsigned blocksize_, uint4 compress_min_)
 {
-    AssertRel(blocksize_,>=,2048);
+    AssertRel(blocksize_,>=,GLASS_MIN_BLOCKSIZE);
     root = 0;
     level = 0;
     num_entries = 0;
