@@ -1,7 +1,7 @@
 /** @file
  * @brief GlassVersion class
  */
-/* Copyright (C) 2006,2007,2008,2009,2010,2013,2014,2015,2016,2018 Olly Betts
+/* Copyright (C) 2006-2024 Olly Betts
  * Copyright (C) 2011 Dan Colish
  *
  * This program is free software; you can redistribute it and/or modify
@@ -62,7 +62,8 @@ class RootInfo {
     bool get_root_is_fake() const { return root_is_fake; }
     bool get_sequential() const { return sequential; }
     unsigned get_blocksize() const {
-	AssertRel(blocksize,>=,2048);
+	AssertRel(blocksize,>=,GLASS_MIN_BLOCKSIZE);
+	AssertRel(blocksize,<=,GLASS_MAX_BLOCKSIZE);
 	return blocksize;
     }
     uint4 get_compress_min() const { return compress_min; }
@@ -74,7 +75,8 @@ class RootInfo {
     void set_sequential(bool f) { sequential = f; }
     void set_root(glass_block_t root_) { root = root_; }
     void set_blocksize(unsigned b) {
-	AssertRel(b,>=,2048);
+	AssertRel(b,>=,GLASS_MIN_BLOCKSIZE);
+	AssertRel(b,<=,GLASS_MAX_BLOCKSIZE);
 	blocksize = b;
     }
     void set_free_list(const std::string & s) { fl_serialised = s; }
@@ -155,7 +157,7 @@ class GlassVersion {
     void unserialise_stats();
 
   public:
-    explicit GlassVersion(const std::string & db_dir_)
+    explicit GlassVersion(std::string_view db_dir_)
 	: rev(0), fd(-1), offset(0), db_dir(db_dir_), changes(NULL),
 	  doccount(0), total_doclen(0), last_docid(0),
 	  doclen_lbound(0), doclen_ubound(0),
@@ -228,6 +230,13 @@ class GlassVersion {
 	return oldest_changeset;
     }
 
+    Xapian::termcount get_unique_terms_lower_bound() const {
+	if (total_doclen == 0) return 0;
+	Assert(doclen_lbound != 0);
+	Assert(wdf_ubound != 0);
+	return (doclen_lbound - 1) / wdf_ubound + 1;
+    }
+
     void set_last_docid(Xapian::docid did) { last_docid = did; }
 
     void set_oldest_changeset(glass_revision_number_t changeset) const {
