@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002-2023 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  * Copyright 2006,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
 
 #include "debuglog.h"
 
+#include "backends/contiguousalldocspostlist.h"
 #include "expand/expandweight.h"
 #include "inmemory_document.h"
 #include "inmemory_alltermslist.h"
@@ -36,13 +37,14 @@
 
 #include <algorithm>
 #include <string>
+#include <string_view>
 #include <vector>
 #include <map>
 
 #include <xapian/error.h>
 #include <xapian/valueiterator.h>
 
-using std::make_pair;
+using namespace std;
 using Xapian::Internal::intrusive_ptr;
 
 inline void
@@ -112,15 +114,16 @@ InMemoryDoc::add_posting(const string& tname,
 
 InMemoryPostList::InMemoryPostList(const InMemoryDatabase* db_,
 				   const InMemoryTerm & imterm,
-				   const std::string & term_)
+				   std::string_view term_)
 	: LeafPostList(term_),
 	  pos(imterm.docs.begin()),
 	  end(imterm.docs.end()),
-	  termfreq(imterm.term_freq),
 	  started(false),
 	  db(db_),
 	  wdf_upper_bound(0)
 {
+    termfreq = imterm.term_freq;
+    collfreq = imterm.collection_freq;
     while (pos != end && !pos->valid) ++pos;
     if (pos != end) {
 	auto first_wdf = (*pos).wdf;
@@ -128,12 +131,6 @@ InMemoryPostList::InMemoryPostList(const InMemoryDatabase* db_,
     }
 }
 
-Xapian::doccount
-InMemoryPostList::get_termfreq() const
-{
-    return termfreq;
-}
-
 Xapian::docid
 InMemoryPostList::get_docid() const
 {
@@ -184,30 +181,36 @@ InMemoryPostList::at_end() const
     return (pos == end);
 }
 
+void
+InMemoryPostList::get_docid_range(Xapian::docid& first,
+				  Xapian::docid& last) const
+{
+    Assert(!started);
+    if (pos != end) {
+	first = pos->did;
+	last = (end - 1)->did;
+    } else {
+	last = 0;
+    }
+}
+
 string
 InMemoryPostList::get_description() const
 {
-    return "InMemoryPostList " + str(termfreq);
+    return term + ":" + str(termfreq);
 }
 
 Xapian::termcount
-InMemoryPostList::get_doclength() const
+InMemoryPostList::get_wdfdocmax() const
 {
-    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    return db->get_doclength(get_docid());
-}
-
-Xapian::termcount
-InMemoryPostList::get_unique_terms() const
-{
-    return db->get_unique_terms(get_docid());
+   return db->get_wdfdocmax(get_docid());
 }
 
 PositionList *
 InMemoryPostList::read_position_list()
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    mypositions.set_data(pos->positions);
+    mypositions.assign(pos->positions.copy());
     return &mypositions;
 }
 
@@ -215,7 +218,8 @@ PositionList *
 InMemoryPostList::open_position_list() const
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    return new InMemoryPositionList(pos->positions);
+    if (pos->positions.empty()) return nullptr;
+    return new InMemoryPositionList(pos->positions.copy());
 }
 
 Xapian::termcount
@@ -252,7 +256,7 @@ InMemoryTermList::get_wdf() const
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
-    Assert(!at_end());
+    Assert(pos != end);
     return (*pos).wdf;
 }
 
@@ -261,7 +265,7 @@ InMemoryTermList::get_termfreq() const
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
-    Assert(!at_end());
+    Assert(pos != end);
 
     Xapian::doccount tf;
     db->get_freqs((*pos).tname, &tf, NULL);
@@ -280,37 +284,31 @@ InMemoryTermList::accumulate_stats(Xapian::Internal::ExpandStats & stats) const
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     Assert(started);
-    Assert(!at_end());
+    Assert(pos != end);
     stats.accumulate(shard_index,
 		     InMemoryTermList::get_wdf(), document_length,
 		     InMemoryTermList::get_termfreq(),
 		     db->get_doccount());
 }
 
-string
-InMemoryTermList::get_termname() const
-{
-    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    Assert(started);
-    Assert(!at_end());
-    return (*pos).tname;
-}
-
 TermList *
 InMemoryTermList::next()
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
     if (started) {
-	Assert(!at_end());
+	Assert(pos != end);
 	++pos;
     } else {
 	started = true;
     }
+    if (pos == end)
+	return this;
+    current_term = pos->tname;
     return NULL;
 }
 
-TermList *
-InMemoryTermList::skip_to(const string & term)
+TermList*
+InMemoryTermList::skip_to(string_view term)
 {
     if (rare(db->is_closed()))
 	InMemoryDatabase::throw_database_closed();
@@ -320,17 +318,12 @@ InMemoryTermList::skip_to(const string & term)
     }
 
     started = true;
+    if (pos == end)
+	return this;
+    current_term = pos->tname;
     return NULL;
 }
 
-bool
-InMemoryTermList::at_end() const
-{
-    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    Assert(started);
-    return (pos == end);
-}
-
 Xapian::termcount
 InMemoryTermList::positionlist_count() const
 {
@@ -338,11 +331,11 @@ InMemoryTermList::positionlist_count() const
     return db->positionlist_count(did, (*pos).tname);
 }
 
-Xapian::PositionIterator
+PositionList*
 InMemoryTermList::positionlist_begin() const
 {
     if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    return Xapian::PositionIterator(db->open_position_list(did, (*pos).tname));
+    return db->open_position_list(did, (*pos).tname);
 }
 
 /////////////////////////////
@@ -350,15 +343,9 @@ InMemoryTermList::positionlist_begin() const
 /////////////////////////////
 
 InMemoryAllDocsPostList::InMemoryAllDocsPostList(const InMemoryDatabase* db_)
-	: LeafPostList(std::string()), did(0), db(db_)
+	: LeafPostList({}), did(0), db(db_)
 {
-}
-
-Xapian::doccount
-InMemoryAllDocsPostList::get_termfreq() const
-{
-    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    return db->totdocs;
+    collfreq = termfreq = db->totdocs;
 }
 
 Xapian::docid
@@ -371,19 +358,6 @@ InMemoryAllDocsPostList::get_docid() const
     return did;
 }
 
-Xapian::termcount
-InMemoryAllDocsPostList::get_doclength() const
-{
-    if (db->is_closed()) InMemoryDatabase::throw_database_closed();
-    return db->get_doclength(did);
-}
-
-Xapian::termcount
-InMemoryAllDocsPostList::get_unique_terms() const
-{
-    return db->get_unique_terms(did);
-}
-
 Xapian::termcount
 InMemoryAllDocsPostList::get_wdf() const
 {
@@ -450,14 +424,13 @@ InMemoryAllDocsPostList::get_description() const
 // Actual database class //
 ///////////////////////////
 
+// Updates are applied immediately so we can't support transactions.
 InMemoryDatabase::InMemoryDatabase()
-	: totdocs(0), totlen(0), positions_present(false), closed(false)
+    : Xapian::Database::Internal(TRANSACTION_UNIMPLEMENTED),
+      totdocs(0), totlen(0), positions_present(false), closed(false)
 {
-    // Updates are applied immediately so we can't support transactions.
-    transaction_state = TRANSACTION_UNIMPLEMENTED;
-
     // We keep an empty entry in postlists for convenience of implementing
-    // allterms iteration and returning a PostList for an absent term.
+    // allterms iteration.
     postlists.insert(make_pair(string(), InMemoryTerm()));
 }
 
@@ -487,20 +460,35 @@ InMemoryDatabase::close()
     closed = true;
 }
 
-LeafPostList *
-InMemoryDatabase::open_post_list(const string & tname) const
+PostList*
+InMemoryDatabase::open_post_list(string_view term) const
 {
+    return InMemoryDatabase::open_leaf_post_list(term, false);
+}
+
+LeafPostList*
+InMemoryDatabase::open_leaf_post_list(string_view term,
+				      bool need_read_pos) const
+{
+    (void)need_read_pos;
     if (closed) InMemoryDatabase::throw_database_closed();
-    if (tname.empty()) {
+    if (term.empty()) {
+	Assert(!need_read_pos);
+	Xapian::doccount doccount = totdocs;
+	if (rare(doccount == 0)) {
+	    return nullptr;
+	}
+	if (doccount == termlists.size()) {
+	    // The used docid range is exactly 1 to doccount inclusive.
+	    return new ContiguousAllDocsPostList(doccount);
+	}
 	return new InMemoryAllDocsPostList(this);
     }
-    map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
+    auto i = postlists.find(term);
     if (i == postlists.end() || i->second.term_freq == 0) {
-	i = postlists.begin();
-	// Check that our dummy entry for string() is present.
-	Assert(i->first.empty());
+	return nullptr;
     }
-    return new InMemoryPostList(this, i->second, tname);
+    return new InMemoryPostList(this, i->second, term);
 }
 
 bool
@@ -511,12 +499,12 @@ InMemoryDatabase::doc_exists(Xapian::docid did) const
 }
 
 void
-InMemoryDatabase::get_freqs(const string & term,
-			    Xapian::doccount * termfreq_ptr,
-			    Xapian::termcount * collfreq_ptr) const
+InMemoryDatabase::get_freqs(string_view term,
+			    Xapian::doccount* termfreq_ptr,
+			    Xapian::termcount* collfreq_ptr) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    map<string, InMemoryTerm>::const_iterator i = postlists.find(term);
+    auto i = postlists.find(term);
     if (i != postlists.end()) {
 	if (termfreq_ptr)
 	    *termfreq_ptr = i->second.term_freq;
@@ -534,7 +522,7 @@ Xapian::doccount
 InMemoryDatabase::get_value_freq(Xapian::valueno slot) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    map<Xapian::valueno, ValueStats>::const_iterator i = valuestats.find(slot);
+    auto i = valuestats.find(slot);
     if (i == valuestats.end()) return 0;
     return i->second.freq;
 }
@@ -543,7 +531,7 @@ std::string
 InMemoryDatabase::get_value_lower_bound(Xapian::valueno slot) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    map<Xapian::valueno, ValueStats>::const_iterator i = valuestats.find(slot);
+    auto i = valuestats.find(slot);
     if (i == valuestats.end()) return string();
     return i->second.lower_bound;
 }
@@ -552,11 +540,37 @@ std::string
 InMemoryDatabase::get_value_upper_bound(Xapian::valueno slot) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    map<Xapian::valueno, ValueStats>::const_iterator i = valuestats.find(slot);
+    auto i = valuestats.find(slot);
     if (i == valuestats.end()) return string();
     return i->second.upper_bound;
 }
 
+Xapian::termcount
+InMemoryDatabase::get_doclength_lower_bound() const
+{
+    // A zero-length document can't contain any terms, so we ignore such
+    // documents for the purposes of this lower bound.
+    return 1;
+}
+
+Xapian::termcount
+InMemoryDatabase::get_doclength_upper_bound() const
+{
+    // Not a very tight bound in general, but InMemory isn't really built for
+    // performance.
+    return min(get_total_length(), Xapian::totallength(Xapian::termcount(-1)));
+}
+
+Xapian::termcount
+InMemoryDatabase::get_wdf_upper_bound(string_view term) const
+{
+    // Not a very tight bound in general, but InMemory isn't really built for
+    // performance.
+    Xapian::termcount cf;
+    get_freqs(term, NULL, &cf);
+    return cf;
+}
+
 Xapian::doccount
 InMemoryDatabase::get_doccount() const
 {
@@ -602,6 +616,20 @@ InMemoryDatabase::get_unique_terms(Xapian::docid did) const
     return std::min(terms, Xapian::termcount(doclengths[did - 1]));
 }
 
+Xapian::termcount
+InMemoryDatabase::get_wdfdocmax(Xapian::docid did) const
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+    if (did == 0 || did > termlists.size() || !termlists[did - 1].is_valid)
+	throw Xapian::DocNotFoundError(string("Docid ") + str(did) +
+				 string(" not found"));
+    Xapian::termcount max_wdf = 0;
+    for (auto&& i : termlists[did - 1].terms) {
+	if (i.wdf > max_wdf) max_wdf = i.wdf;
+    }
+    return max_wdf;
+}
+
 TermList *
 InMemoryDatabase::open_term_list(Xapian::docid did) const
 {
@@ -616,6 +644,12 @@ InMemoryDatabase::open_term_list(Xapian::docid did) const
 				termlists[did - 1], doclengths[did - 1]);
 }
 
+TermList *
+InMemoryDatabase::open_term_list_direct(Xapian::docid did) const
+{
+    return InMemoryDatabase::open_term_list(did);
+}
+
 Xapian::Document::Internal *
 InMemoryDatabase::open_document(Xapian::docid did, bool lazy) const
 {
@@ -630,17 +664,17 @@ InMemoryDatabase::open_document(Xapian::docid did, bool lazy) const
 }
 
 std::string
-InMemoryDatabase::get_metadata(const std::string & key) const
+InMemoryDatabase::get_metadata(std::string_view key) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    map<string, string>::const_iterator i = metadata.find(key);
+    auto i = metadata.find(key);
     if (i == metadata.end())
 	return string();
     return i->second;
 }
 
 TermList *
-InMemoryDatabase::open_metadata_keylist(const string &) const
+InMemoryDatabase::open_metadata_keylist(string_view) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     if (metadata.empty()) return NULL;
@@ -649,20 +683,28 @@ InMemoryDatabase::open_metadata_keylist(const string &) const
 }
 
 void
-InMemoryDatabase::set_metadata(const std::string & key,
-			       const std::string & value)
+InMemoryDatabase::set_metadata(std::string_view key,
+			       std::string_view value)
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     if (!value.empty()) {
-	metadata[key] = value;
+#ifdef __cpp_lib_associative_heterogeneous_insertion // C++26
+	metadata.insert_or_assign(key, value);
+#else
+	metadata.insert_or_assign(string(key), value);
+#endif
     } else {
+#ifdef __cpp_lib_associative_heterogeneous_erasure // C++23
 	metadata.erase(key);
+#else
+	metadata.erase(string(key));
+#endif
     }
 }
 
 Xapian::termcount
 InMemoryDatabase::positionlist_count(Xapian::docid did,
-				     const string & tname) const
+				     string_view tname) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     if (!doc_exists(did)) {
@@ -680,9 +722,9 @@ InMemoryDatabase::positionlist_count(Xapian::docid did,
     return 0;
 }
 
-PositionList *
+PositionList*
 InMemoryDatabase::open_position_list(Xapian::docid did,
-				     const string & tname) const
+				     string_view tname) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     if (usual(doc_exists(did))) {
@@ -696,7 +738,7 @@ InMemoryDatabase::open_position_list(Xapian::docid did,
 	    return new InMemoryPositionList(t->positions);
 	}
     }
-    return new InMemoryPositionList(false);
+    return nullptr;
 }
 
 void
@@ -710,24 +752,22 @@ InMemoryDatabase::add_values(Xapian::docid did,
     valuelists[did - 1] = values_;
 
     // Update the statistics.
-    map<Xapian::valueno, string>::const_iterator j;
-    for (j = values_.begin(); j != values_.end(); ++j) {
-	std::pair<map<Xapian::valueno, ValueStats>::iterator, bool> i;
-	i = valuestats.insert(make_pair(j->first, ValueStats()));
+    for (auto&& j : values_) {
+	auto i = valuestats.insert(make_pair(j.first, ValueStats()));
 
 	// Now, modify the stored statistics.
 	if ((i.first->second.freq)++ == 0) {
 	    // If the value count was previously zero, set the upper and lower
 	    // bounds to the newly added value.
-	    i.first->second.lower_bound = j->second;
-	    i.first->second.upper_bound = j->second;
+	    i.first->second.lower_bound = j.second;
+	    i.first->second.upper_bound = j.second;
 	} else {
 	    // Otherwise, simply make sure they reflect the new value.
-	    if (j->second < i.first->second.lower_bound) {
-		i.first->second.lower_bound = j->second;
+	    if (j.second < i.first->second.lower_bound) {
+		i.first->second.lower_bound = j.second;
 	    }
-	    if (j->second > i.first->second.upper_bound) {
-		i.first->second.upper_bound = j->second;
+	    if (j.second > i.first->second.upper_bound) {
+		i.first->second.upper_bound = j.second;
 	    }
 	}
     }
@@ -755,10 +795,8 @@ InMemoryDatabase::delete_document(Xapian::docid did)
     }
     termlists[did - 1].is_valid = false;
     doclists[did - 1] = string();
-    map<Xapian::valueno, string>::const_iterator j;
-    for (j = valuelists[did - 1].begin(); j != valuelists[did - 1].end(); ++j) {
-	map<Xapian::valueno, ValueStats>::iterator i;
-	i = valuestats.find(j->first);
+    for (auto&& j : valuelists[did - 1]) {
+	auto i = valuestats.find(j.first);
 	if (--(i->second.freq) == 0) {
 	    i->second.lower_bound.resize(0);
 	    i->second.upper_bound.resize(0);
@@ -773,13 +811,10 @@ InMemoryDatabase::delete_document(Xapian::docid did)
     // InMemory structure without being very inefficient.
     if (totdocs == 0) positions_present = false;
 
-    vector<InMemoryTermEntry>::const_iterator i;
-    for (i = termlists[did - 1].terms.begin();
-	 i != termlists[did - 1].terms.end();
-	 ++i) {
-	map<string, InMemoryTerm>::iterator t = postlists.find(i->tname);
+    for (auto&& i : termlists[did - 1].terms) {
+	auto t = postlists.find(i.tname);
 	Assert(t != postlists.end());
-	t->second.collection_freq -= i->wdf;
+	t->second.collection_freq -= i.wdf;
 	--t->second.term_freq;
 
 	// Just invalidate erased doc ids - otherwise we need to erase
@@ -805,10 +840,8 @@ InMemoryDatabase::replace_document(Xapian::docid did,
     if (closed) InMemoryDatabase::throw_database_closed();
 
     if (doc_exists(did)) {
-	map<Xapian::valueno, string>::const_iterator j;
-	for (j = valuelists[did - 1].begin(); j != valuelists[did - 1].end(); ++j) {
-	    map<Xapian::valueno, ValueStats>::iterator i;
-	    i = valuestats.find(j->first);
+	for (auto&& j : valuelists[did - 1]) {
+	    auto i = valuestats.find(j.first);
 	    if (--(i->second.freq) == 0) {
 		i->second.lower_bound.resize(0);
 		i->second.upper_bound.resize(0);
@@ -827,13 +860,10 @@ InMemoryDatabase::replace_document(Xapian::docid did,
 	termlists[did - 1].is_valid = true;
     }
 
-    vector<InMemoryTermEntry>::const_iterator i;
-    for (i = termlists[did - 1].terms.begin();
-	 i != termlists[did - 1].terms.end();
-	 ++i) {
-	map<string, InMemoryTerm>::iterator t = postlists.find(i->tname);
+    for (auto&& i : termlists[did - 1].terms) {
+	auto t = postlists.find(i.tname);
 	Assert(t != postlists.end());
-	t->second.collection_freq -= i->wdf;
+	t->second.collection_freq -= i.wdf;
 	--t->second.term_freq;
 
 	// Just invalidate erased doc ids - otherwise we need to erase
@@ -949,11 +979,13 @@ void InMemoryDatabase::make_posting(InMemoryDoc * doc,
 }
 
 bool
-InMemoryDatabase::term_exists(const string & tname) const
+InMemoryDatabase::term_exists(string_view term) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
-    Assert(!tname.empty());
-    map<string, InMemoryTerm>::const_iterator i = postlists.find(tname);
+    if (term.empty()) {
+	return totdocs != 0;
+    }
+    auto i = postlists.find(term);
     if (i == postlists.end()) return false;
     return (i->second.term_freq != 0);
 }
@@ -965,8 +997,8 @@ InMemoryDatabase::has_positions() const
     return positions_present;
 }
 
-TermList *
-InMemoryDatabase::open_allterms(const string & prefix) const
+TermList*
+InMemoryDatabase::open_allterms(string_view prefix) const
 {
     if (closed) InMemoryDatabase::throw_database_closed();
     return new InMemoryAllTermsList(&postlists,
@@ -974,8 +1006,40 @@ InMemoryDatabase::open_allterms(const string & prefix) const
 				    prefix);
 }
 
+void
+InMemoryDatabase::get_used_docid_range(Xapian::docid& first,
+				       Xapian::docid& last) const
+{
+    if (closed) InMemoryDatabase::throw_database_closed();
+    first = 1;
+    last = Xapian::docid(termlists.size());
+    if (last == 0 || last == totdocs) {
+	// Empty database or contiguous range starting at 1.
+	return;
+    }
+    while (!termlists[first - 1].is_valid) ++first;
+    while (!termlists[last - 1].is_valid) --last;
+}
+
+Xapian::Database::Internal*
+InMemoryDatabase::update_lock(int /*flags*/)
+{
+    // InMemoryDatabase doesn't really distinguish writable and read-only.
+    return this;
+}
+
 void
 InMemoryDatabase::throw_database_closed()
 {
     throw Xapian::DatabaseClosedError("Database has been closed");
 }
+
+string
+InMemoryDatabase::get_description() const
+{
+    return "InMemory";
+}
+
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
