@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2014,2015 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  * Copyright 2006,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -22,24 +22,23 @@
  * USA
  */
 
-#ifndef OM_HGUARD_INMEMORY_DATABASE_H
-#define OM_HGUARD_INMEMORY_DATABASE_H
+#ifndef XAPIAN_INCLUDED_INMEMORY_DATABASE_H
+#define XAPIAN_INCLUDED_INMEMORY_DATABASE_H
 
-#include "api/leafpostlist.h"
+#include "api/smallvector.h"
 #include "api/termlist.h"
 #include "backends/backends.h"
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
+#include "backends/leafpostlist.h"
 #include "backends/valuestats.h"
-#include <map>
-#include <vector>
 #include <algorithm>
+#include <map>
+#include <string_view>
+#include <vector>
 #include <xapian/document.h>
 #include "inmemory_positionlist.h"
 #include "internaltypes.h"
 #include "omassert.h"
-#include "noreturn.h"
-
-using namespace std;
 
 // Class representing a posting (a term/doc pair, and
 // all the relevant positional information, is a single posting)
@@ -47,7 +46,7 @@ class InMemoryPosting {
   public:
     Xapian::docid did;
     bool valid;
-    vector<Xapian::termpos> positions; // Sorted vector of positions
+    Xapian::VecCOW<Xapian::termpos> positions; // Sorted vector of positions
     Xapian::termcount wdf;
 
     // Add new position entry preserving sorted order.
@@ -60,8 +59,8 @@ class InMemoryPosting {
 
 class InMemoryTermEntry {
   public:
-    string tname;
-    vector<Xapian::termpos> positions; // Sorted vector of positions
+    std::string tname;
+    Xapian::VecCOW<Xapian::termpos> positions; // Sorted vector of positions
     Xapian::termcount wdf;
 
     // Add new position entry preserving sorted order.
@@ -96,7 +95,7 @@ class InMemoryTermEntryLessThan {
 class InMemoryTerm {
   public:
     // Sorted list of documents indexing this term.
-    vector<InMemoryPosting> docs;
+    std::vector<InMemoryPosting> docs;
 
     Xapian::termcount term_freq;
     Xapian::termcount collection_freq;
@@ -114,7 +113,7 @@ class InMemoryDoc {
   public:
     bool is_valid;
     // Sorted list of terms indexing this document.
-    vector<InMemoryTermEntry> terms;
+    std::vector<InMemoryTermEntry> terms;
 
     /* Initialise invalid by default, so that resizing the termlist array
      * doesn't create valid documents. */
@@ -137,9 +136,8 @@ class InMemoryPostList : public LeafPostList {
     friend class InMemoryDatabase;
 
   private:
-    vector<InMemoryPosting>::const_iterator pos;
-    vector<InMemoryPosting>::const_iterator end;
-    Xapian::doccount termfreq;
+    std::vector<InMemoryPosting>::const_iterator pos;
+    std::vector<InMemoryPosting>::const_iterator end;
     bool started;
 
     /** List of positions of the current term.
@@ -152,14 +150,12 @@ class InMemoryPostList : public LeafPostList {
     Xapian::termcount wdf_upper_bound;
 
     InMemoryPostList(const InMemoryDatabase* db,
-		     const InMemoryTerm & imterm, const std::string & term_);
+		     const InMemoryTerm& imterm, std::string_view term_);
   public:
-    Xapian::doccount get_termfreq() const;
-
     Xapian::docid get_docid() const;     // Gets current docid
-    Xapian::termcount get_doclength() const; // Length of current document
-    Xapian::termcount get_unique_terms() const; // number of terms in current document
     Xapian::termcount get_wdf() const;	   // Within Document Frequency
+    // Max wdf of terms in current document
+    Xapian::termcount get_wdfdocmax() const;
     PositionList * read_position_list();
     PositionList * open_position_list() const;
 
@@ -173,7 +169,9 @@ class InMemoryPostList : public LeafPostList {
 
     Xapian::termcount get_wdf_upper_bound() const;
 
-    string get_description() const;
+    void get_docid_range(Xapian::docid& first, Xapian::docid& last) const;
+
+    std::string get_description() const;
 };
 
 /** A PostList over all docs in an inmemory database.
@@ -189,8 +187,6 @@ class InMemoryAllDocsPostList : public LeafPostList {
     InMemoryAllDocsPostList(const InMemoryDatabase* db);
 
   public:
-    Xapian::doccount get_termfreq() const;
-
     Xapian::docid get_docid() const;     // Gets current docid
     Xapian::termcount get_doclength() const; // Length of current document
     // number of terms in current document
@@ -209,7 +205,7 @@ class InMemoryAllDocsPostList : public LeafPostList {
 
     Xapian::termcount get_wdf_upper_bound() const;
 
-    string get_description() const;
+    std::string get_description() const;
 };
 
 // Term List
@@ -217,8 +213,8 @@ class InMemoryTermList : public TermList {
     friend class InMemoryDatabase;
 
   private:
-    vector<InMemoryTermEntry>::const_iterator pos;
-    vector<InMemoryTermEntry>::const_iterator end;
+    std::vector<InMemoryTermEntry>::const_iterator pos;
+    std::vector<InMemoryTermEntry>::const_iterator end;
     Xapian::termcount terms;
     bool started;
 
@@ -237,15 +233,13 @@ class InMemoryTermList : public TermList {
     /// Collate weighting information for the current term.
     void accumulate_stats(Xapian::Internal::ExpandStats & stats) const;
 
-    string get_termname() const;
     // Number of occurrences of term in current doc
     Xapian::termcount get_wdf() const;
     Xapian::doccount get_termfreq() const;  // Number of docs indexed by term
     TermList * next();
-    TermList * skip_to(const std::string & term);
-    bool at_end() const;
+    TermList* skip_to(std::string_view term);
     Xapian::termcount positionlist_count() const;
-    Xapian::PositionIterator positionlist_begin() const;
+    PositionList* positionlist_begin() const;
 };
 
 class InMemoryDocument;
@@ -258,15 +252,15 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     friend class InMemoryAllDocsPostList;
     friend class InMemoryDocument;
 
-    map<string, InMemoryTerm> postlists;
-    vector<InMemoryDoc> termlists;
-    vector<std::string> doclists;
-    vector<std::map<Xapian::valueno, string>> valuelists;
+    std::map<std::string, InMemoryTerm, std::less<>> postlists;
+    std::vector<InMemoryDoc> termlists;
+    std::vector<std::string> doclists;
+    std::vector<std::map<Xapian::valueno, std::string>> valuelists;
     std::map<Xapian::valueno, ValueStats> valuestats;
 
-    vector<Xapian::termcount> doclengths;
+    std::vector<Xapian::termcount> doclengths;
 
-    std::map<string, string> metadata;
+    std::map<std::string, std::string, std::less<>> metadata;
 
     Xapian::doccount totdocs;
 
@@ -281,18 +275,18 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     InMemoryDatabase& operator=(const InMemoryDatabase &);
     InMemoryDatabase(const InMemoryDatabase &);
 
-    void make_term(const string & tname);
+    void make_term(const std::string& tname);
 
     bool doc_exists(Xapian::docid did) const;
-    Xapian::docid make_doc(const string & docdata);
+    Xapian::docid make_doc(const std::string& docdata);
 
     /* The common parts of add_doc and replace_doc */
     void finish_add_doc(Xapian::docid did, const Xapian::Document& document);
     void add_values(Xapian::docid did,
-		    const map<Xapian::valueno, string>& values_);
+		    const std::map<Xapian::valueno, std::string>& values_);
 
-    void make_posting(InMemoryDoc * doc,
-		      const string & tname,
+    void make_posting(InMemoryDoc* doc,
+		      const std::string& tname,
 		      Xapian::docid did,
 		      Xapian::termpos position,
 		      Xapian::termcount wdf,
@@ -336,39 +330,57 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     Xapian::totallength get_total_length() const;
     Xapian::termcount get_doclength(Xapian::docid did) const;
     Xapian::termcount get_unique_terms(Xapian::docid did) const;
+    Xapian::termcount get_wdfdocmax(Xapian::docid did) const;
 
-    void get_freqs(const string & term,
-		   Xapian::doccount * termfreq_ptr,
-		   Xapian::termcount * collfreq_ptr) const;
+    void get_freqs(std::string_view term,
+		   Xapian::doccount* termfreq_ptr,
+		   Xapian::termcount* collfreq_ptr) const;
     Xapian::doccount get_value_freq(Xapian::valueno slot) const;
     std::string get_value_lower_bound(Xapian::valueno slot) const;
     std::string get_value_upper_bound(Xapian::valueno slot) const;
-    bool term_exists(const string & tname) const;
+    Xapian::termcount get_doclength_lower_bound() const;
+    Xapian::termcount get_doclength_upper_bound() const;
+    Xapian::termcount get_wdf_upper_bound(std::string_view term) const;
+    bool term_exists(std::string_view term) const;
     bool has_positions() const;
 
-    LeafPostList * open_post_list(const string & tname) const;
+    PostList* open_post_list(std::string_view tname) const;
+    LeafPostList* open_leaf_post_list(std::string_view term,
+				      bool need_read_pos) const;
     TermList * open_term_list(Xapian::docid did) const;
+    TermList * open_term_list_direct(Xapian::docid did) const;
     Xapian::Document::Internal* open_document(Xapian::docid did,
 					      bool lazy) const;
 
-    std::string get_metadata(const std::string & key) const;
-    TermList * open_metadata_keylist(const std::string &prefix) const;
-    void set_metadata(const std::string & key, const std::string & value);
+    std::string get_metadata(std::string_view key) const;
+    TermList* open_metadata_keylist(std::string_view prefix) const;
+    void set_metadata(std::string_view key, std::string_view value);
 
     Xapian::termcount positionlist_count(Xapian::docid did,
-					 const string & tname) const;
-    PositionList * open_position_list(Xapian::docid did,
-				      const string & tname) const;
-    TermList * open_allterms(const string & prefix) const;
+					 std::string_view tname) const;
+    PositionList* open_position_list(Xapian::docid did,
+				     std::string_view tname) const;
+    TermList* open_allterms(std::string_view prefix) const;
 
-    XAPIAN_NORETURN(static void throw_database_closed());
+    [[noreturn]]
+    static void throw_database_closed();
 
-    int get_backend_info(string * path) const {
-	if (path) *path = string();
+    int get_backend_info(std::string* path) const {
+	if (path) *path = std::string();
 	return BACKEND_INMEMORY;
     }
 
+    void get_used_docid_range(Xapian::docid& first, Xapian::docid& last) const;
+
     bool locked() const { return !closed; }
+
+    Xapian::Database::Internal* update_lock(int flags);
+
+    std::string get_description() const;
 };
 
-#endif /* OM_HGUARD_INMEMORY_DATABASE_H */
+#ifdef DISABLE_GPL_LIBXAPIAN
+# error GPL source we cannot relicense included in libxapian
+#endif
+
+#endif /* XAPIAN_INCLUDED_INMEMORY_DATABASE_H */
