@@ -1,7 +1,7 @@
 /** @file
  * @brief Class for merging AllTermsList objects from subdatabases.
  */
-/* Copyright (C) 2007,2008,2011 Olly Betts
+/* Copyright (C) 2007,2008,2011,2017,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,10 +23,11 @@
 
 #include "backends/alltermslist.h"
 
-#include "backends/database.h"
-
 #include <string>
-#include <vector>
+
+namespace Xapian {
+class Database;
+}
 
 /// Class for merging AllTermsList objects from subdatabases.
 class MultiAllTermsList : public AllTermsList {
@@ -36,22 +37,29 @@ class MultiAllTermsList : public AllTermsList {
     /// Don't allow copying.
     MultiAllTermsList(const MultiAllTermsList &);
 
-    /// Current termname (or empty if we haven't started yet).
-    std::string current_term;
+    /* Current termname (now in base class).
+     *
+     * If current_term.empty(), then either we haven't started yet (and
+     * count != 0) or we've reached the end (and count == 0).
+     */
 
-    /// Vector of sub-termlists which we use as a heap.
-    std::vector<TermList *> termlists;
+    /// Current termfreq (or 0 if not yet calculated).
+    mutable Xapian::doccount current_termfreq;
+
+    /// Number of TermList* entries in @a termlists.
+    mutable size_t count;
+
+    /// Sub-termlists which we use as a heap.
+    TermList** termlists;
 
   public:
     /// Constructor.
-    MultiAllTermsList(const std::vector<Xapian::Internal::intrusive_ptr<Xapian::Database::Internal> > & dbs,
-		      const std::string & prefix);
+    MultiAllTermsList(size_t count_, TermList** termlists_);
 
     /// Destructor.
     ~MultiAllTermsList();
 
-    /// Return the termname at the current position.
-    std::string get_termname() const;
+    Xapian::termcount get_approx_size() const;
 
     /// Return the term frequency for the term at the current position.
     Xapian::doccount get_termfreq() const;
@@ -64,10 +72,7 @@ class MultiAllTermsList : public AllTermsList {
      *  If the specified term isn't in the list, position ourselves on the
      *  first term after @a term (or at_end() if no terms after @a term exist).
      */
-    TermList *skip_to(const std::string &term);
-
-    /// Return true if the current position is past the last term in this list.
-    bool at_end() const;
+    TermList* skip_to(std::string_view term);
 };
 
 #endif // XAPIAN_INCLUDED_MULTI_ALLTERMSLIST_H
