@@ -49,6 +49,9 @@ class PositionIterator::Internal : public Xapian::Internal::intrusive_base
     /// Return approximate size of this positionlist.
     virtual Xapian::termcount get_approx_size() const = 0;
 
+    /// Return the final entry in this positionlist.
+    virtual Xapian::termpos back() const = 0;
+
     /// Return the current position.
     virtual Xapian::termpos get_position() const = 0;
 
