@@ -1,7 +1,7 @@
 /** @file
  *  @brief Remote backend database class
  */
-/* Copyright (C) 2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2019,2020 Olly Betts
+/* Copyright (C) 2006-2024 Olly Betts
  * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -26,24 +26,29 @@
 #include <signal.h>
 #include "safesyssocket.h" // For MSG_NOSIGNAL.
 
-#include "autoptr.h"
+#include "api/msetinternal.h"
+#include "api/smallvector.h"
+#include "backends/contiguousalldocspostlist.h"
 #include "backends/inmemory/inmemory_positionlist.h"
 #include "net_postlist.h"
-#include "net_termlist.h"
-#include "noreturn.h"
 #include "remote-document.h"
 #include "omassert.h"
 #include "realtime.h"
-#include "net/length.h"
 #include "net/serialise.h"
 #include "net/serialise-error.h"
+#include "pack.h"
+#include "remote_alltermslist.h"
+#include "remote_keylist.h"
+#include "remote_termlist.h"
 #include "serialise-double.h"
 #include "str.h"
 #include "stringutils.h" // For STRINGIZE().
 #include "weight/weightinternal.h"
 
 #include <cerrno>
+#include <memory>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include "xapian/constants.h"
@@ -55,20 +60,22 @@ using Xapian::Internal::intrusive_ptr;
 
 /// Return true if further replies should be expected.
 static inline bool
-is_intermediate_reply(int msg_code, int reply_code)
+is_intermediate_reply(int reply_code)
 {
-    return reply_code == REPLY_ALLTERMS ||
-	   reply_code == REPLY_DOCDATA ||
+    return reply_code == REPLY_DOCDATA ||
 	   reply_code == REPLY_VALUE ||
-	   reply_code == REPLY_TERMLIST ||
-	   reply_code == REPLY_POSITIONLIST ||
-	   reply_code == REPLY_POSTLISTSTART ||
-	   reply_code == REPLY_POSTLISTITEM ||
-	   reply_code == REPLY_METADATAKEYLIST ||
-	   (msg_code == MSG_TERMLIST && reply_code == REPLY_DOCLENGTH);
+	   reply_code == REPLY_TERMLISTHEADER ||
+	   reply_code == REPLY_POSTLISTHEADER;
 }
 
-XAPIAN_NORETURN(static void throw_handshake_failed(const string & context));
+[[noreturn]]
+static void
+throw_invalid_operation(const char* message)
+{
+    throw Xapian::InvalidOperationError(message);
+}
+
+[[noreturn]]
 static void
 throw_handshake_failed(const string & context)
 {
@@ -76,78 +83,77 @@ throw_handshake_failed(const string & context)
 			       context);
 }
 
-XAPIAN_NORETURN(static void throw_connection_closed_unexpectedly());
+[[noreturn]]
 static void
 throw_connection_closed_unexpectedly()
 {
     throw Xapian::NetworkError("Connection closed unexpectedly");
 }
 
-RemoteDatabase::RemoteDatabase(int fd, double timeout_,
-			       const string & context_, bool writable,
+RemoteDatabase::RemoteDatabase(pair<int, string> fd_and_context,
+			       double timeout_,
+			       bool writable,
 			       int flags)
-	: link(fd, fd, context_),
-	  context(context_),
-	  cached_stats_valid(),
-	  mru_valstats(),
-	  mru_slot(Xapian::BAD_VALUENO),
-	  timeout(timeout_)
+    : Xapian::Database::Internal(writable ?
+				 TRANSACTION_NONE :
+				 TRANSACTION_READONLY),
+      link(fd_and_context.first, fd_and_context.first, fd_and_context.second),
+      cached_stats_valid(),
+      mru_valstats(),
+      mru_slot(Xapian::BAD_VALUENO),
+      timeout(timeout_)
 {
-    if (!writable) {
-	// Transactions only make sense when writing, so flag them as
-	// "unimplemented" so that our destructor doesn't call dtor_called()
-	// since that might try to call commit() which will cause a message to
-	// be sent to the remote server and probably an InvalidOperationError
-	// exception message to be returned.
-	transaction_state = TRANSACTION_UNIMPLEMENTED;
-    }
-
     update_stats(MSG_MAX);
 
     if (writable) {
 	if (flags & Xapian::DB_RETRY_LOCK) {
-	    const string & body = encode_length(flags & Xapian::DB_RETRY_LOCK);
-	    update_stats(MSG_WRITEACCESS, body);
+	    string message;
+	    pack_uint_last(message, unsigned(flags & Xapian::DB_RETRY_LOCK));
+	    update_stats(MSG_WRITEACCESS, message);
 	} else {
 	    update_stats(MSG_WRITEACCESS);
 	}
     }
 }
 
+Xapian::termcount
+RemoteDatabase::positionlist_count(Xapian::docid did,
+				   std::string_view term) const
+{
+    if (cached_stats_valid && !has_positional_info)
+	return 0;
+
+    string message;
+    pack_uint(message, did);
+    message += term;
+    send_message(MSG_POSITIONLISTCOUNT, message);
+
+    get_message(message, REPLY_POSITIONLISTCOUNT);
+    const char * p = message.data();
+    const char * p_end = p + message.size();
+    Xapian::termcount count;
+    if (!unpack_uint_last(&p, p_end, &count)) {
+	throw Xapian::NetworkError("Bad REPLY_POSITIONLISTCOUNT",
+				   link.get_context());
+    }
+    return count;
+}
+
 void
 RemoteDatabase::keep_alive()
 {
-    send_message(MSG_KEEPALIVE, string());
+    send_message(MSG_KEEPALIVE, {});
     string message;
     get_message(message, REPLY_DONE);
 }
 
-TermList *
-RemoteDatabase::open_metadata_keylist(const std::string &prefix) const
+TermList*
+RemoteDatabase::open_metadata_keylist(std::string_view prefix) const
 {
-    // Ensure that total_length and doccount are up-to-date.
-    if (!cached_stats_valid) update_stats();
-
     send_message(MSG_METADATAKEYLIST, prefix);
-
     string message;
-    AutoPtr<NetworkTermList> tlist(
-	new NetworkTermList(0, doccount,
-			    intrusive_ptr<const RemoteDatabase>(this),
-			    0));
-    vector<NetworkTermListItem> & items = tlist->items;
-
-    string term = prefix;
-    while (get_message_or_done(message, REPLY_METADATAKEYLIST)) {
-	NetworkTermListItem item;
-	term.resize(size_t(static_cast<unsigned char>(message[0])));
-	term.append(message, 1, string::npos);
-	item.tname = term;
-	items.push_back(item);
-    }
-
-    tlist->current_position = tlist->items.begin();
-    return tlist.release();
+    get_message(message, REPLY_METADATAKEYLIST);
+    return new RemoteKeyList(prefix, std::move(message));
 }
 
 TermList *
@@ -158,116 +164,108 @@ RemoteDatabase::open_term_list(Xapian::docid did) const
     // Ensure that total_length and doccount are up-to-date.
     if (!cached_stats_valid) update_stats();
 
-    send_message(MSG_TERMLIST, encode_length(did));
-
     string message;
-    get_message(message, REPLY_DOCLENGTH);
+    pack_uint_last(message, did);
+    send_message(MSG_TERMLIST, message);
+
+    get_message(message, REPLY_TERMLISTHEADER);
     const char * p = message.c_str();
     const char * p_end = p + message.size();
     Xapian::termcount doclen;
-    decode_length(&p, p_end, doclen);
-    if (p != p_end) {
-	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH message received", context);
+    Xapian::termcount num_entries;
+    if (!unpack_uint(&p, p_end, &doclen) ||
+	!unpack_uint_last(&p, p_end, &num_entries)) {
+	throw Xapian::NetworkError("Bad REPLY_TERMLISTHEADER",
+				   link.get_context());
     }
-
-    AutoPtr<NetworkTermList> tlist(
-	new NetworkTermList(doclen, doccount,
-			    intrusive_ptr<const RemoteDatabase>(this),
-			    did));
-    vector<NetworkTermListItem> & items = tlist->items;
-
-    string term;
-    while (get_message_or_done(message, REPLY_TERMLIST)) {
-	NetworkTermListItem item;
-	p = message.data();
-	p_end = p + message.size();
-	decode_length(&p, p_end, item.wdf);
-	decode_length(&p, p_end, item.termfreq);
-	term.resize(size_t(static_cast<unsigned char>(*p++)));
-	term.append(p, p_end);
-	item.tname = term;
-	items.push_back(item);
-    }
-
-    tlist->current_position = tlist->items.begin();
-    return tlist.release();
+    get_message(message, REPLY_TERMLIST);
+    return new RemoteTermList(num_entries, doclen, doccount, this, did,
+			      std::move(message));
 }
 
 TermList *
-RemoteDatabase::open_allterms(const string & prefix) const {
-    // Ensure that total_length and doccount are up-to-date.
-    if (!cached_stats_valid) update_stats();
+RemoteDatabase::open_term_list_direct(Xapian::docid did) const
+{
+    return RemoteDatabase::open_term_list(did);
+}
 
+TermList*
+RemoteDatabase::open_allterms(string_view prefix) const
+{
     send_message(MSG_ALLTERMS, prefix);
-
-    AutoPtr<NetworkTermList> tlist(
-	new NetworkTermList(0, doccount,
-			    intrusive_ptr<const RemoteDatabase>(this),
-			    0));
-    vector<NetworkTermListItem> & items = tlist->items;
-
-    string term = prefix;
     string message;
-    while (get_message_or_done(message, REPLY_ALLTERMS)) {
-	NetworkTermListItem item;
-	const char * p = message.data();
-	const char * p_end = p + message.size();
-	decode_length(&p, p_end, item.termfreq);
-	term.resize(size_t(static_cast<unsigned char>(*p++)));
-	term.append(p, p_end);
-	item.tname = term;
-	items.push_back(item);
+    get_message(message, REPLY_ALLTERMS);
+    return new RemoteAllTermsList(prefix, std::move(message));
+}
+
+PostList*
+RemoteDatabase::open_post_list(string_view term) const
+{
+    if (term.empty()) {
+	if (!cached_stats_valid) update_stats();
+	if (rare(doccount == 0))
+	    return nullptr;
+	if (doccount == lastdocid) {
+	    // The used docid range is exactly 1 to doccount inclusive.
+	    return new ContiguousAllDocsPostList(doccount);
+	}
     }
 
-    tlist->current_position = tlist->items.begin();
-    return tlist.release();
-}
-
-LeafPostList *
-RemoteDatabase::open_post_list(const string &term) const
-{
-    return new NetworkPostList(intrusive_ptr<const RemoteDatabase>(this), term);
-}
-
-Xapian::doccount
-RemoteDatabase::read_post_list(const string &term, NetworkPostList & pl) const
-{
     send_message(MSG_POSTLIST, term);
 
     string message;
-    get_message(message, REPLY_POSTLISTSTART);
+    get_message(message, REPLY_POSTLISTHEADER);
 
     const char * p = message.data();
     const char * p_end = p + message.size();
     Xapian::doccount termfreq;
-    decode_length(&p, p_end, termfreq);
-
-    while (get_message_or_done(message, REPLY_POSTLISTITEM)) {
-	pl.append_posting(message);
+    if (!unpack_uint_last(&p, p_end, &termfreq)) {
+	unpack_throw_serialisation_error(p);
     }
 
-    return termfreq;
+    get_message(message, REPLY_POSTLIST);
+
+    return new NetworkPostList(intrusive_ptr<const RemoteDatabase>(this),
+			       term,
+			       termfreq,
+			       std::move(message));
 }
 
-PositionList *
-RemoteDatabase::open_position_list(Xapian::docid did, const string &term) const
+LeafPostList*
+RemoteDatabase::open_leaf_post_list(string_view, bool) const
 {
-    send_message(MSG_POSITIONLIST, encode_length(did) + term);
-
-    vector<Xapian::termpos> positions;
+    // This method is only called during the match, and remote shards are
+    // handled by running the match on the remote.
+    Assert(false);
+    return nullptr;
+}
 
+PositionList*
+RemoteDatabase::open_position_list(Xapian::docid did, string_view term) const
+{
     string message;
+    pack_uint(message, did);
+    message += term;
+    send_message(MSG_POSITIONLIST, message);
+
+    get_message(message, REPLY_POSITIONLIST);
+    if (message.empty())
+	return nullptr;
+
+    Xapian::VecCOW<Xapian::termpos> positions;
     Xapian::termpos lastpos = static_cast<Xapian::termpos>(-1);
-    while (get_message_or_done(message, REPLY_POSITIONLIST)) {
-	const char * p = message.data();
-	const char * p_end = p + message.size();
+    const char* p = message.data();
+    const char* p_end = p + message.size();
+    while (p != p_end) {
 	Xapian::termpos inc;
-	decode_length(&p, p_end, inc);
-	lastpos += inc + 1;
+	if (!unpack_uint(&p, p_end, &inc)) {
+	    unpack_throw_serialisation_error(p);
+	}
+	UNSIGNED_OVERFLOW_OK(lastpos += inc + 1);
 	positions.push_back(lastpos);
     }
 
-    return new InMemoryPositionList(positions);
+    return new InMemoryPositionList(std::move(positions));
 }
 
 bool
@@ -317,21 +315,26 @@ RemoteDatabase::open_document(Xapian::docid did, bool /*lazy*/) const
 {
     Assert(did);
 
-    send_message(MSG_DOCUMENT, encode_length(did));
+    string message;
+    pack_uint_last(message, did);
+    send_message(MSG_DOCUMENT, message);
+
     string doc_data;
-    map<Xapian::valueno, string> values;
     get_message(doc_data, REPLY_DOCDATA);
 
-    string message;
+    map<Xapian::valueno, string> values;
     while (get_message_or_done(message, REPLY_VALUE)) {
 	const char * p = message.data();
 	const char * p_end = p + message.size();
 	Xapian::valueno slot;
-	decode_length(&p, p_end, slot);
+	if (!unpack_uint(&p, p_end, &slot)) {
+	    unpack_throw_serialisation_error(p);
+	}
 	values.insert(make_pair(slot, string(p, p_end)));
     }
 
-    return new RemoteDocument(this, did, doc_data, values);
+    return new RemoteDocument(this, did, std::move(doc_data),
+			      std::move(values));
 }
 
 bool
@@ -349,7 +352,7 @@ RemoteDatabase::update_stats(message_type msg_code, const string & body) const
     }
 
     if (message.size() < 3) {
-	throw_handshake_failed(context);
+	throw_handshake_failed(link.get_context());
     }
     const char *p = message.c_str();
     const char *p_end = p + message.size();
@@ -376,20 +379,20 @@ RemoteDatabase::update_stats(message_type msg_code, const string & body) const
 	    STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MAJOR_VERSION)
 	    "."
 	    STRINGIZE(XAPIAN_REMOTE_PROTOCOL_MINOR_VERSION);
-	throw Xapian::NetworkError(errmsg, context);
+	throw Xapian::NetworkError(errmsg, link.get_context());
     }
 
-    decode_length(&p, p_end, doccount);
-    decode_length(&p, p_end, lastdocid);
-    lastdocid += doccount;
-    decode_length(&p, p_end, doclen_lbound);
-    decode_length(&p, p_end, doclen_ubound);
-    doclen_ubound += doclen_lbound;
-    if (p == p_end) {
-	throw Xapian::NetworkError("Bad stats update message received", context);
+    if (!unpack_uint(&p, p_end, &doccount) ||
+	!unpack_uint(&p, p_end, &lastdocid) ||
+	!unpack_uint(&p, p_end, &doclen_lbound) ||
+	!unpack_uint(&p, p_end, &doclen_ubound) ||
+	!unpack_bool(&p, p_end, &has_positional_info) ||
+	!unpack_uint(&p, p_end, &total_length)) {
+	throw Xapian::NetworkError("Bad stats update message received",
+				   link.get_context());
     }
-    has_positional_info = (*p++ == '1');
-    decode_length(&p, p_end, total_length);
+    lastdocid += doccount;
+    doclen_ubound += doclen_lbound;
     uuid.assign(p, p_end);
     cached_stats_valid = true;
     return true;
@@ -417,10 +420,12 @@ RemoteDatabase::get_total_length() const
 }
 
 bool
-RemoteDatabase::term_exists(const string & tname) const
+RemoteDatabase::term_exists(string_view term) const
 {
-    Assert(!tname.empty());
-    send_message(MSG_TERMEXISTS, tname);
+    if (term.empty()) {
+	return get_doccount() != 0;
+    }
+    send_message(MSG_TERMEXISTS, term);
     string message;
     reply_type type = get_message(message,
 				  REPLY_TERMEXISTS,
@@ -429,58 +434,64 @@ RemoteDatabase::term_exists(const string & tname) const
 }
 
 void
-RemoteDatabase::get_freqs(const string & term,
-			  Xapian::doccount * termfreq_ptr,
-			  Xapian::termcount * collfreq_ptr) const
+RemoteDatabase::get_freqs(string_view term,
+			  Xapian::doccount* termfreq_ptr,
+			  Xapian::termcount* collfreq_ptr) const
 {
     Assert(!term.empty());
     string message;
-    const char * p;
-    const char * p_end;
-    if (termfreq_ptr) {
-	if (collfreq_ptr) {
-	    send_message(MSG_FREQS, term);
-	    get_message(message, REPLY_FREQS);
-	} else {
-	    send_message(MSG_TERMFREQ, term);
-	    get_message(message, REPLY_TERMFREQ);
+    if (termfreq_ptr && collfreq_ptr) {
+	send_message(MSG_FREQS, term);
+	get_message(message, REPLY_FREQS);
+	const char* p = message.data();
+	const char* p_end = p + message.size();
+	if (unpack_uint(&p, p_end, termfreq_ptr) &&
+	    unpack_uint_last(&p, p_end, collfreq_ptr)) {
+	    return;
+	}
+    } else if (termfreq_ptr) {
+	send_message(MSG_TERMFREQ, term);
+	get_message(message, REPLY_TERMFREQ);
+	const char* p = message.data();
+	const char* p_end = p + message.size();
+	if (unpack_uint_last(&p, p_end, termfreq_ptr)) {
+	    return;
 	}
-	p = message.data();
-	p_end = p + message.size();
-	decode_length(&p, p_end, *termfreq_ptr);
     } else if (collfreq_ptr) {
 	send_message(MSG_COLLFREQ, term);
 	get_message(message, REPLY_COLLFREQ);
-	p = message.data();
-	p_end = p + message.size();
-    }
-    if (collfreq_ptr) {
-	decode_length(&p, p_end, *collfreq_ptr);
+	const char* p = message.data();
+	const char* p_end = p + message.size();
+	if (unpack_uint_last(&p, p_end, collfreq_ptr)) {
+	    return;
+	}
+    } else {
+	Assert(false);
+	return;
     }
+    throw Xapian::NetworkError("Bad REPLY_FREQS/REPLY_TERMFREQ/REPLY_COLLFREQ",
+			       link.get_context());
 }
 
 void
 RemoteDatabase::read_value_stats(Xapian::valueno slot) const
 {
-    if (mru_slot != slot) {
-	send_message(MSG_VALUESTATS, encode_length(slot));
-	string message;
-	get_message(message, REPLY_VALUESTATS);
-	const char * p = message.data();
-	const char * p_end = p + message.size();
-	mru_slot = slot;
-	decode_length(&p, p_end, mru_valstats.freq);
-	size_t len;
-	decode_length_and_check(&p, p_end, len);
-	mru_valstats.lower_bound.assign(p, len);
-	p += len;
-	decode_length_and_check(&p, p_end, len);
-	mru_valstats.upper_bound.assign(p, len);
-	p += len;
-	if (p != p_end) {
-	    throw Xapian::NetworkError("Bad REPLY_VALUESTATS message received", context);
-	}
+    if (mru_slot == slot)
+	return;
+
+    string message;
+    pack_uint_last(message, slot);
+    send_message(MSG_VALUESTATS, message);
+
+    get_message(message, REPLY_VALUESTATS);
+    const char* p = message.data();
+    const char* p_end = p + message.size();
+    mru_slot = slot;
+    if (!unpack_uint(&p, p_end, &mru_valstats.freq) ||
+	!unpack_string(&p, p_end, mru_valstats.lower_bound)) {
+	throw Xapian::NetworkError("Bad REPLY_VALUESTATS", link.get_context());
     }
+    mru_valstats.upper_bound.assign(p, p_end);
 }
 
 Xapian::doccount
@@ -517,7 +528,7 @@ RemoteDatabase::get_doclength_upper_bound() const
 }
 
 Xapian::termcount
-RemoteDatabase::get_wdf_upper_bound(const string &) const
+RemoteDatabase::get_wdf_upper_bound(string_view) const
 {
     // The default implementation returns get_collection_freq(), but we
     // don't want the overhead of a remote message and reply per query
@@ -530,15 +541,16 @@ Xapian::termcount
 RemoteDatabase::get_doclength(Xapian::docid did) const
 {
     Assert(did != 0);
-    send_message(MSG_DOCLENGTH, encode_length(did));
     string message;
+    pack_uint_last(message, did);
+    send_message(MSG_DOCLENGTH, message);
+
     get_message(message, REPLY_DOCLENGTH);
-    const char * p = message.c_str();
-    const char * p_end = p + message.size();
+    const char* p = message.c_str();
+    const char* p_end = p + message.size();
     Xapian::termcount doclen;
-    decode_length(&p, p_end, doclen);
-    if (p != p_end) {
-	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH message received", context);
+    if (!unpack_uint_last(&p, p_end, &doclen)) {
+	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH", link.get_context());
     }
     return doclen;
 }
@@ -547,19 +559,38 @@ Xapian::termcount
 RemoteDatabase::get_unique_terms(Xapian::docid did) const
 {
     Assert(did != 0);
-    send_message(MSG_UNIQUETERMS, encode_length(did));
     string message;
+    pack_uint_last(message, did);
+    send_message(MSG_UNIQUETERMS, message);
+
     get_message(message, REPLY_UNIQUETERMS);
-    const char * p = message.c_str();
-    const char * p_end = p + message.size();
+    const char* p = message.c_str();
+    const char* p_end = p + message.size();
     Xapian::termcount doclen;
-    decode_length(&p, p_end, doclen);
-    if (p != p_end) {
-	throw Xapian::NetworkError("Bad REPLY_UNIQUETERMS message received", context);
+    if (!unpack_uint_last(&p, p_end, &doclen)) {
+	throw Xapian::NetworkError("Bad REPLY_DOCLENGTH", link.get_context());
     }
     return doclen;
 }
 
+Xapian::termcount
+RemoteDatabase::get_wdfdocmax(Xapian::docid did) const
+{
+    Assert(did != 0);
+    string message;
+    pack_uint_last(message, did);
+    send_message(MSG_WDFDOCMAX, message);
+
+    get_message(message, REPLY_WDFDOCMAX);
+    const char* p = message.c_str();
+    const char* p_end = p + message.size();
+    Xapian::termcount wdfdocmax;
+    if (!unpack_uint_last(&p, p_end, &wdfdocmax)) {
+	throw Xapian::NetworkError("Bad REPLY_WDFDOCMAX", link.get_context());
+    }
+    return wdfdocmax;
+}
+
 reply_type
 RemoteDatabase::get_message(string &result,
 			    reply_type required_type,
@@ -567,20 +598,20 @@ RemoteDatabase::get_message(string &result,
 {
     double end_time = RealTime::end_time(timeout);
     int type = link.get_message(result, end_time);
-    if (pending_reply >= 0 && !is_intermediate_reply(pending_reply, type)) {
-	pending_reply = -1;
+    if (pending_reply && !is_intermediate_reply(type)) {
+	pending_reply = false;
     }
     if (type < 0)
 	throw_connection_closed_unexpectedly();
     if (rare(type) >= REPLY_MAX) {
 	if (required_type == REPLY_UPDATE)
-	    throw_handshake_failed(context);
+	    throw_handshake_failed(link.get_context());
 	string errmsg("Invalid reply type ");
 	errmsg += str(type);
 	throw Xapian::NetworkError(errmsg);
     }
     if (type == REPLY_EXCEPTION) {
-	unserialise_error(result, "REMOTE:", context);
+	unserialise_error(result, "REMOTE:", link.get_context());
     }
     if (type != required_type && type != required_type2) {
 	string errmsg("Expecting reply type ");
@@ -598,40 +629,29 @@ RemoteDatabase::get_message(string &result,
 }
 
 void
-RemoteDatabase::send_message(message_type type, const string &message) const
+RemoteDatabase::send_message(message_type type, string_view message) const
 {
     double end_time = RealTime::end_time(timeout);
-    while (pending_reply >= 0) {
+    while (pending_reply) {
 	string dummy;
 	int reply_code = link.get_message(dummy, end_time);
 	if (reply_code < 0)
 	    throw_connection_closed_unexpectedly();
-	if (!is_intermediate_reply(pending_reply, reply_code)) {
-	    pending_reply = -1;
+	if (!is_intermediate_reply(reply_code)) {
+	    pending_reply = false;
 	}
     }
     link.send_message(static_cast<unsigned char>(type), message, end_time);
-    if (type == MSG_REMOVESPELLING) {
-	// MSG_REMOVESPELLING is the only message we send which doesn't expect
-	// a reply (except MSG_SHUTDOWN which causes us to exit anyway).
-	pending_reply = -1;
-    } else {
-	pending_reply = int(type);
-    }
+    pending_reply = true;
 }
 
 void
 RemoteDatabase::do_close()
 {
-    // In the constructor, we set transaction_state to
-    // TRANSACTION_UNIMPLEMENTED if we aren't writable so that we can check
-    // it here.
-    bool writable = (transaction_state != TRANSACTION_UNIMPLEMENTED);
-
-    if (writable) {
+    if (!is_read_only()) {
 	try {
 	    if (transaction_active()) {
-		cancel_transaction();
+		end_transaction(false);
 	    } else {
 		commit();
 	    }
@@ -656,107 +676,107 @@ RemoteDatabase::do_close()
 void
 RemoteDatabase::set_query(const Xapian::Query& query,
 			  Xapian::termcount qlen,
-			  Xapian::doccount collapse_max,
 			  Xapian::valueno collapse_key,
+			  Xapian::doccount collapse_max,
 			  Xapian::Enquire::docid_order order,
 			  Xapian::valueno sort_key,
 			  Xapian::Enquire::Internal::sort_setting sort_by,
 			  bool sort_value_forward,
 			  double time_limit,
-			  int percent_cutoff, double weight_cutoff,
-			  const Xapian::Weight *wtscheme,
+			  int percent_threshold, double weight_threshold,
+			  const Xapian::Weight& wtscheme,
 			  const Xapian::RSet &omrset,
-			  const vector<Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy>> & matchspies)
+			  const vector<opt_ptr_spy>& matchspies) const
 {
-    string tmp = query.serialise();
-    string message = encode_length(tmp.size());
-    message += tmp;
+    string message;
+    pack_string(message, query.serialise());
 
     // Serialise assorted Enquire settings.
-    message += encode_length(qlen);
-    message += encode_length(collapse_max);
-    if (collapse_max) message += encode_length(collapse_key);
-    message += char('0' + order);
-    message += encode_length(sort_key);
-    message += char('0' + sort_by);
-    message += char('0' + sort_value_forward);
+    pack_uint(message, qlen);
+    pack_uint(message, collapse_max);
+    if (collapse_max) pack_uint(message, collapse_key);
+    message += char(order);
+    message += char(sort_by);
+    if (sort_by != Xapian::Enquire::Internal::REL) {
+	pack_uint(message, sort_key);
+    }
+    pack_bool(message, sort_value_forward);
     message += serialise_double(time_limit);
-    message += char(percent_cutoff);
-    message += serialise_double(weight_cutoff);
+    message += char(percent_threshold);
+    message += serialise_double(weight_threshold);
 
-    tmp = wtscheme->name();
-    message += encode_length(tmp.size());
-    message += tmp;
+    pack_string(message, wtscheme.name());
 
-    tmp = wtscheme->serialise();
-    message += encode_length(tmp.size());
-    message += tmp;
+    pack_string(message, wtscheme.serialise());
 
-    tmp = serialise_rset(omrset);
-    message += encode_length(tmp.size());
-    message += tmp;
+    pack_string(message, serialise_rset(omrset));
 
     for (auto i : matchspies) {
-	tmp = i->name();
-	if (tmp.empty()) {
+	const string& name = i->name();
+	if (name.empty()) {
 	    throw Xapian::UnimplementedError("MatchSpy subclass not suitable for use with remote searches - name() method returned empty string");
 	}
-	message += encode_length(tmp.size());
-	message += tmp;
-
-	tmp = i->serialise();
-	message += encode_length(tmp.size());
-	message += tmp;
+	pack_string(message, name);
+	pack_string(message, i->serialise());
     }
 
     send_message(MSG_QUERY, message);
 }
 
-bool
-RemoteDatabase::get_remote_stats(bool nowait, Xapian::Weight::Internal &out)
+void
+RemoteDatabase::accumulate_remote_stats(Xapian::Weight::Internal& total) const
 {
-    if (nowait && !link.ready_to_read()) return false;
-
     string message;
     get_message(message, REPLY_STATS);
     const char* p = message.data();
-    unserialise_stats(p, p + message.size(), out);
-
-    return true;
+    Xapian::Weight::Internal remote_stats;
+    unserialise_stats(p, p + message.size(), remote_stats);
+    total += remote_stats;
 }
 
 void
 RemoteDatabase::send_global_stats(Xapian::doccount first,
 				  Xapian::doccount maxitems,
 				  Xapian::doccount check_at_least,
-				  const Xapian::Weight::Internal &stats)
+				  const Xapian::KeyMaker* sorter,
+				  const Xapian::Weight::Internal &stats) const
 {
-    string message = encode_length(first);
-    message += encode_length(maxitems);
-    message += encode_length(check_at_least);
+    string message;
+    pack_uint(message, first);
+    pack_uint(message, maxitems);
+    pack_uint(message, check_at_least);
+    if (!sorter) {
+	pack_string_empty(message);
+    } else {
+	const string& name = sorter->name();
+	if (name.empty()) {
+	    throw_invalid_operation("sorter reported empty name");
+	}
+	pack_string(message, name);
+	pack_string(message, sorter->serialise());
+    }
     message += serialise_stats(stats);
     send_message(MSG_GETMSET, message);
 }
 
-void
-RemoteDatabase::get_mset(Xapian::MSet &mset,
-			 const vector<Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy>> & matchspies)
+Xapian::MSet
+RemoteDatabase::get_mset(const vector<opt_ptr_spy>& matchspies) const
 {
     string message;
     get_message(message, REPLY_RESULTS);
     const char * p = message.data();
     const char * p_end = p + message.size();
 
+    string spyresults;
     for (auto i : matchspies) {
-	if (p == p_end)
+	if (!unpack_string(&p, p_end, spyresults)) {
 	    throw Xapian::NetworkError("Expected serialised matchspy");
-	size_t len;
-	decode_length_and_check(&p, p_end, len);
-	string spyresults(p, len);
-	p += len;
+	}
 	i->merge_results(spyresults);
     }
-    mset = unserialise_mset(p, p_end);
+    Xapian::MSet mset;
+    mset.internal->unserialise(p, p_end);
+    return mset;
 }
 
 void
@@ -764,7 +784,7 @@ RemoteDatabase::commit()
 {
     if (!uncommitted_changes) return;
 
-    send_message(MSG_COMMIT, string());
+    send_message(MSG_COMMIT, {});
 
     // We need to wait for a response to ensure documents have been committed.
     string message;
@@ -781,7 +801,7 @@ RemoteDatabase::cancel()
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
 
-    send_message(MSG_CANCEL, string());
+    send_message(MSG_CANCEL, {});
     string dummy;
     get_message(dummy, REPLY_DONE);
 
@@ -800,10 +820,12 @@ RemoteDatabase::add_document(const Xapian::Document & doc)
     string message;
     get_message(message, REPLY_ADDDOCUMENT);
 
-    const char * p = message.data();
-    const char * p_end = p + message.size();
+    const char* p = message.data();
+    const char* p_end = p + message.size();
     Xapian::docid did;
-    decode_length(&p, p_end, did);
+    if (!unpack_uint_last(&p, p_end, &did)) {
+	unpack_throw_serialisation_error(p);
+    }
     return did;
 }
 
@@ -814,13 +836,15 @@ RemoteDatabase::delete_document(Xapian::docid did)
     mru_slot = Xapian::BAD_VALUENO;
     uncommitted_changes = true;
 
-    send_message(MSG_DELETEDOCUMENT, encode_length(did));
-    string dummy;
-    get_message(dummy, REPLY_DONE);
+    string message;
+    pack_uint_last(message, did);
+    send_message(MSG_DELETEDOCUMENT, message);
+
+    get_message(message, REPLY_DONE);
 }
 
 void
-RemoteDatabase::delete_document(const std::string & unique_term)
+RemoteDatabase::delete_document(std::string_view unique_term)
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
@@ -839,34 +863,37 @@ RemoteDatabase::replace_document(Xapian::docid did,
     mru_slot = Xapian::BAD_VALUENO;
     uncommitted_changes = true;
 
-    string message = encode_length(did);
+    string message;
+    pack_uint(message, did);
     message += serialise_document(doc);
 
     send_message(MSG_REPLACEDOCUMENT, message);
-    string dummy;
-    get_message(dummy, REPLY_DONE);
+
+    get_message(message, REPLY_DONE);
 }
 
 Xapian::docid
-RemoteDatabase::replace_document(const std::string & unique_term,
+RemoteDatabase::replace_document(std::string_view unique_term,
 				 const Xapian::Document & doc)
 {
     cached_stats_valid = false;
     mru_slot = Xapian::BAD_VALUENO;
     uncommitted_changes = true;
 
-    string message = encode_length(unique_term.size());
-    message += unique_term;
+    string message;
+    pack_string(message, unique_term);
     message += serialise_document(doc);
 
     send_message(MSG_REPLACEDOCUMENTTERM, message);
 
     get_message(message, REPLY_ADDDOCUMENT);
 
-    const char * p = message.data();
-    const char * p_end = p + message.size();
+    const char* p = message.data();
+    const char* p_end = p + message.size();
     Xapian::docid did;
-    decode_length(&p, p_end, did);
+    if (!unpack_uint_last(&p, p_end, &did)) {
+	unpack_throw_serialisation_error(p);
+    }
     return did;
 }
 
@@ -877,7 +904,7 @@ RemoteDatabase::get_uuid() const
 }
 
 string
-RemoteDatabase::get_metadata(const string & key) const
+RemoteDatabase::get_metadata(string_view key) const
 {
     send_message(MSG_GETMETADATA, key);
     string metadata;
@@ -886,40 +913,114 @@ RemoteDatabase::get_metadata(const string & key) const
 }
 
 void
-RemoteDatabase::set_metadata(const string & key, const string & value)
+RemoteDatabase::set_metadata(string_view key, string_view value)
 {
     uncommitted_changes = true;
 
-    string data = encode_length(key.size());
-    data += key;
-    data += value;
-    send_message(MSG_SETMETADATA, data);
-    string dummy;
-    get_message(dummy, REPLY_DONE);
+    string message;
+    pack_string(message, key);
+    message += value;
+    send_message(MSG_SETMETADATA, message);
+
+    get_message(message, REPLY_DONE);
 }
 
 void
-RemoteDatabase::add_spelling(const string & word,
+RemoteDatabase::request_document(Xapian::docid did) const
+{
+    string message;
+    pack_uint(message, did);
+    send_message(MSG_REQUESTDOCUMENT, message);
+
+    get_message(message, REPLY_DONE);
+}
+
+void
+RemoteDatabase::add_spelling(string_view word,
 			     Xapian::termcount freqinc) const
 {
     uncommitted_changes = true;
 
-    string data = encode_length(freqinc);
-    data += word;
-    send_message(MSG_ADDSPELLING, data);
-    string dummy;
-    get_message(dummy, REPLY_DONE);
+    string message;
+    pack_uint(message, freqinc);
+    message += word;
+    send_message(MSG_ADDSPELLING, message);
+
+    get_message(message, REPLY_DONE);
 }
 
-void
-RemoteDatabase::remove_spelling(const string & word,
+Xapian::termcount
+RemoteDatabase::remove_spelling(string_view word,
 				Xapian::termcount freqdec) const
 {
     uncommitted_changes = true;
 
-    string data = encode_length(freqdec);
-    data += word;
-    send_message(MSG_REMOVESPELLING, data);
+    string message;
+    pack_uint(message, freqdec);
+    message += word;
+    send_message(MSG_REMOVESPELLING, message);
+
+    get_message(message, REPLY_REMOVESPELLING);
+    const char * p = message.data();
+    const char * p_end = p + message.size();
+    Xapian::termcount result;
+    if (!unpack_uint_last(&p, p_end, &result)) {
+	throw Xapian::NetworkError("Bad REPLY_REMOVESPELLING",
+				   link.get_context());
+    }
+    return result;
+}
+
+TermList*
+RemoteDatabase::open_synonym_termlist(string_view word) const
+{
+    string message;
+    send_message(MSG_SYNONYMTERMLIST, word);
+    get_message(message, REPLY_SYNONYMTERMLIST);
+    return new RemoteKeyList({}, std::move(message));
+}
+
+TermList*
+RemoteDatabase::open_synonym_keylist(string_view prefix) const
+{
+    string message;
+    send_message(MSG_SYNONYMKEYLIST, prefix);
+    get_message(message, REPLY_SYNONYMKEYLIST);
+    return new RemoteKeyList({}, std::move(message));
+}
+
+void
+RemoteDatabase::add_synonym(string_view word, string_view synonym) const
+{
+    uncommitted_changes = true;
+
+    string message;
+    pack_string(message, word);
+    message += synonym;
+    send_message(MSG_ADDSYNONYM, message);
+    get_message(message, REPLY_DONE);
+}
+
+void
+RemoteDatabase::remove_synonym(string_view word, string_view synonym) const
+{
+    uncommitted_changes = true;
+
+    string message;
+    pack_string(message, word);
+    message += synonym;
+    send_message(MSG_REMOVESYNONYM, message);
+    get_message(message, REPLY_DONE);
+}
+
+void
+RemoteDatabase::clear_synonyms(string_view word) const
+{
+    uncommitted_changes = true;
+
+    string message;
+    send_message(MSG_CLEARSYNONYMS, word);
+    get_message(message, REPLY_DONE);
 }
 
 bool
@@ -927,3 +1028,31 @@ RemoteDatabase::locked() const
 {
     throw Xapian::UnimplementedError("Database::locked() not implemented for remote backend");
 }
+
+string
+RemoteDatabase::reconstruct_text(Xapian::docid did,
+				 size_t length,
+				 string_view prefix,
+				 Xapian::termpos start_pos,
+				 Xapian::termpos end_pos) const
+{
+    string message;
+    pack_uint(message, did);
+    pack_uint(message, length);
+    pack_uint(message, start_pos);
+    pack_uint(message, end_pos);
+    message += prefix;
+    send_message(MSG_RECONSTRUCTTEXT, message);
+
+    get_message(message, REPLY_RECONSTRUCTTEXT);
+    return message;
+}
+
+string
+RemoteDatabase::get_description() const
+{
+    string desc = "Remote(context=";
+    desc += link.get_context();
+    desc += ')';
+    return desc;
+}
