@@ -1,7 +1,7 @@
 /** @file
  *  @brief RemoteDatabase is the baseclass for remote database implementations.
  */
-/* Copyright (C) 2006,2007,2009,2010,2011,2014,2015,2019,2020 Olly Betts
+/* Copyright (C) 2006-2024 Olly Betts
  * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -23,13 +23,15 @@
 #define XAPIAN_INCLUDED_REMOTE_DATABASE_H
 
 #include "backends/backends.h"
-#include "backends/database.h"
-#include "api/omenquireinternal.h"
+#include "backends/databaseinternal.h"
+#include "api/enquireinternal.h"
 #include "api/queryinternal.h"
 #include "net/remoteconnection.h"
 #include "backends/valuestats.h"
 #include "xapian/weight.h"
 
+#include <utility>
+
 namespace Xapian {
     class RSet;
 }
@@ -77,20 +79,11 @@ class RemoteDatabase : public Xapian::Database::Internal {
      *  sending another message.  This flag allows us to detect that situation
      *  and discard the unwanted reply rather than trying to read it as the
      *  response to the new message.
-     *
-     *  Unhelpfully the remote protocol in 1.4.x can send REPLY_DOCLENGTH in
-     *  response to MSG_DOCLENGTH (when it's a final reply) or in response to
-     *  MSG_TERMLIST (when further replies are expected).  To allow use to
-     *  distinguish these cases, pending_reply is set to the MSG_* code, or
-     *  -1 if we're not currently expecting a reply.
      */
-    mutable int pending_reply = -1;
+    mutable bool pending_reply = false;
 
     /// The UUID of the remote database.
-    mutable string uuid;
-
-    /// The context to return with any error messages
-    string context;
+    mutable std::string uuid;
 
     mutable bool cached_stats_valid;
 
@@ -119,18 +112,24 @@ class RemoteDatabase : public Xapian::Database::Internal {
      *  can't be created - a derived class must be instantiated which
      *  has code in the constructor to open the socket.
      *
-     *  @param fd	The file descriptor for the connection to the server.
+     *  @param fd_and_context
+     *			A std::pair containing the file descriptor for the
+     *			connection to the server and a context string to
+     *			return with any error messages.  (These are passed
+     *			together so the caller can return them from a
+     *			single helper function easily).
      *  @param timeout_ The timeout used with the network operations.
      *			Generally a Xapian::NetworkTimeoutError exception will
      *			be thrown if the remote end doesn't respond for this
      *			length of time (in seconds).  A timeout of 0 means that
      *			operations will never timeout.
-     *  @param context_ The context to return with any error messages.
      *	@param writable	Is this a WritableDatabase?
      *	@param flags	Xapian::DB_RETRY_LOCK or 0.
      */
-    RemoteDatabase(int fd, double timeout_, const string & context_,
-		   bool writable, int flags);
+    RemoteDatabase(std::pair<int, std::string> fd_and_context,
+		   double timeout_,
+		   bool writable,
+		   int flags);
 
     /// Receive a message from the server.
     reply_type get_message(std::string& message,
@@ -148,78 +147,95 @@ class RemoteDatabase : public Xapian::Database::Internal {
     }
 
     /// Send a message to the server.
-    void send_message(message_type type, const string & data) const;
+    void send_message(message_type type, std::string_view data) const;
 
     /// Close the socket
     void do_close();
 
-    bool get_posting(Xapian::docid &did, double &w, string &value);
+    bool get_posting(Xapian::docid& did, double& w, std::string& value);
 
     /// The timeout value used in network communications, in seconds.
     double timeout;
 
   public:
+    /** Get the length of the position list.
+     *
+     *  Extra method for RemoteDatabase.
+     */
+    Xapian::termcount positionlist_count(Xapian::docid did,
+					 std::string_view term) const;
+
     /// Send a keep-alive message.
     void keep_alive();
 
+    typedef Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy> opt_ptr_spy;
+
     /** Set the query
      *
      * @param query			The query.
      * @param qlen			The query length.
+     * @param collapse_key		The value number to collapse matches on.
      * @param collapse_max		Max number of items with the same key
      *					to leave after collapsing (0 for don't
      *					collapse).
-     * @param collapse_key		The value number to collapse matches on.
      * @param order			Sort order for docids.
      * @param sort_key			The value number to sort on.
      * @param sort_by			Which order to apply sorts in.
      * @param sort_value_forward	Sort order for values.
      * @param time_limit_		Seconds to reduce check_at_least after
      *					(or <= 0 for no limit).
-     * @param percent_cutoff		Percentage cutoff.
-     * @param weight_cutoff		Weight cutoff.
+     * @param percent_threshold		Lower bound on percentage score.
+     * @param weight_threshold		Lower bound on weight.
      * @param wtscheme			Weighting scheme.
      * @param omrset			The rset.
      * @param matchspies                The matchspies to use.
      */
     void set_query(const Xapian::Query& query,
 		   Xapian::termcount qlen,
-		   Xapian::doccount collapse_max,
 		   Xapian::valueno collapse_key,
+		   Xapian::doccount collapse_max,
 		   Xapian::Enquire::docid_order order,
 		   Xapian::valueno sort_key,
 		   Xapian::Enquire::Internal::sort_setting sort_by,
 		   bool sort_value_forward,
 		   double time_limit,
-		   int percent_cutoff, double weight_cutoff,
-		   const Xapian::Weight *wtscheme,
+		   int percent_threshold, double weight_threshold,
+		   const Xapian::Weight& wtscheme,
 		   const Xapian::RSet &omrset,
-		   const vector<Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy>> & matchspies);
+		   const std::vector<opt_ptr_spy>& matchspies) const;
 
-    /** Get the stats from the remote server.
+    /** Get the underlying fd this remote connection reads from.
      *
-     *  @return	true if we got the remote stats; false if we should try again.
+     *  This allows the matcher to efficiently wait for remote databases to be
+     *  ready in parallel using poll() or select().
      */
-    bool get_remote_stats(bool nowait, Xapian::Weight::Internal &out);
+    int get_read_fd() const {
+	return link.get_read_fd();
+    }
+
+    /// Accumulate stats from the remote server.
+    void accumulate_remote_stats(Xapian::Weight::Internal& total) const;
 
     /// Send the global stats to the remote server.
     void send_global_stats(Xapian::doccount first,
 			   Xapian::doccount maxitems,
 			   Xapian::doccount check_at_least,
-			   const Xapian::Weight::Internal &stats);
+			   const Xapian::KeyMaker* sorter,
+			   const Xapian::Weight::Internal &stats) const;
 
     /// Get the MSet from the remote server.
-    void get_mset(Xapian::MSet &mset,
-		  const vector<Xapian::Internal::opt_intrusive_ptr<Xapian::MatchSpy>> & matchspies);
+    Xapian::MSet get_mset(const std::vector<opt_ptr_spy>& matchspies) const;
 
     /// Get remote metadata key list.
-    TermList * open_metadata_keylist(const std::string & prefix) const;
+    TermList* open_metadata_keylist(std::string_view prefix) const;
 
     /// Get remote termlist.
     TermList * open_term_list(Xapian::docid did) const;
 
+    TermList * open_term_list_direct(Xapian::docid did) const;
+
     /// Iterate all terms.
-    TermList * open_allterms(const string & prefix) const;
+    TermList* open_allterms(std::string_view prefix) const;
 
     bool has_positions() const;
 
@@ -227,12 +243,14 @@ class RemoteDatabase : public Xapian::Database::Internal {
 
     void close();
 
-    LeafPostList * open_post_list(const string & tname) const;
+    PostList* open_post_list(std::string_view term) const;
 
-    Xapian::doccount read_post_list(const string &term, NetworkPostList & pl) const;
+    LeafPostList* open_leaf_post_list(std::string_view term, bool) const;
 
-    PositionList * open_position_list(Xapian::docid did,
-				      const string & tname) const;
+    Xapian::doccount read_post_list(const std::string& term, NetworkPostList & pl) const;
+
+    PositionList* open_position_list(Xapian::docid did,
+				     std::string_view tname) const;
 
     /// Get a remote document.
     Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy) const;
@@ -247,13 +265,14 @@ class RemoteDatabase : public Xapian::Database::Internal {
 
     Xapian::termcount get_doclength(Xapian::docid did) const;
     Xapian::termcount get_unique_terms(Xapian::docid did) const;
+    Xapian::termcount get_wdfdocmax(Xapian::docid did) const;
 
     /// Check if term exists.
-    bool term_exists(const string & tname) const;
+    bool term_exists(std::string_view term) const;
 
-    void get_freqs(const string & term,
-		   Xapian::doccount * termfreq_ptr,
-		   Xapian::termcount * collfreq_ptr) const;
+    void get_freqs(std::string_view term,
+		   Xapian::doccount* termfreq_ptr,
+		   Xapian::termcount* collfreq_ptr) const;
 
     /// Read the value statistics for a value from a remote database.
     void read_value_stats(Xapian::valueno slot) const;
@@ -263,7 +282,7 @@ class RemoteDatabase : public Xapian::Database::Internal {
 
     Xapian::termcount get_doclength_lower_bound() const;
     Xapian::termcount get_doclength_upper_bound() const;
-    Xapian::termcount get_wdf_upper_bound(const string & term) const;
+    Xapian::termcount get_wdf_upper_bound(std::string_view term) const;
 
     void commit();
 
@@ -272,28 +291,50 @@ class RemoteDatabase : public Xapian::Database::Internal {
     Xapian::docid add_document(const Xapian::Document & doc);
 
     void delete_document(Xapian::docid did);
-    void delete_document(const std::string & unique_term);
+    void delete_document(std::string_view unique_term);
 
     void replace_document(Xapian::docid did, const Xapian::Document & doc);
-    Xapian::docid replace_document(const std::string & unique_term,
-				   const Xapian::Document & document);
+    Xapian::docid replace_document(std::string_view unique_term,
+				   const Xapian::Document& document);
 
     std::string get_uuid() const;
 
-    string get_metadata(const string & key) const;
+    std::string get_metadata(std::string_view key) const;
 
-    void set_metadata(const string & key, const string & value);
+    void set_metadata(std::string_view key, std::string_view value);
 
-    void add_spelling(const std::string&, Xapian::termcount) const;
+    void request_document(Xapian::docid did) const;
 
-    void remove_spelling(const std::string&, Xapian::termcount freqdec) const;
+    void add_spelling(std::string_view word, Xapian::termcount freqinc) const;
 
-    int get_backend_info(string * path) const {
-	if (path) *path = context;
+    TermList* open_synonym_termlist(std::string_view term) const;
+
+    TermList* open_synonym_keylist(std::string_view prefix) const;
+
+    void add_synonym(std::string_view word, std::string_view synonym) const;
+
+    void remove_synonym(std::string_view word,
+			std::string_view synonym) const;
+
+    void clear_synonyms(std::string_view word) const;
+
+    Xapian::termcount remove_spelling(std::string_view word,
+				      Xapian::termcount freqdec) const;
+
+    int get_backend_info(std::string* path) const {
+	if (path) *path = link.get_context();
 	return BACKEND_REMOTE;
     }
 
     bool locked() const;
+
+    std::string reconstruct_text(Xapian::docid did,
+				 size_t length,
+				 std::string_view prefix,
+				 Xapian::termpos start_pos,
+				 Xapian::termpos end_pos) const;
+
+    std::string get_description() const;
 };
 
 #endif // XAPIAN_INCLUDED_REMOTE_DATABASE_H
