@@ -30,7 +30,7 @@ struct ValueStats {
     /** The number of documents which have a (non-empty) value stored in the
      *  slot.
      */
-    Xapian::doccount freq;
+    Xapian::doccount freq = 0;
 
     /** A lower bound on the values stored in the given value slot.
      */
@@ -41,7 +41,7 @@ struct ValueStats {
     std::string upper_bound;
 
     /// Construct an empty ValueStats object.
-    ValueStats() : freq(0), lower_bound(), upper_bound() {}
+    ValueStats() { }
 
     /** Clear the statistics.
      */
