@@ -25,10 +25,13 @@
 #include <xapian.h>
 
 #include <cstdlib>
+#include <cstring>
 #include <iostream>
 
 #include "gnu_getopt.h"
 
+#include "backends/glass/glass_defs.h"
+
 using namespace std;
 
 #define PROG_NAME "xapian-compact"
@@ -43,6 +46,10 @@ static void show_usage() {
 "Options:\n"
 "  -b, --blocksize=B  Set the blocksize in bytes (e.g. 4096) or K (e.g. 4K)\n"
 "                     (must be between 2K and 64K and a power of 2, default 8K)\n"
+"  -B, --backend=B    Set the output backend.  Supported values are 'glass'\n"
+"                     and 'honey'.  By default input's backend is used.  At\n"
+"                     present only glass to honey conversion is implemented -\n"
+"                     otherwise the backend must be the same.\n"
 "  -n, --no-full      Disable full compaction\n"
 "  -F, --fuller       No effect for glass since Xapian 1.4.31\n"
 "  -m, --multipass    If merging more than 3 databases, merge the postlists in\n"
@@ -53,7 +60,7 @@ static void show_usage() {
 "                     unique ids from an external source).  Currently this\n"
 "                     option is only supported when merging databases if they\n"
 "                     have disjoint ranges of used document ids\n"
-"  -s, --single-file  Produce a single file database (not supported for chert)\n"
+"  -s, --single-file  Produce a single file database\n"
 "  --help             display this help and exit\n"
 "  --version          output version information and exit\n";
 }
@@ -105,12 +112,13 @@ MyCompactor::resolve_duplicate_metadata(const string & key,
 int
 main(int argc, char **argv)
 {
-    const char * opts = "b:nFmqs";
+    const char * opts = "b:B:nFmqs";
     static const struct option long_opts[] = {
 	{"fuller",	no_argument, 0, 'F'},
 	{"no-full",	no_argument, 0, 'n'},
 	{"multipass",	no_argument, 0, 'm'},
 	{"blocksize",	required_argument, 0, 'b'},
+	{"backend",	required_argument, 0, 'B'},
 	{"no-renumber", no_argument, 0, OPT_NO_RENUMBER},
 	{"single-file", no_argument, 0, 's'},
 	{"quiet",	no_argument, 0, 'q'},
@@ -121,28 +129,46 @@ main(int argc, char **argv)
 
     MyCompactor compactor;
     Xapian::Compactor::compaction_level level = Xapian::Compactor::FULL;
+    unsigned backend = 0;
     unsigned flags = 0;
-    size_t block_size = 0;
+    unsigned block_size = 0;
 
     int c;
     while ((c = gnu_getopt_long(argc, argv, opts, long_opts, 0)) != -1) {
 	switch (c) {
 	    case 'b': {
 		char *p;
-		block_size = strtoul(optarg, &p, 10);
-		if (block_size <= 64 && (*p == 'K' || *p == 'k')) {
+		unsigned long value = strtoul(optarg, &p, 10);
+		if (value <= GLASS_MAX_BLOCKSIZE / 1024 &&
+		    (*p == 'K' || *p == 'k')) {
 		    ++p;
-		    block_size *= 1024;
+		    value *= 1024;
 		}
-		if (*p || block_size < 2048 || block_size > 65536 ||
-		    (block_size & (block_size - 1)) != 0) {
+		if (*p ||
+		    value < GLASS_MIN_BLOCKSIZE ||
+		    value > GLASS_MAX_BLOCKSIZE ||
+		    (value & (value - 1)) != 0) {
+		    cerr << PROG_NAME": Bad value '" << optarg << "' passed "
+			    "for blocksize, must be a power of 2 between "
+			 << (GLASS_MIN_BLOCKSIZE / 1024) << "K and "
+			 << (GLASS_MAX_BLOCKSIZE / 1024) << "K\n";
+		    exit(1);
+		}
+		block_size = unsigned(value);
+		break;
+	    }
+	    case 'B':
+		if (strcmp(optarg, "honey") == 0) {
+		    backend = Xapian::DB_BACKEND_HONEY;
+		} else if (strcmp(optarg, "glass") == 0) {
+		    backend = Xapian::DB_BACKEND_GLASS;
+		} else {
 		    cerr << PROG_NAME": Bad value '" << optarg
-			 << "' passed for blocksize, must be a power of 2 "
-			    "between 2K and 64K\n";
+			 << "' passed for backend - must be 'glass' or "
+			    "'honey'\n";
 		    exit(1);
 		}
 		break;
-	    }
 	    case 'n':
 		level = compactor.STANDARD;
 		break;
@@ -182,12 +208,14 @@ main(int argc, char **argv)
     // Path to the database to create.
     string destdir = argv[argc - 1];
 
+    flags |= backend | level;
+
     try {
 	Xapian::Database src;
 	for (int i = optind; i < argc - 1; ++i) {
 	    src.add_database(Xapian::Database(argv[i]));
 	}
-	src.compact(destdir, level | flags, block_size, compactor);
+	src.compact(destdir, flags, block_size, compactor);
     } catch (const Xapian::Error &error) {
 	cerr << argv[0] << ": " << error.get_description() << '\n';
 	exit(1);
