@@ -37,7 +37,7 @@
 using namespace std;
 
 #define PROG_NAME "xapian-inspect"
-#define PROG_DESC "Inspect the contents of a glass table for development or debugging"
+#define PROG_DESC "Inspect a glass table for development or debugging"
 
 #define OPT_HELP 1
 #define OPT_VERSION 2
@@ -295,8 +295,7 @@ main(int argc, char** argv)
     } else if (!arg_is_directory) {
 	single_file_fd = open(db_path.c_str(), O_RDONLY | O_BINARY);
 	if (single_file_fd < 0) {
-	    cerr << argv[0] << ": Couldn't open file '" << db_path << "'"
-		 << endl;
+	    cerr << argv[0] << ": Couldn't open file '" << db_path << "'\n";
 	    exit(1);
 	}
     }
@@ -353,10 +352,11 @@ open_different_table:
 	    cout << "No entries!\n";
 	    exit(0);
 	}
+
 	cout << "Table has " << table.get_entry_count() << " entries\n";
 
 	GlassCursor cursor(&table);
-	cursor.find_entry_ge(string());
+	cursor.rewind();
 	cursor.next();
 
 	while (!cin.eof()) {
@@ -410,7 +410,7 @@ wait_for_input:
 		do_until(cursor, string(), false);
 		goto wait_for_input;
 	    } else if (input == "f" || input == "first") {
-		cursor.find_entry_ge(string());
+		cursor.rewind();
 		cursor.next();
 		continue;
 	    } else if (input == "l" || input == "last") {
