@@ -1,7 +1,7 @@
 /** @file
  * @brief Classes to encode/decode a bitstream.
  */
-/* Copyright (C) 2004,2005,2006,2008,2013,2014,2016,2018 Olly Betts
+/* Copyright (C) 2004,2005,2006,2008,2013,2014,2016,2017,2018 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,8 +28,6 @@
 #include "omassert.h"
 #include "pack.h"
 
-#include <vector>
-
 using namespace std;
 
 // Find the position of the most significant set bit counting from 1 with
@@ -63,7 +61,7 @@ highest_order_bit(T mask)
     };
 
     int result = 0;
-    if (sizeof(T) > 4) {
+    if constexpr(sizeof(T) > 4) {
 	if (mask >= 0x100000000ul) {
 	    mask >>= 32;
 	    result += 32;
@@ -96,7 +94,11 @@ BitWriter::encode(Xapian::termpos value, Xapian::termpos outof)
 {
     Assert(value < outof);
     unsigned bits = highest_order_bit(outof - Xapian::termpos(1));
-    const Xapian::termpos spare = safe_shl(Xapian::termpos(1), bits) - outof;
+    // If the top bit of (outof - Xapian::termpos(1)) is set then
+    // the shift will shift the bit out and give zero and the
+    // subtraction will result in an unsigned overflow.
+    const Xapian::termpos spare =
+	UNSIGNED_OVERFLOW_OK(safe_shl(Xapian::termpos(1), bits) - outof);
     if (spare) {
 	/* If we have spare values, we can use one fewer bit to encode some
 	 * values.  We shorten the values in the middle of the range, as
@@ -154,7 +156,8 @@ BitWriter::encode(Xapian::termpos value, Xapian::termpos outof)
 }
 
 void
-BitWriter::encode_interpolative(const vector<Xapian::termpos>& pos, int j, int k)
+BitWriter::encode_interpolative(const Xapian::VecCOW<Xapian::termpos>& pos,
+				int j, int k)
 {
     // "Interpolative code" - for an algorithm description, see "Managing
     // Gigabytes" - pages 126-127 in the second edition.  You can probably
@@ -178,19 +181,23 @@ BitReader::decode(Xapian::termpos outof, bool force)
     (void)force;
     Assert(force == di_current.is_initialized());
     Xapian::termpos bits = highest_order_bit(outof - Xapian::termpos(1));
-    const Xapian::termpos spare = safe_shl(Xapian::termpos(1), bits) - outof;
+    // If the top bit of (outof - Xapian::termpos(1)) is set then
+    // the shift will shift the bit out and give zero and the
+    // subtraction will result in an unsigned overflow.
+    const Xapian::termpos spare =
+	UNSIGNED_OVERFLOW_OK(safe_shl(Xapian::termpos(1), bits) - outof);
     const Xapian::termpos mid_start = (outof - spare) / 2;
-    Xapian::termpos p;
+    Xapian::termpos pos;
     if (spare) {
-	p = read_bits(bits - 1);
-	if (p < mid_start) {
-	    if (read_bits(1)) p += mid_start + spare;
+	pos = read_bits(bits - 1);
+	if (pos < mid_start) {
+	    if (read_bits(1)) pos += mid_start + spare;
 	}
     } else {
-	p = read_bits(bits);
+	pos = read_bits(bits);
     }
-    Assert(p < outof);
-    return p;
+    Assert(pos < outof);
+    return pos;
 }
 
 Xapian::termpos
@@ -209,9 +216,8 @@ BitReader::read_bits(int count)
 	return result | (read_bits(count - half_the_bits) << half_the_bits);
     }
     while (n_bits < count) {
-	Assert(idx < buf.size());
-	unsigned char byte = buf[idx++];
-	acc |= Xapian::termpos(byte) << n_bits;
+	Assert(p < end);
+	acc |= Xapian::termpos(static_cast<unsigned char>(*p++)) << n_bits;
 	n_bits += 8;
     }
     result = acc & ((Xapian::termpos(1) << count) - Xapian::termpos(1));
