@@ -225,12 +225,8 @@ Xapian::Internal::closefrom(int fd)
     // getdirentries() then this code can be used.  AIX is an example of
     // a platform of the former, but apparently has F_CLOSEM.
     char path[6 + sizeof(pid_t) * 3 + 4];
-# ifdef SNPRINTF
     snprintf(path, sizeof(path), "/proc/%ld/fd", long(getpid()));
     path[sizeof(path) - 1] = '\0';
-# else
-    sprintf(path, "/proc/%ld/fd", long(getpid()));
-# endif
 #endif
     if (maxfd < 0)
 	maxfd = get_maxfd();
