@@ -60,8 +60,7 @@ CompressionStream::compress(const char* buf, size_t* p_size) {
 	out = new char[size];
     }
     deflate_zstream->avail_in = static_cast<uInt>(size);
-    deflate_zstream->next_in =
-	reinterpret_cast<Bytef*>(const_cast<char*>(buf));
+    deflate_zstream->next_in = reinterpret_cast<const Bytef*>(buf);
     deflate_zstream->next_out = reinterpret_cast<Bytef*>(out);
     // Specify the output buffer size as being the size of the input so zlib
     // will give up once it discovers it can't compress (while it might seem
@@ -88,8 +87,7 @@ CompressionStream::decompress_chunk(const char* p, int len, string& buf)
 {
     Bytef blk[8192];
 
-    inflate_zstream->next_in =
-	reinterpret_cast<Bytef*>(const_cast<char*>(p));
+    inflate_zstream->next_in = reinterpret_cast<const Bytef*>(p);
     inflate_zstream->avail_in = static_cast<uInt>(len);
 
     while (true) {
