@@ -1,7 +1,7 @@
 /** @file
  * @brief Wrappers for low-level POSIX I/O routines.
  */
-/* Copyright (C) 2006,2007,2008,2009,2011,2014,2015,2016 Olly Betts
+/* Copyright (C) 2006-2025 Olly Betts
  * Copyright (C) 2010 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,6 +65,40 @@ inline int io_open_block_wr(const std::string& filename, bool anew)
     return io_open_block_wr(filename.c_str(), anew);
 }
 
+/** Open a stream-based file for reading.
+ *
+ *  @param filename  The path of the file to open.
+ */
+inline int io_open_stream_rd(const char* filename) {
+    return ::open(filename, O_RDONLY | O_BINARY | O_CLOEXEC);
+}
+
+/** Open a stream-based file for reading.
+ *
+ *  @param filename  The path of the file to open.
+ */
+inline int io_open_stream_rd(const std::string& filename)
+{
+    return io_open_stream_rd(filename.c_str());
+}
+
+/** Open a stream-based file for writing.
+ *
+ *  @param filename  The path of the file to open.
+ *  @param anew   If true, open the file anew (create or truncate it).
+ */
+int io_open_stream_wr(const char* filename, bool anew);
+
+/** Open a stream-based file for writing.
+ *
+ *  @param filename  The path of the file to open.
+ *  @param anew  If true, open the file anew (create or truncate it).
+ */
+inline int io_open_stream_wr(const std::string& filename, bool anew)
+{
+    return io_open_stream_wr(filename.c_str(), anew);
+}
+
 /** Ensure all data previously written to file descriptor fd has been written to
  *  disk.
  *
@@ -123,6 +157,29 @@ inline void io_write(int fd, const unsigned char * p, size_t n) {
     io_write(fd, reinterpret_cast<const char *>(p), n);
 }
 
+/** Read n bytes (or until EOF) into block pointed to by p from file descriptor
+ *  fd starting at position o.
+ *
+ *  If a read error occurs, throws DatabaseError.
+ *
+ *  If @a min is specified and EOF is reached after less than @a min bytes,
+ *  throws DatabaseCorruptError.
+ *
+ *  The current file position may or may not be updated.
+ *
+ *  Returns the number of bytes actually read.
+ */
+size_t io_pread(int fd, char * p, size_t n, off_t o, size_t min = 0);
+
+/** Write n bytes from block pointed to by p to file descriptor fd starting at
+ *  position o.
+ *
+ *  If a read error occurs, throws DatabaseError.
+ *
+ *  The current file position may or may not be updated.
+ */
+void io_pwrite(int fd, const char * p, size_t n, off_t o);
+
 /** Readahead block b size n bytes from file descriptor fd.
  *
  *  Returns false if we can't readahead on this fd.
