@@ -1,7 +1,7 @@
 /** @file
  * @brief Pack types into strings and unpack them again.
  */
-/* Copyright (C) 2009,2015,2016,2017 Olly Betts
+/* Copyright (C) 2009,2015,2016,2017,2018,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,8 @@
 # error config.h must be included first in each C++ source file
 #endif
 
-#include <cstring>
 #include <string>
+#include <string_view>
 #include <type_traits>
 
 #include "omassert.h"
@@ -48,6 +48,13 @@ const unsigned int SORTABLE_UINT_MAX_BYTES = 1 << SORTABLE_UINT_LOG2_MAX_BYTES;
 const unsigned int SORTABLE_UINT_1ST_BYTE_MASK =
 	(0xffu >> SORTABLE_UINT_LOG2_MAX_BYTES);
 
+/** Throw appropriate SerialisationError.
+ *
+ *  @param p If NULL, out of data; otherwise type overflow.
+ */
+[[noreturn]]
+void unpack_throw_serialisation_error(const char* p);
+
 /** Append an encoded bool to a string.
  *
  *  @param s		The string to append to.
@@ -92,7 +99,7 @@ template<class U>
 inline void
 pack_uint_last(std::string& s, U value)
 {
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
 
     while (value) {
 	s += char(value & 0xff);
@@ -110,7 +117,7 @@ template<class U>
 inline bool
 unpack_uint_last(const char** p, const char* end, U* result)
 {
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
     Assert(result);
 
     const char* ptr = *p;
@@ -130,95 +137,6 @@ unpack_uint_last(const char** p, const char* end, U* result)
     return true;
 }
 
-/** Append an encoded unsigned integer to a string, preserving the sort order.
- *
- *  [Chert variant]
- *
- *  The appended string data will sort in the same order as the unsigned
- *  integer being encoded.
- *
- *  Note that the first byte of the encoding will never be \xff, so it is
- *  safe to store the result of this function immediately after the result of
- *  pack_string_preserving_sort().
- *
- *  @param s		The string to append to.
- *  @param value	The unsigned integer to encode.
- */
-template<class U>
-inline void
-C_pack_uint_preserving_sort(std::string & s, U value)
-{
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
-#if 0
-    // FIXME: Doesn't work with 64-bit Xapian::docid, etc.
-    static_assert(sizeof(U) <= SORTABLE_UINT_MAX_BYTES,
-		  "Template type U too wide for database format");
-#endif
-
-    char tmp[sizeof(U) + 1];
-    char * p = tmp + sizeof(tmp);
-
-    do {
-	*--p = char(value & 0xff);
-	value >>= 8;
-    } while (value &~ SORTABLE_UINT_1ST_BYTE_MASK);
-
-    unsigned char len = static_cast<unsigned char>(tmp + sizeof(tmp) - p);
-    *--p = char((len - 1) << (8 - SORTABLE_UINT_LOG2_MAX_BYTES) | value);
-    s.append(p, len + 1);
-    Assert(s[0] != '\xff');
-}
-
-/** Decode an "sort preserved" unsigned integer from a string.
- *
- *  [Chert variant]
- *
- *  The unsigned integer must have been encoded with
- *  C_pack_uint_preserving_sort().
- *
- *  @param p	    Pointer to pointer to the current position in the string.
- *  @param end	    Pointer to the end of the string.
- *  @param result   Where to store the result.
- */
-template<class U>
-inline bool
-C_unpack_uint_preserving_sort(const char ** p, const char * end, U * result)
-{
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
-    static_assert(sizeof(U) < 256,
-		  "Template type U too wide for database format");
-    Assert(result);
-
-    const char * ptr = *p;
-    Assert(ptr);
-
-    if (rare(ptr == end)) {
-	return false;
-    }
-
-    unsigned char len_byte = static_cast<unsigned char>(*ptr++);
-    *result = len_byte & SORTABLE_UINT_1ST_BYTE_MASK;
-    size_t len = (len_byte >> (8 - SORTABLE_UINT_LOG2_MAX_BYTES)) + 1;
-
-    if (rare(size_t(end - ptr) < len)) {
-	return false;
-    }
-
-    end = ptr + len;
-    *p = end;
-
-    // Check for overflow.
-    if (rare(len > int(sizeof(U)))) {
-	return false;
-    }
-
-    while (ptr != end) {
-	*result = (*result << 8) | U(static_cast<unsigned char>(*ptr++));
-    }
-
-    return true;
-}
-
 #if HAVE_DECL___BUILTIN_CLZ && \
     HAVE_DECL___BUILTIN_CLZL && \
     HAVE_DECL___BUILTIN_CLZLL
@@ -251,8 +169,6 @@ do_clz(unsigned long long value) {
 #endif
 
 /** Append an encoded unsigned integer to a string, preserving the sort order.
- *
- *  [Glass and newer variant]
  *
  *  The appended string data will sort in the same order as the unsigned
  *  integer being encoded.
@@ -268,7 +184,7 @@ template<class U>
 inline void
 pack_uint_preserving_sort(std::string& s, U value)
 {
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
     static_assert(sizeof(U) <= 8,
 		  "Template type U too wide for database format");
     // The clz() functions are undefined for 0, so handle the smallest band
@@ -303,8 +219,6 @@ pack_uint_preserving_sort(std::string& s, U value)
 }
 
 /** Decode a "sort preserved" unsigned integer from a string.
- *
- *  [Glass and newer variant]
  *
  *  The unsigned integer must have been encoded with
  *  pack_uint_preserving_sort().
@@ -317,7 +231,7 @@ template<class U>
 inline bool
 unpack_uint_preserving_sort(const char** p, const char* end, U* result)
 {
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
     static_assert(sizeof(U) <= 8,
 		  "Template type U too wide for database format");
     Assert(result);
@@ -355,7 +269,7 @@ unpack_uint_preserving_sort(const char** p, const char* end, U* result)
 
     // Check for overflow.
     if (rare(len > int(sizeof(U)))) return false;
-    if (sizeof(U) != 8) {
+    if constexpr(sizeof(U) != 8) {
 	// Need to check the top byte too.
 	if (rare(len == int(sizeof(U)) && len_byte != 0)) return false;
     }
@@ -381,7 +295,7 @@ template<class U>
 inline void
 pack_uint(std::string& s, U value)
 {
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
 
     while (value >= 128) {
 	s += static_cast<char>(static_cast<unsigned char>(value) | 0x80);
@@ -412,7 +326,7 @@ template<class U>
 inline bool
 unpack_uint(const char** p, const char* end, U* result)
 {
-    static_assert(std::is_unsigned<U>::value, "Unsigned type required");
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
 
     const char* ptr = *p;
     Assert(ptr);
@@ -468,30 +382,61 @@ unpack_uint(const char** p, const char* end, U* result)
     return true;
 }
 
+/** Decode an unsigned integer from a string, going backwards.
+ *
+ *  @param p	    Pointer to pointer just after the position in the string.
+ *  @param start    Pointer to the start of the string.
+ *  @param result   Where to store the result (or NULL to just skip it).
+ */
+template<class U>
+inline bool
+unpack_uint_backwards(const char** p, const char* start, U* result)
+{
+    static_assert(std::is_unsigned_v<U>, "Unsigned type required");
+
+    const char* ptr = *p;
+    Assert(ptr);
+
+    // Check it's not empty and that the final byte is valid.
+    if (rare(ptr == start || static_cast<unsigned char>(ptr[-1]) >= 128)) {
+	// Out of data.
+	*p = NULL;
+	return false;
+    }
+
+    do {
+	if (rare(--ptr == start))
+	    break;
+    } while (static_cast<unsigned char>(ptr[-1]) >= 128);
+
+    const char* end = *p;
+    *p = ptr;
+    return unpack_uint(&ptr, end, result);
+}
+
 /** Append an encoded std::string to a string.
  *
  *  @param s		The string to append to.
  *  @param value	The std::string to encode.
  */
 inline void
-pack_string(std::string& s, const std::string& value)
+pack_string(std::string& s, std::string_view value)
 {
     pack_uint(s, value.size());
     s += value;
 }
 
-/** Append an encoded C-style string to a string.
+/** Append an empty encoded std::string to a string.
+ *
+ *  This is equivalent to pack_string(s, ""sv) but is probably a bit more
+ *  efficient.
  *
  *  @param s		The string to append to.
- *  @param ptr		The C-style string to encode.
  */
 inline void
-pack_string(std::string& s, const char* ptr)
+pack_string_empty(std::string& s)
 {
-    Assert(ptr);
-    size_t len = std::strlen(ptr);
-    pack_uint(s, len);
-    s.append(ptr, len);
+    s += '\0';
 }
 
 /** Decode a std::string from a string.
@@ -519,6 +464,31 @@ unpack_string(const char** p, const char* end, std::string& result)
     return true;
 }
 
+/** Decode a std::string from a string and append.
+ *
+ *  @param p	    Pointer to pointer to the current position in the string.
+ *  @param end	    Pointer to the end of the string.
+ *  @param result   Where to store the result.
+ */
+inline bool
+unpack_string_append(const char** p, const char* end, std::string& result)
+{
+    size_t len;
+    if (rare(!unpack_uint(p, end, &len))) {
+	return false;
+    }
+
+    const char*& ptr = *p;
+    if (rare(len > size_t(end - ptr))) {
+	ptr = NULL;
+	return false;
+    }
+
+    result.append(ptr, len);
+    ptr += len;
+    return true;
+}
+
 /** Append an encoded std::string to a string, preserving the sort order.
  *
  *  The byte which follows this encoded value *must not* be \xff, or the sort
@@ -536,7 +506,7 @@ unpack_string(const char** p, const char* end, std::string& result)
  *  where nothing does and get a shorter encoding in those cases.
  */
 inline void
-pack_string_preserving_sort(std::string& s, const std::string& value,
+pack_string_preserving_sort(std::string& s, std::string_view value,
 			    bool last = false)
 {
     std::string::size_type b = 0, e;
@@ -582,7 +552,7 @@ unpack_string_preserving_sort(const char** p, const char* end,
 }
 
 inline std::string
-pack_chert_postlist_key(const std::string& term)
+pack_glass_postlist_key(std::string_view term)
 {
     // Special case for doclen lists.
     if (term.empty())
@@ -594,35 +564,7 @@ pack_chert_postlist_key(const std::string& term)
 }
 
 inline std::string
-pack_chert_postlist_key(const std::string& term, Xapian::docid did)
-{
-    // Special case for doclen lists.
-    if (term.empty()) {
-	std::string key("\x00\xe0", 2);
-	C_pack_uint_preserving_sort(key, did);
-	return key;
-    }
-
-    std::string key;
-    pack_string_preserving_sort(key, term);
-    C_pack_uint_preserving_sort(key, did);
-    return key;
-}
-
-inline std::string
-pack_glass_postlist_key(const std::string& term)
-{
-    // Special case for doclen lists.
-    if (term.empty())
-	return std::string("\x00\xe0", 2);
-
-    std::string key;
-    pack_string_preserving_sort(key, term, true);
-    return key;
-}
-
-inline std::string
-pack_glass_postlist_key(const std::string& term, Xapian::docid did)
+pack_glass_postlist_key(std::string_view term, Xapian::docid did)
 {
     // Special case for doclen lists.
     if (term.empty()) {
@@ -637,4 +579,23 @@ pack_glass_postlist_key(const std::string& term, Xapian::docid did)
     return key;
 }
 
+inline std::string
+pack_honey_postlist_key(std::string_view term)
+{
+    Assert(!term.empty());
+    std::string key;
+    pack_string_preserving_sort(key, term, true);
+    return key;
+}
+
+inline std::string
+pack_honey_postlist_key(std::string_view term, Xapian::docid did)
+{
+    Assert(!term.empty());
+    std::string key;
+    pack_string_preserving_sort(key, term);
+    pack_uint_preserving_sort(key, did);
+    return key;
+}
+
 #endif // XAPIAN_INCLUDED_PACK_H
