@@ -1,8 +1,8 @@
-/** @file posixy_wrapper.h
+/** @file
  * @brief Provides wrappers with POSIXy semantics.
  */
 /* Copyright 2007 Lemur Consulting Ltd
- * Copyright 2007,2012,2014,2018 Olly Betts
+ * Copyright 2007,2012,2014,2018,2025 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,9 @@
 #define XAPIAN_INCLUDED_POSIXY_WRAPPER_H
 
 #ifdef __WIN32__
+/** Call GetLastError() and set errno appropriately. */
+int posixy_set_errno_from_getlasterror();
+
 /** Version of open() which allows the file to be unlinked while open. */
 int posixy_open(const char *filename, int flags);
 
