@@ -1,7 +1,7 @@
 /** @file
  * @brief Convert types to pretty representations
  */
-/* Copyright (C) 2010,2011,2012,2014,2016,2023 Olly Betts
+/* Copyright (C) 2010,2011,2012,2014,2016,2017,2019,2023,2024,2026 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,11 @@
 #include <map>
 #include <ostream>
 #include <string>
+#include <string_view>
 #include <vector>
 
+#include "api/smallvector.h"
+
 #include "xapian/intrusive_ptr.h"
 #include "xapian/types.h"
 
@@ -139,8 +142,8 @@ inline PrettyOStream<S> &
 operator<<(PrettyOStream<S> &ps, const std::string & str)
 {
     ps.os << '"';
-    for (std::string::const_iterator i = str.begin(); i != str.end(); ++i) {
-	write_ch(ps.os, *i);
+    for (char ch : str) {
+	write_ch(ps.os, ch);
     }
     ps.os << '"';
     return ps;
@@ -162,6 +165,29 @@ operator<<(PrettyOStream<S> &ps, std::string *)
     return ps;
 }
 
+template<class S>
+inline PrettyOStream<S>&
+operator<<(PrettyOStream<S>& ps, std::string_view str)
+{
+    ps.os << '"';
+    for (char ch : str) {
+	write_ch(ps.os, ch);
+    }
+    ps.os << '"';
+    return ps;
+}
+
+template<class S>
+inline PrettyOStream<S>&
+operator<<(PrettyOStream<S>& ps, const std::string_view* p_str)
+{
+    if (p_str)
+	ps.os << *p_str;
+    else
+	ps.os << "NULL";
+    return ps;
+}
+
 template<class S>
 inline PrettyOStream<S> &
 operator<<(PrettyOStream<S> &ps, unsigned char ch)
@@ -259,6 +285,22 @@ operator<<(PrettyOStream<S> &ps, const std::vector<T> & v) {
     return ps;
 }
 
+template<class S, typename T>
+inline PrettyOStream<S> &
+operator<<(PrettyOStream<S> &ps, const Xapian::Vec<T>& v) {
+    ps.os << "Vec(" << v.size() << ')';
+    // FIXME: could show first up to N elements.
+    return ps;
+}
+
+template<class S, typename T>
+inline PrettyOStream<S> &
+operator<<(PrettyOStream<S> &ps, const Xapian::VecCOW<T>& v) {
+    ps.os << "VecCOW(" << v.size() << ')';
+    // FIXME: could show first up to N elements.
+    return ps;
+}
+
 template<class S, typename T, typename U>
 inline PrettyOStream<S> &
 operator<<(PrettyOStream<S> &ps, const std::pair<T, U>& v) {
@@ -267,16 +309,25 @@ operator<<(PrettyOStream<S> &ps, const std::pair<T, U>& v) {
 }
 
 namespace Xapian {
+    class Centroid;
+    class Cluster;
+    class ClusterSet;
     class ExpandDecider;
+    class FreqSource;
+    class KeyMaker;
     class LatLongMetric;
     class MatchDecider;
+    class Point;
+    class PointType;
     class Registry;
+    class TermListGroup;
     class Weight;
     namespace Internal {
 	class AndContext;
 	class ExpandStats;
 	class ExpandWeight;
 	class OrContext;
+	struct PostListAndEstimate;
     }
 }
 
@@ -284,14 +335,13 @@ namespace Glass {
     class RootInfo;
 }
 
-class ChertCursor;
-class ChertDatabase;
-class ChertTable;
 class GlassCursor;
 class GlassDatabase;
 class GlassFreeListChecker;
 class GlassTable;
 
+class HoneyTable;
+
 #define XAPIAN_PRETTY_AS_CLASSNAME(C)\
 template<class S>\
 inline PrettyOStream<S> &\
@@ -300,23 +350,30 @@ operator<<(PrettyOStream<S> &ps, const C &) {\
     return ps;\
 }
 
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Centroid)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Cluster)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::ClusterSet)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::ExpandDecider)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::FreqSource)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::KeyMaker)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::LatLongMetric)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::MatchDecider)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Point)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::PointType)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Registry)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::TermListGroup)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Weight)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Internal::AndContext)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Internal::ExpandStats)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Internal::ExpandWeight)
 XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Internal::OrContext)
-XAPIAN_PRETTY_AS_CLASSNAME(ChertCursor)
-XAPIAN_PRETTY_AS_CLASSNAME(ChertDatabase)
-XAPIAN_PRETTY_AS_CLASSNAME(ChertTable)
+XAPIAN_PRETTY_AS_CLASSNAME(Xapian::Internal::PostListAndEstimate)
 XAPIAN_PRETTY_AS_CLASSNAME(Glass::RootInfo)
 XAPIAN_PRETTY_AS_CLASSNAME(GlassCursor)
 XAPIAN_PRETTY_AS_CLASSNAME(GlassFreeListChecker)
 XAPIAN_PRETTY_AS_CLASSNAME(GlassDatabase)
 XAPIAN_PRETTY_AS_CLASSNAME(GlassTable)
+XAPIAN_PRETTY_AS_CLASSNAME(HoneyTable)
 
 template<class S>
 inline PrettyOStream<S> &
@@ -334,7 +391,7 @@ operator<<(PrettyOStream<S> &ps, const RemoteConnection &) {
     return ps;
 }
 
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
 
 template<class S>
 inline PrettyOStream<S> &
