@@ -1,7 +1,7 @@
 /** @file
  *  @brief Socket handling utilities.
  */
-/* Copyright (C) 2006,2007,2008,2015 Olly Betts
+/* Copyright (C) 2006,2007,2008,2015,2018,2023 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,12 @@
 
 using namespace std;
 
-#ifdef __WIN32__
+#include "stringutils.h"
+
+#ifndef __WIN32__
+# include <arpa/inet.h>
+# include <netinet/in.h>
+#else
 # include <io.h>
 # include "msvcignoreinvalidparam.h"
 # include <cerrno>
@@ -40,7 +45,12 @@ extern HANDLE fd_to_handle(int fd) {
     if (handle != INVALID_HANDLE_VALUE) return handle;
     // On WIN32, a socket fd isn't the same as a non-socket fd - in fact it's
     // already a HANDLE!
-    return reinterpret_cast<HANDLE>(fd);
+    //
+    // We need to convert to intptr_t first to suppress a compiler warning here
+    // about casting an integer to a wider pointer type which is a reasonable
+    // warning in general, but we check that the value isn't truncated before
+    // we cast the HANDLE to int (see common/safesyssocket.h).
+    return reinterpret_cast<HANDLE>(intptr_t(fd));
 }
 
 /// Close an fd, which might be a socket.
@@ -98,3 +108,61 @@ set_socket_timeouts(int fd, double timeout)
     }
 #endif
 }
+
+int
+pretty_ip6(const void* p, char* buf)
+{
+    const sockaddr* sa = reinterpret_cast<const sockaddr*>(p);
+    auto af = sa->sa_family;
+    int port;
+#ifndef __WIN32__
+    const void* src;
+#endif
+    if (af == AF_INET6) {
+	auto sa6 = reinterpret_cast<const sockaddr_in6*>(p);
+	port = sa6->sin6_port;
+#ifndef __WIN32__
+	src = &sa6->sin6_addr;
+#endif
+    } else if (af == AF_INET) {
+	auto sa4 = reinterpret_cast<const sockaddr_in*>(p);
+	port = sa4->sin_port;
+#ifndef __WIN32__
+	src = &sa4->sin_addr;
+#endif
+    } else {
+	return -1;
+    }
+
+#ifndef __WIN32__
+    const char* r = inet_ntop(af, src, buf, PRETTY_IP6_LEN);
+    if (!r)
+	return -1;
+#else
+    // inet_ntop() isn't always available (at least with mingw) but
+    // WSAAddressToString() supports both IPv4 and IPv6, so just use that.
+    //
+    // WSAAddressToString() has a non-const first parameter so we have to cast
+    // away const.
+    DWORD in_size = (af == AF_INET6 ?
+		     sizeof(struct sockaddr_in6) :
+		     sizeof(struct sockaddr_in));
+    DWORD size = PRETTY_IP6_LEN;
+    if (WSAAddressToString(const_cast<struct sockaddr*>(sa),
+			   in_size, NULL, buf, &size) != 0) {
+	return -1;
+    }
+    const char* r = buf;
+#endif
+
+    if (startswith(r, "::ffff:") || startswith(r, "::FFFF:")) {
+	if (strchr(r + 7, '.')) {
+	    r += 7;
+	}
+    }
+
+    if (r != buf)
+	memmove(buf, r, strlen(r) + 1);
+
+    return port;
+}
