@@ -1,7 +1,7 @@
 /** @file
- * @brief Various handy helpers which std::string really should provide.
+ * @brief Various handy string-related helpers
  */
-/* Copyright (C) 2004-2022 Olly Betts
+/* Copyright (C) 2004-2023 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,15 @@
 #ifndef XAPIAN_INCLUDED_STRINGUTILS_H
 #define XAPIAN_INCLUDED_STRINGUTILS_H
 
+// Hack to allow inclusion from xapian-omega.
+// FIXME: Move C_isalpha(), etc to the public API?
+#define XAPIAN_IN_XAPIAN_H
 #include <xapian/constinfo.h>
+#undef XAPIAN_IN_XAPIAN_H
 
 #include <algorithm>
 #include <string>
+#include <string_view>
 #include <cstring>
 
 /** Helper macro for STRINGIZE - the nested call is required because of how
@@ -42,56 +47,85 @@
  */
 #define CONST_STRLEN(S) (sizeof(S"") - 1)
 
+/* C++20 added starts_with(), ends_with() and contains() methods to std::string
+ * and std::string_view which provide this functionality, but we don't yet
+ * require C++20.
+ */
+
 inline bool
-startswith(const std::string & s, char pfx)
+startswith(std::string_view s, char pfx)
 {
     return !s.empty() && s[0] == pfx;
 }
 
 inline bool
-startswith(const std::string & s, const char * pfx, size_t len)
+startswith(std::string_view s, const char* pfx, size_t len)
 {
     return s.size() >= len && (std::memcmp(s.data(), pfx, len) == 0);
 }
 
 inline bool
-startswith(const std::string & s, const char * pfx)
+startswith(std::string_view s, const char* pfx)
 {
     return startswith(s, pfx, std::strlen(pfx));
 }
 
 inline bool
-startswith(const std::string & s, const std::string & pfx)
+startswith(std::string_view s, std::string_view pfx)
 {
     return startswith(s, pfx.data(), pfx.size());
 }
 
 inline bool
-endswith(const std::string & s, char sfx)
+endswith(std::string_view s, char sfx)
 {
     return !s.empty() && s[s.size() - 1] == sfx;
 }
 
 inline bool
-endswith(const std::string & s, const char * sfx, size_t len)
+endswith(std::string_view s, const char* sfx, size_t len)
 {
     return s.size() >= len && (std::memcmp(s.data() + s.size() - len, sfx, len) == 0);
 }
 
 inline bool
-endswith(const std::string & s, const char * sfx)
+endswith(std::string_view s, const char* sfx)
 {
     return endswith(s, sfx, std::strlen(sfx));
 }
 
 inline bool
-endswith(const std::string & s, const std::string & sfx)
+endswith(std::string_view s, std::string_view sfx)
 {
     return endswith(s, sfx.data(), sfx.size());
 }
 
+inline bool
+contains(std::string_view s, char substring)
+{
+    return s.find(substring) != s.npos;
+}
+
+inline bool
+contains(std::string_view s, const char* substring, size_t len)
+{
+    return s.find(substring, 0, len) != s.npos;
+}
+
+inline bool
+contains(std::string_view s, const char* substring)
+{
+    return s.find(substring) != s.npos;
+}
+
+inline bool
+contains(std::string_view s, std::string_view substring)
+{
+    return s.find(substring) != s.npos;
+}
+
 inline std::string::size_type
-common_prefix_length(const std::string &a, const std::string &b)
+common_prefix_length(std::string_view a, std::string_view b)
 {
     std::string::size_type minlen = std::min(a.size(), b.size());
     std::string::size_type common;
@@ -101,6 +135,20 @@ common_prefix_length(const std::string &a, const std::string &b)
     return common;
 }
 
+inline std::string::size_type
+common_prefix_length(std::string_view a, std::string_view b,
+		     std::string::size_type max_prefix_len)
+{
+    std::string::size_type minlen = std::min({a.size(),
+					      b.size(),
+					      max_prefix_len});
+    std::string::size_type common;
+    for (common = 0; common < minlen; ++common) {
+	if (a[common] != b[common]) break;
+    }
+    return common;
+}
+
 // Like C's isXXXXX() but:
 //  (a) always work in the C locale
 //  (b) handle signed char as well as unsigned char
