@@ -1,7 +1,7 @@
 /** @file
  * @brief Debug positional data
  */
-/* Copyright 2018 Olly Betts
+/* Copyright 2018-2022 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -26,12 +26,13 @@
 
 #include <iostream>
 
-#include <algorithm>
 #include <cerrno>
 #include <cstdlib>
 #include <limits>
 
 #include "gnu_getopt.h"
+#include "heap.h"
+#include "parseint.h"
 #include "stringutils.h"
 
 using namespace std;
@@ -47,11 +48,12 @@ show_usage()
 {
     cout << "Usage: " PROG_NAME " [OPTIONS] DATABASE\n\n"
 "Options:\n"
-"  -d, --doc=DOCID  Show positions for document DOCID\n"
-"  -s, --start=POS  Specifies the first position to show\n"
-"  -e, --end=POS    Specifies the last position to show\n"
-"  --help           display this help and exit\n"
-"  --version        output version information and exit" << endl;
+"  -d, --doc=DOCID             show positions for document DOCID\n"
+"  -s, --start=POS             specifies the first position to show\n"
+"  -e, --end=POS               specifies the last position to show\n"
+"  -r, --reconstruct[=PREFIX]  reconstruct text for prefix PREFIX\n"
+"  --help                      display this help and exit\n"
+"  --version                   output version information and exit\n";
 }
 
 class Pos {
@@ -87,27 +89,6 @@ struct PosCmp {
     }
 };
 
-template<typename T>
-bool to_unsigned_int(const char* s, T& result)
-{
-    errno = 0;
-    char* e;
-    auto v = strtoull(s, &e, 0);
-    if (errno == 0) {
-	if (*e || e == s) {
-	    // Junk after or empty input.
-	    errno = EINVAL;
-	} else if (v > numeric_limits<T>::max()) {
-	    // Exceeds the type.
-	    errno = ERANGE;
-	} else {
-	    result = T(v);
-	    return true;
-	}
-    }
-    return false;
-}
-
 int
 main(int argc, char **argv)
 try {
@@ -115,6 +96,7 @@ try {
 	{"doc",		required_argument, 0, 'd'},
 	{"start",	required_argument, 0, 's'},
 	{"end",		required_argument, 0, 'e'},
+	{"reconstruct",	optional_argument, 0, 'r'},
 	{"help",	no_argument, 0, OPT_HELP},
 	{"version",	no_argument, 0, OPT_VERSION},
 	{NULL,		0, 0, 0}
@@ -123,37 +105,41 @@ try {
     Xapian::docid did = 0;
     Xapian::termpos startpos = 0;
     Xapian::termpos endpos = numeric_limits<Xapian::termpos>::max();
+    bool reconstruct = false;
+    string reconstruct_prefix;
     int c;
-    while ((c = gnu_getopt_long(argc, argv, "d:e:s:", long_opts, 0)) != -1) {
+    while ((c = gnu_getopt_long(argc, argv, "d:e:s:r::", long_opts, 0)) != -1) {
 	switch (c) {
 	    case 'd':
-		if (!to_unsigned_int(optarg, did) || did == 0) {
-		    if (errno == 0) errno = ERANGE;
-		    cerr << "Bad docid value '" << optarg << "': "
-			 << strerror(errno) << endl;
+		if (!parse_unsigned(optarg, did) || did == 0) {
+		    cerr << "Bad docid value '" << optarg << "'\n";
 		    exit(1);
 		}
 		break;
 	    case 's':
-		if (!to_unsigned_int(optarg, startpos)) {
-		    cerr << "Bad start position '" << optarg << "': "
-			 << strerror(errno) << endl;
+		if (!parse_unsigned(optarg, startpos)) {
+		    cerr << "Bad start position '" << optarg << "'\n";
 		    exit(1);
 		}
 		break;
 	    case 'e':
-		if (!to_unsigned_int(optarg, endpos)) {
-		    cerr << "Bad end position '" << optarg << "': "
-			 << strerror(errno) << endl;
+		if (!parse_unsigned(optarg, endpos)) {
+		    cerr << "Bad end position '" << optarg << "'\n";
 		    exit(1);
 		}
 		break;
+	    case 'r':
+		reconstruct = true;
+		if (optarg) {
+		    reconstruct_prefix = optarg;
+		}
+		break;
 	    case OPT_HELP:
 		cout << PROG_NAME " - " PROG_DESC "\n\n";
 		show_usage();
 		exit(0);
 	    case OPT_VERSION:
-		cout << PROG_NAME " - " PACKAGE_STRING << endl;
+		cout << PROG_NAME " - " PACKAGE_STRING "\n";
 		exit(0);
 	    default:
 		show_usage();
@@ -168,14 +154,21 @@ try {
     }
 
     if (did == 0) {
-	cerr << "--doc=DOCID option required." << endl;
+	cerr << "--doc=DOCID option required.\n";
 	exit(1);
     }
 
-    vector<Pos*> heap;
-
     Xapian::Database db(argv[optind]);
 
+    if (reconstruct) {
+	cout << db.reconstruct_text(did, 0, reconstruct_prefix,
+				    startpos, endpos)
+	     << '\n';
+	exit(0);
+    }
+
+    vector<Pos*> heap;
+
     for (auto term_it = db.termlist_begin(did);
 	 term_it != db.termlist_end(did); ++term_it) {
 	const string& term = *term_it;
@@ -186,7 +179,7 @@ try {
 	}
     }
 
-    make_heap(heap.begin(), heap.end(), PosCmp());
+    Heap::make(heap.begin(), heap.end(), PosCmp());
 
     Xapian::termpos old_pos = startpos - 1;
     while (!heap.empty()) {
@@ -212,16 +205,15 @@ try {
 	old_pos = pos;
 
 	if (tip->next()) {
-	    pop_heap(heap.begin(), heap.end(), PosCmp());
-	    push_heap(heap.begin(), heap.end(), PosCmp());
+	    Heap::replace(heap.begin(), heap.end(), PosCmp());
 	} else {
-	    pop_heap(heap.begin(), heap.end(), PosCmp());
+	    Heap::pop(heap.begin(), heap.end(), PosCmp());
 	    heap.resize(heap.size() - 1);
 	}
     }
 
-    cout << endl;
+    cout << '\n';
 } catch (const Xapian::Error & e) {
-    cerr << '\n' << argv[0] << ": " << e.get_description() << endl;
+    cerr << '\n' << argv[0] << ": " << e.get_description() << '\n';
     exit(1);
 }
