@@ -1,7 +1,7 @@
 /** @file
  * @brief Merge two TermList objects using an OR operation.
  */
-/* Copyright (C) 2007,2010 Olly Betts
+/* Copyright (C) 2007,2010,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,13 +34,11 @@ class OrTermList : public TermList {
     /// The two TermList objects we're merging.
     TermList *left, *right;
 
-    /** The current term for left and right respectively.
+    /** The result of left->get_termname().compare(right->get_termname()).
      *
-     *  Until next() is first called, these will be empty strings.  Once next()
-     *  has been called, they won't be empty (since the empty string isn't a
-     *  valid term).
+     *  Until next() is first called, this will be zero.
      */
-    std::string left_current, right_current;
+    int cmp = 0;
 
     /// Check that next() has already been called.
     void check_started() const;
@@ -55,21 +53,17 @@ class OrTermList : public TermList {
 
     void accumulate_stats(Xapian::Internal::ExpandStats & stats) const;
 
-    std::string get_termname() const;
-
     Xapian::termcount get_wdf() const;
 
     Xapian::doccount get_termfreq() const;
 
     TermList *next();
 
-    TermList * skip_to(const std::string & term);
-
-    bool at_end() const;
+    TermList* skip_to(std::string_view term);
 
     Xapian::termcount positionlist_count() const;
 
-    Xapian::PositionIterator positionlist_begin() const;
+    PositionList* positionlist_begin() const;
 };
 
 /** A termlist which ORs two termlists together, adding term frequencies.
