@@ -54,7 +54,6 @@
 
 // Exceptions
 #include <xapian/error.h>
-#include <xapian/errorhandler.h>
 
 // Access to databases, documents, etc.
 #include <xapian/database.h>
@@ -74,13 +73,18 @@
 #include <xapian/mset.h>
 #include <xapian/expanddecider.h>
 #include <xapian/keymaker.h>
+#include <xapian/matchdecider.h>
 #include <xapian/matchspy.h>
 #include <xapian/postingsource.h>
 #include <xapian/query.h>
 #include <xapian/queryparser.h>
+#include <xapian/rset.h>
 #include <xapian/valuesetmatchdecider.h>
 #include <xapian/weight.h>
 
+// Clustering
+#include <xapian/cluster.h>
+
 // Stemming
 #include <xapian/stem.h>
 
