@@ -46,7 +46,8 @@ class XAPIAN_VISIBILITY_DEFAULT PositionIterator {
     /// @private @internal Reference counted internals.
     Internal * internal;
 
-    /// @private @internal Construct given internals.
+    /// @private @internal Wrap an existing Internal.
+    XAPIAN_VISIBILITY_INTERNAL
     explicit PositionIterator(Internal *internal_);
 
     /// Copy constructor.
@@ -55,7 +56,6 @@ class XAPIAN_VISIBILITY_DEFAULT PositionIterator {
     /// Assignment.
     PositionIterator & operator=(const PositionIterator & o);
 
-#ifdef XAPIAN_MOVE_SEMANTICS
     /// Move constructor.
     PositionIterator(PositionIterator && o)
 	: internal(o.internal) {
@@ -71,14 +71,13 @@ class XAPIAN_VISIBILITY_DEFAULT PositionIterator {
 	}
 	return *this;
     }
-#endif
 
     /** Default constructor.
      *
      *  Creates an uninitialised iterator, which can't be used before being
      *  assigned to, but is sometimes syntactically convenient.
      */
-    XAPIAN_NOTHROW(PositionIterator())
+    PositionIterator() noexcept
 	: internal() { }
 
     /// Destructor.
@@ -134,24 +133,18 @@ class XAPIAN_VISIBILITY_DEFAULT PositionIterator {
     // @}
 };
 
-bool
-XAPIAN_NOTHROW(operator==(const PositionIterator &a, const PositionIterator &b));
-
 /// Equality test for PositionIterator objects.
 inline bool
-operator==(const PositionIterator &a, const PositionIterator &b) XAPIAN_NOEXCEPT
+operator==(const PositionIterator& a, const PositionIterator& b) noexcept
 {
     // Use a pointer comparison - this ensures both that (a == a) and correct
     // handling of end iterators (which we ensure have NULL internals).
     return a.internal == b.internal;
 }
 
-bool
-XAPIAN_NOTHROW(operator!=(const PositionIterator &a, const PositionIterator &b));
-
 /// Inequality test for PositionIterator objects.
 inline bool
-operator!=(const PositionIterator &a, const PositionIterator &b) XAPIAN_NOEXCEPT
+operator!=(const PositionIterator& a, const PositionIterator& b) noexcept
 {
     return !(a == b);
 }
