@@ -1,7 +1,7 @@
 /** @file
  * @brief parsing a user query string to build a Xapian::Query object
  */
-/* Copyright (C) 2005-2023 Olly Betts
+/* Copyright (C) 2005-2026 Olly Betts
  * Copyright (C) 2010 Adam Sjøgren
  *
  * This program is free software; you can redistribute it and/or
@@ -28,14 +28,14 @@
 #endif
 
 #include <xapian/attributes.h>
-#include <xapian/deprecated.h>
 #include <xapian/intrusive_ptr.h>
 #include <xapian/query.h>
 #include <xapian/termiterator.h>
 #include <xapian/visibility.h>
 
-#include <set>
 #include <string>
+#include <string_view>
+#include <unordered_set>
 
 namespace Xapian {
 
@@ -50,10 +50,10 @@ class Stem;
 class XAPIAN_VISIBILITY_DEFAULT Stopper
     : public Xapian::Internal::opt_intrusive_base {
     /// Don't allow assignment.
-    void operator=(const Stopper &);
+    void operator=(const Stopper &) = delete;
 
     /// Don't allow copying.
-    Stopper(const Stopper &);
+    Stopper(const Stopper &) = delete;
 
   public:
     /// Default constructor.
@@ -98,7 +98,7 @@ class XAPIAN_VISIBILITY_DEFAULT Stopper
 
 /// Simple implementation of Stopper class - this will suit most users.
 class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
-    std::set<std::string> stop_words;
+    std::unordered_set<std::string> stop_words;
 
   public:
     /// Default constructor.
@@ -191,7 +191,7 @@ class XAPIAN_VISIBILITY_DEFAULT RangeProcessor
      *			   (e.g. `2..12kg` or `2..kg`).
      */
     explicit RangeProcessor(Xapian::valueno slot_,
-			    const std::string& str_ = std::string(),
+			    std::string_view str_ = {},
 			    unsigned flags_ = 0)
 	: slot(slot_), str(str_), flags(flags_) { }
 
@@ -272,7 +272,7 @@ class XAPIAN_VISIBILITY_DEFAULT DateRangeProcessor : public RangeProcessor {
     explicit DateRangeProcessor(Xapian::valueno slot_,
 				unsigned flags_ = 0,
 				int epoch_year_ = 1970)
-	: RangeProcessor(slot_, std::string(), flags_),
+	: RangeProcessor(slot_, {}, flags_),
 	  epoch_year(epoch_year_) { }
 
     /** Constructor.
@@ -310,7 +310,7 @@ class XAPIAN_VISIBILITY_DEFAULT DateRangeProcessor : public RangeProcessor {
      *  Xapian::RP_SUFFIX and Xapian::RP_REPEATED are irrelevant) - anything
      *  which looks like a date range will be processed.
      */
-    DateRangeProcessor(Xapian::valueno slot_, const std::string &str_,
+    DateRangeProcessor(Xapian::valueno slot_, std::string_view str_,
 		       unsigned flags_ = 0, int epoch_year_ = 1970)
 	: RangeProcessor(slot_, str_, flags_),
 	  epoch_year(epoch_year_) { }
@@ -400,7 +400,7 @@ class XAPIAN_VISIBILITY_DEFAULT NumberRangeProcessor : public RangeProcessor {
      *  valid ranges.
      */
     NumberRangeProcessor(Xapian::valueno slot_,
-			 const std::string &str_ = std::string(),
+			 std::string_view str_ = {},
 			 unsigned flags_ = 0)
 	: RangeProcessor(slot_, str_, flags_) { }
 
@@ -419,328 +419,46 @@ class XAPIAN_VISIBILITY_DEFAULT NumberRangeProcessor : public RangeProcessor {
     Xapian::Query operator()(const std::string& begin, const std::string& end);
 };
 
-/// Base class for value range processors.
-class XAPIAN_VISIBILITY_DEFAULT ValueRangeProcessor
-    : public Xapian::Internal::opt_intrusive_base {
-    /// Don't allow assignment.
-    void operator=(const ValueRangeProcessor &);
-
-    /// Don't allow copying.
-    ValueRangeProcessor(const ValueRangeProcessor &);
-
-  public:
-    /// Default constructor.
-    ValueRangeProcessor() { }
-
-    /// Destructor.
-    virtual ~ValueRangeProcessor();
-
-    /** Check for a valid range of this type.
-     *
-     *  @param[in,out] begin	The start of the range as specified in the query
-     *				string by the user.  This parameter is a
-     *				non-const reference so the ValueRangeProcessor
-     *				can modify it to return the value to start the
-     *				range with.
-     *  @param[in,out] end	The end of the range.  This is also a non-const
-     *				reference so it can be modified.
-     *
-     *  @return	If this ValueRangeProcessor recognises the range BEGIN..END it
-     *		returns the value slot number to range filter on.  Otherwise it
-     *		returns Xapian::BAD_VALUENO.
-     */
-    virtual Xapian::valueno operator()(std::string &begin, std::string &end) = 0;
-
-    /** Start reference counting this object.
-     *
-     *  You can hand ownership of a dynamically allocated ValueRangeProcessor
-     *  object to Xapian by calling release() and then passing the object to a
-     *  Xapian method.  Xapian will arrange to delete the object once it is no
-     *  longer required.
-     */
-    ValueRangeProcessor * release() {
-	opt_intrusive_base::release();
-	return this;
-    }
-
-    /** Start reference counting this object.
-     *
-     *  You can hand ownership of a dynamically allocated ValueRangeProcessor
-     *  object to Xapian by calling release() and then passing the object to a
-     *  Xapian method.  Xapian will arrange to delete the object once it is no
-     *  longer required.
-     */
-    const ValueRangeProcessor * release() const {
-	opt_intrusive_base::release();
-	return this;
-    }
-};
-
-/** Handle a string range.
- *
- *  The end points can be any strings.
- *
- *  @deprecated Use Xapian::RangeProcessor instead (added in 1.3.6).
- */
-class XAPIAN_DEPRECATED_CLASS_EX XAPIAN_VISIBILITY_DEFAULT StringValueRangeProcessor : public ValueRangeProcessor {
-  protected:
-    /** The value slot to process. */
-    Xapian::valueno valno;
-
-    /** Whether to look for @a str as a prefix or suffix. */
-    bool prefix;
-
-    /** The prefix (or suffix if prefix==false) string to look for. */
-    std::string str;
-
-  public:
-    /** Constructor.
-     *
-     *  @param slot_	The value number to return from operator().
-     */
-    explicit StringValueRangeProcessor(Xapian::valueno slot_)
-	: valno(slot_), str() { }
-
-    /** Constructor.
-     *
-     *  @param slot_	The value number to return from operator().
-     *  @param str_     A string to look for to recognise values as belonging
-     *                  to this range.
-     *  @param prefix_	Flag specifying whether to check for str_ as a prefix
-     *			or a suffix.
-     */
-    StringValueRangeProcessor(Xapian::valueno slot_, const std::string &str_,
-			      bool prefix_ = true)
-	: valno(slot_), prefix(prefix_), str(str_) { }
-
-    /** Check for a valid string range.
-     *
-     *  @param[in,out] begin	The start of the range as specified in the
-     *				query string by the user.  This parameter is a
-     *				non-const reference so the ValueRangeProcessor
-     *				can modify it to return the value to start the
-     *				range with.
-     *  @param[in,out] end	The end of the range.  This is also a non-const
-     *				reference so it can be modified.
-     *
-     *  @return	A StringValueRangeProcessor always accepts a range it is
-     *		offered, and returns the value of slot_ passed at construction
-     *		time.  It doesn't modify @a begin or @a end.
-     */
-    Xapian::valueno operator()(std::string &begin, std::string &end);
-};
-
-/** Handle a date range.
- *
- *  Begin and end must be dates in a recognised format.
- *
- *  @deprecated Use Xapian::DateRangeProcessor instead (added in 1.3.6).
- */
-class XAPIAN_DEPRECATED_CLASS_EX XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public StringValueRangeProcessor {
-    bool prefer_mdy;
-    int epoch_year;
-
-  public:
-    /** Constructor.
-     *
-     *  @param slot_	    The value number to return from operator().
-     *  @param prefer_mdy_  Should ambiguous dates be interpreted as
-     *			    month/day/year rather than day/month/year?
-     *			    (default: false)
-     *  @param epoch_year_  Year to use as the epoch for dates with 2 digit
-     *			    years (default: 1970, so 1/1/69 is 2069 while
-     *			    1/1/70 is 1970).
-     */
-    DateValueRangeProcessor(Xapian::valueno slot_, bool prefer_mdy_ = false,
-			    int epoch_year_ = 1970)
-	: StringValueRangeProcessor(slot_),
-	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
-
-    /** Constructor.
-     *
-     *  @param slot_	    The value number to return from operator().
-     *
-     *  @param str_     A string to look for to recognise values as belonging
-     *                  to this date range.
-     *
-     *  @param prefix_  Whether to look for the string at the start or end of
-     *                  the values.  If true, the string is a prefix; if
-     *                  false, the string is a suffix (default: true).
-     *
-     *  @param prefer_mdy_  Should ambiguous dates be interpreted as
-     *			    month/day/year rather than day/month/year?
-     *			    (default: false)
-     *
-     *  @param epoch_year_  Year to use as the epoch for dates with 2 digit
-     *			    years (default: 1970, so 1/1/69 is 2069 while
-     *			    1/1/70 is 1970).
-     *
-     *  The string supplied in str_ is used by @a operator() to decide whether
-     *  the pair of strings supplied to it constitute a valid range.  If
-     *  prefix_ is true, the first value in a range must begin with str_ (and
-     *  the second value may optionally begin with str_);
-     *  if prefix_ is false, the second value in a range must end with str_
-     *  (and the first value may optionally end with str_).
-     *
-     *  If str_ is empty, the setting of prefix_ is irrelevant, and no special
-     *  strings are required at the start or end of the strings defining the
-     *  range.
-     *
-     *  The remainder of both strings defining the endpoints must be valid
-     *  dates.
-     *
-     *  For example, if str_ is "created:" and prefix_ is true, and the range
-     *  processor has been added to the queryparser, the queryparser will
-     *  accept "created:1/1/2000..31/12/2001".
-     */
-    DateValueRangeProcessor(Xapian::valueno slot_, const std::string &str_,
-			    bool prefix_ = true,
-			    bool prefer_mdy_ = false, int epoch_year_ = 1970)
-	: StringValueRangeProcessor(slot_, str_, prefix_),
-	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
-
-#ifndef SWIG
-    /** Constructor.
-     *
-     *  This is like the previous version, but with const char * instead of
-     *  std::string - we need this overload as otherwise
-     *  DateValueRangeProcessor(1, "date:") quietly interprets the second
-     *  argument as a boolean in preference to std::string.  If you want to
-     *  be compatible with 1.2.12 and earlier, then explicitly convert to
-     *  std::string, i.e.: DateValueRangeProcessor(1, std::string("date:"))
-     *
-     *  @param slot_	    The value number to return from operator().
-     *
-     *  @param str_     A string to look for to recognise values as belonging
-     *                  to this date range.
-     *
-     *  @param prefix_  Whether to look for the string at the start or end of
-     *                  the values.  If true, the string is a prefix; if
-     *                  false, the string is a suffix (default: true).
-     *
-     *  @param prefer_mdy_  Should ambiguous dates be interpreted as
-     *			    month/day/year rather than day/month/year?
-     *			    (default: false)
-     *
-     *  @param epoch_year_  Year to use as the epoch for dates with 2 digit
-     *			    years (default: 1970, so 1/1/69 is 2069 while
-     *			    1/1/70 is 1970).
-     *
-     *  The string supplied in str_ is used by @a operator() to decide whether
-     *  the pair of strings supplied to it constitute a valid range.  If
-     *  prefix_ is true, the first value in a range must begin with str_ (and
-     *  the second value may optionally begin with str_);
-     *  if prefix_ is false, the second value in a range must end with str_
-     *  (and the first value may optionally end with str_).
-     *
-     *  If str_ is empty, the setting of prefix_ is irrelevant, and no special
-     *  strings are required at the start or end of the strings defining the
-     *  range.
-     *
-     *  The remainder of both strings defining the endpoints must be valid
-     *  dates.
-     *
-     *  For example, if str_ is "created:" and prefix_ is true, and the range
-     *  processor has been added to the queryparser, the queryparser will
-     *  accept "created:1/1/2000..31/12/2001".
-     */
-    DateValueRangeProcessor(Xapian::valueno slot_, const char * str_,
-			    bool prefix_ = true,
-			    bool prefer_mdy_ = false, int epoch_year_ = 1970)
-	: StringValueRangeProcessor(slot_, str_, prefix_),
-	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
-#endif
-
-    /** Check for a valid date range.
-     *
-     *  @param[in,out] begin	The start of the range as specified in the
-     *				query string by the user.  This parameter is a
-     *				non-const reference so the ValueRangeProcessor
-     *				can modify it to return the value to start the
-     *				range with.
-     *  @param[in,out] end	The end of the range.  This is also a non-const
-     *				reference so it can be modified.
-     *
-     *  @return	If BEGIN..END is a sensible date range, this method modifies
-     *		them into the format YYYYMMDD and returns the value of slot_
-     *		passed at construction time.  Otherwise it returns
-     *		Xapian::BAD_VALUENO.
-     */
-    Xapian::valueno operator()(std::string &begin, std::string &end);
-};
-
-/** Handle a number range.
+/** Handle a byte unit range.
  *
  *  This class must be used on values which have been encoded using
  *  Xapian::sortable_serialise() which turns numbers into strings which
  *  will sort in the same order as the numbers (the same values can be
  *  used to implement a numeric sort).
- *
- *  @deprecated Use Xapian::NumberRangeProcessor instead (added in 1.3.6).
  */
-class XAPIAN_DEPRECATED_CLASS_EX XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public StringValueRangeProcessor {
+class XAPIAN_VISIBILITY_DEFAULT UnitRangeProcessor : public RangeProcessor {
   public:
     /** Constructor.
      *
-     *  @param slot_   The value number to return from operator().
-     */
-    explicit NumberValueRangeProcessor(Xapian::valueno slot_)
-	: StringValueRangeProcessor(slot_) { }
-
-    /** Constructor.
-     *
-     *  @param slot_    The value number to return from operator().
+     *  @param slot_    The value slot number to query.
      *
      *  @param str_     A string to look for to recognise values as belonging
      *                  to this numeric range.
      *
-     *  @param prefix_  Whether to look for the string at the start or end of
-     *                  the values.  If true, the string is a prefix; if
-     *                  false, the string is a suffix (default: true).
+     *  The string supplied in str_ is the prefix plus ":" that defines the field
+     *  upon which the range query is performed.
      *
-     *  The string supplied in str_ is used by @a operator() to decide whether
-     *  the pair of strings supplied to it constitute a valid range.  If
-     *  prefix_ is true, the first value in a range must begin with str_ (and
-     *  the second value may optionally begin with str_);
-     *  if prefix_ is false, the second value in a range must end with str_
-     *  (and the first value may optionally end with str_).
-     *
-     *  If str_ is empty, the setting of prefix_ is irrelevant, and no special
-     *  strings are required at the start or end of the strings defining the
-     *  range.
-     *
-     *  The remainder of both strings defining the endpoints must be valid
-     *  floating point numbers. (FIXME: define format recognised).
-     *
-     *  For example, if str_ is "$" and prefix_ is true, and the range
+     *  For example, if str_ is "size:", and the range
      *  processor has been added to the queryparser, the queryparser will
-     *  accept "$10..50" or "$10..$50", but not "10..50" or "10..$50" as valid
-     *  ranges.  If str_ is "kg" and prefix_ is false, the queryparser will
-     *  accept "10..50kg" or "10kg..50kg", but not "10..50" or "10kg..50" as
-     *  valid ranges.
+     *  accept "size:3K..10K" as a valid range.
      */
-    NumberValueRangeProcessor(Xapian::valueno slot_, const std::string &str_,
-			      bool prefix_ = true)
-	: StringValueRangeProcessor(slot_, str_, prefix_) { }
+    UnitRangeProcessor(Xapian::valueno slot_,
+		       std::string_view str_ = {})
+	: RangeProcessor(slot_, str_) { }
 
-    /** Check for a valid numeric range.
+    /** Check for a valid byte value range.
      *
-     *  @param[in,out] begin	The start of the range as specified in the
-     *				query string by the user.  This parameter is a
-     *				non-const reference so the ValueRangeProcessor
-     *				can modify it to return the value to start the
-     *				range with.
-     *  @param[in,out] end	The end of the range.  This is also a non-const
-     *				reference so it can be modified.
+     *  If BEGIN..END is a valid numeric byte range with the specified prefix
+     *  the prefix is removed, the string converted to a number, unit
+     *  normalized and encoded with Xapian::sortable_serialise(),
+     *  and a value range query is built.
      *
-     *  @return	If BEGIN..END is a valid numeric range with the specified
-     *		prefix/suffix (if one was specified), this method modifies
-     *		them by removing the prefix/suffix, converting to a number,
-     *		and encoding with Xapian::sortable_serialise(), and returns the
-     *		value of slot_ passed at construction time.  Otherwise it
-     *		returns Xapian::BAD_VALUENO.
+     *  @param begin	The start of the range as specified in the query string
+     *			by the user.
+     *  @param end	The end of the range as specified in the query string
+     *			by the user.
      */
-    Xapian::valueno operator()(std::string &begin, std::string &end);
+    Xapian::Query operator()(const std::string& begin, const std::string& end);
 };
 
 /** Base class for field processors.
@@ -799,7 +517,7 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     /// Class representing the queryparser internals.
     class Internal;
     /// @private @internal Reference counted internals.
-    Xapian::Internal::intrusive_ptr<Internal> internal;
+    Xapian::Internal::intrusive_ptr_nonnull<Internal> internal;
 
     /// Enum of feature flags.
     typedef enum {
@@ -813,7 +531,9 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	FLAG_BOOLEAN_ANY_CASE = 8,
 	/** Support wildcards.
 	 *
-	 *  At present only right truncation (e.g. Xap*) is supported.
+	 *  This flag only enables support for right truncation (e.g. Xap*) -
+	 *  see FLAG_WILDCARD_MULTI, FLAG_WILDCARD_SINGLE and FLAG_WILDCARD_GLOB
+	 *  for extended wildcard support.
 	 *
 	 *  Currently you can't use wildcards with boolean filter prefixes,
 	 *  or in a phrase (either an explicitly quoted one, or one implicitly
@@ -922,6 +642,73 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	 */
 	FLAG_CJK_NGRAM = FLAG_NGRAMS,
 
+	/** Find word breaks for text in scripts without explicit word breaks.
+	 *
+	 *  With this option enabled, spans of text written in such scripts are
+	 *  split into words using ICU (which uses heuristics and/or
+	 *  dictionaries to do so).  Text in other scripts is split into words
+	 *  as normal.
+	 *
+	 *  The TermGenerator::FLAG_WORD_BREAKS flag needs to have been used at
+	 *  index time.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	FLAG_WORD_BREAKS = 4096,
+
+	/** Support extended wildcard '*'.
+	 *
+	 *  This flag enables support for wildcard '*' matching zero
+	 *  or more characters, which may be used anywhere in a word.
+	 *
+	 *  Such wildcards can be relatively expensive to expand and to match
+	 *  so benchmark your system carefully if you have a lot of documents
+	 *  and/or a high search load.
+	 *
+	 *  FLAG_WILDCARD is ignored if this flag is specified.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	FLAG_WILDCARD_MULTI = 8192,
+
+	/** Support extended wildcard '?'.
+	 *
+	 *  This flag enables support for wildcard '?' matching exactly one
+	 *  character, which may be use anywhere in a word.
+	 *
+	 *  Such wildcards can be relatively expensive to expand and to match
+	 *  so benchmark your system carefully if you have a lot of documents
+	 *  and/or a high search load.
+	 *
+	 *  FLAG_WILDCARD is ignored if this flag is specified.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	FLAG_WILDCARD_SINGLE = 16384,
+
+	/** Enable glob-style wildcarding.
+	 *
+	 *  This enables all supported glob-style wildcard pattern flags
+	 *  - currently that's FLAG_WILDCARD_MULTI and FLAG_WILDCARD_SINGLE.
+	 *
+	 *  FLAG_WILDCARD is ignored if this flag is specified.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	FLAG_WILDCARD_GLOB = FLAG_WILDCARD_MULTI | FLAG_WILDCARD_SINGLE,
+
+	/** Support fuzzy matching.
+	 *
+	 *  E.g. "unserten~3" would expand to "uncertain" (and likely other
+	 *  terms).
+	 *
+	 *  foo~ uses edit distance of 2
+	 *  since~0.2 uses edit distance of length("since") * 0.2 = 5 * 0.2 = 1
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	FLAG_FUZZY = 32768,
+
 	/** Accumulate unstem and stoplist results.
 	 *
 	 *  By default, the unstem and stoplist data is reset by a call to
@@ -948,12 +735,50 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	 */
 	FLAG_NO_POSITIONS = 0x20000,
 
+	/** Turn off special handling of capitalised words.
+	 *
+	 *  By default capitalising a word prevents it from being considered
+	 *  for stemming.  This only has any effect when both stemmed and
+	 *  non-stemmed terms are available (i.e. when stem_strategy is the
+	 *  default STEM_SOME or STEM_SOME_FULL_POS), and since Xapian 2.0.0 it
+	 *  is only done for languages where it is helpful.
+	 *
+	 *  This attempts to limit problems which stemming can cause with
+	 *  proper names.  For example, the English stemmer conflates `Tony`
+	 *  with `Toni` and `Keats` with `Keating`, which can lead to unwanted
+	 *  matches with irrelevant documents.
+	 *
+	 *  One downside is it prevents stemming of words that aren't proper
+	 *  nouns but are capitalised for other reasons (e.g. in a title or
+	 *  at the start of a sentence).
+	 *
+	 *  Another is that proper nouns can be inflected in some languages.
+	 *  In English some can be pluralised ("How many Tonys do you know?"),
+	 *  and in some languages proper nouns can take an -s suffix to
+	 *  indicate the genitive case (e.g. "Köpenhamns" in Swedish is
+	 *  like "Copenhagen's" in English).
+	 *
+	 *  If inflection is limited to plurals and genitive-s, the benefits
+	 *  of this special handling still seem to outweigh the downsides.
+	 *  However for languages with more extensive inflection of proper
+	 *  nouns it seems too problematic so (since Xapian 2.0.0) we don't
+	 *  enable it for such languages.  It's also off for German where all
+	 *  nouns are capitalised (not just proper nouns).
+	 *
+	 *  This special handling is also only useful in languages where proper
+	 *  nouns are capitalised (so for example, it's not useful for
+	 *  languages written in alphabets without upper case).
+	 *
+	 *  @since FLAG_NO_PROPER_NOUN_HEURISTIC was added in Xapian 2.0.0.
+	 */
+	FLAG_NO_PROPER_NOUN_HEURISTIC = 0x40000,
+
 	/** The default flags.
 	 *
 	 *  Used if you don't explicitly pass any to @a parse_query().
 	 *  The default flags are FLAG_PHRASE|FLAG_BOOLEAN|FLAG_LOVEHATE.
 	 *
-	 *  Added in Xapian 1.0.11.
+	 *  @since Added in Xapian 1.0.11.
 	 */
 	FLAG_DEFAULT = FLAG_PHRASE|FLAG_BOOLEAN|FLAG_LOVEHATE
     } feature_flag;
@@ -963,19 +788,20 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	STEM_NONE, STEM_SOME, STEM_ALL, STEM_ALL_Z, STEM_SOME_FULL_POS
     } stem_strategy;
 
+    /// Stopper strategies, for use with set_stopper_strategy().
+    typedef enum { STOP_NONE, STOP_ALL, STOP_STEMMED } stop_strategy;
+
     /// Copy constructor.
     QueryParser(const QueryParser & o);
 
     /// Assignment.
     QueryParser & operator=(const QueryParser & o);
 
-#ifdef XAPIAN_MOVE_SEMANTICS
     /// Move constructor.
     QueryParser(QueryParser && o);
 
     /// Move assignment operator.
     QueryParser & operator=(QueryParser && o);
-#endif
 
     /// Default constructor.
     QueryParser();
@@ -1029,6 +855,31 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      */
     void set_stopper(const Stopper *stop = NULL);
 
+    /** Set the stopper strategy.
+     *
+     *  This method controls how the stopper is used.
+     *
+     *  You need to also call @a set_stopper() for this to have any effect.
+     *
+     *  @param strategy The strategy to use - possible values are:
+     *   - STOP_NONE:     Don't use the stopper.
+     *   - STOP_ALL:      If a word is identified as a stop word, skip it
+     *                    completely.  This makes some queries less useful
+     *                    (e.g. `"to be or not to be that is the question"`
+     *                    would become a search for just `question` if the
+     *                    other words were all stopwords).  If you index
+     *                    with `STOP_ALL` you should use it when parsing
+     *                    queries too.
+     *   - STOP_STEMMED:  If a word is identified as a stop word, assume it
+     *                    was still indexed unstemmed and don't treat it as
+     *                    a stopword in contexts where we would use the
+     *                    unstemmed form (for example, phrase searches, ADJ,
+     *                    NEAR).  (This is the default mode).
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    void set_stopper_strategy(stop_strategy strategy);
+
     /** Set the default operator.
      *
      *  @param default_op	The operator to use to combine non-filter
@@ -1056,15 +907,16 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      */
     void set_database(const Database &db);
 
-    /** Specify the maximum expansion of a wildcard and/or partial term.
+    /** Specify the maximum expansion of a wildcard and/or partial and/or
+     *  fuzzy term.
      *
-     *  Note: you must also set FLAG_WILDCARD and/or FLAG_PARTIAL in the flags
-     *  parameter to @a parse_query() for this setting to have anything to
-     *  affect.
+     *  Note: you must also set FLAG_WILDCARD and/or FLAG_PARTIAL and/or
+     *  FLAG_FUZZY in the flags parameter to @a parse_query() for this setting
+     *  to have anything to affect.
      *
      *  If you don't call this method, the default settings are no limit on
-     *  wildcard expansion, and partial terms expanding to the most frequent
-     *  100 terms - i.e. as if you'd called:
+     *  wildcard and fuzzy expansion, and partial terms expanding to the most
+     *  frequent 100 terms - i.e. as if you'd called:
      *
      *  set_max_expansion(0);
      *  set_max_expansion(100, Xapian::Query::WILDCARD_LIMIT_MOST_FREQUENT, Xapian::QueryParser::FLAG_PARTIAL);
@@ -1077,28 +929,42 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *			@a Xapian::Query::WILDCARD_LIMIT_MOST_FREQUENT
      *			(default: Xapian::Query::WILDCARD_LIMIT_ERROR).
      *  @param flags	What to set the limit for (default:
-     *			FLAG_WILDCARD|FLAG_PARTIAL, setting the limit for both
-     *			wildcards and partial terms).
+     *			FLAG_WILDCARD|FLAG_PARTIAL|FLAG_FUZZY, setting the
+     *			limit for all types).
      *
      *  @since 1.3.3
      */
     void set_max_expansion(Xapian::termcount max_expansion,
 			   int max_type = Xapian::Query::WILDCARD_LIMIT_ERROR,
-			   unsigned flags = FLAG_WILDCARD|FLAG_PARTIAL);
+			   unsigned flags = FLAG_WILDCARD |
+					    FLAG_PARTIAL |
+					    FLAG_FUZZY);
 
-    /** Specify the maximum expansion of a wildcard.
+    /** Specify minimum length for fixed initial portion in wildcard patterns.
      *
-     *  If any wildcard expands to more than @a max_expansion terms, an
-     *  exception will be thrown.
+     *  It can be desirable for performance reasons to restrict use of
+     *  wildcards to patterns with a fixed initial portion, so this method
+     *  provides a way to specify a minimum length.  A wildcard pattern
+     *  with a shorter (or no) fixed initial portion will result in
+     *  Xapian::QueryParser being thrown.  The default minimum length is 0.
      *
-     *  This method is provided for API compatibility with Xapian 1.2.x and is
-     *  deprecated - replace it with:
+     *  It also provides a way to specify the minimum length of a word to
+     *  expand for partial matching (see @a FLAG_PARTIAL).  In this case
+     *  a shorter word at the end of the query simply result in no partial
+     *  matching.  The default minimum length for this case is 2 (since
+     *  2.0.0 - in earlier versions it was effectively 0).
      *
-     *  set_max_wildcard_expansion(max_expansion,
-     *				   Xapian::Query::WILDCARD_LIMIT_ERROR,
-     *				   Xapian::QueryParser::FLAG_WILDCARD);
+     *  @param min_prefix_len	Minimum length of fixed initial portion in
+     *			        Unicode characters.
+     *  @param flags		What to set the minimum length for
+     *				(default: FLAG_WILDCARD|FLAG_PARTIAL, setting
+     *				the limit for both wildcards and partial
+     *				terms).
+     *
+     *  @since Added in Xapian 2.0.0.
      */
-    XAPIAN_DEPRECATED(void set_max_wildcard_expansion(Xapian::termcount));
+    void set_min_wildcard_prefix(unsigned min_prefix_len,
+				 unsigned flags = FLAG_WILDCARD|FLAG_PARTIAL);
 
     /** Parse a query.
      *
@@ -1123,9 +989,9 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *		   @li Syntax: &lt;expression&gt; OR &lt;expression&gt;
      *		   @li Syntax: &lt;expression&gt; XOR &lt;expression&gt;
      */
-    Query parse_query(const std::string &query_string,
+    Query parse_query(std::string_view query_string,
 		      unsigned flags = FLAG_DEFAULT,
-		      const std::string &default_prefix = std::string());
+		      std::string_view default_prefix = {});
 
     /** Add a free-text field term prefix.
      *
@@ -1171,11 +1037,11 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *		       consistency with how range prefixes are specified.
      *  @param prefix  The term prefix to map this to.
      */
-    void add_prefix(const std::string& field, const std::string& prefix);
+    void add_prefix(std::string_view field, std::string_view prefix);
 
     /** Register a FieldProcessor.
      */
-    void add_prefix(const std::string& field, Xapian::FieldProcessor * proc);
+    void add_prefix(std::string_view field, Xapian::FieldProcessor* proc);
 
     /** Add a boolean term prefix allowing the user to restrict a
      *  search with a boolean filter specified in the free text query.
@@ -1236,7 +1102,7 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *			document can have multiple terms with this prefix).
      *			[default: NULL]
      */
-    void add_boolean_prefix(const std::string &field, const std::string &prefix,
+    void add_boolean_prefix(std::string_view field, std::string_view prefix,
 			    const std::string* grouping = NULL);
 
     /** Add a boolean term prefix allowing the user to restrict a
@@ -1261,7 +1127,7 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *			each filter (this is sometimes useful when each
      *			document can have multiple terms with this prefix).
      */
-    void add_boolean_prefix(const std::string &field, const std::string &prefix,
+    void add_boolean_prefix(std::string_view field, std::string_view prefix,
 			    bool exclusive) {
 	if (exclusive) {
 	    add_boolean_prefix(field, prefix);
@@ -1273,7 +1139,8 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 
     /** Register a FieldProcessor for a boolean prefix.
      */
-    void add_boolean_prefix(const std::string &field, Xapian::FieldProcessor *proc,
+    void add_boolean_prefix(std::string_view field,
+			    Xapian::FieldProcessor* proc,
 			    const std::string* grouping = NULL);
 
     /** Register a FieldProcessor for a boolean prefix.
@@ -1281,7 +1148,8 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *  This is an older version of this method - use the version with
      *  the `grouping` parameter in preference to this one.
      */
-    void add_boolean_prefix(const std::string &field, Xapian::FieldProcessor *proc,
+    void add_boolean_prefix(std::string_view field,
+			    Xapian::FieldProcessor* proc,
 			    bool exclusive) {
 	if (exclusive) {
 	    add_boolean_prefix(field, proc);
@@ -1295,15 +1163,15 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     TermIterator stoplist_begin() const;
 
     /// End iterator over terms omitted from the query as stopwords.
-    TermIterator XAPIAN_NOTHROW(stoplist_end() const) {
+    TermIterator stoplist_end() const noexcept {
 	return TermIterator();
     }
 
     /// Begin iterator over unstemmed forms of the given stemmed query term.
-    TermIterator unstem_begin(const std::string &term) const;
+    TermIterator unstem_begin(std::string_view term) const;
 
     /// End iterator over unstemmed forms of the given stemmed query term.
-    TermIterator XAPIAN_NOTHROW(unstem_end(const std::string &) const) {
+    TermIterator unstem_end(std::string_view) const noexcept {
 	return TermIterator();
     }
 
@@ -1311,42 +1179,6 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     void add_rangeprocessor(Xapian::RangeProcessor * range_proc,
 			    const std::string* grouping = NULL);
 
-    /** Register a ValueRangeProcessor.
-     *
-     *  This method is provided for API compatibility with Xapian 1.2.x and is
-     *  deprecated - use @a add_rangeprocessor() with a RangeProcessor instead.
-     */
-    XAPIAN_DEPRECATED(void add_valuerangeprocessor(Xapian::ValueRangeProcessor * vrproc)) {
-#ifdef __GNUC__
-// Avoid deprecation warnings if compiling without optimisation.
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
-	/// Compatibility shim.
-	class ShimRangeProcessor : public RangeProcessor {
-	    Xapian::Internal::opt_intrusive_ptr<Xapian::ValueRangeProcessor> vrp;
-
-	  public:
-	    ShimRangeProcessor(Xapian::ValueRangeProcessor * vrp_)
-		: RangeProcessor(Xapian::BAD_VALUENO), vrp(vrp_) { }
-
-	    Xapian::Query
-	    operator()(const std::string &begin, const std::string &end)
-	    {
-		std::string b = begin, e = end;
-		slot = (*vrp)(b, e);
-		if (slot == Xapian::BAD_VALUENO)
-		    return Xapian::Query(Xapian::Query::OP_INVALID);
-		return RangeProcessor::operator()(b, e);
-	    }
-	};
-
-	add_rangeprocessor((new ShimRangeProcessor(vrproc))->release());
-#ifdef __GNUC__
-# pragma GCC diagnostic pop
-#endif
-    }
-
     /** Get the spelling-corrected query string.
      *
      *  This will only be set if FLAG_SPELLING_CORRECTION is specified when
@@ -1360,17 +1192,9 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
     std::string get_description() const;
 };
 
-inline void
-QueryParser::set_max_wildcard_expansion(Xapian::termcount max_expansion)
-{
-    set_max_expansion(max_expansion,
-		      Xapian::Query::WILDCARD_LIMIT_ERROR,
-		      FLAG_WILDCARD);
-}
-
 /// @private @internal Helper for sortable_serialise().
 XAPIAN_VISIBILITY_DEFAULT
-size_t XAPIAN_NOTHROW(sortable_serialise_(double value, char * buf));
+size_t sortable_serialise_(double value, char* buf) noexcept;
 
 /** Convert a floating point number to a string, preserving sort order.
  *
@@ -1418,7 +1242,7 @@ inline std::string sortable_serialise(double value) {
  *  @param serialised	The serialised string to decode.
  */
 XAPIAN_VISIBILITY_DEFAULT
-double XAPIAN_NOTHROW(sortable_unserialise(const std::string & serialised));
+double sortable_unserialise(std::string_view serialised) noexcept;
 
 }
 
