@@ -1,7 +1,7 @@
 /** @file
  * @brief parse free text and generate terms
  */
-/* Copyright (C) 2007,2009,2011,2012,2013,2014,2018,2023 Olly Betts
+/* Copyright (C) 2007,2009,2011,2012,2013,2014,2018,2023,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <xapian/visibility.h>
 
 #include <string>
+#include <string_view>
 
 namespace Xapian {
 
@@ -50,7 +51,7 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
     /// @private @internal Class representing the TermGenerator internals.
     class Internal;
     /// @private @internal Reference counted internals.
-    Xapian::Internal::intrusive_ptr<Internal> internal;
+    Xapian::Internal::intrusive_ptr_nonnull<Internal> internal;
 
     /// Copy constructor.
     TermGenerator(const TermGenerator & o);
@@ -58,13 +59,11 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
     /// Assignment.
     TermGenerator & operator=(const TermGenerator & o);
 
-#ifdef XAPIAN_MOVE_SEMANTICS
     /// Move constructor.
     TermGenerator(TermGenerator && o);
 
     /// Move assignment operator.
     TermGenerator & operator=(TermGenerator && o);
-#endif
 
     /// Default constructor.
     TermGenerator();
@@ -133,7 +132,21 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
 	 *
 	 *  @since Added in Xapian 1.3.4 and 1.2.22.
 	 */
-	FLAG_CJK_NGRAM = FLAG_NGRAMS // Value matches QueryParser flag.
+	FLAG_CJK_NGRAM = FLAG_NGRAMS, // Value matches QueryParser flag.
+
+	/** Find word breaks for text in scripts without explicit word breaks.
+	 *
+	 *  With this option enabled, spans of text written in such scripts are
+	 *  split into words using ICU (which uses heuristics and/or
+	 *  dictionaries to do so).  Text in other scripts is split into words
+	 *  as normal.
+	 *
+	 *  The QueryParser::FLAG_WORD_BREAKS flag needs to be passed to
+	 *  QueryParser.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	FLAG_WORD_BREAKS = 4096 // Value matches QueryParser flag
     };
 
     /// Stemming strategies, for use with set_stemming_strategy().
@@ -160,8 +173,7 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
 
     /** Set the stemming strategy.
      *
-     *  This method controls how the stemming algorithm is applied.  It was
-     *  new in Xapian 1.3.1.
+     *  This method controls how the stemming algorithm is applied.
      *
      *  @param strategy	The strategy to use - possible values are:
      *   - STEM_NONE:	Don't perform any stemming - only unstemmed terms
@@ -175,13 +187,14 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      *			1.4.8.
      *   - STEM_ALL:	Generate only stemmed terms (but without a "Z" prefix).
      *   - STEM_ALL_Z:	Generate only stemmed terms (with a "Z" prefix).
+     *
+     *  @since Added in Xapian 1.3.1.
      */
     void set_stemming_strategy(stem_strategy strategy);
 
     /** Set the stopper strategy.
      *
-     *  The method controls how the stopper is used.  It was added in Xapian
-     *  1.4.1.
+     *  The method controls how the stopper is used.
      *
      *  You need to also call @a set_stopper() for this to have any effect.
      *
@@ -195,6 +208,8 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      *			  so this allows searches for phrases containing
      *			  stopwords to be supported.  (This is the default
      *			  mode).
+     *
+     *  @since Added in Xapian 1.4.1.
      */
     void set_stopper_strategy(stop_strategy strategy);
 
@@ -208,10 +223,10 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      *  help prevent the index being bloated by useless junk terms from trying
      *  to indexing things like binary data, uuencoded data, ASCII art, etc.
      *
-     *  This method was new in Xapian 1.3.1.
-     *
      *  @param max_word_length	The maximum length word to index, in bytes in
      *				UTF-8 representation.  Default is 64.
+     *
+     *  @since Added in Xapian 1.3.1.
      */
     void set_max_word_length(unsigned max_word_length);
 
@@ -223,17 +238,17 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      */
     void index_text(const Xapian::Utf8Iterator & itor,
 		    Xapian::termcount wdf_inc = 1,
-		    const std::string & prefix = std::string());
+		    std::string_view prefix = {});
 
-    /** Index some text in a std::string.
+    /** Index some text.
      *
      * @param text	The text to index.
      * @param wdf_inc	The wdf increment (default 1).
      * @param prefix	The term prefix to use (default is no prefix).
      */
-    void index_text(const std::string & text,
+    void index_text(std::string_view text,
 		    Xapian::termcount wdf_inc = 1,
-		    const std::string & prefix = std::string()) {
+		    std::string_view prefix = {}) {
 	index_text(Utf8Iterator(text), wdf_inc, prefix);
     }
 
@@ -249,9 +264,9 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      */
     void index_text_without_positions(const Xapian::Utf8Iterator & itor,
 				      Xapian::termcount wdf_inc = 1,
-				      const std::string & prefix = std::string());
+				      std::string_view prefix = {});
 
-    /** Index some text in a std::string without positional information.
+    /** Index some text without positional information.
      *
      * Just like index_text, but no positional information is generated.  This
      * means that the database will be significantly smaller, but that phrase
@@ -261,9 +276,9 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      * @param wdf_inc	The wdf increment (default 1).
      * @param prefix	The term prefix to use (default is no prefix).
      */
-    void index_text_without_positions(const std::string & text,
+    void index_text_without_positions(std::string_view text,
 				      Xapian::termcount wdf_inc = 1,
-				      const std::string & prefix = std::string()) {
+				      std::string_view prefix = {}) {
 	index_text_without_positions(Utf8Iterator(text), wdf_inc, prefix);
     }
 
@@ -286,6 +301,15 @@ class XAPIAN_VISIBILITY_DEFAULT TermGenerator {
      */
     void set_termpos(Xapian::termpos termpos);
 
+    /** Set the term position limit.
+     *
+     *  @param termpos_limit  Upper bound on term positions that can be added.
+     *
+     *  By default the only limit is the maximum value of the Xapian::termpos
+     *  type.
+     */
+    void set_termpos_limit(Xapian::termpos termpos_limit);
+
     /// Return a string describing this object.
     std::string get_description() const;
 };
