@@ -1,7 +1,7 @@
 /** @file
  *  @brief Class for iterating over a list of terms
  */
-/* Copyright (C) 2007,2008,2009,2010,2011,2012,2013,2014,2015,2023 Olly Betts
+/* Copyright (C) 2007-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 
 #include <iterator>
 #include <string>
+#include <string_view>
 
 #include <xapian/attributes.h>
 #include <xapian/derefwrapper.h>
@@ -45,7 +46,8 @@ class XAPIAN_VISIBILITY_DEFAULT TermIterator {
     /// @private @internal Reference counted internals.
     Internal * internal;
 
-    /// @private @internal Construct given internals.
+    /// @private @internal Wrap an existing Internal.
+    XAPIAN_VISIBILITY_INTERNAL
     explicit TermIterator(Internal *internal_);
 
     /// Copy constructor.
@@ -54,7 +56,6 @@ class XAPIAN_VISIBILITY_DEFAULT TermIterator {
     /// Assignment.
     TermIterator & operator=(const TermIterator & o);
 
-#ifdef XAPIAN_MOVE_SEMANTICS
     /// Move constructor.
     TermIterator(TermIterator && o)
 	: internal(o.internal) {
@@ -70,14 +71,13 @@ class XAPIAN_VISIBILITY_DEFAULT TermIterator {
 	}
 	return *this;
     }
-#endif
 
     /** Default constructor.
      *
      *  Creates an uninitialised iterator, which can't be used before being
      *  assigned to, but is sometimes syntactically convenient.
      */
-    XAPIAN_NOTHROW(TermIterator())
+    TermIterator() noexcept
 	: internal() { }
 
     /// Destructor.
@@ -107,7 +107,7 @@ class XAPIAN_VISIBILITY_DEFAULT TermIterator {
     PositionIterator positionlist_begin() const;
 
     /// Return an end PositionIterator for the current term.
-    PositionIterator XAPIAN_NOTHROW(positionlist_end() const) {
+    PositionIterator positionlist_end() const noexcept {
 	return PositionIterator();
     }
 
@@ -130,7 +130,7 @@ class XAPIAN_VISIBILITY_DEFAULT TermIterator {
      *			the stream being iterated, then the iterator is moved
      *			to the next term after it which is.
      */
-    void skip_to(const std::string &term);
+    void skip_to(std::string_view term);
 
     /// Return a string describing this object.
     std::string get_description() const;
@@ -161,27 +161,22 @@ class XAPIAN_VISIBILITY_DEFAULT TermIterator {
   private:
     void decref();
 
+    XAPIAN_VISIBILITY_INTERNAL
     void post_advance(Internal * res);
 };
 
-bool
-XAPIAN_NOTHROW(operator==(const TermIterator &a, const TermIterator &b));
-
 /// Equality test for TermIterator objects.
 inline bool
-operator==(const TermIterator &a, const TermIterator &b) XAPIAN_NOEXCEPT
+operator==(const TermIterator& a, const TermIterator& b) noexcept
 {
     // Use a pointer comparison - this ensures both that (a == a) and correct
     // handling of end iterators (which we ensure have NULL internals).
     return a.internal == b.internal;
 }
 
-bool
-XAPIAN_NOTHROW(operator!=(const TermIterator &a, const TermIterator &b));
-
 /// Inequality test for TermIterator objects.
 inline bool
-operator!=(const TermIterator &a, const TermIterator &b) XAPIAN_NOEXCEPT
+operator!=(const TermIterator& a, const TermIterator& b) noexcept
 {
     return !(a == b);
 }
