@@ -1,7 +1,7 @@
 /** @file
  * @brief Return document ids from an external source.
  */
-/* Copyright 2008,2009,2010,2011 Olly Betts
+/* Copyright 2008-2026 Olly Betts
  * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,20 +23,21 @@
 
 #include "externalpostlist.h"
 
-#include <xapian/error.h>
 #include <xapian/postingsource.h>
 
 #include "debuglog.h"
+#include "estimateop.h"
 #include "omassert.h"
 
 using namespace std;
 
 ExternalPostList::ExternalPostList(const Xapian::Database& db,
 				   Xapian::PostingSource* source_,
+				   EstimateOp* estimate_op,
 				   double factor_,
-				   MultiMatch* matcher,
+				   bool* max_weight_cached_flag_ptr,
 				   Xapian::doccount shard_index)
-    : current(0), factor(factor_)
+    : factor(factor_)
 {
     Assert(source_);
     Xapian::PostingSource* newsource = source_->clone();
@@ -51,39 +52,14 @@ ExternalPostList::ExternalPostList(const Xapian::Database& db,
 					    "implement clone() to support use "
 					    "with a sharded database");
     }
-    source->register_matcher_(static_cast<void*>(matcher));
-    source->init(db);
-}
-
-Xapian::doccount
-ExternalPostList::get_termfreq_min() const
-{
-    Assert(source.get());
-    return source->get_termfreq_min();
-}
-
-Xapian::doccount
-ExternalPostList::get_termfreq_est() const
-{
-    Assert(source.get());
-    return source->get_termfreq_est();
-}
-
-Xapian::doccount
-ExternalPostList::get_termfreq_max() const
-{
-    Assert(source.get());
-    return source->get_termfreq_max();
-}
-
-double
-ExternalPostList::get_maxweight() const
-{
-    LOGCALL(MATCH, double, "ExternalPostList::get_maxweight", NO_ARGS);
-    // source will be NULL here if we've reached the end.
-    if (source.get() == NULL) RETURN(0.0);
-    if (factor == 0.0) RETURN(0.0);
-    RETURN(factor * source->get_maxweight());
+    source->set_max_weight_cached_flag_ptr_(max_weight_cached_flag_ptr);
+    source->reset(db, shard_index);
+    termfreq = source->get_termfreq_est();
+    if (estimate_op) {
+	estimate_op->report_termfreqs(source->get_termfreq_min(),
+				      termfreq,
+				      source->get_termfreq_max());
+    }
 }
 
 Xapian::docid
@@ -95,32 +71,24 @@ ExternalPostList::get_docid() const
 }
 
 double
-ExternalPostList::get_weight() const
+ExternalPostList::get_weight(Xapian::termcount,
+			     Xapian::termcount,
+			     Xapian::termcount) const
 {
     LOGCALL(MATCH, double, "ExternalPostList::get_weight", NO_ARGS);
-    Assert(source.get());
+    Assert(source);
     if (factor == 0.0) RETURN(factor);
     RETURN(factor * source->get_weight());
 }
 
-Xapian::termcount
-ExternalPostList::get_doclength() const
-{
-    Assert(false);
-    return 0;
-}
-
-Xapian::termcount
-ExternalPostList::get_unique_terms() const
-{
-    Assert(false);
-    return 0;
-}
-
 double
 ExternalPostList::recalc_maxweight()
 {
-    return ExternalPostList::get_maxweight();
+    LOGCALL(MATCH, double, "ExternalPostList::recalc_maxweight", NO_ARGS);
+    // source will be NULL here if we've reached the end.
+    if (!source) RETURN(0.0);
+    if (factor == 0.0) RETURN(0.0);
+    RETURN(factor * source->get_maxweight());
 }
 
 PositionList *
@@ -132,7 +100,7 @@ ExternalPostList::read_position_list()
 PostList *
 ExternalPostList::update_after_advance() {
     LOGCALL(MATCH, PostList *, "ExternalPostList::update_after_advance", NO_ARGS);
-    Assert(source.get());
+    Assert(source);
     if (source->at_end()) {
 	LOGLINE(MATCH, "ExternalPostList now at end");
 	source = NULL;
@@ -146,7 +114,7 @@ PostList *
 ExternalPostList::next(double w_min)
 {
     LOGCALL(MATCH, PostList *, "ExternalPostList::next", w_min);
-    Assert(source.get());
+    Assert(source);
     source->next(w_min);
     RETURN(update_after_advance());
 }
@@ -155,7 +123,7 @@ PostList *
 ExternalPostList::skip_to(Xapian::docid did, double w_min)
 {
     LOGCALL(MATCH, PostList *, "ExternalPostList::skip_to", did | w_min);
-    Assert(source.get());
+    Assert(source);
     if (did <= current) RETURN(NULL);
     source->skip_to(did, w_min);
     RETURN(update_after_advance());
@@ -165,7 +133,7 @@ PostList *
 ExternalPostList::check(Xapian::docid did, double w_min, bool &valid)
 {
     LOGCALL(MATCH, PostList *, "ExternalPostList::check", did | w_min | valid);
-    Assert(source.get());
+    Assert(source);
     if (did <= current) {
 	valid = true;
 	RETURN(NULL);
@@ -184,7 +152,7 @@ bool
 ExternalPostList::at_end() const
 {
     LOGCALL(MATCH, bool, "ExternalPostList::at_end", NO_ARGS);
-    RETURN(source.get() == NULL);
+    RETURN(!source);
 }
 
 Xapian::termcount
@@ -197,7 +165,7 @@ string
 ExternalPostList::get_description() const
 {
     string desc = "ExternalPostList(";
-    if (source.get()) desc += source->get_description();
+    if (source) desc += source->get_description();
     desc += ")";
     return desc;
 }
