@@ -1,7 +1,7 @@
 /** @file
  * @brief Return document ids from an external source.
  */
-/* Copyright 2008,2009,2011 Olly Betts
+/* Copyright 2008-2022 Olly Betts
  * Copyright 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,13 +22,14 @@
 #ifndef XAPIAN_INCLUDED_EXTERNALPOSTLIST_H
 #define XAPIAN_INCLUDED_EXTERNALPOSTLIST_H
 
-#include "api/postlist.h"
+#include "backends/postlist.h"
 
 namespace Xapian {
     class PostingSource;
 }
 
-class MultiMatch;
+class EstimateOp;
+class PostListTree;
 
 class ExternalPostList : public PostList {
     /// Disallow copying.
@@ -39,7 +40,7 @@ class ExternalPostList : public PostList {
 
     Xapian::Internal::opt_intrusive_ptr<Xapian::PostingSource> source;
 
-    Xapian::docid current;
+    Xapian::docid current = 0;
 
     double factor;
 
@@ -48,29 +49,22 @@ class ExternalPostList : public PostList {
   public:
     /** Constructor.
      *
-     *  @param matcher	The matcher to notify when maximum weight changes.
+     *  @param estimate_op		    Object to report min/est/max to.
+     *  @param max_weight_cached_flag_ptr   Pointer to flag to clear when max
+     *					    weight changes.
      */
     ExternalPostList(const Xapian::Database & db,
 		     Xapian::PostingSource *source_,
+		     EstimateOp* estimate_op,
 		     double factor_,
-		     MultiMatch* matcher,
+		     bool* max_weight_cached_flag_ptr,
 		     Xapian::doccount shard_index);
 
-    Xapian::doccount get_termfreq_min() const;
-
-    Xapian::doccount get_termfreq_est() const;
-
-    Xapian::doccount get_termfreq_max() const;
-
-    double get_maxweight() const;
-
     Xapian::docid get_docid() const;
 
-    double get_weight() const;
-
-    Xapian::termcount get_doclength() const;
-
-    Xapian::termcount get_unique_terms() const;
+    double get_weight(Xapian::termcount doclen,
+		      Xapian::termcount unique_terms,
+		      Xapian::termcount wdfdocmax) const;
 
     double recalc_maxweight();
 
