@@ -1,7 +1,7 @@
 /** @file
  *  @brief SubMatch class for a local database.
  */
-/* Copyright (C) 2006,2007,2009,2010,2011,2013,2014,2015,2016,2018,2020 Olly Betts
+/* Copyright (C) 2006-2026 Olly Betts
  * Copyright (C) 2007,2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,10 @@
 
 #include "localsubmatch.h"
 
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
+#include "backends/leafpostlist.h"
 #include "debuglog.h"
-#include "api/emptypostlist.h"
 #include "extraweightpostlist.h"
-#include "api/leafpostlist.h"
 #include "omassert.h"
 #include "queryoptimiser.h"
 #include "synonympostlist.h"
@@ -36,7 +35,7 @@
 
 #include "xapian/error.h"
 
-#include "autoptr.h"
+#include <memory>
 #include <string>
 
 using namespace std;
@@ -60,6 +59,8 @@ class LazyWeight : public Xapian::Weight {
 
     double factor;
 
+    const Xapian::Database::Internal* shard;
+
     LazyWeight* clone() const override;
 
     void init(double factor_) override;
@@ -70,13 +71,15 @@ class LazyWeight : public Xapian::Weight {
 	       Xapian::Weight::Internal * stats_,
 	       Xapian::termcount qlen_,
 	       Xapian::termcount wqf__,
-	       double factor_)
+	       double factor_,
+	       const Xapian::Database::Internal* shard_)
 	: pl(pl_),
 	  real_wt(real_wt_),
 	  stats(stats_),
 	  qlen(qlen_),
 	  wqf(wqf__),
-	  factor(factor_)
+	  factor(factor_),
+	  shard(shard_)
     { }
 
     std::string name() const override;
@@ -86,11 +89,13 @@ class LazyWeight : public Xapian::Weight {
 
     double get_sumpart(Xapian::termcount wdf,
 		       Xapian::termcount doclen,
-		       Xapian::termcount uniqterms) const override;
+		       Xapian::termcount uniqterms,
+		       Xapian::termcount wdfdocmax) const override;
     double get_maxpart() const override;
 
     double get_sumextra(Xapian::termcount doclen,
-			Xapian::termcount uniqterms) const override;
+			Xapian::termcount uniqterms,
+			Xapian::termcount wdfdocmax) const override;
     double get_maxextra() const override;
 };
 
@@ -131,20 +136,24 @@ LazyWeight::unserialise(const string &) const
 double
 LazyWeight::get_sumpart(Xapian::termcount wdf,
 			Xapian::termcount doclen,
-			Xapian::termcount uniqterms) const
+			Xapian::termcount uniqterms,
+			Xapian::termcount wdfdocmax) const
 {
     (void)wdf;
     (void)doclen;
     (void)uniqterms;
+    (void)wdfdocmax;
     throw Xapian::InvalidOperationError("LazyWeight::get_sumpart()");
 }
 
 double
 LazyWeight::get_sumextra(Xapian::termcount doclen,
-			 Xapian::termcount uniqterms) const
+			 Xapian::termcount uniqterms,
+			 Xapian::termcount wdfdocmax) const
 {
     (void)doclen;
     (void)uniqterms;
+    (void)wdfdocmax;
     throw Xapian::InvalidOperationError("LazyWeight::get_sumextra()");
 }
 
@@ -152,7 +161,7 @@ double
 LazyWeight::get_maxpart() const
 {
     // This gets called first for the case we care about.
-    return pl->resolve_lazy_termweight(real_wt, stats, qlen, wqf, factor);
+    return pl->resolve_lazy_termweight(real_wt, stats, qlen, wqf, factor, shard);
 }
 
 double
@@ -161,121 +170,113 @@ LazyWeight::get_maxextra() const
     throw Xapian::InvalidOperationError("LazyWeight::get_maxextra()");
 }
 
-bool
-LocalSubMatch::prepare_match(bool nowait,
-			     Xapian::Weight::Internal & total_stats)
+PostListAndEstimate
+LocalSubMatch::get_postlist(PostListTree * matcher,
+			    Xapian::termcount * total_subqs_ptr)
 {
-    LOGCALL(MATCH, bool, "LocalSubMatch::prepare_match", nowait | total_stats);
-    (void)nowait;
-    Assert(db);
-    total_stats.accumulate_stats(*db, rset);
-    RETURN(true);
-}
-
-void
-LocalSubMatch::start_match(Xapian::doccount first,
-			   Xapian::doccount maxitems,
-			   Xapian::doccount check_at_least,
-			   Xapian::Weight::Internal & total_stats)
-{
-    LOGCALL_VOID(MATCH, "LocalSubMatch::start_match", first | maxitems | check_at_least | total_stats);
-    (void)first;
-    (void)maxitems;
-    (void)check_at_least;
-    // Store a pointer to the total stats to use when building the Query tree.
-    stats = &total_stats;
-}
-
-PostList *
-LocalSubMatch::get_postlist(MultiMatch * matcher,
-			    Xapian::termcount* total_subqs_ptr,
-			    Xapian::Weight::Internal&)
-{
-    LOGCALL(MATCH, PostList*, "LocalSubMatch::get_postlist", matcher | total_subqs_ptr | Literal("[total_subqs]"));
+    LOGCALL(MATCH, PostList *, "LocalSubMatch::get_postlist", matcher | total_subqs_ptr);
 
     if (query.empty() || db->get_doccount() == 0)
-	RETURN(new EmptyPostList); // MatchNothing
+	return {nullptr, nullptr}; // MatchNothing
 
     // Build the postlist tree for the query.  This calls
     // LocalSubMatch::open_post_list() for each term in the query.
-    PostList * pl;
+    PostListAndEstimate plest;
     {
 	QueryOptimiser opt(*db, *this, matcher, shard_index);
-	double factor = wt_factory->is_bool_weight_() ? 0.0 : 1.0;
-	pl = query.internal->postlist(&opt, factor);
+	double factor = wt_factory.is_bool_weight_() ? 0.0 : 1.0;
+	plest = query.internal->postlist(&opt, factor, NULL);
 	*total_subqs_ptr = opt.get_total_subqs();
     }
 
-    AutoPtr<Xapian::Weight> extra_wt(wt_factory->clone());
-    // Only uses term-independent stats.
-    extra_wt->init_(*stats, qlen);
-    if (extra_wt->get_maxextra() != 0.0) {
-	// There's a term-independent weight contribution, so we combine the
-	// postlist tree with an ExtraWeightPostList which adds in this
-	// contribution.
-	pl = new ExtraWeightPostList(pl, extra_wt.release(), matcher);
+    if (plest.pl) {
+	unique_ptr<Xapian::Weight> extra_wt(wt_factory.clone());
+	// Only uses term-independent stats.
+	extra_wt->init_(*total_stats, qlen, db);
+	if (extra_wt->get_maxextra() != 0.0) {
+	    // There's a term-independent weight contribution, so we combine
+	    // the postlist tree with an ExtraWeightPostList which adds in this
+	    // contribution.
+	    plest.pl = new ExtraWeightPostList(plest.pl, extra_wt.release(),
+					       matcher);
+	}
     }
 
-    RETURN(pl);
+    return plest;
 }
 
-PostList *
-LocalSubMatch::make_synonym_postlist(PostList * or_pl, MultiMatch * matcher,
+PostListAndEstimate
+LocalSubMatch::make_synonym_postlist(PostListTree* pltree,
+				     PostListAndEstimate or_pl,
 				     double factor,
-				     bool wdf_disjoint)
+				     const TermFreqs& termfreqs)
 {
-    LOGCALL(MATCH, PostList *, "LocalSubMatch::make_synonym_postlist", or_pl | matcher | factor | wdf_disjoint);
-    if (rare(or_pl->get_termfreq_max() == 0)) {
-	// or_pl is an EmptyPostList or equivalent.
-	return or_pl;
-    }
-    LOGVALUE(MATCH, or_pl->get_termfreq_est());
-    Xapian::termcount len_lb = db->get_doclength_lower_bound();
-    AutoPtr<SynonymPostList> res(new SynonymPostList(or_pl, matcher, len_lb,
-						     wdf_disjoint));
-    AutoPtr<Xapian::Weight> wt(wt_factory->clone());
+    LOGCALL(MATCH, PostListAndEstimate, "LocalSubMatch::make_synonym_postlist", pltree | or_pl | factor | termfreqs);
+    bool needs_doclen = wt_factory.get_sumpart_needs_doclength_();
+    unique_ptr<SynonymPostList> res(new SynonymPostList(or_pl.pl, pltree,
+							needs_doclen));
+    unique_ptr<Xapian::Weight> wt(wt_factory.clone());
 
-    TermFreqs freqs;
-    // Avoid calling get_termfreq_est_using_stats() if the database is empty
-    // so we don't need to special case that repeatedly when implementing it.
-    // FIXME: it would be nicer to handle an empty database higher up, though
-    // we need to catch the case where all the non-empty subdatabases have
-    // failed, so we can't just push this right up to the start of get_mset().
-    if (usual(stats->collection_size != 0)) {
-	freqs = or_pl->get_termfreq_est_using_stats(*stats);
-    }
-    wt->init_(*stats, qlen, factor,
-	      freqs.termfreq, freqs.reltermfreq, freqs.collfreq);
+    // We shortcut an empty shard and avoid creating a postlist tree for it,
+    // and all shards must be empty for collection_size to be zero.
+    Assert(total_stats->collection_size);
+    wt->init_(*total_stats, qlen, factor,
+	      termfreqs.termfreq, termfreqs.reltermfreq, termfreqs.collfreq,
+	      db);
 
     res->set_weight(wt.release());
-    RETURN(res.release());
+    RETURN({res.release(), std::move(or_pl.est)});
 }
 
-LeafPostList *
+PostListAndEstimate
 LocalSubMatch::open_post_list(const string& term,
 			      Xapian::termcount wqf,
 			      double factor,
 			      bool need_positions,
-			      bool in_synonym,
-			      QueryOptimiser * qopt,
-			      bool lazy_weight)
+			      bool compound_weight,
+			      QueryOptimiser* qopt,
+			      bool lazy_weight,
+			      TermFreqs* termfreqs)
 {
-    LOGCALL(MATCH, LeafPostList *, "LocalSubMatch::open_post_list", term | wqf | factor | need_positions | qopt | lazy_weight);
+    LOGCALL(MATCH, PostListAndEstimate, "LocalSubMatch::open_post_list", term | wqf | factor | need_positions | qopt | lazy_weight | termfreqs);
 
-    bool weighted = (factor != 0.0 && !term.empty());
+    bool weighted = false;
 
     LeafPostList * pl = NULL;
-    if (!term.empty() && !need_positions) {
-	if ((!weighted && !in_synonym) ||
-	    !wt_factory->get_sumpart_needs_wdf_()) {
-	    Xapian::doccount sub_tf;
-	    db->get_freqs(term, &sub_tf, NULL);
-	    if (sub_tf == qopt->db_size) {
+    if (term.empty()) {
+	Assert(!need_positions);
+	pl = db->open_leaf_post_list(term, false);
+    } else {
+	weighted = (factor != 0.0);
+	const LeafPostList* hint = qopt->get_hint_postlist();
+	if (!hint || !hint->open_nearby_postlist(term, need_positions, pl)) {
+	    pl = db->open_leaf_post_list(term, need_positions);
+	}
+	if (pl) qopt->set_hint_postlist(pl);
+	if (pl && !need_positions) {
+	    bool need_wdf = (weighted || compound_weight) &&
+			    wt_factory.get_sumpart_needs_wdf_();
+	    if (!need_wdf && pl->get_termfreq() == qopt->db_size) {
 		// If we're not going to use the wdf or term positions, and the
 		// term indexes all documents, we can replace it with the
 		// MatchAll postlist, which is especially efficient if there
 		// are no gaps in the docids.
-		pl = db->open_post_list(string());
+		//
+		// We opened the real PostList already as that's more efficient
+		// than asking the Database for the termfreq in the common case
+		// when the term doesn't index all documents, and is similar
+		// work in the case where it does (for glass, there's the extra
+		// overhead of creating a cursor but we still need to read and
+		// decode the same data).
+		//
+		// The real PostList got set as the QueryOptimiser's hint above
+		// so we can just hand ownership of it to the QueryOptimiser.
+		qopt->own_hint_postlist();
+		pl = db->open_leaf_post_list(string(), false);
+		// We shortcut an empty shard and avoid creating a postlist
+		// tree for it, so an alldocs postlist can't be NULL here.
+		Assert(pl);
+
 		// Set the term name so the postlist looks up the correct term
 		// frequencies - this is necessary if the weighting scheme
 		// needs collection frequency or reltermfreq (termfreq would be
@@ -286,40 +287,42 @@ LocalSubMatch::open_post_list(const string& term,
 	}
     }
 
-    if (!pl) {
-	const LeafPostList * hint = qopt->get_hint_postlist();
-	if (hint)
-	    pl = hint->open_nearby_postlist(term);
-	if (!pl)
-	    pl = db->open_post_list(term);
-	qopt->set_hint_postlist(pl);
-    }
-
-    if (lazy_weight) {
-	auto res = stats->termfreqs.emplace(term, TermFreqs());
-	if (res.second) {
-	    // Term came from a wildcard, but the same term may be elsewhere
-	    // in the query so only accumulate its TermFreqs if emplace()
-	    // created a new element.
-	    db->get_freqs(term,
-			  &res.first->second.termfreq,
-			  &res.first->second.collfreq);
-	}
-    }
-
-    if (weighted) {
-	Xapian::Weight * wt = wt_factory->clone();
+    if (pl && weighted) {
+	Xapian::Weight * wt = wt_factory.clone();
 	if (!lazy_weight) {
-	    wt->init_(*stats, qlen, term, wqf, factor, pl);
+	    wt->init_(*total_stats, qlen, term, wqf, factor, db, pl);
 	    if (pl->get_termfreq() > 0)
-		stats->set_max_part(term, wt->get_maxpart());
+		total_stats->set_max_part(term, wt->get_maxpart());
 	} else {
 	    // Delay initialising the actual weight object, so that we can
 	    // gather stats for the terms lazily expanded from a wildcard
 	    // (needed for the remote database case).
-	    wt = new LazyWeight(pl, wt, stats, qlen, wqf, factor);
+	    wt = new LazyWeight(pl, wt, total_stats, qlen, wqf, factor, db);
 	}
 	pl->set_termweight(wt);
     }
-    RETURN(pl);
+
+    if (termfreqs) {
+	if (term.empty()) {
+	    *termfreqs = TermFreqs(total_stats->collection_size,
+				   total_stats->rset_size,
+				   total_stats->total_length);
+	} else if (!lazy_weight) {
+	    auto i = total_stats->termfreqs.find(term);
+	    Assert(i != total_stats->termfreqs.end());
+	    *termfreqs = i->second;
+	}
+    }
+
+    if (!pl) {
+	RETURN({nullptr, nullptr});
+    }
+
+    Xapian::docid first = 1, last = Xapian::docid(-1);
+    pl->get_docid_range(first, last);
+
+    EstimateOp* est = nullptr;
+    if (!qopt->get_no_estimates())
+	est = new EstimateOp(pl->get_termfreq(), first, last);
+    RETURN({pl, est});
 }
