@@ -1,7 +1,7 @@
 /** @file
  * @brief N-way OR postlist with wt=max(wt_i)
  */
-/* Copyright (C) 2007,2009,2010,2011,2012,2013 Olly Betts
+/* Copyright (C) 2007,2009,2010,2011,2012,2013,2017,2018 Olly Betts
  * Copyright (C) 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -22,11 +22,10 @@
 #ifndef XAPIAN_INCLUDED_MAXPOSTLIST_H
 #define XAPIAN_INCLUDED_MAXPOSTLIST_H
 
-#include "multimatch.h"
-#include "api/postlist.h"
-#include <algorithm>
+#include "backends/postlist.h"
+#include "postlisttree.h"
 
-class MultiMatch;
+#include <algorithm>
 
 /// N-way OR postlist with wt=max(wt_i).
 class MaxPostList : public PostList {
@@ -37,22 +36,16 @@ class MaxPostList : public PostList {
     MaxPostList(const MaxPostList &);
 
     /// The current docid, or zero if we haven't started or are at_end.
-    Xapian::docid did;
+    Xapian::docid did = 0;
 
     /// The number of sub-postlists.
     size_t n_kids;
 
     /// Array of pointers to sub-postlists.
-    PostList ** plist;
-
-    /// Cached answer to get_maxweight.
-    double max_cached;
-
-    /// The number of documents in the database.
-    Xapian::doccount db_size;
+    PostList** plist = nullptr;
 
     /// Pointer to the matcher object, so we can report pruning.
-    MultiMatch *matcher;
+    PostListTree *matcher;
 
     /// Erase a sub-postlist.
     void erase_sublist(size_t i) {
@@ -61,7 +54,7 @@ class MaxPostList : public PostList {
 	for (size_t j = i; j < n_kids; ++j) {
 	    plist[j] = plist[j + 1];
 	}
-	matcher->recalc_maxweight();
+	matcher->force_recalc();
     }
 
   public:
@@ -70,31 +63,34 @@ class MaxPostList : public PostList {
      */
     template<class RandomItor>
     MaxPostList(RandomItor pl_begin, RandomItor pl_end,
-		MultiMatch * matcher_, Xapian::doccount db_size_)
-	: did(0), n_kids(pl_end - pl_begin), plist(NULL),
-	  max_cached(0), db_size(db_size_), matcher(matcher_)
+		PostListTree* matcher_, Xapian::doccount db_size)
+	: n_kids(pl_end - pl_begin), matcher(matcher_)
     {
 	plist = new PostList * [n_kids];
 	std::copy(pl_begin, pl_end, plist);
+
+	// We shortcut an empty shard and avoid creating a postlist tree for it.
+	Assert(db_size);
+
+	// We calculate the estimate assuming independence.  The simplest
+	// way to calculate this seems to be a series of (n_kids - 1) pairwise
+	// calculations, which gives the same answer regardless of the order.
+	double scale = 1.0 / db_size;
+	double P_est = plist[0]->get_termfreq() * scale;
+	for (size_t i = 1; i < n_kids; ++i) {
+	    double P_i = plist[i]->get_termfreq() * scale;
+	    P_est += P_i - P_est * P_i;
+	}
+	termfreq = static_cast<Xapian::doccount>(P_est * db_size + 0.5);
     }
 
     ~MaxPostList();
 
-    Xapian::doccount get_termfreq_min() const;
-
-    Xapian::doccount get_termfreq_max() const;
-
-    Xapian::doccount get_termfreq_est() const;
-
-    double get_maxweight() const;
-
     Xapian::docid get_docid() const;
 
-    Xapian::termcount get_doclength() const;
-
-    Xapian::termcount get_unique_terms() const;
-
-    double get_weight() const;
+    double get_weight(Xapian::termcount doclen,
+		      Xapian::termcount unique_terms,
+		      Xapian::termcount wdfdocmax) const;
 
     bool at_end() const;
 
@@ -108,6 +104,8 @@ class MaxPostList : public PostList {
 
     PostList* skip_to(Xapian::docid, double w_min);
 
+    void get_docid_range(Xapian::docid& first, Xapian::docid& last) const;
+
     std::string get_description() const;
 
     /** get_wdf() for MaxPostlist returns the sum of the wdfs of the
@@ -119,9 +117,6 @@ class MaxPostList : public PostList {
      */
     Xapian::termcount get_wdf() const;
 
-    // Note - we don't need to implement get_termfreq_est_using_stats() because
-    // an OP_MAX when used as a child of a synonym will be optimised to an OR.
-
     Xapian::termcount count_matching_subqs() const;
 };
 
