@@ -1,5 +1,5 @@
 /** @file
- * @brief MSetItem comparison functions.
+ * @brief Result comparison functions.
  */
 /* Copyright (C) 2006,2007,2011,2017 Olly Betts
  *
@@ -22,13 +22,16 @@
 #ifndef XAPIAN_INCLUDED_MSETCMP_H
 #define XAPIAN_INCLUDED_MSETCMP_H
 
-#include "api/omenquireinternal.h"
+#include "api/enquireinternal.h"
 
-// typedef for MSetItem comparison function.
-typedef bool (* MSetCmp)(const Xapian::Internal::MSetItem &,
-			 const Xapian::Internal::MSetItem &);
+class Result;
+
+// typedef for Result comparison function.
+typedef bool (* MSetCmp)(const Result&, const Result&);
 
 /// Select the appropriate msetcmp function.
-MSetCmp get_msetcmp_function(Xapian::Enquire::Internal::sort_setting sort_by, bool sort_forward, bool sort_value_forward);
+MSetCmp get_msetcmp_function(Xapian::Enquire::Internal::sort_setting sort_by,
+			     bool sort_forward,
+			     bool sort_value_forward);
 
 #endif // XAPIAN_INCLUDED_MSETCMP_H
