@@ -1,7 +1,7 @@
-/** @file nearpostlist.h
+/** @file
  * @brief Return docs containing terms within a specified window.
  *
- * Copyright (C) 2006,2015 Olly Betts
+ * Copyright (C) 2006,2015,2017 Olly Betts
  * Copyright (C) 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,9 @@
 #include "selectpostlist.h"
 #include <vector>
 
+class EstimateOp;
+class PostListTree;
+
 /** Postlist which matches terms occurring within a specified window.
  *
  *  NearPostList only returns a posting for documents contains all the terms
@@ -47,19 +50,16 @@ class NearPostList : public SelectPostList {
 
   public:
     NearPostList(PostList *source_,
+		 EstimateOp* estimate_op_,
 		 Xapian::termpos window_,
 		 const std::vector<PostList*>::const_iterator &terms_begin,
-		 const std::vector<PostList*>::const_iterator &terms_end);
+		 const std::vector<PostList*>::const_iterator &terms_end,
+		 PostListTree* pltree_);
 
     ~NearPostList();
 
     Xapian::termcount get_wdf() const;
 
-    Xapian::doccount get_termfreq_est() const;
-
-    TermFreqs get_termfreq_est_using_stats(
-	const Xapian::Weight::Internal & stats) const;
-
     std::string get_description() const;
 };
 
