@@ -22,7 +22,7 @@
 #define XAPIAN_INCLUDED_ORPOSITIONLIST_H
 
 #include "backends/positionlist.h"
-#include "api/postlist.h"
+#include "backends/postlist.h"
 
 #include "xapian/error.h"
 #include <algorithm>
@@ -59,6 +59,8 @@ class OrPositionList : public PositionList {
 
     Xapian::termcount get_approx_size() const;
 
+    Xapian::termpos back() const;
+
     Xapian::termpos get_position() const;
 
     bool next();
