@@ -1,7 +1,7 @@
 /** @file
  * @brief Details passed around while building PostList tree from Query tree
  */
-/* Copyright (C) 2007,2008,2009,2010,2011,2013,2014,2015,2016,2018 Olly Betts
+/* Copyright (C) 2007-2026 Olly Betts
  * Copyright (C) 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -22,12 +22,16 @@
 #ifndef XAPIAN_INCLUDED_QUERYOPTIMISER_H
 #define XAPIAN_INCLUDED_QUERYOPTIMISER_H
 
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
+#include "backends/leafpostlist.h"
+#include "backends/postlist.h"
 #include "localsubmatch.h"
-#include "api/postlist.h"
 
 class LeafPostList;
-class MultiMatch;
+class PostListTree;
+
+namespace Xapian {
+namespace Internal {
 
 class QueryOptimiser {
     /// Prevent assignment.
@@ -43,16 +47,18 @@ class QueryOptimiser {
      *  Used for scaling percentages when the highest weighted document doesn't
      *  "match all terms".
      */
-    Xapian::termcount total_subqs;
+    Xapian::termcount total_subqs = 0;
 
-    LeafPostList * hint;
+    LeafPostList* hint = nullptr;
 
-    bool hint_owned;
+    bool hint_owned = false;
+
+    bool no_estimates = false;
 
   public:
-    bool need_positions;
+    bool need_positions = false;
 
-    bool in_synonym;
+    bool compound_weight = false;
 
     Xapian::doccount shard_index;
 
@@ -60,15 +66,13 @@ class QueryOptimiser {
 
     Xapian::doccount db_size;
 
-    MultiMatch * matcher;
+    PostListTree * matcher;
 
     QueryOptimiser(const Xapian::Database::Internal & db_,
 		   LocalSubMatch & localsubmatch_,
-		   MultiMatch* matcher_,
+		   PostListTree * matcher_,
 		   Xapian::doccount shard_index_)
-	: localsubmatch(localsubmatch_), total_subqs(0),
-	  hint(0), hint_owned(false),
-	  need_positions(false), in_synonym(false),
+	: localsubmatch(localsubmatch_),
 	  shard_index(shard_index_),
 	  db(db_), db_size(db.get_doccount()),
 	  matcher(matcher_) { }
@@ -77,35 +81,66 @@ class QueryOptimiser {
 	if (hint_owned) delete hint;
     }
 
-    bool full_db_has_positions() const {
-	return matcher->full_db_has_positions();
-    }
-
     void inc_total_subqs() { ++total_subqs; }
 
     Xapian::termcount get_total_subqs() const { return total_subqs; }
 
     void set_total_subqs(Xapian::termcount n) { total_subqs = n; }
 
-    LeafPostList * open_post_list(const std::string& term,
-				  Xapian::termcount wqf,
-				  double factor) {
+    bool get_no_estimates() const { return no_estimates; }
+
+    void set_no_estimates(bool f) { no_estimates = f; }
+
+    /** Create a PostList object for @a term.
+     *
+     *  @param[out] termfreqs If not NULL, the pointed to object is set to the
+     *		TermFreqs for @a term.  If the database is sharded, these will
+     *		be for the whole database not just the current shard.  This
+     *		is used to estimate TermFreqs for an OP_SYNONYM.
+     */
+    PostListAndEstimate
+    open_post_list(const std::string& term,
+		   Xapian::termcount wqf,
+		   double factor,
+		   TermFreqs* termfreqs) {
 	return localsubmatch.open_post_list(term, wqf, factor, need_positions,
-					    in_synonym, this, false);
+					    compound_weight, this, false,
+					    termfreqs);
     }
 
-    LeafPostList * open_lazy_post_list(const std::string& term,
-				       Xapian::termcount wqf,
-				       double factor) {
-	return localsubmatch.open_post_list(term, wqf, factor, false,
-					    in_synonym, this, true);
+    PostListAndEstimate
+    open_lazy_post_list(const std::string& term,
+			Xapian::termcount wqf,
+			double factor) {
+	return localsubmatch.open_post_list(term, wqf, factor, need_positions,
+					    compound_weight, this, true,
+					    NULL);
     }
 
-    PostList * make_synonym_postlist(PostList * pl,
-				     double factor,
-				     bool wdf_disjoint) {
-	return localsubmatch.make_synonym_postlist(pl, matcher, factor,
-						   wdf_disjoint);
+    /** Register a lazily-created LeafPostList for stats.
+     *
+     *  @param pl   An object previously returned by open_lazy_post_list().
+     *  @param[out] termfreqs If not NULL, the pointed to object is set to the
+     *		TermFreqs for @a term.  If the database is sharded, these will
+     *		be for the whole database not just the current shard.  This
+     *		is used to estimate TermFreqs for an OP_SYNONYM.
+     */
+    void register_lazy_postlist_for_stats(LeafPostList* pl,
+					  TermFreqs* termfreqs) {
+	localsubmatch.register_lazy_postlist_for_stats(pl, termfreqs);
+    }
+
+    /** Create a SynonymPostList object.
+     *
+     *  @param or_pl	    An unweighted BoolOrPostList or OrPostList of the
+     *			    PostList objects for children of the OP_SYNONYM.
+     *  @param termfreqs    Estimated TermFreqs for @a or_pl.
+     */
+    PostListAndEstimate make_synonym_postlist(PostListAndEstimate or_pl,
+					      double factor,
+					      const TermFreqs& termfreqs) {
+	return localsubmatch.make_synonym_postlist(matcher, std::move(or_pl),
+						   factor, termfreqs);
     }
 
     const LeafPostList * get_hint_postlist() const { return hint; }
@@ -118,7 +153,10 @@ class QueryOptimiser {
 	hint = new_hint;
     }
 
+    void own_hint_postlist() { hint_owned = true; }
+
     void destroy_postlist(PostList* pl) {
+	if (!pl) return;
 	if (pl == static_cast<PostList*>(hint)) {
 	    hint_owned = true;
 	} else {
@@ -133,6 +171,19 @@ class QueryOptimiser {
 	    delete pl;
 	}
     }
+
+    bool need_wdf_for_compound_weight() const {
+	return compound_weight && !localsubmatch.weight_needs_wdf();
+    }
+
+    const Xapian::Weight::Internal* get_stats() const {
+	return localsubmatch.get_stats();
+    }
 };
 
+}
+}
+
+using Xapian::Internal::QueryOptimiser;
+
 #endif // XAPIAN_INCLUDED_QUERYOPTIMISER_H
