@@ -1,4 +1,4 @@
-/** @file valuegepostlist.h
+/** @file
  * @brief Return document ids matching a >= test on a specified doc value.
  */
 /* Copyright 2007,2011 Olly Betts
@@ -24,6 +24,8 @@
 
 #include "valuerangepostlist.h"
 
+class EstimateOp;
+
 class ValueGePostList: public ValueRangePostList {
     /// Disallow copying.
     ValueGePostList(const ValueGePostList &);
@@ -33,9 +35,12 @@ class ValueGePostList: public ValueRangePostList {
 
   public:
     ValueGePostList(const Xapian::Database::Internal *db_,
+		    EstimateOp* estimate_op_,
+		    Xapian::doccount termfreq_,
 		    Xapian::valueno slot_,
 		    const std::string &begin_)
-	: ValueRangePostList(db_, slot_, begin_, std::string()) {}
+	: ValueRangePostList(db_, estimate_op_, termfreq_,
+			     slot_, begin_, std::string()) {}
 
     PostList * next(double w_min);
 
