@@ -1,7 +1,7 @@
 /** @file
  *  @brief Implementation of RemoteDatabase using a spawned server.
  */
-/* Copyright (C) 2007,2010,2011,2014,2019 Olly Betts
+/* Copyright (C) 2007,2010,2011,2014,2019,2023,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,9 @@
 #ifndef XAPIAN_INCLUDED_PROGCLIENT_H
 #define XAPIAN_INCLUDED_PROGCLIENT_H
 
+#include <string>
+#include <string_view>
+
 #include <sys/types.h>
 
 #include "backends/remote/remote-database.h"
@@ -33,10 +36,10 @@
  */
 class ProgClient : public RemoteDatabase {
     /// Don't allow assignment.
-    void operator=(const ProgClient &);
+    ProgClient& operator=(const ProgClient&) = delete;
 
     /// Don't allow copying.
-    ProgClient(const ProgClient &);
+    ProgClient(const ProgClient&) = delete;
 
 #ifndef __WIN32__
     /// Process id of the child process.
@@ -52,48 +55,43 @@ class ProgClient : public RemoteDatabase {
      *  @param args	Any arguments to the program.
      *  @param child	Reference to store the child process pid/HANDLE in.
      *
-     *  @return	file descriptor for reading from/writing to the child process.
+     *  @return A std::pair containing the file descriptor for the connection
+     *		to the child process and a context string to return with any
+     *		error messages.
      *
      *  Note: this method is called early on during class construction before
      *  any member variables or even the base class have been initialised.
      *  To help avoid accidentally trying to use member variables, this method
      *  has been deliberately made "static".
      */
-    static int run_program(const std::string &progname,
-			   const std::string &args,
+    static std::pair<int, std::string> run_program(std::string_view progname,
+						   std::string_view args,
 #ifndef __WIN32__
-			   pid_t& child
+						   pid_t& child
 #else
-			   HANDLE& child
+						   HANDLE& child
 #endif
-			   );
-
-    /** Generate context string for Xapian::Error exception objects.
-     *
-     *  @param progname	The program used to create the connection.
-     *  @param args	Any arguments to the program.
-     *
-     *  Note: this method is used from constructors so has been made static to
-     *  avoid problems with trying to use uninitialised member variables.  In
-     *  particular, it can't be made a virtual method of the base class.
-     */
-    static std::string get_progcontext(const std::string &progname,
-				       const std::string &args);
+						   );
 
   public:
     /** Constructor.
      *
      *  @param progname	The program used to create the connection.
      *  @param args	Any arguments to the program.
-     *  @param timeout	Timeout for communication (in seconds).
+     *  @param timeout_	Timeout for communication (in seconds).
      *  @param writable	Is this a WritableDatabase?
      *  @param flags	Xapian::DB_RETRY_LOCK or 0.
      */
-    ProgClient(const std::string &progname,
-	       const std::string &arg,
-	       double timeout,
+    ProgClient(std::string_view progname,
+	       std::string_view args,
+	       double timeout_,
 	       bool writable,
-	       int flags);
+	       int flags)
+	: RemoteDatabase(run_program(progname, args, child),
+			 timeout_,
+			 writable,
+			 flags)
+    {}
 
     /** Destructor. */
     ~ProgClient();
