@@ -1,7 +1,7 @@
 /** @file
  *  @brief RemoteConnection class used by the remote backend.
  */
-/* Copyright (C) 2006-2024 Olly Betts
+/* Copyright (C) 2006-2025 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,33 +45,32 @@
 #include "debuglog.h"
 #include "fd.h"
 #include "filetests.h"
-#include "noreturn.h"
 #include "omassert.h"
 #include "overflow.h"
+#include "pack.h"
 #include "posixy_wrapper.h"
 #include "realtime.h"
-#include "length.h"
 #include "socket_utils.h"
 
 using namespace std;
 
 #define CHUNKSIZE 4096
 
-XAPIAN_NORETURN(static void throw_database_closed());
+[[noreturn]]
 static void
 throw_database_closed()
 {
     throw Xapian::DatabaseClosedError("Database has been closed");
 }
 
-XAPIAN_NORETURN(static void throw_network_error_insane_message_length());
+[[noreturn]]
 static void
-throw_network_error_insane_message_length()
+throw_network_error_message_too_long_for_off_t()
 {
-    throw Xapian::NetworkError("Insane message length specified!");
+    throw Xapian::NetworkError("Message too long for size to fit in off_t");
 }
 
-XAPIAN_NORETURN(static void throw_timeout(const char*, const string&));
+[[noreturn]]
 static void
 throw_timeout(const char* msg, const string& context)
 {
@@ -279,39 +278,6 @@ RemoteConnection::read_at_least(size_t min_len, double end_time)
     RETURN(true);
 }
 
-bool
-RemoteConnection::ready_to_read() const
-{
-    LOGCALL(REMOTE, bool, "RemoteConnection::ready_to_read", NO_ARGS);
-    if (fdin == -1)
-	throw_database_closed();
-
-    if (!buffer.empty()) RETURN(true);
-
-    // See if there's data available to be read.
-#ifdef HAVE_POLL
-    struct pollfd fds;
-    fds.fd = fdin;
-    fds.events = POLLIN;
-    RETURN(poll(&fds, 1, 0) > 0);
-#else
-# ifndef __WIN32__
-    if (fdin >= FD_SETSIZE) {
-	// Not ideal, but OK for how this method is currently used.
-	RETURN(true);
-    }
-# endif
-    fd_set fdset;
-    FD_ZERO(&fdset);
-    FD_SET(fdin, &fdset);
-
-    struct timeval tv;
-    tv.tv_sec = 0;
-    tv.tv_usec = 0;
-    RETURN(select(fdin + 1, &fdset, 0, 0, &tv) > 0);
-#endif
-}
-
 #ifndef __WIN32__
 ssize_t
 RemoteConnection::send_or_write(const void* p, size_t len)
@@ -331,8 +297,7 @@ RemoteConnection::send_or_write(const void* p, size_t len)
 #endif
 
 void
-RemoteConnection::send_message(char type, const string &message,
-			       double end_time)
+RemoteConnection::send_message(char type, string_view message, double end_time)
 {
     LOGCALL_VOID(REMOTE, "RemoteConnection::send_message", type | message | end_time);
     if (fdout == -1)
@@ -340,11 +305,12 @@ RemoteConnection::send_message(char type, const string &message,
 
     string header;
     header += type;
-    header += encode_length(message.size());
+    pack_uint(header, message.size());
+    string_view header_view = header;
 
 #ifdef __WIN32__
     HANDLE hout = fd_to_handle(fdout);
-    const string * str = &header;
+    const string_view* str = &header_view;
 
     size_t count = 0;
     while (true) {
@@ -385,7 +351,7 @@ RemoteConnection::send_message(char type, const string &message,
 				   context, errno);
     }
 
-    const string * str = &header;
+    const string_view* str = &header_view;
 
     size_t count = 0;
     while (true) {
@@ -465,7 +431,7 @@ RemoteConnection::send_file(char type, int fd, double end_time)
     if (fdout == -1)
 	throw_database_closed();
 
-    off_t size = file_size(fd);
+    auto size = file_size(fd);
     if (errno)
 	throw Xapian::NetworkError("Couldn't stat file to send", errno);
     // FIXME: Use sendfile() or similar if available?
@@ -474,7 +440,8 @@ RemoteConnection::send_file(char type, int fd, double end_time)
     buf[0] = type;
     size_t c = 1;
     {
-	string enc_size = encode_length(size);
+	string enc_size;
+	pack_uint(enc_size, size);
 	c += enc_size.size();
 	// An encoded length should be just a few bytes.
 	AssertRel(c, <=, sizeof(buf));
@@ -631,30 +598,29 @@ RemoteConnection::get_message(string &result, double end_time)
 
     if (!read_at_least(2, end_time))
 	RETURN(-1);
+    // This code assume things about the pack_uint() encoding in order to
+    // handle partial reads.
     size_t len = static_cast<unsigned char>(buffer[1]);
-    if (!read_at_least(len + 2, end_time))
-	RETURN(-1);
-    if (len != 0xff) {
+    if (len < 128) {
+	if (!read_at_least(len + 2, end_time))
+	    RETURN(-1);
 	result.assign(buffer.data() + 2, len);
 	unsigned char type = buffer[0];
 	buffer.erase(0, len + 2);
 	RETURN(type);
     }
-    len = 0;
-    string::const_iterator i = buffer.begin() + 2;
-    unsigned char ch;
-    int shift = 0;
-    do {
-	if (i == buffer.end() || shift > 28) {
-	    // Something is very wrong...
-	    throw_network_error_insane_message_length();
-	}
-	ch = *i++;
-	len |= size_t(ch & 0x7f) << shift;
-	shift += 7;
-    } while ((ch & 0x80) == 0);
-    len += 255;
-    size_t header_len = (i - buffer.begin());
+
+    // We know the message payload is at least 128 bytes of data, and if we
+    // read that much we'll definitely have the whole of the length.
+    if (!read_at_least(128 + 2, end_time))
+	RETURN(-1);
+    const char* p = buffer.data();
+    const char* p_end = p + buffer.size();
+    ++p;
+    if (!unpack_uint(&p, p_end, &len)) {
+	RETURN(-1);
+    }
+    size_t header_len = (p - buffer.data());
     if (!read_at_least(header_len + len, end_time))
 	RETURN(-1);
     result.assign(buffer.data() + header_len, len);
@@ -673,47 +639,33 @@ RemoteConnection::get_message_chunked(double end_time)
 
     if (!read_at_least(2, end_time))
 	RETURN(-1);
+    // This code assume things about the pack_uint() encoding in order to
+    // handle partial reads.
     uint_least64_t len = static_cast<unsigned char>(buffer[1]);
-    if (len != 0xff) {
+    if (len < 128) {
 	chunked_data_left = off_t(len);
 	char type = buffer[0];
 	buffer.erase(0, 2);
 	RETURN(type);
     }
-    if (!read_at_least(len + 2, end_time))
+
+    // We know the message payload is at least 128 bytes of data, and if we
+    // read that much we'll definitely have the whole of the length.
+    if (!read_at_least(128 + 2, end_time))
+	RETURN(-1);
+    const char* p = buffer.data();
+    const char* p_end = p + buffer.size();
+    ++p;
+    if (!unpack_uint(&p, p_end, &len)) {
 	RETURN(-1);
-    len = 0;
-    string::const_iterator i = buffer.begin() + 2;
-    unsigned char ch;
-    int shift = 0;
-    do {
-	// Allow at most 63 bits for message lengths - it's neatly a multiple
-	// of 7 bits and anything longer than this is almost certainly a
-	// corrupt value.
-	// The value also needs to be representable as an
-	// off_t (which is a signed type), so use that size if it is smaller.
-	const int SHIFT_LIMIT = 63;
-	if (rare(i == buffer.end() || shift >= SHIFT_LIMIT)) {
-	    // Something is very wrong...
-	    throw_network_error_insane_message_length();
-	}
-	ch = *i++;
-	uint_least64_t bits = ch & 0x7f;
-	len |= bits << shift;
-	shift += 7;
-    } while ((ch & 0x80) == 0);
-    len += 255;
-
-    chunked_data_left = off_t(len);
-    if (sizeof(off_t) * CHAR_BIT < 63) {
-	// Check that the value of len fits in an off_t without loss.
-	if (rare(uint_least64_t(chunked_data_left) != len)) {
-	    throw_network_error_insane_message_length();
-	}
     }
-
+    chunked_data_left = off_t(len);
+    // Check that the value of len fits in an off_t without loss.
+    if (rare(uint_least64_t(chunked_data_left) != len)) {
+	throw_network_error_message_too_long_for_off_t();
+    }
+    size_t header_len = (p - buffer.data());
     unsigned char type = buffer[0];
-    size_t header_len = (i - buffer.begin());
     buffer.erase(0, header_len);
     RETURN(type);
 }
@@ -791,7 +743,7 @@ RemoteConnection::shutdown()
 
     // We can be called from a destructor, so we can't throw an exception.
     try {
-	send_message(MSG_SHUTDOWN, string(), 0.0);
+	send_message(MSG_SHUTDOWN, {}, 0.0);
 #ifdef __WIN32__
 	HANDLE hin = fd_to_handle(fdin);
 	char dummy;
