@@ -1,7 +1,7 @@
 /** @file
  *  @brief Xapian remote backend server base class
  */
-/* Copyright (C) 2006,2007,2008,2009,2010,2014,2019 Olly Betts
+/* Copyright (C) 2006-2024 Olly Betts
  * Copyright (C) 2007,2009,2010 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -44,10 +44,10 @@ class XAPIAN_VISIBILITY_DEFAULT RemoteServer : private RemoteConnection {
      *
      *  If we're writable, this is the same as wdb.
      */
-    Xapian::Database * db;
+    Xapian::Database* db = nullptr;
 
     /// The WritableDatabase we're using, or NULL if we're read-only.
-    Xapian::WritableDatabase * wdb;
+    Xapian::WritableDatabase* wdb = nullptr;
 
     /// Do we support writing?
     bool writable;
@@ -70,121 +70,174 @@ class XAPIAN_VISIBILITY_DEFAULT RemoteServer : private RemoteConnection {
     Xapian::Registry reg;
 
     /// Accept a message from the client.
+    XAPIAN_VISIBILITY_INTERNAL
     message_type get_message(double timeout, std::string & result,
 			     message_type required_type = MSG_MAX);
 
     /// Send a message to the client.
-    void send_message(reply_type type, const std::string &message);
+    XAPIAN_VISIBILITY_INTERNAL
+    void send_message(reply_type type, std::string_view message);
 
     /// Send a message to the client, with specific end_time.
-    void send_message(reply_type type, const std::string &message,
+    XAPIAN_VISIBILITY_INTERNAL
+    void send_message(reply_type type, std::string_view message,
 		      double end_time) {
 	unsigned char type_as_char = static_cast<unsigned char>(type);
 	RemoteConnection::send_message(type_as_char, message, end_time);
     }
 
     // all terms
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_allterms(const std::string & message);
 
     // get document
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_document(const std::string & message);
 
     // term exists?
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_termexists(const std::string & message);
 
     // get collection freq
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_collfreq(const std::string & message);
 
     // get termfreq
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_termfreq(const std::string & message);
 
     // get termfreq and collection freq
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_freqs(const std::string & message);
 
     // get value statistics
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_valuestats(const std::string & message);
 
     // keep alive
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_keepalive(const std::string & message);
 
     // get doclength
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_doclength(const std::string & message);
 
     // set the query; return the mset
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_query(const std::string & message);
 
     // get termlist
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_termlist(const std::string & message);
 
     // get postlist
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_postlist(const std::string & message);
 
     // get positionlist
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_positionlist(const std::string &message);
 
+    // get positionlist count
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_positionlistcount(const std::string &message);
+
     // get write access
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_writeaccess(const std::string & message);
 
     // reopen
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_reopen(const std::string & message);
 
     // get updated doccount and avlength
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_update(const std::string &message);
 
     // commit
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_commit(const std::string & message);
 
-    // cancel (compat)
-    void msg_cancel_(const std::string &message);
+    // cancel
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_cancel(const std::string &message);
 
     // add document
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_adddocument(const std::string & message);
 
     // delete document
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_deletedocument(const std::string & message);
 
-    // delete document with unique term (compat)
-    void msg_deletedocumentterm_(const std::string & message);
-
-    // replace document (compat)
-    void msg_replacedocument_(const std::string & message);
-
-    // replace document with unique term
-    void msg_replacedocumentterm(const std::string & message);
-
-    // get metadata
-    void msg_getmetadata(const std::string & message);
-
-    // read metadata key list
-    void msg_openmetadatakeylist(const std::string & message);
-
-    // set metadata (compat)
-    void msg_setmetadata_(const std::string & message);
-
-    // add a spelling (compat)
-    void msg_addspelling_(const std::string & message);
-
-    // remove a spelling
-    void msg_removespelling(const std::string & message);
-
-    // get number of unique terms
-    void msg_uniqueterms(const std::string & message);
-
     // delete document with unique term
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_deletedocumentterm(const std::string & message);
 
     // replace document
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_replacedocument(const std::string & message);
 
-    // cancel
-    void msg_cancel(const std::string &message);
+    // replace document with unique term
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_replacedocumentterm(const std::string & message);
+
+    // get metadata
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_getmetadata(const std::string & message);
+
+    // read metadata key list
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_metadatakeylist(const std::string & message);
 
     // set metadata
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_setmetadata(const std::string & message);
 
+    // request a document (pre-fetch hint)
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_requestdocument(const std::string& message);
+
     // add a spelling
+    XAPIAN_VISIBILITY_INTERNAL
     void msg_addspelling(const std::string & message);
 
+    // remove a spelling
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_removespelling(const std::string & message);
+
+    // get number of unique terms
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_uniqueterms(const std::string & message);
+
+    // get max_wdf
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_wdfdocmax(const std::string& message);
+
+    // reconstruct document text
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_reconstructtext(const std::string& message);
+
+    // get synonyms for a term
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_synonymtermlist(const std::string& message);
+
+    // get terms with an entry in synonym table, starting with a prefix
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_synonymkeylist(const std::string& message);
+
+    // add a synonym
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_addsynonym(const std::string& message);
+
+    // remove a synonym
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_removesynonym(const std::string& message);
+
+    // clear synonyms for a term
+    XAPIAN_VISIBILITY_INTERNAL
+    void msg_clearsynonyms(const std::string& message);
+
   public:
     /** Construct a RemoteServer.
      *
