@@ -1,7 +1,7 @@
 /** @file
  *  @brief TCP/IP socket based RemoteDatabase implementation
  */
-/* Copyright (C) 2007,2008,2010,2011,2014 Olly Betts
+/* Copyright (C) 2007,2008,2010,2011,2014,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,10 @@
 #define XAPIAN_INCLUDED_REMOTETCPCLIENT_H
 
 #include "backends/remote/remote-database.h"
+#include "socket_utils.h"
+
+#include <string>
+#include <string_view>
 
 #ifdef __WIN32__
 # define SOCKET_INITIALIZER_MIXIN private WinsockInitializer,
@@ -45,21 +49,18 @@ class RemoteTcpClient : SOCKET_INITIALIZER_MIXIN public RemoteDatabase {
      *  Connect to xapian-tcpsrv running on port @a port of host @a hostname.
      *  Give up trying to connect after @a timeout_connect seconds.
      *
+     *  @return A std::pair containing the file descriptor for the connection
+     *		to the child process and a context string to return with any
+     *		error messages.
+     *
      *  Note: this method is called early on during class construction before
      *  any member variables or even the base class have been initialised.
      *  To help avoid accidentally trying to use member variables or call other
      *  methods which do, this method has been deliberately made "static".
      */
-    static int open_socket(const std::string & hostname, int port,
-			   double timeout_connect);
-
-    /** Get a context string for use when constructing Xapian::NetworkError.
-     *
-     *  Note: this method is used from constructors so has been made static to
-     *  avoid problems with trying to use uninitialised member variables.  In
-     *  particular, it can't be made a virtual method of the base class.
-     */
-    static std::string get_tcpcontext(const std::string & hostname, int port);
+    static std::pair<int, std::string> open_socket(std::string_view hostname,
+						   int port,
+						   double timeout_connect);
 
   public:
     /** Constructor.
@@ -73,12 +74,11 @@ class RemoteTcpClient : SOCKET_INITIALIZER_MIXIN public RemoteDatabase {
      *	@param writable		Is this a WritableDatabase?
      *	@param flags		Xapian::DB_RETRY_LOCK or 0.
      */
-    RemoteTcpClient(const std::string & hostname, int port,
+    RemoteTcpClient(std::string_view hostname, int port,
 		    double timeout_, double timeout_connect, bool writable,
 		    int flags)
 	: RemoteDatabase(open_socket(hostname, port, timeout_connect),
-			 timeout_, get_tcpcontext(hostname, port),
-			 writable, flags) { }
+			 timeout_, writable, flags) { }
 
     /** Destructor. */
     ~RemoteTcpClient();
