@@ -22,6 +22,7 @@
 #define XAPIAN_INCLUDED_REPLICATETCPCLIENT_H
 
 #include "remoteconnection.h"
+#include "socket_utils.h"
 
 #include "xapian/visibility.h"
 #include "api/replication.h"
@@ -56,6 +57,7 @@ class XAPIAN_VISIBILITY_DEFAULT ReplicateTcpClient SOCKET_INITIALIZER_MIXIN {
      *  To help avoid accidentally trying to use member variables or call other
      *  methods which do, this method has been deliberately made "static".
      */
+    XAPIAN_VISIBILITY_INTERNAL
     static int open_socket(const std::string & hostname, int port,
 			   double timeout_connect);
 
