@@ -22,7 +22,7 @@
 
 #include <xapian/error.h>
 
-#include "length.h"
+#include "pack.h"
 #include "serialise-error.h"
 
 #include <string>
@@ -34,10 +34,8 @@ serialise_error(const Xapian::Error &e)
 {
     // The byte before the type name is the type code.
     string result(1, (e.get_type())[-1]);
-    result += encode_length(e.get_context().length());
-    result += e.get_context();
-    result += encode_length(e.get_msg().length());
-    result += e.get_msg();
+    pack_string(result, e.get_context());
+    pack_string(result, e.get_msg());
     // The "error string" goes last so we don't need to store its length.
     const char * err = e.get_error_string();
     if (err) result += err;
@@ -54,15 +52,12 @@ unserialise_error(const string &serialised_error, const string &prefix,
     if (p != end) {
 	char type = *p++;
 
-	size_t len;
-	decode_length_and_check(&p, end, len);
-	string context(p, len);
-	p += len;
-
-	decode_length_and_check(&p, end, len);
+	string context;
 	string msg(prefix);
-	msg.append(p, len);
-	p += len;
+	if (!unpack_string(&p, end, context) ||
+	    !unpack_string_append(&p, end, msg)) {
+	    unpack_throw_serialisation_error(p);
+	}
 
 	const char * error_string = (p == end) ? NULL : p;
 
