@@ -1,9 +1,7 @@
 /** @file
- * @brief Open a TCP connection to a server.
+ *  @brief Open a TCP connection to a server.
  */
-/* Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002 Ananova Ltd
- * Copyright 2004,2005,2006,2007,2008,2010,2012,2013,2015,2017 Olly Betts
+/* Copyright 2007-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,25 +15,22 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
 
 #include "tcpclient.h"
 
-#include "remoteconnection.h"
-#include "resolver.h"
-#include "str.h"
 #include <xapian/error.h>
 
 #include "realtime.h"
-#include "safefcntl.h"
-#include "safenetdb.h"
-#include "safesyssocket.h"
+#include "resolver.h"
 #include "socket_utils.h"
 
+#include "safefcntl.h"
+#include "safesyssocket.h"
+
 #ifdef HAVE_POLL_H
 # include <poll.h>
 #else
@@ -43,27 +38,33 @@
 #endif
 
 #include <cerrno>
-#include <cstring>
 #ifndef __WIN32__
 # include <netinet/in.h>
 # include <netinet/tcp.h>
 #endif
 
+#include <string_view>
+
 using namespace std;
 
 int
-TcpClient::open_socket(const std::string & hostname, int port,
-		       double timeout_connect, bool tcp_nodelay)
+TcpClient::open_socket(std::string_view hostname, int port,
+		       double timeout_connect, bool tcp_nodelay,
+		       const string& context)
 {
     int socketfd = -1;
     int connect_errno = 0;
     for (auto&& r : Resolver(hostname, port)) {
 	int socktype = r.ai_socktype | SOCK_CLOEXEC;
 #ifdef SOCK_NONBLOCK
+	// Set the socket as non-blocking so we can implement a timeout on
+	// the connection attempt by using select() or poll().  If
+	// SOCK_NONBLOCK is available we can get socket() to do this, but
+	// if not we call fcntl()/ioctlsocket() below to set it.
 	socktype |= SOCK_NONBLOCK;
 #endif
 	int fd = socket(r.ai_family, socktype, r.ai_protocol);
-	if (fd == -1)
+	if (fd < 0)
 	    continue;
 
 #if !defined __WIN32__ && defined F_SETFD && defined FD_CLOEXEC
@@ -76,8 +77,10 @@ TcpClient::open_socket(const std::string & hostname, int port,
 
 #ifndef SOCK_NONBLOCK
 #ifdef __WIN32__
-	ULONG enabled = 1;
-	int rc = ioctlsocket(fd, FIONBIO, &enabled);
+	int rc = [&]() {
+	    ULONG on = 1;
+	    return ioctlsocket(fd, FIONBIO, &on);
+	}();
 #define FLAG_NAME "FIONBIO"
 #elif defined O_NONBLOCK
 	int rc = fcntl(fd, F_SETFL, O_NONBLOCK);
@@ -87,24 +90,28 @@ TcpClient::open_socket(const std::string & hostname, int port,
 #define FLAG_NAME "O_NDELAY"
 #endif
 	if (rc < 0) {
-	    int saved_errno = socket_errno(); // note down in case close hits an error
+	    int saved_errno = socket_errno();
 	    CLOSESOCKET(fd);
-	    throw Xapian::NetworkError("Couldn't set " FLAG_NAME, saved_errno);
+	    throw Xapian::NetworkError("Couldn't set " FLAG_NAME,
+				       context,
+				       saved_errno);
 #undef FLAG_NAME
 	}
 #endif
 
 	if (tcp_nodelay) {
-	    int optval = 1;
+	    int on = 1;
 	    // 4th argument might need to be void* or char* - cast it to char*
 	    // since C++ allows implicit conversion to void* but not from
 	    // void*.
 	    if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY,
-			   reinterpret_cast<char *>(&optval),
-			   sizeof(optval)) < 0) {
-		int saved_errno = socket_errno(); // note down in case close hits an error
+			   reinterpret_cast<char*>(&on),
+			   sizeof(on)) < 0) {
+		int setsockopt_errno = socket_errno();
 		CLOSESOCKET(fd);
-		throw Xapian::NetworkError("Couldn't set TCP_NODELAY", saved_errno);
+		throw Xapian::NetworkError("Couldn't set TCP_NODELAY",
+					   context,
+					   setsockopt_errno);
 	    }
 	}
 
@@ -116,14 +123,14 @@ TcpClient::open_socket(const std::string & hostname, int port,
 
 	int err = socket_errno();
 	if (
-#ifdef __WIN32__
-	    WSAGetLastError() == WSAEWOULDBLOCK
-#else
+#ifndef __WIN32__
 	    err == EINPROGRESS
+#else
+	    err == WSAEWOULDBLOCK
 #endif
-	    ) {
+	   ) {
 	    // Wait for the socket to be writable or give an error, with a
-	    // timeout.
+	    // timeout.  FIXME: Reduce the timeout if we retry.
 #ifdef HAVE_POLL
 	    struct pollfd fds;
 	    fds.fd = fd;
@@ -131,26 +138,33 @@ TcpClient::open_socket(const std::string & hostname, int port,
 	    do {
 		retval = poll(&fds, 1, int(timeout_connect * 1000));
 	    } while (retval < 0 && (errno == EINTR || errno == EAGAIN));
+# define FUNC_NAME "poll()"
 #else
 	    fd_set fdset;
 	    FD_ZERO(&fdset);
 	    do {
 		FD_SET(fd, &fdset);
-		// FIXME: Reduce the timeout if we retry on EINTR.
 		struct timeval tv;
 		RealTime::to_timeval(timeout_connect, &tv);
 		retval = select(fd + 1, 0, &fdset, 0, &tv);
 	    } while (retval < 0 && (errno == EINTR || errno == EAGAIN));
+# define FUNC_NAME "select()"
 #endif
 
 	    if (retval <= 0) {
 		int saved_errno = errno;
 		CLOSESOCKET(fd);
-		if (retval < 0)
-		    throw Xapian::NetworkError("Couldn't connect (poll() or "
-					       "select() on socket failed)",
+		if (retval < 0) {
+		    throw Xapian::NetworkError("Couldn't connect ("
+					       FUNC_NAME " on socket failed)",
+					       context,
 					       saved_errno);
-		throw Xapian::NetworkTimeoutError("Timed out waiting to connect", ETIMEDOUT);
+#undef FUNC_NAME
+		}
+		throw Xapian::NetworkTimeoutError("Timed out waiting to "
+						  "connect",
+						  context,
+						  ETIMEDOUT);
 	    }
 
 	    err = 0;
@@ -159,12 +173,14 @@ TcpClient::open_socket(const std::string & hostname, int port,
 	    // 4th argument might need to be void* or char* - cast it to char*
 	    // since C++ allows implicit conversion to void* but not from void*.
 	    retval = getsockopt(fd, SOL_SOCKET, SO_ERROR,
-				reinterpret_cast<char *>(&err), &len);
+				reinterpret_cast<char*>(&err), &len);
 
 	    if (retval < 0) {
-		int saved_errno = socket_errno(); // note down in case close hits an error
+		int getsockopt_errno = socket_errno();
 		CLOSESOCKET(fd);
-		throw Xapian::NetworkError("Couldn't get socket options", saved_errno);
+		throw Xapian::NetworkError("getsockopt failed",
+					   context,
+					   getsockopt_errno);
 	    }
 	    if (err == 0) {
 		// Connected successfully.
@@ -183,15 +199,19 @@ TcpClient::open_socket(const std::string & hostname, int port,
 	CLOSESOCKET(fd);
     }
 
-    if (socketfd == -1) {
-	throw Xapian::NetworkError("Couldn't connect", connect_errno);
+    if (socketfd < 0) {
+	throw Xapian::NetworkError("connect failed",
+				   context,
+				   connect_errno);
     }
 
-#ifdef __WIN32__
-    ULONG enabled = 0;
-    ioctlsocket(socketfd, FIONBIO, &enabled);
-#else
+    // Set the socket to be blocking.
+#ifndef __WIN32__
     fcntl(socketfd, F_SETFL, 0);
+#else
+    ULONG off = 0;
+    ioctlsocket(socketfd, FIONBIO, &off);
 #endif
+
     return socketfd;
 }
