@@ -1,7 +1,7 @@
 /** @file
  *  @brief Generic TCP/IP socket based server base class.
  */
-/* Copyright (C) 2007,2008 Olly Betts
+/* Copyright (C) 2007,2008,2023 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,58 +22,33 @@
 #define XAPIAN_INCLUDED_TCPSERVER_H
 
 #ifdef __WIN32__
-# include "remoteconnection.h"
+# include "socket_utils.h"
 # define SOCKET_INITIALIZER_MIXIN : private WinsockInitializer
 #else
 # define SOCKET_INITIALIZER_MIXIN
 #endif
 
-#if defined __CYGWIN__ || defined __WIN32__
-# include "safewindows.h" // Only for HANDLE!
-#endif
-
 #include <xapian/visibility.h>
 
 #include <string>
 
-/** TCP/IP socket based server for RemoteDatabase.
- *
- *  This class implements the server used by xapian-tcpsrv.
- */
+/** Generic TCP/IP socket based server base class. */
 class XAPIAN_VISIBILITY_DEFAULT TcpServer SOCKET_INITIALIZER_MIXIN {
     /// Don't allow assignment.
-    void operator=(const TcpServer &);
+    TcpServer& operator=(const TcpServer&) = delete;
 
     /// Don't allow copying.
-    TcpServer(const TcpServer &);
-
-#if defined __CYGWIN__ || defined __WIN32__
-    /// Mutex to stop two TcpServers running on the same port.
-    HANDLE mutex = NULL;
-#endif
+    TcpServer(const TcpServer&) = delete;
 
     /** The socket we're listening on. */
-    int listen_socket;
-
-    /** Create a listening socket ready to accept connections.
-     *
-     *  @param host	hostname or address to listen on or an empty string to
-     *			accept connections on any interface.
-     *  @param port	TCP port to listen on.
-     *  @param tcp_nodelay	If true, enable TCP_NODELAY option.
-     */
-    static int get_listening_socket(const std::string & host, int port,
-				    bool tcp_nodelay
-#if defined __CYGWIN__ || defined __WIN32__
-				    , HANDLE &mutex
-#endif
-	    );
+    int listener;
 
   protected:
     /** Should we produce output when connections are made or lost? */
     bool verbose;
 
     /** Accept a connection and return the file descriptor for it. */
+    XAPIAN_VISIBILITY_INTERNAL
     int accept_connection();
 
   public:
@@ -86,11 +61,16 @@ class XAPIAN_VISIBILITY_DEFAULT TcpServer SOCKET_INITIALIZER_MIXIN {
      *	@param verbose	Should we produce output when connections are
      *			made or lost?
      */
-    TcpServer(const std::string &host, int port, bool tcp_nodelay,
-	      bool verbose);
+    TcpServer(const std::string& host, int port, bool tcp_nodelay,
+	      bool verbose_);
 
-    /** Destructor. */
-    virtual ~TcpServer();
+    /** Destructor.
+     *
+     *  Note: We don't need a virtual destructor despite having a virtual
+     *  method as we don't ever delete a subclass using a pointer to the
+     *  base class.
+     */
+    ~TcpServer();
 
     /** Accept connections and service requests indefinitely.
      *
@@ -103,8 +83,11 @@ class XAPIAN_VISIBILITY_DEFAULT TcpServer SOCKET_INITIALIZER_MIXIN {
     /** Accept a single connection, service requests on it, then stop.  */
     void run_once();
 
+    /// Should we produce output when connections are made or lost?
+    bool get_verbose() const { return verbose; }
+
     /// Handle a single connection on an already connected socket.
     virtual void handle_one_connection(int socket) = 0;
 };
 
-#endif  // XAPIAN_INCLUDED_TCPSERVER_H
+#endif // XAPIAN_INCLUDED_TCPSERVER_H
