@@ -1,7 +1,7 @@
-/** @file termgenerator.cc
+/** @file
  * @brief TermGenerator class implementation
  */
-/* Copyright (C) 2007,2012 Olly Betts
+/* Copyright (C) 2007,2012,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,16 +28,15 @@
 
 #include "str.h"
 
+#include <string_view>
+
 using namespace std;
 using namespace Xapian;
 
-TermGenerator::TermGenerator(const TermGenerator & o) : internal(o.internal) { }
+TermGenerator::TermGenerator(const TermGenerator &) = default;
 
 TermGenerator &
-TermGenerator::operator=(const TermGenerator & o) {
-    internal = o.internal;
-    return *this;
-}
+TermGenerator::operator=(const TermGenerator &) = default;
 
 TermGenerator::TermGenerator(TermGenerator &&) = default;
 
@@ -108,7 +107,7 @@ TermGenerator::set_max_word_length(unsigned max_word_length)
 void
 TermGenerator::index_text(const Xapian::Utf8Iterator & itor,
 			  Xapian::termcount weight,
-			  const string & prefix)
+			  string_view prefix)
 {
     internal->index_text(itor, weight, prefix, true);
 }
@@ -116,7 +115,7 @@ TermGenerator::index_text(const Xapian::Utf8Iterator & itor,
 void
 TermGenerator::index_text_without_positions(const Xapian::Utf8Iterator & itor,
 					    Xapian::termcount weight,
-					    const string & prefix)
+					    string_view prefix)
 {
     internal->index_text(itor, weight, prefix, false);
 }
@@ -139,12 +138,18 @@ TermGenerator::set_termpos(Xapian::termpos termpos)
     internal->cur_pos = termpos;
 }
 
+void
+TermGenerator::set_termpos_limit(Xapian::termpos termpos_limit)
+{
+    internal->pos_limit = termpos_limit;
+}
+
 string
 TermGenerator::get_description() const
 {
     string s("Xapian::TermGenerator(stem=");
     s += internal->stemmer.get_description();
-    if (internal->stopper.get()) {
+    if (internal->stopper) {
 	s += ", stopper set";
     }
     s += ", doc=";
