@@ -1,7 +1,7 @@
 /** @file
  * @brief TermGenerator class internals
  */
-/* Copyright (C) 2007-2023 Olly Betts
+/* Copyright (C) 2007-2026 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "termgenerator_internal.h"
 
-#include "api/omenquireinternal.h"
+#include "api/msetinternal.h"
 #include "api/queryinternal.h"
 
 #include <xapian/document.h>
@@ -38,6 +38,7 @@
 #include <limits>
 #include <list>
 #include <string>
+#include <string_view>
 #include <unordered_map>
 #include <vector>
 
@@ -90,7 +91,12 @@ check_infix(unsigned ch)
     // 0x2019 is Unicode apostrophe and single closing quote.
     // 0x201b is Unicode single opening quote with the tail rising.
     if (ch == 0x2019 || ch == 0x201b) return '\'';
-    if (ch >= 0x200b && (ch <= 0x200d || ch == 0x2060 || ch == 0xfeff))
+    // 0x200c and 0x200d are zero width non-joiner and joiner respectively.
+    // 0x2060 and 0xfeff are word joiners (0xfeff deprecated since Unicode 3.2).
+    if (ch >= 0x200c && (ch <= 0x200d || ch == 0x2060 || ch == 0xfeff))
+	return UNICODE_IGNORE;
+    // 0xad is SOFT HYPHEN which marks a potential hyphenation point in a word.
+    if (ch == 0xad)
 	return UNICODE_IGNORE;
     return 0;
 }
@@ -131,6 +137,47 @@ check_suffix(unsigned ch)
     return 0;
 }
 
+static_assert(int(MSet::SNIPPET_WORD_BREAKS) == TermGenerator::FLAG_WORD_BREAKS,
+	      "WORD_BREAKS flags have same value");
+
+template<typename ACTION>
+static bool
+break_words(Utf8Iterator& itor, unsigned break_flags, bool with_positions,
+	    ACTION action)
+{
+#ifdef USE_ICU
+    if (break_flags & MSet::SNIPPET_WORD_BREAKS) {
+	const char* start = itor.raw();
+	// get_unbroken() returns the number of codepoints, which we aren't
+	// interested in here.
+	(void)get_unbroken(itor);
+	size_t left = itor.raw() - start;
+	for (WordIterator tk(start, left); tk != WordIterator(); ++tk) {
+	    const string& token = *tk;
+	    left -= token.length();
+	    if (!action(token, with_positions, itor.left() + left))
+		return false;
+	}
+	return true;
+    }
+#else
+    (void)break_flags;
+#endif
+
+    NgramIterator tk(itor);
+    while (tk != NgramIterator()) {
+	const string& token = *tk;
+	// FLAG_NGRAMS only sets positions for tokens of length 1.
+	bool with_pos = with_positions && tk.unigram();
+	if (!action(token, with_pos, tk.get_utf8iterator().left()))
+	    return false;
+	++tk;
+    }
+    // Update itor to point the end of the span of text in an unbroken script.
+    itor = tk.get_utf8iterator();
+    return true;
+}
+
 /** Templated framework for processing terms.
  *
  *  Calls action(term, positional) for each term to add, where term is a
@@ -139,8 +186,8 @@ check_suffix(unsigned ch)
  */
 template<typename ACTION>
 static void
-parse_terms(Utf8Iterator itor, bool try_word_break, bool with_positions,
-            ACTION action)
+parse_terms(Utf8Iterator itor, unsigned break_flags, bool with_positions,
+	    ACTION action)
 {
     while (true) {
 	// Advance to the start of the next term.
@@ -175,19 +222,9 @@ parse_terms(Utf8Iterator itor, bool try_word_break, bool with_positions,
 	}
 
 	while (true) {
-	    if (try_word_break &&
-		is_unbroken_script(*itor) &&
-		Unicode::is_wordchar(*itor)) {
-		NgramIterator tk(itor);
-		while (tk != NgramIterator()) {
-		    const string& ngram = *tk;
-		    if (!action(ngram, with_positions && tk.unigram(),
-				tk.get_utf8iterator()))
-			return;
-		    ++tk;
-		}
-		// Update itor to end of CJK text span.
-		itor = tk.get_utf8iterator();
+	    if (break_flags && is_unbroken_wordchar(*itor)) {
+		if (!break_words(itor, break_flags, with_positions, action))
+		    return;
 		while (true) {
 		    if (itor == Utf8Iterator()) return;
 		    ch = check_wordchar(*itor);
@@ -201,7 +238,7 @@ parse_terms(Utf8Iterator itor, bool try_word_break, bool with_positions,
 		Unicode::append_utf8(term, ch);
 		prevch = ch;
 		if (++itor == Utf8Iterator() ||
-		    (try_word_break && is_unbroken_script(*itor)))
+		    (break_flags && is_unbroken_script(*itor)))
 		    goto endofterm;
 		ch = check_wordchar(*itor);
 	    } while (ch);
@@ -242,32 +279,57 @@ parse_terms(Utf8Iterator itor, bool try_word_break, bool with_positions,
 	}
 
 endofterm:
-	if (!action(term, with_positions, itor))
+	if (!action(term, with_positions, itor.left()))
 	    return;
     }
 }
 
 void
 TermGenerator::Internal::index_text(Utf8Iterator itor, termcount wdf_inc,
-				    const string & prefix, bool with_positions)
+				    string_view prefix, bool with_positions)
 {
-    bool try_word_break = (flags & FLAG_NGRAMS) || is_ngram_enabled();
+#ifndef USE_ICU
+    if (flags & FLAG_WORD_BREAKS) {
+	throw Xapian::FeatureUnavailableError("FLAG_WORD_BREAKS requires "
+					      "building Xapian to use ICU");
+    }
+#endif
+    unsigned break_flags = flags & (FLAG_NGRAMS | FLAG_WORD_BREAKS);
+    if (break_flags == 0 && is_ngram_enabled()) {
+	break_flags = FLAG_NGRAMS;
+    }
 
     stop_strategy current_stop_mode;
-    if (!stopper.get()) {
+    if (!stopper) {
 	current_stop_mode = TermGenerator::STOP_NONE;
     } else {
 	current_stop_mode = stop_mode;
     }
 
-    parse_terms(itor, try_word_break, with_positions,
-	[=
+    // Create two std::string objects which we effectively use as buffers to
+    // build the prefixed terms we generate (one for non-stemmed and one for
+    // stemmed terms).  This is a simple way to avoid creating a temporary
+    // std::string object each time we generate a prefixed term.
+    string prefixed_term;
+    auto prefix_size = prefix.size();
+    prefixed_term.reserve(prefix_size + max_word_length);
+    prefixed_term.assign(prefix);
+
+    string prefixed_stemmed_term;
+    int add_z = (strategy != TermGenerator::STEM_ALL);
+    prefixed_stemmed_term.reserve(add_z + prefix_size + max_word_length);
+    if (add_z) prefixed_stemmed_term.assign(1, 'Z');
+    prefixed_stemmed_term.append(prefix);
+    auto prefixed_stemmed_size = prefixed_stemmed_term.size();
+
+    parse_terms(itor, break_flags, with_positions,
+	[=, &prefixed_term, &prefixed_stemmed_term
 #if __cplusplus >= 201907L
 // C++20 no longer supports implicit `this` in lambdas but older C++ versions
 // don't allow `this` here.
 	, this
 #endif
-	](const string & term, bool positional, const Utf8Iterator &) {
+	](const string & term, bool positional, size_t) {
 	    if (term.size() > max_word_length) return true;
 
 	    if (current_stop_mode == TermGenerator::STOP_ALL &&
@@ -278,16 +340,20 @@ TermGenerator::Internal::index_text(Utf8Iterator itor, termcount wdf_inc,
 	    if (strategy == TermGenerator::STEM_SOME ||
 		strategy == TermGenerator::STEM_NONE ||
 		strategy == TermGenerator::STEM_SOME_FULL_POS) {
+		prefixed_term.append(term);
 		if (positional) {
-		    doc.add_posting(prefix + term, ++cur_pos, wdf_inc);
+		    if (rare(cur_pos >= pos_limit))
+			throw Xapian::RangeError("termpos limit exceeded");
+		    doc.add_posting(prefixed_term, ++cur_pos, wdf_inc);
 		} else {
-		    doc.add_term(prefix + term, wdf_inc);
+		    doc.add_term(prefixed_term, wdf_inc);
 		}
+		prefixed_term.resize(prefix_size);
 	    }
 
 	    // MSVC seems to need "this->" on member variables in this
 	    // situation.
-	    if ((this->flags & FLAG_SPELLING) && prefix.empty())
+	    if ((this->flags & FLAG_SPELLING) && prefix_size == 0)
 		db.add_spelling(term);
 
 	    if (strategy == TermGenerator::STEM_NONE || stemmer.is_none())
@@ -307,18 +373,19 @@ TermGenerator::Internal::index_text(Utf8Iterator itor, termcount wdf_inc,
 	    // Add stemmed form without positional information.
 	    const string& stem = stemmer(term);
 	    if (rare(stem.empty())) return true;
-	    string stemmed_term;
-	    if (strategy != TermGenerator::STEM_ALL) {
-		stemmed_term += "Z";
-	    }
-	    stemmed_term += prefix;
-	    stemmed_term += stem;
-	    if (strategy != TermGenerator::STEM_SOME && with_positions) {
-		if (strategy != TermGenerator::STEM_SOME_FULL_POS) ++cur_pos;
-		doc.add_posting(stemmed_term, cur_pos, wdf_inc);
+	    prefixed_stemmed_term.append(stem);
+	    if (strategy != TermGenerator::STEM_SOME && positional) {
+		if (strategy != TermGenerator::STEM_SOME_FULL_POS) {
+		    if (rare(cur_pos >= pos_limit))
+			throw Xapian::RangeError("termpos limit exceeded");
+		    ++cur_pos;
+		}
+		doc.add_posting(prefixed_stemmed_term, cur_pos, wdf_inc);
 	    } else {
-		doc.add_term(stemmed_term, wdf_inc);
+		doc.add_term(prefixed_stemmed_term, wdf_inc);
 	    }
+	    prefixed_stemmed_term.resize(prefixed_stemmed_size);
+
 	    return true;
 	});
 }
@@ -364,11 +431,11 @@ class SnipPipe {
 
     void done();
 
-    bool drain(const string & input,
-	       const string & hi_start,
-	       const string & hi_end,
-	       const string & omit,
-	       string & output);
+    bool drain(string_view input,
+	       string_view hi_start,
+	       string_view hi_end,
+	       string_view omit,
+	       string& output);
 };
 
 #define DECAY 2.0
@@ -541,11 +608,11 @@ append_escaping_xml(const char* p, const char* end, string& output)
 }
 
 inline bool
-SnipPipe::drain(const string & input,
-		const string & hi_start,
-		const string & hi_end,
-		const string & omit,
-		string & output)
+SnipPipe::drain(string_view input,
+		string_view hi_start,
+		string_view hi_end,
+		string_view omit,
+		string& output)
 {
     if (best_pipe.empty() && !pipe.empty()) {
 	swap(best_pipe, pipe);
@@ -684,7 +751,8 @@ static void
 check_query(const Xapian::Query & query,
 	    list<vector<string>> & exact_phrases,
 	    unordered_map<string, double> & loose_terms,
-	    list<string> & wildcards,
+	    list<const Xapian::Internal::QueryWildcard*> & wildcards,
+	    list<const Xapian::Internal::QueryEditDistance*> & fuzzies,
 	    size_t & longest_phrase)
 {
     // FIXME: OP_NEAR, non-tight OP_PHRASE, OP_PHRASE with non-term subqueries
@@ -695,9 +763,15 @@ check_query(const Xapian::Query & query,
 	    *static_cast<const Xapian::Internal::QueryTerm *>(query.internal.get());
 	loose_terms.insert(make_pair(qt.get_term(), 0));
     } else if (op == query.OP_WILDCARD) {
-	const Xapian::Internal::QueryWildcard & qw =
-	    *static_cast<const Xapian::Internal::QueryWildcard *>(query.internal.get());
-	wildcards.push_back(qw.get_pattern());
+	using Xapian::Internal::QueryWildcard;
+	const QueryWildcard* qw =
+	    static_cast<const QueryWildcard*>(query.internal.get());
+	wildcards.push_back(qw);
+    } else if (op == query.OP_EDIT_DISTANCE) {
+	using Xapian::Internal::QueryEditDistance;
+	const QueryEditDistance* qed =
+	    static_cast<const QueryEditDistance*>(query.internal.get());
+	fuzzies.push_back(qed);
     } else if (op == query.OP_PHRASE) {
 	const Xapian::Internal::QueryPhrase & phrase =
 	    *static_cast<const Xapian::Internal::QueryPhrase *>(query.internal.get());
@@ -725,7 +799,7 @@ check_query(const Xapian::Query & query,
 non_term_subquery:
     for (size_t i = 0; i != n_subqs; ++i)
 	check_query(query.get_subquery(i), exact_phrases, loose_terms,
-		    wildcards, longest_phrase);
+		    wildcards, fuzzies, longest_phrase);
 }
 
 static double*
@@ -751,22 +825,29 @@ check_term(unordered_map<string, double> & loose_terms,
 }
 
 string
-MSet::Internal::snippet(const string & text,
+MSet::Internal::snippet(string_view text,
 			size_t length,
 			const Xapian::Stem & stemmer,
 			unsigned flags,
-			const string & hi_start,
-			const string & hi_end,
-			const string & omit) const
+			string_view hi_start,
+			string_view hi_end,
+			string_view omit) const
 {
     if (hi_start.empty() && hi_end.empty() && text.size() <= length) {
 	// Too easy!
-	return text;
+	return string{text};
     }
 
-    bool try_word_break = (flags & MSet::SNIPPET_CJK_NGRAM);
-    if (!try_word_break) {
-	try_word_break = true;
+#ifndef USE_ICU
+    if (flags & MSet::SNIPPET_WORD_BREAKS) {
+	throw Xapian::FeatureUnavailableError("SNIPPET_WORD_BREAKS requires "
+					      "building Xapian to use ICU");
+    }
+#endif
+    auto SNIPPET_BREAK_MASK = MSet::SNIPPET_NGRAMS | MSet::SNIPPET_WORD_BREAKS;
+    unsigned break_flags = flags & SNIPPET_BREAK_MASK;
+    if (break_flags == 0 && is_ngram_enabled()) {
+	break_flags = MSet::SNIPPET_NGRAMS;
     }
 
     size_t term_start = 0;
@@ -783,17 +864,18 @@ MSet::Internal::snippet(const string & text,
     }
 
     Xapian::Query query;
-    if (enquire.get()) {
+    if (enquire) {
 	query = enquire->query;
     }
     SnipPipe snip(length);
 
     list<vector<string>> exact_phrases;
     unordered_map<string, double> loose_terms;
-    list<string> wildcards;
+    list<const Xapian::Internal::QueryWildcard*> wildcards;
+    list<const Xapian::Internal::QueryEditDistance*> fuzzies;
     size_t longest_phrase = 0;
     check_query(query, exact_phrases, loose_terms,
-		wildcards, longest_phrase);
+		wildcards, fuzzies, longest_phrase);
 
     vector<double> exact_phrases_relevance;
     exact_phrases_relevance.reserve(exact_phrases.size());
@@ -803,13 +885,21 @@ MSet::Internal::snippet(const string & text,
     }
 
     vector<double> wildcards_relevance;
-    wildcards_relevance.reserve(exact_phrases.size());
+    wildcards_relevance.reserve(wildcards.size());
     for (auto&& pattern : wildcards) {
 	// FIXME: What relevance to use?
 	(void)pattern;
 	wildcards_relevance.push_back(max_tw + min_tw);
     }
 
+    vector<double> fuzzies_relevance;
+    fuzzies_relevance.reserve(fuzzies.size());
+    for (auto&& pattern : fuzzies) {
+	// FIXME: What relevance to use?
+	(void)pattern;
+	fuzzies_relevance.push_back(max_tw + min_tw);
+    }
+
     // Background relevance is the same for a given MSet, so cache it
     // between calls to MSet::snippet() on the same object.
     unordered_map<string, double>& background = snippet_bg_relevance;
@@ -818,8 +908,8 @@ MSet::Internal::snippet(const string & text,
     if (longest_phrase) phrase.resize(longest_phrase - 1);
     size_t phrase_next = 0;
     bool matchfound = false;
-    parse_terms(Utf8Iterator(text), try_word_break, true,
-	[&](const string & term, bool positional, const Utf8Iterator & it) {
+    parse_terms(Utf8Iterator(text), break_flags, true,
+	[&](const string & term, bool positional, size_t left) {
 	    // FIXME: Don't hardcode this here.
 	    const size_t max_word_length = 64;
 
@@ -829,7 +919,7 @@ MSet::Internal::snippet(const string & text,
 	    // We get segments with any "inter-word" characters in front of
 	    // each word, e.g.:
 	    // [The][ cat][ sat][ on][ the][ mat]
-	    size_t term_end = text.size() - it.left();
+	    size_t term_end = text.size() - left;
 
 	    double* relevance = 0;
 	    size_t highlight = 0;
@@ -839,7 +929,7 @@ MSet::Internal::snippet(const string & text,
 		    if (term == terms.back()) {
 			size_t n = terms.size() - 1;
 			bool match = true;
-			while (n--) {
+			while (UNSIGNED_OVERFLOW_OK(n--)) {
 			    if (terms[n] != phrase[(n + phrase_next) % (longest_phrase - 1)]) {
 				match = false;
 				break;
@@ -855,7 +945,7 @@ MSet::Internal::snippet(const string & text,
 		    ++i;
 		}
 
-		relevance = check_term(loose_terms, stats, term, max_tw);
+		relevance = check_term(loose_terms, stats.get(), term, max_tw);
 		if (relevance) {
 		    // Matched unstemmed term.
 		    highlight = 1;
@@ -864,7 +954,7 @@ MSet::Internal::snippet(const string & text,
 
 		string stem = "Z";
 		stem += stemmer(term);
-		relevance = check_term(loose_terms, stats, stem, max_tw);
+		relevance = check_term(loose_terms, stats.get(), stem, max_tw);
 		if (relevance) {
 		    // Matched stemmed term.
 		    highlight = 1;
@@ -872,10 +962,10 @@ MSet::Internal::snippet(const string & text,
 		}
 
 		// Check wildcards.
-		// FIXME: Sort wildcards, shortest pattern first or something?
+		// FIXME: Sort wildcards, cheapest to check first or something?
 		i = 0;
-		for (auto&& pattern : wildcards) {
-		    if (startswith(term, pattern)) {
+		for (auto&& qw : wildcards) {
+		    if (qw->test(term)) {
 			relevance = &wildcards_relevance[i];
 			highlight = 1;
 			goto relevance_done;
@@ -883,6 +973,24 @@ MSet::Internal::snippet(const string & text,
 		    ++i;
 		}
 
+		// Check fuzzies.
+		// FIXME: Sort fuzzies, cheapest to check first or something?
+		i = 0;
+		for (auto&& qed : fuzzies) {
+		    // test() returns 0 for no match, or edit_distance + 1.
+		    int ed_result = qed->test(term);
+		    if (ed_result) {
+			// FIXME: Reduce relevance the more edits there are?
+			// We can't just divide by ed_result here as this
+			// relevance is used by any term matching this
+			// subquery.
+			relevance = &fuzzies_relevance[i];
+			highlight = 1;
+			goto relevance_done;
+		    }
+		    ++i;
+		}
+
 		if (flags & Xapian::MSet::SNIPPET_BACKGROUND_MODEL) {
 		    // Background document model.
 		    auto bgit = background.find(term);
