@@ -255,7 +255,8 @@ DEFINE_TESTCASE(expandweights1, backend) {
     TEST_EQUAL_DOUBLE(eset[2].get_weight(), 4.73383620844021);
 
     // Test non-default k too.
-    eset = enquire.get_eset(3, myrset, enquire.USE_EXACT_TERMFREQ, 2.0);
+    enquire.set_expansion_scheme("prob", 2.0);
+    eset = enquire.get_eset(3, myrset, enquire.USE_EXACT_TERMFREQ);
     TEST_EQUAL(eset.size(), 3);
     TEST_REL(eset.get_ebound(), >=, eset.size());
     TEST_EQUAL_DOUBLE(eset[0].get_weight(), 5.88109547674955);
@@ -373,7 +374,7 @@ DEFINE_TESTCASE(expandweights6, backend) {
     TEST_EQUAL_DOUBLE(eset[1].get_weight(), 6.08904001099445);
     TEST_EQUAL_DOUBLE(eset[2].get_weight(), 4.73383620844021);
 
-    // Older scheme name "trad" (alias for "prob").
+    // Test deprecated scheme name "trad" (alias for "prob").
     enquire.set_expansion_scheme("trad");
     eset = enquire.get_eset(3, myrset, enquire.USE_EXACT_TERMFREQ);
 
@@ -405,7 +406,8 @@ DEFINE_TESTCASE(expandweights8, backend) {
     myrset.add_document(*(++i));
 
     // Set expand_k to 1.0 and min_wt to 0
-    Xapian::ESet eset = enquire.get_eset(50, myrset, 0, 1.0, 0, 0);
+    enquire.set_expansion_scheme("prob", 1.0);
+    Xapian::ESet eset = enquire.get_eset(50, myrset, 0, 0, 0);
     // With a multi backend, the top three terms all happen to occur in both
     // shard so their termfreq is exactly known even without
     // USE_EXACT_TERMFREQ and so the weights should be the same for all
@@ -1465,6 +1467,7 @@ DEFINE_TESTCASE(qterminfo1, backend) {
 		    Xapian::Query(Xapian::Query::OP_OR,
 			    Xapian::Query(term2),
 			    Xapian::Query(term3)));
+    myquery = myquery &~ Xapian::Query("Boolean");
     enquire1.set_query(myquery);
     enquire2.set_query(myquery);
 
@@ -1490,12 +1493,13 @@ DEFINE_TESTCASE(qterminfo1, backend) {
 	TEST_EQUAL(mymset1a.get_termweight(term3), 0);
 
 	TEST_EQUAL(mymset1a.get_termfreq(stemmer("banana")), 1);
-	TEST_EXCEPTION(Xapian::InvalidArgumentError,
-		       mymset1a.get_termweight(stemmer("banana")));
+	TEST_EQUAL(mymset1a.get_termweight(stemmer("banana")), 0.0);
 
 	TEST_EQUAL(mymset1a.get_termfreq("sponge"), 0);
-	TEST_EXCEPTION(Xapian::InvalidArgumentError,
-		       mymset1a.get_termweight("sponge"));
+	TEST_EQUAL(mymset1a.get_termweight(stemmer("sponge")), 0.0);
+
+	TEST_EQUAL(mymset1a.get_termfreq("Boolean"), 0);
+	TEST_EQUAL(mymset1a.get_termweight("Boolean"), 0.0);
 
 	// Repeat tests with TradWeight.  (Regression test to ensure
 	// non-existent terms get zero weight with TradWeight.)
@@ -1545,8 +1549,6 @@ DEFINE_TESTCASE(msetzeroitems1, backend) {
 
 // test that the matches_* of a simple query are as expected
 DEFINE_TESTCASE(matches1, backend) {
-    bool remote = get_dbtype().find("remote") != string::npos;
-
     Xapian::Database db = get_database("apitest_simpledata");
     Xapian::Enquire enquire(db);
     Xapian::Query myquery;
@@ -1595,17 +1597,20 @@ DEFINE_TESTCASE(matches1, backend) {
     myquery = query(Xapian::Query::OP_AND, "simple", "word");
     enquire.set_query(myquery);
     mymset = enquire.get_mset(0, 0);
-    if (db.size() == 1) {
-	// This isn't true for sharded DBs since there one sub-database has 3
-	// documents and simple and word both have termfreq of 2, so the
-	// matcher can tell at least one document must match!)
-	TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
+    if (db.size() > 1) {
+	// We get a tighter lower bound because each shard is handled
+	// separately and that happens to give us the same tight range for
+	// both terms in one shard, and no matches for one term in the other.
+	TEST_EQUAL(mymset.get_matches_lower_bound(), 2);
+    } else {
+	// The matcher can tell at least 1 document must match by taking into
+	// account the ranges of matched docids.
+	TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
     }
-    TEST_REL(mymset.get_matches_lower_bound(),<=,mymset.get_matches_estimated());
-    TEST_EQUAL(mymset.get_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
     TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),<=,mymset.get_uncollapsed_matches_estimated());
-    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
     TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
 
     mymset = enquire.get_mset(0, 1);
@@ -1628,27 +1633,19 @@ DEFINE_TESTCASE(matches1, backend) {
     enquire.set_query(myquery);
     mymset = enquire.get_mset(0, 0);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
-    TEST_EQUAL(mymset.get_matches_estimated(), 2);
-    TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
     TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 1);
-    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
-    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 1);
 
     mymset = enquire.get_mset(0, 1);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
+    TEST_EQUAL(mymset.get_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
     TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 1);
-    if (db.size() > 1 && remote) {
-	// The matcher can tell there's only one match in this case.
-	TEST_EQUAL(mymset.get_matches_estimated(), 1);
-	TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
-	TEST_EQUAL(mymset.get_matches_upper_bound(), 1);
-	TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 1);
-    } else {
-	TEST_EQUAL(mymset.get_matches_estimated(), 2);
-	TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 2);
-	TEST_EQUAL(mymset.get_matches_upper_bound(), 2);
-	TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 2);
-    }
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 1);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 1);
 
     mymset = enquire.get_mset(0, 2);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 1);
