@@ -1,7 +1,7 @@
 /** @file
  * @brief Backend-related tests.
  */
-/* Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2018,2019 Olly Betts
+/* Copyright (C) 2008-2023 Olly Betts
  * Copyright (C) 2010 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -24,13 +24,14 @@
 
 #include "api_backend.h"
 
-#define XAPIAN_DEPRECATED(X) X
 #include <xapian.h>
 
 #include "backendmanager.h"
 #include "errno_to_string.h"
 #include "filetests.h"
+#include "net/resolver.h"
 #include "str.h"
+#include "socket_utils.h"
 #include "testrunner.h"
 #include "testsuite.h"
 #include "testutils.h"
@@ -39,6 +40,7 @@
 #include "apitest.h"
 
 #include "safefcntl.h"
+#include "safenetdb.h"
 #include "safesysstat.h"
 #include "safeunistd.h"
 #ifdef HAVE_SOCKETPAIR
@@ -54,7 +56,7 @@
 using namespace std;
 
 /// Regression test - lockfile should honour umask, was only user-readable.
-DEFINE_TESTCASE(lockfileumask1, chert || glass) {
+DEFINE_TESTCASE(lockfileumask1, glass) {
 #if !defined __WIN32__ && !defined __CYGWIN__ && !defined __OS2__
     mode_t old_umask = umask(022);
     try {
@@ -113,41 +115,43 @@ DEFINE_TESTCASE(exceed32bitcombineddb1, writable) {
     // Test case is for 64-bit Xapian::docid.
     // FIXME: Though we should check that the overflow is handled gracefully
     // for 32-bit...
-    if (sizeof(Xapian::docid) == 4) return;
+    if constexpr(sizeof(Xapian::docid) == 4) {
+	SKIP_TEST("Not supported with 32-bit docid currently");
+    } else {
+	// The InMemory backend uses a vector for the documents, so trying to
+	// add a document with the maximum docid is likely to fail because we
+	// can't allocate enough memory!
+	SKIP_TEST_FOR_BACKEND("inmemory");
 
-    // The InMemory backend uses a vector for the documents, so trying to add
-    // a document with the maximum docid is likely to fail because we can't
-    // allocate enough memory!
-    SKIP_TEST_FOR_BACKEND("inmemory");
+	Xapian::WritableDatabase db1 = get_writable_database();
+	Xapian::Document doc;
+	doc.set_data("prose");
+	doc.add_term("word");
+	Xapian::docid max_32bit_id = 0xffffffff;
+	db1.replace_document(max_32bit_id, doc);
+	db1.commit();
 
-    Xapian::WritableDatabase db1 = get_writable_database();
-    Xapian::Document doc;
-    doc.set_data("prose");
-    doc.add_term("word");
-    Xapian::docid max_32bit_id = 0xffffffff;
-    db1.replace_document(max_32bit_id, doc);
-    db1.commit();
+	Xapian::Database db2 = get_writable_database_as_database();
 
-    Xapian::Database db2 = get_writable_database_as_database();
+	Xapian::Database db;
+	db.add_database(db1);
+	db.add_database(db2);
 
-    Xapian::Database db;
-    db.add_database(db1);
-    db.add_database(db2);
+	Xapian::Enquire enquire(db);
+	enquire.set_query(Xapian::Query::MatchAll);
+	Xapian::MSet mymset = enquire.get_mset(0, 10);
 
-    Xapian::Enquire enquire(db);
-    enquire.set_query(Xapian::Query::MatchAll);
-    Xapian::MSet mymset = enquire.get_mset(0, 10);
+	TEST_EQUAL(2, mymset.size());
 
-    TEST_EQUAL(2, mymset.size());
-
-    // We can't usefully check the shard docid if the testharness backend is
-    // multi.
-    bool multi = (db1.size() > 1);
-    for (Xapian::MSetIterator i = mymset.begin(); i != mymset.end(); ++i) {
-	doc = i.get_document();
-	if (!multi)
-	    TEST_EQUAL(doc.get_docid(), max_32bit_id);
-	TEST_EQUAL(doc.get_data(), "prose");
+	// We can't usefully check the shard docid if the testharness backend
+	// is multi.
+	bool multi = (db1.size() > 1);
+	for (Xapian::MSetIterator i = mymset.begin(); i != mymset.end(); ++i) {
+	    doc = i.get_document();
+	    if (!multi)
+		TEST_EQUAL(doc.get_docid(), max_32bit_id);
+	    TEST_EQUAL(doc.get_data(), "prose");
+	}
     }
 }
 
@@ -183,6 +187,13 @@ DEFINE_TESTCASE(dbstats1, backend) {
     // This failed with an assertion during development between 1.3.1 and
     // 1.3.2.
     TEST_EQUAL(db.get_wdf_upper_bound(""), 0);
+
+    if (get_dbtype() == "honey") {
+	const Xapian::termcount min_unique_len = 2;
+	const Xapian::termcount max_unique_len = 272;
+	TEST_EQUAL(db.get_unique_terms_lower_bound(), min_unique_len);
+	TEST_EQUAL(db.get_unique_terms_upper_bound(), max_unique_len);
+    }
 }
 
 // Check stats with a single document.  In a multi-database situation, this
@@ -217,6 +228,13 @@ DEFINE_TESTCASE(dbstats2, backend) {
     }
 
     TEST_EQUAL(db.get_wdf_upper_bound(""), 0);
+
+    if (get_dbtype() == "honey") {
+	const Xapian::termcount min_unique_len = 9;
+	const Xapian::termcount max_unique_len = 9;
+	TEST_EQUAL(db.get_unique_terms_lower_bound(), min_unique_len);
+	TEST_EQUAL(db.get_unique_terms_upper_bound(), max_unique_len);
+    }
 }
 
 /// Check handling of alldocs on an empty database.
@@ -226,6 +244,10 @@ DEFINE_TESTCASE(alldocspl3, backend) {
     TEST_EQUAL(db.get_termfreq(string()), 0);
     TEST_EQUAL(db.get_collection_freq(string()), 0);
     TEST(db.postlist_begin(string()) == db.postlist_end(string()));
+
+    TEST_EQUAL(db.get_termfreq(string_view()), 0);
+    TEST_EQUAL(db.get_collection_freq(string_view()), 0);
+    TEST(db.postlist_begin(string_view()) == db.postlist_end(string_view()));
 }
 
 /// Regression test for bug#392 in ModifiedPostList iteration, fixed in 1.0.15.
@@ -269,7 +291,7 @@ DEFINE_TESTCASE(valuesaftercommit1, writable) {
     TEST_EQUAL(db.get_document(3).get_value(0), "value");
 }
 
-DEFINE_TESTCASE(lockfilefd0or1, chert || glass) {
+DEFINE_TESTCASE(lockfilefd0or1, glass) {
 #if !defined __WIN32__ && !defined __CYGWIN__ && !defined __OS2__
     int old_stdin = dup(0);
     int old_stdout = dup(1);
@@ -310,7 +332,7 @@ DEFINE_TESTCASE(lockfilefd0or1, chert || glass) {
 }
 
 /// Regression test for bug fixed in 1.2.13 and 1.3.1.
-DEFINE_TESTCASE(lockfilealreadyopen1, chert || glass) {
+DEFINE_TESTCASE(lockfilealreadyopen1, glass) {
     // Ensure database has been created.
     (void)get_named_writable_database("lockfilealreadyopen1");
     string path = get_named_writable_database_path("lockfilealreadyopen1");
@@ -329,7 +351,7 @@ DEFINE_TESTCASE(lockfilealreadyopen1, chert || glass) {
 }
 
 /// Feature tests for Database::locked().
-DEFINE_TESTCASE(testlock1, chert || glass) {
+DEFINE_TESTCASE(testlock1, glass) {
     Xapian::Database rdb;
     TEST(!rdb.locked());
     {
@@ -399,18 +421,66 @@ DEFINE_TESTCASE(testlock3, inmemory) {
     TEST(!db.locked());
 }
 
-/// Test locked() on closed WritableDatabase.
-DEFINE_TESTCASE(testlock4, chert || glass) {
-    Xapian::Database db = get_writable_database("apitest_simpledata");
-    // Even if we don't have a way to test the lock on the current platform,
-    // this should know the database is locked because this object holds the
-    // lock.
-    TEST(db.locked());
-    db.close();
-    try {
+/// Feature tests for Database::lock() and unlock().
+DEFINE_TESTCASE(testlock4, glass) {
+    Xapian::Database rdb;
+    TEST(!rdb.locked());
+
+    {
+	Xapian::WritableDatabase db = get_named_writable_database("testlock4");
+	TEST(db.locked());
+	Xapian::Database db_as_database = db;
+	TEST(db_as_database.locked());
+	TEST(!rdb.locked());
+
+	{
+	    rdb = get_writable_database_as_database();
+	    // Test lock() fails (already open to write as db).
+	    TEST_EXCEPTION(Xapian::DatabaseLockError,
+			   auto wdb = rdb.lock());
+	}
+
+	rdb = db.unlock();
+	try {
+	    TEST(!rdb.locked());
+	    // unlock() should have closed the underlying WritableDatabase so
+	    // locked() should either report that it isn't locked, or throw
+	    // Xapian::DatabaseClosedError.
+	    try {
+		TEST(!db.locked());
+	    } catch (const Xapian::DatabaseClosedError&) {
+	    }
+	    try {
+		TEST(!db_as_database.locked());
+	    } catch (const Xapian::DatabaseClosedError&) {
+	    }
+	} catch (const Xapian::FeatureUnavailableError&) {
+	    SKIP_TEST("Database::locked() not supported on this platform");
+	}
+
+	db.close();
 	TEST(!db.locked());
-    } catch (const Xapian::FeatureUnavailableError&) {
-	SKIP_TEST("Database::locked() not supported on this platform");
+	TEST(!db_as_database.locked());
+	TEST(!rdb.locked());
+	TEST_EXCEPTION(Xapian::DatabaseClosedError,
+		       db.lock());
+	TEST_EXCEPTION(Xapian::DatabaseClosedError,
+		       db.unlock());
+	TEST_EXCEPTION(Xapian::DatabaseClosedError,
+		       db_as_database.lock());
+	TEST_EXCEPTION(Xapian::DatabaseClosedError,
+		       db_as_database.unlock());
+
+	{
+	    auto wdb = rdb.lock();
+	    TEST(rdb.locked());
+	}
+
+	rdb.close();
+	TEST_EXCEPTION(Xapian::DatabaseClosedError,
+		       rdb.lock());
+	TEST_EXCEPTION(Xapian::DatabaseClosedError,
+		       rdb.unlock());
     }
 }
 
@@ -443,7 +513,7 @@ DEFINE_TESTCASE(matchdecider4, remote) {
 }
 
 /** Check that replacing an unmodified document doesn't increase the automatic
- *  flush counter.  Regression test for bug fixed in 1.1.4/1.0.18.
+ *  commit counter.  Regression test for bug fixed in 1.1.4/1.0.18.
  */
 DEFINE_TESTCASE(replacedoc7, writable && !inmemory && !remote) {
     // The inmemory backend doesn't batch changes, so there's nothing to
@@ -476,13 +546,13 @@ DEFINE_TESTCASE(replacedoc7, writable && !inmemory && !remote) {
     Xapian::Database rodb(get_writable_database_as_database());
     TEST_EQUAL(rodb.get_doccount(), 1);
 
-    db.flush();
+    db.commit();
     TEST(rodb.reopen());
 
     TEST_EQUAL(rodb.get_doccount(), 2);
 }
 
-/** Check that replacing a document deleted since the last flush works.
+/** Check that replacing a document deleted since the last commit works.
  *  Prior to 1.1.4/1.0.18, this failed to update the collection frequency and
  *  wdf, and caused an assertion failure when assertions were enabled.
  */
@@ -501,7 +571,7 @@ DEFINE_TESTCASE(replacedoc8, writable) {
 	doc.add_term("takeaway", 2);
 	db.replace_document(1, doc);
     }
-    db.flush();
+    db.commit();
     TEST_EQUAL(db.get_collection_freq("takeaway"), 2);
     Xapian::PostingIterator p = db.postlist_begin("takeaway");
     TEST(p != db.postlist_end("takeaway"));
@@ -591,12 +661,12 @@ DEFINE_TESTCASE(qpmemoryleak1, writable && !inmemory) {
     Xapian::Database database(get_writable_database_as_database());
     Xapian::QueryParser queryparser;
     queryparser.set_database(database);
-    TEST_EXCEPTION(Xapian::DatabaseModifiedError,
+    TEST_EXCEPTION3(Xapian::DatabaseModifiedError,
 	for (int k = 0; k < 1000; ++k) {
 	    wdb.add_document(doc);
 	    wdb.commit();
 	    (void)queryparser.parse_query("1", queryparser.FLAG_PARTIAL);
-	}
+	},
 	SKIP_TEST("didn't manage to trigger DatabaseModifiedError");
     );
 }
@@ -865,7 +935,7 @@ DEFINE_TESTCASE(orcheck1, backend) {
  *
  *  We failed to mark the Btree as unmodified after cancel().
  */
-DEFINE_TESTCASE(failedreplace1, chert || glass) {
+DEFINE_TESTCASE(failedreplace1, glass) {
     Xapian::WritableDatabase db(get_writable_database());
     Xapian::Document doc;
     doc.add_term("foo");
@@ -880,7 +950,7 @@ DEFINE_TESTCASE(failedreplace1, chert || glass) {
     TEST_EQUAL(db.get_termfreq("foo"), 0);
 }
 
-DEFINE_TESTCASE(failedreplace2, chert || glass) {
+DEFINE_TESTCASE(failedreplace2, glass) {
     Xapian::WritableDatabase db(get_writable_database("apitest_simpledata"));
     db.commit();
     Xapian::doccount db_size = db.get_doccount();
@@ -1085,21 +1155,20 @@ DEFINE_TESTCASE(itorskiptofromend1, backend) {
 // Regression test for bug fixed in 1.2.17 and 1.3.2 - the size gets fixed
 // but the uncorrected size was passed to the base file.  Also, abort() was
 // called on 0.
-DEFINE_TESTCASE(blocksize1, chert || glass) {
+DEFINE_TESTCASE(blocksize1, glass) {
     string db_dir = "." + get_dbtype();
     mkdir(db_dir.c_str(), 0755);
     db_dir += "/db__blocksize1";
     int flags;
-    if (get_dbtype() == "chert") {
-	flags = Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT;
-    } else {
+    if (get_dbtype() == "glass") {
 	flags = Xapian::DB_CREATE|Xapian::DB_BACKEND_GLASS;
+    } else {
+	FAIL_TEST("Unhandled backend type");
     }
-    static const unsigned bad_sizes[] = {
+    static const int bad_sizes[] = {
 	65537, 8000, 2000, 1024, 16, 7, 3, 1, 0
     };
-    for (size_t i = 0; i < sizeof(bad_sizes) / sizeof(bad_sizes[0]); ++i) {
-	size_t block_size = bad_sizes[i];
+    for (int block_size : bad_sizes) {
 	rm_rf(db_dir);
 	Xapian::WritableDatabase db(db_dir, flags, block_size);
 	Xapian::Document doc;
@@ -1116,9 +1185,7 @@ DEFINE_TESTCASE(notermlist1, glass) {
     mkdir(db_dir.c_str(), 0755);
     db_dir += "/db__notermlist1";
     int flags = Xapian::DB_CREATE|Xapian::DB_NO_TERMLIST;
-    if (get_dbtype() == "chert") {
-	flags |= Xapian::DB_BACKEND_CHERT;
-    } else {
+    if (get_dbtype() == "glass") {
 	flags |= Xapian::DB_BACKEND_GLASS;
     }
     rm_rf(db_dir);
@@ -1157,7 +1224,7 @@ DEFINE_TESTCASE(newfreelistblock1, writable) {
  *  branch which failed to append a "/" when generating a temporary filename
  *  from the database directory.
  */
-DEFINE_TESTCASE(readonlyparentdir1, chert || glass) {
+DEFINE_TESTCASE(readonlyparentdir1, glass) {
 #if !defined __WIN32__ && !defined __CYGWIN__ && !defined __OS2__
     string path = get_named_writable_database_path("readonlyparentdir1");
     // Fix permissions if the previous test was killed.
@@ -1397,7 +1464,7 @@ DEFINE_TESTCASE(cursorbug1, writable && path) {
     Xapian::Enquire enq(db);
     enq.set_query(Xapian::Query::MatchAll);
     Xapian::MSet mset;
-    // The original problem triggers for chert and glass on repeat==7.
+    // The original problem triggered for chert and glass on repeat==7.
     for (int repeat = 0; repeat < 10; ++repeat) {
 	tout.str(string());
 	tout << "iteration #" << repeat << '\n';
@@ -1516,7 +1583,7 @@ DEFINE_TESTCASE(exactxor1, backend) {
 }
 
 /// Feature test for Database::get_revision().
-DEFINE_TESTCASE(getrevision1, chert || glass) {
+DEFINE_TESTCASE(getrevision1, glass) {
     Xapian::WritableDatabase db = get_writable_database();
     TEST_EQUAL(db.get_revision(), 0);
     db.commit();
@@ -1642,6 +1709,27 @@ DEFINE_TESTCASE(nopositionbug1, backend) {
     TEST_EQUAL(enq.get_mset(0, 5).size(), 0);
 }
 
+/** Check that a TermIterator returns the correct termfreqs.
+ *
+ *  Prior to 2.0.0, the termfreq was approximated in the multidatabase case.
+ */
+DEFINE_TESTCASE(termitertf1, backend) {
+    Xapian::Database db = get_database("apitest_simpledata");
+    Xapian::TermIterator t = db.termlist_begin(2);
+
+    t.skip_to("mset");
+    TEST_EQUAL(*t, "mset");
+    TEST_EQUAL(t.get_termfreq(), 1);
+
+    t.skip_to("paragraph");
+    TEST_EQUAL(*t, "paragraph");
+    TEST_EQUAL(t.get_termfreq(), 5);
+
+    t.skip_to("queri");
+    TEST_EQUAL(*t, "queri");
+    TEST_EQUAL(t.get_termfreq(), 3);
+}
+
 /** Regression test for bug with get_mset(0, 0, N) (N > 0).
  *
  *  Fixed in 1.4.6.
@@ -1717,13 +1805,74 @@ DEFINE_TESTCASE(multidb1, backend) {
     TEST_EQUAL(db.size(), db2.size() * 2);
 }
 
-// Test that all the terms returned exist.
-DEFINE_TESTCASE(allterms7, backend) {
-    Xapian::Database db = get_database("etext");
-    for (auto i = db.allterms_begin(); i != db.allterms_end(); ++i) {
-	string term = *i;
-	TEST(db.get_termfreq(term) > 0);
-	TEST(db.postlist_begin(term) != db.postlist_end(term));
+// Regression test for bug in unreleased versions before 2.0.0.
+DEFINE_TESTCASE(matchall3, backend) {
+    Xapian::Database db = get_database("apitest_simpledata");
+    Xapian::Enquire enq(db);
+    Xapian::Query qw(Xapian::Query::OP_WILDCARD, "nosuch");
+    enq.set_query(0 * (Xapian::Query::MatchAll & qw));
+    TEST_EQUAL(enq.get_mset(0, 10).size(), 0);
+    enq.set_query(0 * (qw & Xapian::Query::MatchAll));
+    TEST_EQUAL(enq.get_mset(0, 10).size(), 0);
+}
+
+DEFINE_TESTCASE(reconstruct1, backend) {
+    Xapian::Database db = get_database("apitest_simpledata");
+    TEST_STRINGS_EQUAL(db.reconstruct_text(6),
+		       "and yet anoth this one doe mention banana split "
+		       "though so cant be that bad");
+    TEST_STRINGS_EQUAL(db.reconstruct_text(1, 14), "this is a test");
+    TEST_STRINGS_EQUAL(db.reconstruct_text(1, 10, "S"), "");
+    TEST_STRINGS_EQUAL(db.reconstruct_text(6, 0, "", 1, 3), "and yet anoth");
+}
+
+DEFINE_TESTCASE(reconstruct2, writable) {
+    Xapian::Database db = get_database("reconstruct2",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&)
+				       {
+					   Xapian::Document doc;
+					   doc.add_posting("XMBABxyz", 100);
+					   doc.add_posting("XMBABabc", 101);
+					   wdb.add_document(doc);
+				       });
+    TEST_STRINGS_EQUAL(db.reconstruct_text(1), "");
+}
+
+/** Regression test for bug fixed in git master before 2.0.0.
+ *
+ *  A PositionIterator from a PostingIterator in a multidatabase always used
+ *  the first shard, which could cause a segfault if the term wasn't present
+ *  in that shard.
+ */
+DEFINE_TESTCASE(positfrompostit1, positional) {
+    Xapian::Database db = get_database("apitest_simpledata");
+    {
+	// Wrong results - this was giving (4) instead of (5, 18).
+	auto postit = db.postlist_begin("paragraph");
+	TEST_NOT_EQUAL(postit, db.postlist_end("paragraph"));
+	postit.skip_to(4);
+	TEST_NOT_EQUAL(postit, db.postlist_end("paragraph"));
+	auto p = postit.positionlist_begin();
+	TEST_NOT_EQUAL(p, postit.positionlist_end());
+	TEST_EQUAL(*p, 5);
+	++p;
+	TEST_NOT_EQUAL(p, postit.positionlist_end());
+	TEST_EQUAL(*p, 18);
+	++p;
+	TEST_EQUAL(p, postit.positionlist_end());
+    }
+    {
+	// This was giving a segmentation fault.
+	auto postit = db.postlist_begin("split");
+	TEST_NOT_EQUAL(postit, db.postlist_end("split"));
+	postit.skip_to(6);
+	TEST_NOT_EQUAL(postit, db.postlist_end("split"));
+	auto p = postit.positionlist_begin();
+	TEST_NOT_EQUAL(p, postit.positionlist_end());
+	TEST_EQUAL(*p, 9);
+	++p;
+	TEST_EQUAL(p, postit.positionlist_end());
     }
 }
 
@@ -1746,6 +1895,122 @@ DEFINE_TESTCASE(nosuchterm, backend) {
     }
 }
 
+// Test that all the terms returned exist.
+DEFINE_TESTCASE(allterms7, backend) {
+    Xapian::Database db = get_database("etext");
+    for (auto i = db.allterms_begin(); i != db.allterms_end(); ++i) {
+	string term = *i;
+	TEST(db.get_termfreq(term) > 0);
+	TEST(db.postlist_begin(term) != db.postlist_end(term));
+    }
+}
+
+/* Test that we can't run a second server on the same port.
+ *
+ * This testcase is motivated by Microsoft's stupid and insecure semantics for
+ * SO_REUSEADDR which prior to Server 2003 allowed any process to bind to any
+ * port, even if it was already in use by a different process (and even if the
+ * original process hadn't used SO_REUSEADDR.
+ *
+ * The external behaviour we check here is that another process can't bind to
+ * the port xapian-tcpsrv is using, whatever socket options is uses.
+ *
+ * We use SO_REUSEADDR on Unix so that a connection in TIME_WAIT state doesn't
+ * block binding to the socket.  Apparently that's the default behaviour on
+ * Microsoft Windows.  Ideally we'd have an external test for this too, but it
+ * seems hard to write a reliable one.
+ */
+DEFINE_TESTCASE(remoteportreuse1, remotetcp) {
+    int port;
+    Xapian::Database db = get_remote_database("apitest_simpledata",
+					      300000,
+					      &port);
+
+    // We test with (up to) 3 different socket options combinations:
+    //
+    // 0: no socket options
+    // 1: SO_REUSEADDR
+    // 2: SO_EXCLUSIVEADDRUSE (Microsoft-specific option)
+    //
+    // We don't test with SO_REUSEADDR and SO_EXCLUSIVEADDRUSE together because
+    // that's not a valid combination (second setsockopt() call fails with
+    // WSAEINVAL).
+    for (int reuse_options : { 0, 1,
+#ifdef SO_EXCLUSIVEADDRUSE
+			       2
+#endif
+			     }) {
+	tout << "reuse_options = " << reuse_options << '\n';
+	int socketfd = -1;
+	int bind_errno = 0;
+	for (auto&& r : Resolver("127.0.0.1", port, AI_PASSIVE)) {
+	    int socktype = r.ai_socktype | SOCK_CLOEXEC;
+	    int fd = socket(r.ai_family, socktype, r.ai_protocol);
+	    if (fd == -1)
+		continue;
+
+	    if (reuse_options == 1) {
+		int on = 1;
+		// 4th argument might need to be void* or char* - cast it to
+		// char* since C++ allows implicit conversion to void* but not
+		// from void*.
+		int retval = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
+					reinterpret_cast<char*>(&on),
+					sizeof(on));
+
+		if (retval < 0) {
+		    int setsockopt_errno = socket_errno();
+		    CLOSESOCKET(fd);
+		    FAIL_TEST("setsockopt SO_REUSEADDR failed " +
+			      errno_to_string(setsockopt_errno));
+		}
+#ifdef SO_EXCLUSIVEADDRUSE
+	    } else if (reuse_options == 2) {
+		int on = 1;
+		int retval = setsockopt(fd, SOL_SOCKET, SO_EXCLUSIVEADDRUSE,
+					reinterpret_cast<char*>(&on),
+					sizeof(on));
+
+		if (retval < 0) {
+		    int setsockopt_errno = socket_errno();
+		    CLOSESOCKET(fd);
+		    FAIL_TEST("setsockopt SO_EXCLUSIVEADDRUSE failed " +
+			      errno_to_string(setsockopt_errno));
+		}
+#endif
+	    }
+
+	    if (::bind(fd, r.ai_addr, r.ai_addrlen) == 0) {
+		socketfd = fd;
+		break;
+	    }
+
+	    // Note down the error code for the first address we try, which
+	    // seems likely to be more helpful than the last in the case where
+	    // they differ.
+	    if (bind_errno == 0)
+		bind_errno = socket_errno();
+
+	    CLOSESOCKET(fd);
+	}
+
+	if (socketfd >= 0) {
+	    CLOSESOCKET(socketfd);
+	    FAIL_TEST("Managed to bind to TCP port used by xapian-tcpsrv");
+	}
+
+#ifdef __WIN32__
+	// Gives WSAEACCES instead in some cases involving SO_EXCLUSIVEADDRUSE.
+	if (bind_errno == WSAEACCES) bind_errno = EADDRINUSE;
+#endif
+
+	if (bind_errno != EADDRINUSE) {
+	    FAIL_TEST("bind() failed with unexpected error: " +
+		      errno_to_string(bind_errno));
+	}
+    }
+}
+
 // Test exception for check() on remote via stub.
 DEFINE_TESTCASE(unsupportedcheck1, path) {
     mkdir(".stub", 0755);
@@ -1788,7 +2053,7 @@ DEFINE_TESTCASE(unsupportedcheck3, !backend) {
     // Regression test, exception was DatabaseOpeningError with description:
     // Failed to rewind file descriptor -1 (Bad file descriptor)
     try {
-	Xapian::Database::check(string());
+	Xapian::Database::check(""s);
     } catch (const Xapian::DatabaseOpeningError& e) {
 	string enoent_msg = errno_to_string(ENOENT);
 	TEST_EQUAL(e.get_error_string(), enoent_msg);
