@@ -278,11 +278,7 @@ DEFINE_TESTCASE(compactmerge1, compact) {
 	    if (s) {
 		suffix = s;
 	    } else {
-		if (get_dbtype() == "chert") {
-		    suffix = "/record.DB";
-		} else {
-		    suffix = "/docdata." + get_dbtype();
-		}
+		suffix = "/docdata." + get_dbtype();
 	    }
 	    tout.str(string());
 	    tout << "Trying suffix '" << suffix << "'\n";
@@ -592,8 +588,7 @@ DEFINE_TESTCASE(compactmultipass1, compact) {
 }
 
 // Test compacting to an fd.
-// Chert doesn't support single file databases.
-DEFINE_TESTCASE(compacttofd1, compact && !chert) {
+DEFINE_TESTCASE(compacttofd1, compact) {
     Xapian::Database indb(get_database("apitest_simpledata"));
     string outdbpath = get_compaction_output_path("compacttofd1out");
     rm_rf(outdbpath);
@@ -619,8 +614,7 @@ DEFINE_TESTCASE(compacttofd1, compact && !chert) {
 }
 
 // Test compacting to an fd at at offset.
-// Chert doesn't support single file databases.
-DEFINE_TESTCASE(compacttofd2, compact && !chert) {
+DEFINE_TESTCASE(compacttofd2, compact) {
     Xapian::Database indb(get_database("apitest_simpledata"));
     string outdbpath = get_compaction_output_path("compacttofd2out");
     rm_rf(outdbpath);
@@ -664,9 +658,7 @@ DEFINE_TESTCASE(compacttofd2, compact && !chert) {
 
 // Regression test for bug fixed in 1.3.5.  If you compact a WritableDatabase
 // with uncommitted changes, you get an inconsistent output.
-//
-// Chert doesn't support single file databases.
-DEFINE_TESTCASE(compactsingle1, compact && writable && !chert) {
+DEFINE_TESTCASE(compactsingle1, compact && writable) {
     Xapian::WritableDatabase db = get_writable_database();
     Xapian::Document doc;
     doc.add_term("foo");
@@ -734,3 +726,31 @@ DEFINE_TESTCASE(compact1, compact && writable) {
 
     TEST_EQUAL(Xapian::Database(output).get_doccount(), 3);
 }
+
+// Regression test for compacting honey databases.
+DEFINE_TESTCASE(compact2, compact) {
+    Xapian::Database db = get_database("compact2",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&) {
+					   Xapian::Document doc;
+					   doc.add_term("test");
+					   wdb.add_document(doc);
+					   doc.add_term("test");
+					   for (int i = 1; i < 4000; ++i) {
+					       wdb.add_document(doc);
+					   }
+				       });
+
+    db.add_database(get_database("apitest_simpledata"));
+    auto db_size = db.get_doccount();
+
+    string output = get_compaction_output_path("compact2-out");
+    rm_rf(output);
+
+    db.compact(output);
+    db.close();
+
+    TEST_EQUAL(Xapian::Database::check(output, 0, &tout), 0);
+
+    TEST_EQUAL(Xapian::Database(output).get_doccount(), db_size);
+}
