@@ -180,6 +180,7 @@ DEFINE_TESTCASE(stubdb2, path) {
 	// to connect to it (which hopefully fails), so just test the message
 	// doesn't match the bad 1.4.0 result.
 	TEST(e.get_msg().find("host [") == string::npos);
+	TEST_EQUAL(e.get_context(), "remote:tcp(::1:65535)");
     }
 
     try {
@@ -197,6 +198,7 @@ DEFINE_TESTCASE(stubdb2, path) {
 	// to connect to it (which hopefully fails), so just test the message
 	// doesn't match the bad 1.4.0 result.
 	TEST(e.get_msg().find("host [") == string::npos);
+	TEST_EQUAL(e.get_context(), "remote:tcp(::1:65535)");
     }
 #endif
 
@@ -335,145 +337,6 @@ DEFINE_TESTCASE(stubdb8, inmemory) {
     }
 }
 
-#if 0 // the "force error" mechanism is no longer in place...
-class MyErrorHandler : public Xapian::ErrorHandler {
-    public:
-	int count;
-
-	bool handle_error(Xapian::Error & error) {
-	    ++count;
-	    tout << "Error handling caught: " << error.get_description()
-		 << ", count is now " << count << "\n";
-	    return true;
-	}
-
-	MyErrorHandler() : count (0) {}
-};
-
-// tests error handler in multimatch().
-DEFINE_TESTCASE(multierrhandler1, backend) {
-    MyErrorHandler myhandler;
-
-    Xapian::Database mydb2(get_database("apitest_simpledata"));
-    Xapian::Database mydb3(get_database("apitest_simpledata2"));
-    int errcount = 1;
-    for (int testcount = 0; testcount < 14; testcount ++) {
-	tout << "testcount=" << testcount << "\n";
-	Xapian::Database mydb4(get_database("-e", "apitest_termorder"));
-	Xapian::Database mydb5(get_network_database("apitest_termorder", 1));
-	Xapian::Database mydb6(get_database("-e2", "apitest_termorder"));
-	Xapian::Database mydb7(get_database("-e3", "apitest_simpledata"));
-
-	Xapian::Database dbs;
-	switch (testcount) {
-	    case 0:
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb4);
-		break;
-	    case 1:
-		dbs.add_database(mydb4);
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		break;
-	    case 2:
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb4);
-		dbs.add_database(mydb2);
-		break;
-	    case 3:
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb5);
-		sleep(1);
-		break;
-	    case 4:
-		dbs.add_database(mydb5);
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		sleep(1);
-		break;
-	    case 5:
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb5);
-		dbs.add_database(mydb2);
-		sleep(1);
-		break;
-	    case 6:
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb6);
-		break;
-	    case 7:
-		dbs.add_database(mydb6);
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		break;
-	    case 8:
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb6);
-		dbs.add_database(mydb2);
-		break;
-	    case 9:
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb7);
-		break;
-	    case 10:
-		dbs.add_database(mydb7);
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb3);
-		break;
-	    case 11:
-		dbs.add_database(mydb3);
-		dbs.add_database(mydb7);
-		dbs.add_database(mydb2);
-		break;
-	    case 12:
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb6);
-		dbs.add_database(mydb7);
-		break;
-	    case 13:
-		dbs.add_database(mydb2);
-		dbs.add_database(mydb7);
-		dbs.add_database(mydb6);
-		break;
-	}
-	tout << "db=" << dbs << "\n";
-	Xapian::Enquire enquire(dbs, &myhandler);
-
-	// make a query
-	Xapian::Query myquery = query(Xapian::Query::OP_OR, "inmemory", "word");
-	enquire.set_weighting_scheme(Xapian::BoolWeight());
-	enquire.set_query(myquery);
-
-	tout << "query=" << myquery << "\n";
-	// retrieve the top ten results
-	Xapian::MSet mymset = enquire.get_mset(0, 10);
-
-	switch (testcount) {
-	    case 0: case 3: case 6: case 9:
-		mset_expect_order(mymset, 2, 4, 10);
-		break;
-	    case 1: case 4: case 7: case 10:
-		mset_expect_order(mymset, 3, 5, 11);
-		break;
-	    case 2: case 5: case 8: case 11:
-		mset_expect_order(mymset, 1, 6, 12);
-		break;
-	    case 12:
-	    case 13:
-		mset_expect_order(mymset, 4, 10);
-		errcount += 1;
-		break;
-	}
-	TEST_EQUAL(myhandler.count, errcount);
-	errcount += 1;
-    }
-}
-#endif
-
 class GrepMatchDecider : public Xapian::MatchDecider {
     string needle;
   public:
@@ -677,12 +540,13 @@ DEFINE_TESTCASE(matchdecider3, backend && !remote) {
     TEST_REL(mset1.get_uncollapsed_matches_lower_bound(),<=,mset1.get_uncollapsed_matches_estimated());
     TEST_REL(mset1.get_uncollapsed_matches_estimated(),<=,mset1.get_uncollapsed_matches_upper_bound());
 
-    // The uncollapsed match would match all documents but the one the
-    // matchdecider rejects.
-    TEST_REL(mset1.get_uncollapsed_matches_upper_bound(),>=,db.get_doccount() - 1);
-    TEST_REL(mset1.get_uncollapsed_matches_upper_bound(),<=,db.get_doccount());
-    TEST_REL(mset2.get_uncollapsed_matches_upper_bound(),>=,db.get_doccount() - 1);
-    TEST_REL(mset2.get_uncollapsed_matches_upper_bound(),<=,db.get_doccount());
+    // We ask for 2 or more results so the matcher checks all candidates and
+    // can see that there would be exactly one uncollapsed match.
+    //
+    // In 1.4.x the matcher was less clever and gave doccount or (doccount - 1)
+    // as the uncollapsed upper bound.
+    TEST_EQUAL(mset1.get_uncollapsed_matches_upper_bound(), 1);
+    TEST_EQUAL(mset2.get_uncollapsed_matches_upper_bound(), 1);
 }
 
 // tests that mset iterators on msets compare correctly.
@@ -859,10 +723,11 @@ DEFINE_TESTCASE(collapsekey1, backend) {
 
 // tests that collapse-on-key modifies the predicted bounds for the number of
 // matches appropriately.
-DEFINE_TESTCASE(collapsekey2, backend) {
-    SKIP_TEST("Don't have a suitable database currently");
-    // FIXME: this needs an appropriate database creating, but that's quite
-    // subtle to do it seems.
+DEFINE_TESTCASE(collapsekey2, multi && remote) {
+    // FIXME: This needs an appropriate database creating to work for other
+    // backend combinations - currently those return the same bound after
+    // collapsing (which is valid, but doesn't show the effect we are trying
+    // to test for here).
     Xapian::Enquire enquire(get_database("apitest_simpledata2"));
     enquire.set_query(Xapian::Query("this"));
 
@@ -988,8 +853,6 @@ DEFINE_TESTCASE(allterms1, backend) {
     TEST_EQUAL(*ati, "one");
     TEST_EQUAL(ati.get_termfreq(), 1);
 
-    Xapian::TermIterator ati2 = ati;
-
     ati++;
     TEST(ati != db.allterms_end());
     if (verbose) {
@@ -1002,26 +865,11 @@ DEFINE_TESTCASE(allterms1, backend) {
     TEST(*ati == "three");
     TEST(ati.get_termfreq() == 3);
 
-#if 0
-    TEST(ati2 != db.allterms_end());
-    TEST(*ati2 == "one");
-    TEST(ati2.get_termfreq() == 1);
-#endif
-
     ++ati;
-#if 0
-    ++ati2;
-#endif
     TEST(ati != db.allterms_end());
     TEST(*ati == "two");
     TEST(ati.get_termfreq() == 2);
 
-#if 0
-    TEST(ati2 != db.allterms_end());
-    TEST(*ati2 == "three");
-    TEST(ati2.get_termfreq() == 3);
-#endif
-
     ati++;
     TEST(ati == db.allterms_end());
 }
@@ -1511,78 +1359,53 @@ DEFINE_TESTCASE(consistency1, backend && !remote) {
 }
 
 // Test that specifying a nonexistent input file throws an exception
-// (chert-specific cases).
-DEFINE_TESTCASE(chertdatabasenotfounderror1, chert) {
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-    mkdir(".chert", 0755);
+// (backend-specific cases).
+DEFINE_TESTCASE(databasenotfounderror1, glass || honey) {
+    string db_dir = "." + get_dbtype();
+
+    int db_type_flag;
+    if (get_dbtype() == "glass") {
+	db_type_flag = Xapian::DB_BACKEND_GLASS;
+    } else if (get_dbtype() == "honey") {
+	db_type_flag = Xapian::DB_BACKEND_HONEY;
+    } else {
+	FAIL_TEST("Backend " + get_dbtype() + " not handled by testcase");
+    }
+
+    mkdir(db_dir.c_str(), 0755);
+    db_dir += '/';
 
     TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::Database(".chert/nosuchdirectory",
-		Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::WritableDatabase(".chert/nosuchdirectory",
-		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
+	    Xapian::Database(db_dir + "nosuchdirectory", db_type_flag));
+    if (db_type_flag != Xapian::DB_BACKEND_HONEY) {
+	TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+		Xapian::WritableDatabase(db_dir + "nosuchdirectory",
+		    db_type_flag | Xapian::DB_OPEN));
+    }
 
-    mkdir(".chert/emptydirectory", 0700);
+    string empty_dir = db_dir + "emptydirectory";
+    mkdir(empty_dir.c_str(), 0700);
     TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::Database(".chert/emptydirectory",
-		Xapian::DB_BACKEND_CHERT));
+	    Xapian::Database(empty_dir, db_type_flag));
 
-    touch(".chert/somefile");
+    string some_file = db_dir + "somefile";
+    touch(some_file);
     TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::Database(".chert/somefile",
-		Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::WritableDatabase(".chert/somefile",
-		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::WritableDatabase(".chert/somefile",
-		Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::WritableDatabase(".chert/somefile",
-		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::WritableDatabase(".chert/somefile",
-		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT));
-#endif
-
-}
-
-// Test that specifying a nonexistent input file throws an exception
-// (glass-specific cases).
-DEFINE_TESTCASE(glassdatabasenotfounderror1, glass) {
-#ifdef XAPIAN_HAS_GLASS_BACKEND
-    mkdir(".glass", 0755);
-
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::Database(".glass/nosuchdirectory",
-		Xapian::DB_BACKEND_GLASS));
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::WritableDatabase(".glass/nosuchdirectory",
-		Xapian::DB_OPEN|Xapian::DB_BACKEND_GLASS));
-
-    mkdir(".glass/emptydirectory", 0700);
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::Database(".glass/emptydirectory",
-		Xapian::DB_BACKEND_GLASS));
-
-    touch(".glass/somefile");
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::Database(".glass/somefile",
-		Xapian::DB_BACKEND_GLASS));
-    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
-	    Xapian::WritableDatabase(".glass/somefile",
-		Xapian::DB_OPEN|Xapian::DB_BACKEND_GLASS));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::WritableDatabase(".glass/somefile",
-		Xapian::DB_CREATE|Xapian::DB_BACKEND_GLASS));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::WritableDatabase(".glass/somefile",
-		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_GLASS));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::WritableDatabase(".glass/somefile",
-		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_GLASS));
-#endif
+	    Xapian::Database(some_file, db_type_flag));
+    if (db_type_flag != Xapian::DB_BACKEND_HONEY) {
+	TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+		Xapian::WritableDatabase(some_file,
+		    db_type_flag | Xapian::DB_OPEN));
+	TEST_EXCEPTION(Xapian::DatabaseCreateError,
+		Xapian::WritableDatabase(some_file,
+		    db_type_flag | Xapian::DB_CREATE));
+	TEST_EXCEPTION(Xapian::DatabaseCreateError,
+		Xapian::WritableDatabase(some_file,
+		    db_type_flag | Xapian::DB_CREATE_OR_OPEN));
+	TEST_EXCEPTION(Xapian::DatabaseCreateError,
+		Xapian::WritableDatabase(some_file,
+		    db_type_flag | Xapian::DB_CREATE_OR_OVERWRITE));
+    }
 }
 
 // Test that specifying a nonexistent input file throws an exception
@@ -1604,61 +1427,61 @@ DEFINE_TESTCASE(databasenotfounderror2, !backend) {
 	    Xapian::Database{empty_dir});
 }
 
-/// Test opening of a chert database
-DEFINE_TESTCASE(chertdatabaseopen1, chert) {
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-    const string dbdir = ".chert/test_chertdatabaseopen1";
-    mkdir(".chert", 0755);
+/// Test opening of a glass database
+DEFINE_TESTCASE(glassdatabaseopen1, glass) {
+#ifdef XAPIAN_HAS_GLASS_BACKEND
+    const string dbdir = ".glass/test_glassdatabaseopen1";
+    mkdir(".glass", 0755);
 
     {
 	rm_rf(dbdir);
 	Xapian::WritableDatabase wdb(dbdir,
-		Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE|Xapian::DB_BACKEND_GLASS);
 	TEST_EXCEPTION(Xapian::DatabaseLockError,
 	    Xapian::WritableDatabase(dbdir,
-		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
-	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_GLASS));
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_GLASS);
     }
 
     {
 	rm_rf(dbdir);
 	Xapian::WritableDatabase wdb(dbdir,
-		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_GLASS);
 	TEST_EXCEPTION(Xapian::DatabaseLockError,
 	    Xapian::WritableDatabase(dbdir,
-		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT));
-	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_GLASS));
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_GLASS);
     }
 
     {
 	rm_rf(dbdir);
 	Xapian::WritableDatabase wdb(dbdir,
-		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_GLASS);
 	TEST_EXCEPTION(Xapian::DatabaseLockError,
 	    Xapian::WritableDatabase(dbdir,
-		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT));
-	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_GLASS));
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_GLASS);
     }
 
     {
 	TEST_EXCEPTION(Xapian::DatabaseCreateError,
 	    Xapian::WritableDatabase(dbdir,
-		Xapian::DB_CREATE|Xapian::DB_BACKEND_CHERT));
+		Xapian::DB_CREATE|Xapian::DB_BACKEND_GLASS));
 	Xapian::WritableDatabase wdb(dbdir,
-		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_CHERT);
-	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_GLASS);
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_GLASS);
     }
 
     {
 	Xapian::WritableDatabase wdb(dbdir,
-		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_CHERT);
-	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_GLASS);
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_GLASS);
     }
 
     {
 	Xapian::WritableDatabase wdb(dbdir,
-		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT);
-	Xapian::Database(dbdir, Xapian::DB_BACKEND_CHERT);
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_GLASS);
+	Xapian::Database(dbdir, Xapian::DB_BACKEND_GLASS);
     }
 #endif
 }
@@ -1894,17 +1717,12 @@ class MyWeight : public Xapian::Weight {
 	return new MyWeight;
     }
     double get_sumpart(Xapian::termcount,
+		       Xapian::termcount,
 		       Xapian::termcount,
 		       Xapian::termcount) const override {
 	return scale_factor;
     }
     double get_maxpart() const override { return scale_factor; }
-
-    double get_sumextra(Xapian::termcount,
-			Xapian::termcount) const override {
-	return 0;
-    }
-    double get_maxextra() const override { return 0; }
 };
 
 // tests user weighting scheme.
@@ -1916,8 +1734,8 @@ DEFINE_TESTCASE(userweight1, backend && !remote) {
     static const char * const query[] = {
 	"this", "line", "paragraph", "rubbish"
     };
-    enquire.set_query(Xapian::Query(Xapian::Query::OP_OR, query,
-				    query + sizeof(query) / sizeof(query[0])));
+    enquire.set_query(Xapian::Query(Xapian::Query::OP_OR,
+				    std::begin(query), std::end(query)));
     Xapian::MSet mymset1 = enquire.get_mset(0, 100);
     // MyWeight scores 1 for each matching term, so the weight should equal
     // the number of matching terms.
