@@ -45,7 +45,7 @@ class SimpleMatchSpy : public Xapian::MatchSpy {
     // Vector which will be filled with all the document contents seen.
     std::vector<std::string> seen;
 
-    void operator()(const Xapian::Document &doc, double) {
+    void operator()(const Xapian::Document& doc, double) override {
 	// Note that this is not recommended usage of get_data() - you
 	// generally shouldn't call get_data() from inside a MatchSpy, because
 	// it is (likely to be) a slow operation resulting in considerable IO.
@@ -173,11 +173,6 @@ DEFINE_TESTCASE(matchspy2, backend)
 
 DEFINE_TESTCASE(matchspy4, backend)
 {
-    XFAIL_FOR_BACKEND("multi_remote",
-		      "Matchspy counts hits on remote and locally");
-    XFAIL_FOR_BACKEND("multi_glass_remote",
-		      "Matchspy counts hits on remote and locally");
-
     Xapian::Database db = get_database("matchspy2", make_matchspy2_db);
 
     // We're going to run the match twice - once sorted by relevance, and once
@@ -311,7 +306,7 @@ DEFINE_TESTCASE(matchspy5, backend)
 }
 
 class MySpy : public Xapian::MatchSpy {
-    void operator()(const Xapian::Document &, double) {
+    void operator()(const Xapian::Document&, double) override {
     }
 };
 
@@ -336,7 +331,9 @@ DEFINE_TESTCASE(matchspy7, !backend)
 {
     Xapian::ValueCountMatchSpy myspy(1);
     string s = myspy.serialise_results();
-    s += 'x';
+    // Append a string which overflows a 64-bit type when decoded with
+    // pack_uint().
+    s += "xxxxxxxxx";
     // This merge_results() call used to enter an infinite loop.
-    TEST_EXCEPTION(Xapian::NetworkError, myspy.merge_results(s));
+    TEST_EXCEPTION(Xapian::SerialisationError, myspy.merge_results(s));
 }
