@@ -1,7 +1,7 @@
 /** @file
  * @brief Test the user metadata functionality.
  */
-/* Copyright (C) 2007,2009,2011 Olly Betts
+/* Copyright (C) 2007,2009,2011,2024 Olly Betts
  * Copyright (C) 2007,2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,14 +41,10 @@ DEFINE_TESTCASE(metadata1, metadata) {
 				       {
 					   wdb.set_metadata("empty", "");
 					   wdb.set_metadata("foo", "bar");
-					   wdb.set_metadata(string("", 1),
-							    string("", 1));
-					   wdb.set_metadata(string("\0fo", 3),
-							    string("\0xx", 3));
-					   wdb.set_metadata(string("f\0o", 3),
-							    string("x\0x", 3));
-					   wdb.set_metadata(string("fo\0", 3),
-							    string("xx\0", 3));
+					   wdb.set_metadata("\0"s, "\0"s);
+					   wdb.set_metadata("\0fo"s, "\0xx"s);
+					   wdb.set_metadata("f\0o"s, "x\0x"s);
+					   wdb.set_metadata("fo\0"s, "xx\0"s);
 				       });
     TEST_EQUAL(db.get_doccount(), 0);
     TEST_EQUAL(db.get_metadata("empty"), "");
@@ -56,10 +52,10 @@ DEFINE_TESTCASE(metadata1, metadata) {
     TEST_EQUAL(db.get_metadata("foo"), "bar");
 
     // Check for transparent handling of zero bytes.
-    TEST_EQUAL(db.get_metadata(string("", 1)), string("", 1));
-    TEST_EQUAL(db.get_metadata(string("\0fo", 3)), string("\0xx", 3));
-    TEST_EQUAL(db.get_metadata(string("f\0o", 3)), string("x\0x", 3));
-    TEST_EQUAL(db.get_metadata(string("fo\0", 3)), string("xx\0", 3));
+    TEST_EQUAL(db.get_metadata("\0"s), "\0"s);
+    TEST_EQUAL(db.get_metadata("\0fo"s), "\0xx"s);
+    TEST_EQUAL(db.get_metadata("f\0o"s), "x\0x"s);
+    TEST_EQUAL(db.get_metadata("fo\0"s), "xx\0"s);
 
     TEST_EXCEPTION(Xapian::InvalidArgumentError, db.get_metadata(""));
 }
@@ -81,18 +77,18 @@ DEFINE_TESTCASE(metadata2, metadata && writable) {
 
     // Check for transparent handling of zero bytes.
     db.set_metadata("foo", "value of foo");
-    db.set_metadata(string("foo\0bar", 7), string(1, '\0'));
-    db.set_metadata(string("foo\0", 4), string("foo\0bar", 7));
+    db.set_metadata("foo\0bar"sv, "\0"sv);
+    db.set_metadata("foo\0"sv, "foo\0bar"sv);
 
     TEST_EQUAL(db.get_metadata("foo"), "value of foo");
-    TEST_EQUAL(db.get_metadata(string("foo\0bar", 7)), string(1, '\0'));
-    TEST_EQUAL(db.get_metadata(string("foo\0", 4)), string("foo\0bar", 7));
+    TEST_EQUAL(db.get_metadata("foo\0bar"sv), "\0"sv);
+    TEST_EQUAL(db.get_metadata("foo\0"sv), "foo\0bar"sv);
 
     db.commit();
 
     TEST_EQUAL(db.get_metadata("foo"), "value of foo");
-    TEST_EQUAL(db.get_metadata(string("foo\0bar", 7)), string(1, '\0'));
-    TEST_EQUAL(db.get_metadata(string("foo\0", 4)), string("foo\0bar", 7));
+    TEST_EQUAL(db.get_metadata("foo\0bar"sv), "\0"sv);
+    TEST_EQUAL(db.get_metadata("foo\0"sv), "foo\0bar"sv);
 }
 
 // Test that metadata gets applied at same time as other changes.
