@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2015,2016,2017,2019 Olly Betts
+ * Copyright 2002-2024 Olly Betts
  * Copyright 2006 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -308,12 +308,11 @@ DEFINE_TESTCASE(poscollapse2, !backend) {
     TEST_EQUAL(q.get_length(), 2);
 }
 
-// regression test of querying an uninitialised database: should report an
-// error; used to segfault with 1.0.0.
+// Regression test: query on an uninitialised database segfaulted with 1.0.0.
+// As of 2.0.0, this is just handled as an empty database.
 DEFINE_TESTCASE(uninitdb1, !backend) {
     Xapian::Database db;
-    TEST_EXCEPTION(Xapian::InvalidArgumentError,
-		   Xapian::Enquire enq(db));
+    Xapian::Enquire enq(db);
 }
 
 // Test a scaleweight query applied to a match nothing query
@@ -339,9 +338,12 @@ DEFINE_TESTCASE(rset3, !backend) {
 // Regression test - RSet::get_description() gave a malformed answer in 1.0.7.
 DEFINE_TESTCASE(rset4, !backend) {
     Xapian::RSet rset;
+    TEST_STRINGS_EQUAL(rset.get_description(), "RSet()");
+    rset.add_document(2);
+    // In 1.0.7 this gave: RSet(RSet(RSet::Internal(, 2))
+    TEST_STRINGS_EQUAL(rset.get_description(), "RSet(2)");
     rset.add_document(1);
-    // In 1.0.7 this gave: RSet(RSet(RSet::Internal(, 1))
-    TEST_STRINGS_EQUAL(rset.get_description(), "RSet(RSet::Internal(1))");
+    TEST_STRINGS_EQUAL(rset.get_description(), "RSet(1,2)");
 }
 
 // Direct test of ValueSetMatchDecider
@@ -379,11 +381,13 @@ DEFINE_TESTCASE(valuesetmatchdecider1, !backend) {
     TEST(vsmd3(doc));
 }
 
-// Test that asking for the termfreq on an empty mset raises an exception.
+// Test that requesting termfreq or termweight on an empty mset returns 0.
+// New behaviour as of 2.0.0 - previously both methods threw
+// Xapian::InvalidOperationError.
 DEFINE_TESTCASE(emptymset1, !backend) {
     Xapian::MSet emptymset;
-    TEST_EXCEPTION(Xapian::InvalidOperationError,
-		   emptymset.get_termfreq("foo"));
+    TEST_EQUAL(emptymset.get_termfreq("foo"), 0);
+    TEST_EQUAL(emptymset.get_termweight("foo"), 0.0);
 }
 
 DEFINE_TESTCASE(expanddeciderfilterprefix1, !backend) {
