@@ -32,6 +32,7 @@
 #include "testsuite.h"
 #include "testutils.h"
 
+#include <string_view>
 #include <vector>
 
 using namespace std;
@@ -318,6 +319,13 @@ class TestRangeProcessor : public Xapian::RangeProcessor {
 /// Check reference counting of user-subclassable classes.
 DEFINE_TESTCASE(subclassablerefcount1, !backend) {
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     // Simple test of release().
     {
@@ -399,6 +407,13 @@ class TestFieldProcessor : public Xapian::FieldProcessor {
 /// Check reference counting of user-subclassable classes.
 DEFINE_TESTCASE(subclassablerefcount2, !backend) {
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     // Simple test of release().
     {
@@ -463,6 +478,13 @@ DEFINE_TESTCASE(subclassablerefcount3, backend) {
     Xapian::Database db = get_database("apitest_simpledata");
 
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     // Simple test of release().
     {
@@ -525,6 +547,13 @@ class TestStopper : public Xapian::Stopper {
 /// Check reference counting of Stopper with QueryParser.
 DEFINE_TESTCASE(subclassablerefcount4, !backend) {
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     // Simple test of release().
     {
@@ -595,6 +624,13 @@ DEFINE_TESTCASE(subclassablerefcount4, !backend) {
 /// Check reference counting of Stopper with TermGenerator.
 DEFINE_TESTCASE(subclassablerefcount5, !backend) {
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     // Simple test of release().
     {
@@ -678,6 +714,13 @@ DEFINE_TESTCASE(subclassablerefcount6, backend) {
     Xapian::Database db = get_database("apitest_simpledata");
 
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     // Simple test of release().
     {
@@ -762,6 +805,13 @@ DEFINE_TESTCASE(subclassablerefcount7, backend) {
     rset.add_document(1);
 
     bool gone_auto, gone;
+#ifdef _MSC_VER
+    // MSVC incorrectly warns these are potentially uninitialised.  It's
+    // unhelpful to always initialise these as that could mask if a genuine bug
+    // were introduced (which currently would likely be caught by a warning
+    // from a smarter compiler).
+    gone_auto = gone = false;
+#endif
 
     for (int flags = 0;
 	 flags <= Xapian::Enquire::INCLUDE_QUERY_TERMS;
@@ -833,101 +883,29 @@ DEFINE_TESTCASE(subclassablerefcount7, backend) {
     TEST(gone_auto);
 }
 
-class TestValueRangeProcessor : public Xapian::ValueRangeProcessor {
-    DestroyedFlag destroyed;
-
-  public:
-    TestValueRangeProcessor(bool & destroyed_) : destroyed(destroyed_) { }
-
-    Xapian::valueno operator()(std::string &, std::string &) {
-	return 42;
-    }
-};
-
-/// Check reference counting of user-subclassable classes.
-DEFINE_TESTCASE(subclassablerefcount8, !backend) {
-    bool gone_auto, gone;
-
-    // Simple test of release().
-    {
-	Xapian::ValueRangeProcessor * vrp = new TestValueRangeProcessor(gone);
-	TEST(!gone);
-	Xapian::QueryParser qp;
-	qp.add_valuerangeprocessor(vrp->release());
-	TEST(!gone);
-    }
-    TEST(gone);
-
-    // Check a second call to release() has no effect.
-    {
-	Xapian::ValueRangeProcessor * vrp = new TestValueRangeProcessor(gone);
-	TEST(!gone);
-	Xapian::QueryParser qp;
-	qp.add_valuerangeprocessor(vrp->release());
-	vrp->release();
-	TEST(!gone);
-    }
-    TEST(gone);
-
-    // Test reference counting works, and that a VRP with automatic storage
-    // works OK.
-    {
-	TestValueRangeProcessor vrp_auto(gone_auto);
-	TEST(!gone_auto);
-	{
-	    Xapian::QueryParser qp1;
-	    {
-		Xapian::QueryParser qp2;
-		Xapian::ValueRangeProcessor * vrp;
-		vrp = new TestValueRangeProcessor(gone);
-		TEST(!gone);
-		qp1.add_valuerangeprocessor(vrp->release());
-		TEST(!gone);
-		qp2.add_valuerangeprocessor(vrp);
-		TEST(!gone);
-		qp2.add_valuerangeprocessor(&vrp_auto);
-		TEST(!gone);
-		TEST(!gone_auto);
-	    }
-	    TEST(!gone);
-	}
-	TEST(gone);
-	TEST(!gone_auto);
-    }
-    TEST(gone_auto);
-
-    // Regression test for initial implementation, where ~opt_intrusive_ptr()
-    // checked the reference of the object, which may have already been deleted
-    // if it wasn't been reference counted.
-    {
-	Xapian::QueryParser qp;
-	{
-	    Xapian::ValueRangeProcessor * vrp =
-		new TestValueRangeProcessor(gone);
-	    TEST(!gone);
-	    qp.add_valuerangeprocessor(vrp);
-	    delete vrp;
-	    TEST(gone);
-	}
-	// At the end of this block, qp is destroyed, but mustn't dereference
-	// the pointer it has to vrp.  If it does, that should get caught
-	// when tests are run under valgrind.
-    }
-}
-
 /// Check encoding of non-UTF8 document data.
 DEFINE_TESTCASE(nonutf8docdesc1, !backend) {
     Xapian::Document doc;
     doc.set_data("\xc0\x80\xf5\x80\x80\x80\xfe\xff");
     TEST_EQUAL(doc.get_description(),
-	      "Document(data='\\xc0\\x80\\xf5\\x80\\x80\\x80\\xfe\\xff')");
+	      "Document(docid=0, data=\\xc0\\x80\\xf5\\x80\\x80\\x80\\xfe\\xff)");
     doc.set_data(string("\x00\x1f", 2));
     TEST_EQUAL(doc.get_description(),
-	      "Document(data='\\x00\\x1f')");
+	      "Document(docid=0, data=\\x00\\x1f)");
     // Check that backslashes are encoded so output isn't ambiguous.
     doc.set_data("back\\slash");
     TEST_EQUAL(doc.get_description(),
-	      "Document(data='back\\x5cslash')");
+	      "Document(docid=0, data=back\\x5cslash)");
+}
+
+DEFINE_TESTCASE(orphaneddoctermitor1, !backend) {
+    Xapian::TermIterator t;
+    {
+	Xapian::Document doc;
+	doc.add_term("foo");
+	t = doc.termlist_begin();
+    }
+    TEST_EQUAL(*t, "foo");
 }
 
 /** Test removal of terms from a document while iterating over them.
@@ -978,6 +956,8 @@ DEFINE_TESTCASE(removepostings, !backend) {
     auto num_pos = t.positionlist_count();
     TEST_EQUAL(t.get_wdf(), num_pos);
 
+    Xapian::PositionIterator pi = t.positionlist_begin();
+
     // Out of order is a no-op.
     TEST_EQUAL(doc.remove_postings("foo", 2, 1), 0);
     t = doc.termlist_begin();
@@ -1002,6 +982,12 @@ DEFINE_TESTCASE(removepostings, !backend) {
     TEST_EQUAL(t.positionlist_count(), num_pos - 3);
     TEST_EQUAL(t.get_wdf(), num_pos - 3);
 
+    // Test an existing PositionIterator isn't affected.
+    TEST_EQUAL(*pi, 1);
+    TEST_EQUAL(*++pi, 2);
+    TEST_EQUAL(*++pi, 3);
+    TEST_EQUAL(*++pi, 5);
+
     // Remove the end position.
     TEST_EQUAL(doc.remove_postings("foo", 876, 987), 1);
     t = doc.termlist_begin();
@@ -1022,8 +1008,48 @@ DEFINE_TESTCASE(removepostings, !backend) {
 	TEST_EQUAL(*p, *expect);
 	++expect;
     }
+    TEST_EQUAL(*expect, 9999);
+    TEST_EQUAL(t.get_wdf(), 6);
+
+    pi = t.positionlist_begin();
+
+    // Test remove_position().
+    doc.remove_posting("foo", 8);
+
+    // Test an existing PositionIterator isn't affected.
+    TEST_EQUAL(*pi, 5);
+    TEST_EQUAL(*++pi, 8);
+
+    // Check removing all positions removes the term too if the wdf reaches 0
+    // (since 2.0.0).
+    TEST_EQUAL(doc.remove_postings("foo", 5, 1000), 5);
+    t = doc.termlist_begin();
+    TEST(t == doc.termlist_end());
+
+    // Test the removing all positions doesn't remove the term if the wdf is
+    // still non-zero.
+    doc.add_posting("foo", 123, 2);
+    TEST_EQUAL(doc.remove_postings("foo", 1, 200), 1);
+    t = doc.termlist_begin();
+    TEST(t != doc.termlist_end());
+    TEST(t.positionlist_begin() == t.positionlist_end());
+    TEST_EQUAL(t.get_wdf(), 1);
+
+    // Test removing the last posting is handled correctly (this case is
+    // special-cased internally to be O(1)).
+    t = doc.termlist_begin();
+    doc.add_posting("foo", 12);
+    doc.add_posting("foo", 23);
+    pi = t.positionlist_begin();
+    doc.remove_posting("foo", 23);
+    TEST_EQUAL(*pi, 12);
+    ++pi;
+    TEST_EQUAL(*pi, 23);
+    ++pi;
+    TEST(pi == t.positionlist_end());
 }
 
+[[noreturn]]
 static void
 errorcopyctor_helper(Xapian::Error& error)
 {
@@ -1043,6 +1069,15 @@ DEFINE_TESTCASE(errorcopyctor, !backend) {
     FAIL_TEST("Expected exception to be thrown");
 }
 
+DEFINE_TESTCASE(emptydbbounds, !backend) {
+    Xapian::Database db;
+    TEST_EQUAL(db.get_doclength_lower_bound(), 0);
+    TEST_EQUAL(db.get_doclength_upper_bound(), 0);
+    // We always returned 1 here in the initial implementation.
+    TEST_EQUAL(db.get_unique_terms_lower_bound(), 0);
+    TEST_EQUAL(db.get_unique_terms_upper_bound(), 0);
+}
+
 // Test ESetIterator iterator_traits.
 DEFINE_TESTCASE(stlesetiterator, !backend) {
     Xapian::ESet eset;
