@@ -2,7 +2,7 @@
  * @brief Tests of percentage calculations.
  */
 /* Copyright (C) 2008,2009 Lemur Consulting Ltd
- * Copyright (C) 2008,2009,2010,2011,2012,2014 Olly Betts
+ * Copyright (C) 2008,2009,2010,2011,2012,2014,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "api_percentages.h"
 
+#define XAPIAN_DEPRECATED(X) X
 #include <xapian.h>
 
 #include "apitest.h"
@@ -90,7 +91,9 @@ class MyPostingSource : public Xapian::PostingSource {
 	if (wt > get_maxweight()) set_maxweight(wt);
     }
 
-    void init(const Xapian::Database&) override { started = false; }
+    void reset(const Xapian::Database&, Xapian::doccount) override {
+	started = false;
+    }
 
     double get_weight() const override { return i->second; }
 
@@ -305,7 +308,9 @@ DEFINE_TESTCASE(topercent7, backend) {
 
 class ZWeight : public Xapian::Weight {
   public:
-    ZWeight() { }
+    ZWeight() {
+	need_stat(DOC_LENGTH);
+    }
 
     void init(double) override { }
 
@@ -314,6 +319,7 @@ class ZWeight : public Xapian::Weight {
     }
 
     double get_sumpart(Xapian::termcount,
+		       Xapian::termcount,
 		       Xapian::termcount,
 		       Xapian::termcount) const override {
 	return 0.0;
@@ -324,6 +330,7 @@ class ZWeight : public Xapian::Weight {
     }
 
     double get_sumextra(Xapian::termcount doclen,
+			Xapian::termcount,
 			Xapian::termcount) const override {
 	return 1.0 / doclen;
     }
