@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009,2016 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009,2016,2019 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -534,12 +534,24 @@ DEFINE_TESTCASE(poslist2, positional && writable) {
     Xapian::Document doc4 = db.get_document(did2);
     doc4.remove_posting("hadpos", 1);
     db.replace_document(did2, doc4);
-
+    // Removing the last position should remove the term if the wdf is 0
+    // (since 2.0.0).
+    TEST(!db.term_exists("hadpos"));
     {
 	Xapian::PositionIterator i = db.positionlist_begin(did2, "hadpos");
 	TEST_EQUAL(i, db.positionlist_end(did2, "hadpos"));
     }
 
+    doc4.add_posting("extrawdf", 12, 2);
+    doc4.remove_posting("extrawdf", 12);
+    db.replace_document(did2, doc4);
+    // Removing the last position should leave the term if the wdf is non-zero.
+    TEST(db.term_exists("extrawdf"));
+    {
+	Xapian::PositionIterator i = db.positionlist_begin(did2, "extrawdf");
+	TEST_EQUAL(i, db.positionlist_end(did2, "extrawdf"));
+    }
+
     db.delete_document(did);
     // Check what happens when the document doesn't exist (but once did).
     TEST_EQUAL(db.positionlist_begin(did, "nosuchterm"),
@@ -608,12 +620,8 @@ DEFINE_TESTCASE(positfromtermit1, positional) {
     Xapian::PositionIterator p = t.positionlist_begin();
     TEST_NOT_EQUAL(p, t.positionlist_end());
 
-    try {
-	TEST_EQUAL(t.positionlist_count(), 1);
-	t.skip_to("on");
-	TEST_NOT_EQUAL(t, db.termlist_end(7));
-	TEST_EQUAL(t.positionlist_count(), 2);
-    } catch (const Xapian::UnimplementedError &) {
-	SKIP_TEST("TermList::positionlist_count() not yet implemented for this backend");
-    }
+    TEST_EQUAL(t.positionlist_count(), 1);
+    t.skip_to("on");
+    TEST_NOT_EQUAL(t, db.termlist_end(7));
+    TEST_EQUAL(t.positionlist_count(), 2);
 }
