@@ -1,7 +1,7 @@
 /** @file
  * @brief Query-related tests.
  */
-/* Copyright (C) 2008,2009,2012,2013,2015,2016,2017,2019 Olly Betts
+/* Copyright (C) 2008-2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -341,10 +341,10 @@ DEFINE_TESTCASE(xor3, backend) {
     Xapian::Database db = get_database("apitest_simpledata");
 
     static const char * const subqs[] = {
-	"hack", "which", "paragraph", "is", "return"
+	"this", "hack", "which", "paragraph", "is", "return", "this", "this"
     };
     // Document where the subqueries run out *does* match XOR:
-    Xapian::Query q(Xapian::Query::OP_XOR, subqs, subqs + 5);
+    Xapian::Query q(Xapian::Query::OP_XOR, subqs + 1, subqs + 6);
     Xapian::Enquire enq(db);
     enq.set_query(q);
     Xapian::MSet mset = enq.get_mset(0, 10);
@@ -355,7 +355,7 @@ DEFINE_TESTCASE(xor3, backend) {
     TEST_EQUAL(*mset[2], 3);
 
     // Document where the subqueries run out *does not* match XOR:
-    q = Xapian::Query(Xapian::Query::OP_XOR, subqs, subqs + 4);
+    q = Xapian::Query(Xapian::Query::OP_XOR, subqs + 1, subqs + 5);
     enq.set_query(q);
     mset = enq.get_mset(0, 10);
 
@@ -364,6 +364,32 @@ DEFINE_TESTCASE(xor3, backend) {
     TEST_EQUAL(*mset[1], 4);
     TEST_EQUAL(*mset[2], 2);
     TEST_EQUAL(*mset[3], 3);
+
+    // Tests that XOR subqueries that match all docs are handled well when
+    // calculating min/est/max match counts.
+    q = Xapian::Query(Xapian::Query::OP_XOR, subqs, subqs + 2);
+    enq.set_query(q);
+    mset = enq.get_mset(0, 0);
+    TEST_EQUAL(mset.size(), 0);
+    TEST_EQUAL(mset.get_matches_lower_bound(), 5);
+    TEST_EQUAL(mset.get_matches_estimated(), 5);
+    TEST_EQUAL(mset.get_matches_upper_bound(), 5);
+
+    q = Xapian::Query(Xapian::Query::OP_XOR, subqs + 5, subqs + 7);
+    enq.set_query(q);
+    mset = enq.get_mset(0, 0);
+    TEST_EQUAL(mset.size(), 0);
+    TEST_EQUAL(mset.get_matches_lower_bound(), 5);
+    TEST_EQUAL(mset.get_matches_estimated(), 5);
+    TEST_EQUAL(mset.get_matches_upper_bound(), 5);
+
+    q = Xapian::Query(Xapian::Query::OP_XOR, subqs + 5, subqs + 8);
+    enq.set_query(q);
+    mset = enq.get_mset(0, 0);
+    TEST_EQUAL(mset.size(), 0);
+    TEST_EQUAL(mset.get_matches_lower_bound(), 1);
+    TEST_EQUAL(mset.get_matches_estimated(), 1);
+    TEST_EQUAL(mset.get_matches_upper_bound(), 1);
 }
 
 /// Check encoding of non-UTF8 terms in query descriptions.
@@ -394,6 +420,14 @@ DEFINE_TESTCASE(queryintro1, !backend) {
     TEST_EQUAL(q.get_subquery(0).get_type(), q.LEAF_MATCH_ALL);
     TEST_EQUAL(q.get_subquery(1).get_type(), q.LEAF_TERM);
 
+    q = Xapian::Query("foo", 2, 1);
+    TEST_EQUAL(q.get_leaf_wqf(), 2);
+    TEST_EQUAL(q.get_leaf_pos(), 1);
+
+    q = Xapian::Query("bar");
+    TEST_EQUAL(q.get_leaf_wqf(), 1);
+    TEST_EQUAL(q.get_leaf_pos(), 0);
+
     q = Xapian::Query("foo") & Xapian::Query("bar");
     TEST_EQUAL(q.get_type(), q.OP_AND);
 
@@ -540,6 +574,32 @@ DEFINE_TESTCASE(wildcard2, backend) {
     TEST_EQUAL(mset.size(), 6);
 }
 
+/** Regression test for bug in initial implementation.
+ *
+ *  If any terms started with A-Z then the next term that didn't wasn't
+ *  considered.
+ */
+DEFINE_TESTCASE(wildcard3, backend) {
+    Xapian::Database db = get_database("wildcard3",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&)
+				       {
+					   Xapian::Document doc;
+					   doc.add_term("Zfoo");
+					   doc.add_term("a");
+					   wdb.add_document(doc);
+					   doc.add_term("abc");
+					   wdb.add_document(doc);
+				       });
+
+    Xapian::Enquire enq(db);
+    Xapian::Query q(Xapian::Query::OP_WILDCARD, "?", 0,
+		    Xapian::Query::WILDCARD_PATTERN_GLOB);
+    enq.set_query(q);
+    Xapian::MSet mset = enq.get_mset(0, 10);
+    TEST_EQUAL(mset.size(), 2);
+}
+
 /** Regression test for OP_WILDCARD bug, fixed in 1.4.26.
  *
  *  Fix overly high reported termweight values in some cases.
@@ -585,6 +645,320 @@ DEFINE_TESTCASE(dualprefixwildcard1, backend) {
     TEST_EQUAL(enq.get_mset(0, 5).size(), 2);
 }
 
+/// Test special case wildcards.
+DEFINE_TESTCASE(specialwildcard1, !backend) {
+    const Xapian::Query::op o = Xapian::Query::OP_WILDCARD;
+    const auto f = Xapian::Query::WILDCARD_PATTERN_GLOB;
+
+    // Empty wildcard -> MatchNothing.
+    TEST_EQUAL(Xapian::Query(o, "", 0, f).get_description(), "Query()");
+
+    // "*", "?*", etc -> MatchAll.
+#define QUERY_ALLDOCS "Query(<alldocuments>)"
+    TEST_EQUAL(Xapian::Query(o, "*", 0, f).get_description(), QUERY_ALLDOCS);
+    TEST_EQUAL(Xapian::Query(o, "**", 0, f).get_description(), QUERY_ALLDOCS);
+    TEST_EQUAL(Xapian::Query(o, "?*", 0, f).get_description(), QUERY_ALLDOCS);
+    TEST_EQUAL(Xapian::Query(o, "*?", 0, f).get_description(), QUERY_ALLDOCS);
+    TEST_EQUAL(Xapian::Query(o, "*?*", 0, f).get_description(), QUERY_ALLDOCS);
+}
+
+static void
+gen_singlecharwildcard1_db(Xapian::WritableDatabase& db, const string&)
+{
+    {
+	Xapian::Document doc;
+	doc.add_term("test");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("t\xc3\xaast");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("t\xe1\x80\x80st");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("t\xf3\x80\x80\x80st");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("toast");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("t*t");
+	db.add_document(doc);
+    }
+}
+
+/// Test `?` extended wildcard.
+DEFINE_TESTCASE(singlecharwildcard1, backend) {
+    Xapian::Database db = get_database("singlecharwildcard1",
+				       gen_singlecharwildcard1_db);
+    Xapian::Enquire enq(db);
+    enq.set_weighting_scheme(Xapian::BoolWeight());
+
+    const Xapian::Query::op o = Xapian::Query::OP_WILDCARD;
+    const auto f = Xapian::Query::WILDCARD_PATTERN_SINGLE;
+
+    {
+	// Check that `?` matches one Unicode character.
+	enq.set_query(Xapian::Query(o, "t?st", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 1, 2, 3, 4);
+    }
+
+    {
+	// Check that `??` doesn't match a single two-byte UTF-8 character.
+	enq.set_query(Xapian::Query(o, "t??st", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 5);
+    }
+
+    {
+	// Check that `*` is handled as a literal character not a wildcard.
+	enq.set_query(Xapian::Query(o, "t*t", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 6);
+    }
+}
+
+static void
+gen_multicharwildcard1_db(Xapian::WritableDatabase& db, const string&)
+{
+    {
+	Xapian::Document doc;
+	doc.add_term("ananas");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("annas");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("bananas");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("banannas");
+	db.add_document(doc);
+    }
+    {
+	Xapian::Document doc;
+	doc.add_term("b?nanas");
+	db.add_document(doc);
+    }
+}
+
+/// Test `*` extended wildcard.
+DEFINE_TESTCASE(multicharwildcard1, backend) {
+    Xapian::Database db = get_database("multicharwildcard1",
+				       gen_multicharwildcard1_db);
+    Xapian::Enquire enq(db);
+    enq.set_weighting_scheme(Xapian::BoolWeight());
+
+    const Xapian::Query::op o = Xapian::Query::OP_WILDCARD;
+    const auto f = Xapian::Query::WILDCARD_PATTERN_MULTI;
+
+    {
+	// Check `*` can handle partial matches before and after.
+	enq.set_query(Xapian::Query(o, "b*anas", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 3, 5);
+    }
+
+    {
+	// Check leading `*` works.
+	enq.set_query(Xapian::Query(o, "*anas", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 1, 3, 5);
+    }
+
+    {
+	// Check more than one `*` works.
+	enq.set_query(Xapian::Query(o, "*ann*", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 2, 4);
+    }
+
+    {
+	// Check that `?` is handled as a literal character not a wildcard.
+	enq.set_query(Xapian::Query(o, "b?n*", 0, f));
+	Xapian::MSet mset = enq.get_mset(0, 100);
+	mset_expect_order(mset, 5);
+    }
+}
+
+struct editdist_testcase {
+    const char* target;
+    unsigned edit_distance;
+    Xapian::termcount max_expansion;
+    char max_type;
+    const char* terms[4];
+};
+
+#define EDITDIST_EXCEPTION { 0, 0, 0, "" }
+static const
+editdist_testcase editdist1_testcases[] = {
+    // Tries to expand to 9 terms.
+    { "muse",	2, 8, 'E', EDITDIST_EXCEPTION },
+    { "museum",	3, 3, 'E', { "mset", "must", "use", 0 } },
+    { "thou",	0, 9, 'E', { 0, 0, 0, 0 } },
+    { "though",	0, 9, 'E', { "though", 0, 0, 0 } },
+    { "museum",	3, 1, 'F', { "mset", 0, 0, 0 } },
+    { "museum",	3, 1, 'M', { "use", 0, 0, 0 } },
+};
+
+DEFINE_TESTCASE(editdist1, backend) {
+    // FIXME: The counting of terms the subquery expands to is per subdatabase,
+    // so it may expand to more terms than the limit if some aren't in all
+    // subdatabases.  Also WILDCARD_LIMIT_MOST_FREQUENT uses the frequency from
+    // the subdatabase, and so may select different terms in each subdatabase.
+    SKIP_TEST_FOR_BACKEND("multi");
+    Xapian::Database db = get_database("apitest_simpledata");
+    Xapian::Enquire enq(db);
+    const Xapian::Query::op o = Xapian::Query::OP_EDIT_DISTANCE;
+
+    for (auto&& test : editdist1_testcases) {
+	tout << test.target << '\n';
+	auto tend = test.terms + 4;
+	while (tend > test.terms && tend[-1] == NULL) --tend;
+	bool expect_exception = (tend - test.terms == 4 && tend[-1][0] == '\0');
+	Xapian::Query q;
+	int max_type;
+	switch (test.max_type) {
+	    case 'E':
+		max_type = Xapian::Query::WILDCARD_LIMIT_ERROR;
+		break;
+	    case 'F':
+		max_type = Xapian::Query::WILDCARD_LIMIT_FIRST;
+		break;
+	    case 'M':
+		max_type = Xapian::Query::WILDCARD_LIMIT_MOST_FREQUENT;
+		break;
+	    default:
+		FAIL_TEST("Unexpected max_type value");
+	}
+	q = Xapian::Query(o, test.target, test.max_expansion, max_type,
+			  q.OP_SYNONYM, test.edit_distance);
+	enq.set_query(q);
+	tout << q.get_description() << '\n';
+	try {
+	    Xapian::MSet mset = enq.get_mset(0, 10);
+	    TEST(!expect_exception);
+	    q = Xapian::Query(q.OP_SYNONYM, test.terms, tend);
+	    enq.set_query(q);
+	    Xapian::MSet mset2 = enq.get_mset(0, 10);
+	    TEST_EQUAL(mset.size(), mset2.size());
+	    TEST(mset_range_is_same(mset, 0, mset2, 0, mset.size()));
+	} catch (const Xapian::WildcardError&) {
+	    TEST(expect_exception);
+	}
+    }
+}
+
+// u8"foo" is const char8_t[] in C++20 and later.
+#define UTF8(X) reinterpret_cast<const char*>(u8"" X "")
+
+static const
+editdist_testcase editdist2_testcases[] = {
+    { UTF8("\U00010000"),	1, 8, 'E', { UTF8("a\U00010000"), 0, 0, 0 } },
+};
+
+/// Test Unicode edit distance calculations.
+DEFINE_TESTCASE(editdist2, backend) {
+    Xapian::Database db = get_database("editdist2",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&)
+				       {
+					   Xapian::Document doc;
+					   doc.add_term(UTF8("a\U00010000"));
+					   wdb.add_document(doc);
+				       });
+    Xapian::Enquire enq(db);
+    const Xapian::Query::op o = Xapian::Query::OP_EDIT_DISTANCE;
+
+    for (auto&& test : editdist2_testcases) {
+	tout << test.target << '\n';
+	auto tend = test.terms + 4;
+	while (tend > test.terms && tend[-1] == NULL) --tend;
+	bool expect_exception = (tend - test.terms == 4 && tend[-1][0] == '\0');
+	Xapian::Query q;
+	int max_type;
+	switch (test.max_type) {
+	    case 'E':
+		max_type = Xapian::Query::WILDCARD_LIMIT_ERROR;
+		break;
+	    case 'F':
+		max_type = Xapian::Query::WILDCARD_LIMIT_FIRST;
+		break;
+	    case 'M':
+		max_type = Xapian::Query::WILDCARD_LIMIT_MOST_FREQUENT;
+		break;
+	    default:
+		FAIL_TEST("Unexpected max_type value");
+	}
+	q = Xapian::Query(o, test.target, test.max_expansion, max_type,
+			  q.OP_SYNONYM, test.edit_distance);
+	enq.set_query(q);
+	tout << q.get_description() << '\n';
+	try {
+	    Xapian::MSet mset = enq.get_mset(0, 10);
+	    TEST(!expect_exception);
+	    q = Xapian::Query(q.OP_SYNONYM, test.terms, tend);
+	    enq.set_query(q);
+	    Xapian::MSet mset2 = enq.get_mset(0, 10);
+	    TEST_EQUAL(mset.size(), mset2.size());
+	    TEST(mset_range_is_same(mset, 0, mset2, 0, mset.size()));
+	} catch (const Xapian::WildcardError&) {
+	    TEST(expect_exception);
+	}
+    }
+}
+
+DEFINE_TESTCASE(dualprefixeditdist1, backend) {
+    Xapian::Database db = get_database("dualprefixeditdist1",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&)
+				       {
+					   Xapian::Document doc;
+					   doc.add_term("opossum");
+					   doc.add_term("possum");
+					   wdb.add_document(doc);
+					   doc.clear_terms();
+					   doc.add_term("Spossums");
+					   wdb.add_document(doc);
+				       });
+
+    auto OP_EDIT_DISTANCE = Xapian::Query::OP_EDIT_DISTANCE;
+    auto OP_SYNONYM = Xapian::Query::OP_SYNONYM;
+    Xapian::Query q0(OP_EDIT_DISTANCE, "possum");
+    Xapian::Query q1(OP_EDIT_DISTANCE, "Spossum", 0, 0, OP_SYNONYM, 2, 1);
+    Xapian::Query q(OP_SYNONYM, q0, q1);
+    tout << q.get_description() << '\n';
+    Xapian::Enquire enq(db);
+    enq.set_query(q0);
+    Xapian::MSet mset = enq.get_mset(0, 5);
+    TEST_EQUAL(mset.size(), 1);
+    TEST_EQUAL(*mset[0], 1);
+    enq.set_query(q1);
+    mset = enq.get_mset(0, 5);
+    TEST_EQUAL(mset.size(), 1);
+    TEST_EQUAL(*mset[0], 2);
+    enq.set_query(q);
+    mset = enq.get_mset(0, 5);
+    TEST_EQUAL(mset.size(), 2);
+}
+
 struct positional_testcase {
     int window;
     const char * terms[4];
@@ -802,47 +1176,65 @@ gen_subdbwithoutpos1_db(Xapian::WritableDatabase& db, const string&)
 }
 
 DEFINE_TESTCASE(subdbwithoutpos1, backend) {
-    XFAIL_FOR_BACKEND("remote",
-		      "Known but obscure remote bug which doesn't justify "
-		      "protocol version bump");
-    XFAIL_FOR_BACKEND("multi_remote",
-		      "Known but obscure remote bug which doesn't justify "
-		      "protocol version bump");
-
     Xapian::Database db(get_database("apitest_simpledata"));
+    TEST(db.has_positions());
 
-    Xapian::Query q(Xapian::Query::OP_PHRASE,
-		    Xapian::Query("this"),
-		    Xapian::Query("paragraph"));
+    Xapian::Query q_near(Xapian::Query::OP_NEAR,
+			 Xapian::Query("this"),
+			 Xapian::Query("paragraph"));
+
+    Xapian::Query q_phrase(Xapian::Query::OP_PHRASE,
+			   Xapian::Query("this"),
+			   Xapian::Query("paragraph"));
 
     Xapian::Enquire enq1(db);
-    enq1.set_query(q);
+    enq1.set_query(q_near);
     Xapian::MSet mset1 = enq1.get_mset(0, 10);
     TEST_EQUAL(mset1.size(), 3);
 
+    enq1.set_query(q_phrase);
+    mset1 = enq1.get_mset(0, 10);
+    TEST_EQUAL(mset1.size(), 3);
+
     Xapian::Database db2 =
 	get_database("subdbwithoutpos1", gen_subdbwithoutpos1_db);
+    TEST(!db2.has_positions());
 
-    // If a database has no positional info, OP_PHRASE -> OP_AND.
+    // If a database has no positional info, we used to map OP_PHRASE and
+    // OP_NEAR to OP_AND, but since 2.0.0 we no longer do.
     Xapian::Enquire enq2(db2);
-    enq2.set_query(q);
+    enq2.set_query(q_near);
     Xapian::MSet mset2 = enq2.get_mset(0, 10);
-    TEST_EQUAL(mset2.size(), 1);
+    TEST_EQUAL(mset2.size(), 0);
+
+    enq2.set_query(q_phrase);
+    mset2 = enq2.get_mset(0, 10);
+    TEST_EQUAL(mset2.size(), 0);
 
     // If one sub-database in a combined database has no positional info but
     // other sub-databases do, then we shouldn't convert OP_PHRASE to OP_AND
     // (but prior to 1.4.3 we did).
     db.add_database(db2);
+    TEST(db.has_positions());
+
     Xapian::Enquire enq3(db);
-    enq3.set_query(q);
+    enq3.set_query(q_near);
     Xapian::MSet mset3 = enq3.get_mset(0, 10);
     TEST_EQUAL(mset3.size(), 3);
     // Regression test for bug introduced in 1.4.3 which led to a division by
     // zero and then (at least on Linux) we got 1% here.
     TEST_EQUAL(mset3[0].get_percent(), 100);
 
+    enq3.set_query(q_phrase);
+    mset3 = enq3.get_mset(0, 10);
+    TEST_EQUAL(mset3.size(), 3);
+    // Regression test for bug introduced in 1.4.3 which led to a division by
+    // zero and then (at least on Linux) we got 1% here.
+    TEST_EQUAL(mset3[0].get_percent(), 100);
+
     // Regression test for https://trac.xapian.org/ticket/752
-    enq3.set_query((Xapian::Query("this") & q) | Xapian::Query("wibble"));
+    auto q = (Xapian::Query("this") & q_phrase) | Xapian::Query("wibble");
+    enq3.set_query(q);
     mset3 = enq3.get_mset(0, 10);
     TEST_EQUAL(mset3.size(), 4);
 }
@@ -904,7 +1296,7 @@ DEFINE_TESTCASE(emptynot1, backend) {
     Xapian::MSet mset = enq.get_mset(0, 10);
     TEST_EQUAL(mset.size(), 1);
     // Essentially the same test but with a term which doesn't match anything
-    // instead of a range.
+    // on the right side.
     query = Xapian::Query("document") & Xapian::Query("api");
     query = Xapian::Query(query.OP_AND_NOT,
 			  query,
@@ -912,6 +1304,15 @@ DEFINE_TESTCASE(emptynot1, backend) {
     enq.set_query(query);
     mset = enq.get_mset(0, 10);
     TEST_EQUAL(mset.size(), 1);
+    // Essentially the same test but with a wildcard which doesn't match
+    // anything on right side.
+    query = Xapian::Query("document") & Xapian::Query("api");
+    query = Xapian::Query(query.OP_AND_NOT,
+			  query,
+			  Xapian::Query(query.OP_WILDCARD, "nosuchwildcard"));
+    enq.set_query(query);
+    mset = enq.get_mset(0, 10);
+    TEST_EQUAL(mset.size(), 1);
 }
 
 // Similar case to emptynot1 but for OP_AND_MAYBE.  This case wasn't failing,
@@ -930,7 +1331,7 @@ DEFINE_TESTCASE(emptymaybe1, backend) {
     Xapian::MSet mset = enq.get_mset(0, 10);
     TEST_EQUAL(mset.size(), 1);
     // Essentially the same test but with a term which doesn't match anything
-    // instead of a range.
+    // on the right side.
     query = Xapian::Query("document") & Xapian::Query("api");
     query = Xapian::Query(query.OP_AND_MAYBE,
 			  query,
@@ -938,6 +1339,55 @@ DEFINE_TESTCASE(emptymaybe1, backend) {
     enq.set_query(query);
     mset = enq.get_mset(0, 10);
     TEST_EQUAL(mset.size(), 1);
+    // Essentially the same test but with a wildcard which doesn't match
+    // anything on right side.
+    query = Xapian::Query("document") & Xapian::Query("api");
+    query = Xapian::Query(query.OP_AND_MAYBE,
+			  query,
+			  Xapian::Query(query.OP_WILDCARD, "nosuchwildcard"));
+    enq.set_query(query);
+    mset = enq.get_mset(0, 10);
+    TEST_EQUAL(mset.size(), 1);
+}
+
+// Regression test for optimisation bug on git master before 2.0.0.
+// The query optimiser ignored the NOT part when the LHS contained
+// a MatchAll.
+DEFINE_TESTCASE(allnot1, backend) {
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enq(db);
+    Xapian::Query query;
+    // This case wasn't a problem, but would have been if the index-all term
+    // was handled like MatchAll by this optimisation (which it might be in
+    // future).
+    query = Xapian::Query{query.OP_AND_NOT,
+			  Xapian::Query("this"),
+			  Xapian::Query("the")};
+    enq.set_query(0 * query);
+    Xapian::MSet mset = enq.get_mset(0, 10);
+    TEST_EQUAL(mset.size(), 2);
+    query = Xapian::Query{query.OP_AND_NOT,
+			  query.MatchAll,
+			  Xapian::Query("the")};
+    enq.set_query(0 * query);
+    mset = enq.get_mset(0, 10);
+    TEST_EQUAL(mset.size(), 2);
+}
+
+// Regression test for optimisation bug on git master before 2.0.0.
+// The query optimiser didn't handle the RHS of AND_MAYBE not matching
+// anything.
+DEFINE_TESTCASE(emptymayberhs1, backend) {
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enq(db);
+    // The RHS doesn't match anything, which now gives a NULL PostList*, and
+    // we were trying to dereference that in this case.
+    Xapian::Query query(Xapian::Query::OP_AND_MAYBE,
+			Xapian::Query("document"),
+			Xapian::Query("xyzzy"));
+    enq.set_query(query);
+    Xapian::MSet mset = enq.get_mset(0, 10);
+    TEST_EQUAL(mset.size(), 2);
 }
 
 DEFINE_TESTCASE(phraseweightcheckbug1, backend) {
@@ -964,3 +1414,85 @@ DEFINE_TESTCASE(orphanedhint1, backend) {
     Xapian::MSet mset = enq.get_mset(0, 3);
     TEST_EQUAL(mset.size(), 1);
 }
+
+// Regression test for bugs in initial implementation of query optimisation
+// based on docid range information.
+DEFINE_TESTCASE(docidrangebugs1, backend) {
+    Xapian::Database db(get_database("apitest_simpledata"));
+    Xapian::Enquire enq(db);
+
+    // This triggered a bug in BoolOrPostList::get_docid_range().
+    Xapian::Query query(Xapian::Query::OP_FILTER,
+			Xapian::Query("typo"),
+			Xapian::Query("rubbish") | Xapian::Query("this"));
+    enq.set_query(query);
+    Xapian::MSet mset = enq.get_mset(0, 1);
+    TEST_EQUAL(mset.size(), 1);
+
+    Xapian::Query query2(Xapian::Query::OP_FILTER,
+			 Xapian::Query("typo"),
+			 Xapian::Query("this") | Xapian::Query("rubbish"));
+    enq.set_query(query2);
+    mset = enq.get_mset(0, 1);
+    TEST_EQUAL(mset.size(), 1);
+
+    // Alternative reproducer where the first term doesn't match any
+    // documents.
+    Xapian::Query query3(Xapian::Query::OP_FILTER,
+			 Xapian::Query("typo"),
+			 Xapian::Query("nosuchterm") | Xapian::Query("this"));
+    enq.set_query(query3);
+    mset = enq.get_mset(0, 1);
+    TEST_EQUAL(mset.size(), 1);
+
+    Xapian::Query query4(Xapian::Query::OP_FILTER,
+			 Xapian::Query("typo"),
+			 Xapian::Query("this") | Xapian::Query("nosuchterm"));
+    enq.set_query(query4);
+    mset = enq.get_mset(0, 1);
+    TEST_EQUAL(mset.size(), 1);
+}
+
+DEFINE_TESTCASE(estimateopbug1, backend) {
+    Xapian::Database db = get_database("estimateopbug1",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&)
+				       {
+					   Xapian::Document doc;
+					   doc.add_posting("XFgroups", 7);
+					   doc.add_posting("XSchange", 216);
+					   doc.add_posting("XSmember", 214);
+					   wdb.add_document(doc);
+					   Xapian::Document doc2;
+					   doc2.add_boolean_term("XEP");
+					   wdb.add_document(doc2);
+				       });
+    Xapian::Query q{Xapian::Query::OP_PHRASE,
+		    Xapian::Query{"XSmember"},
+		    Xapian::Query{"XSchange"}};
+    q = Xapian::Query{"XFgroups"} & (q | Xapian::Query{"XSmember"});
+    q &= ~Xapian::Query{"XEP"};
+    Xapian::Enquire enquire(db);
+    enquire.set_query(q);
+    Xapian::MSet matches = enquire.get_mset(0, 10);
+}
+
+struct Exception_estimateop2 {};
+class MDecider_estimateop2 : public Xapian::MatchDecider {
+    bool operator()(const Xapian::Document&) const override {
+	throw Exception_estimateop2();
+    }
+};
+
+DEFINE_TESTCASE(estimateopbug2, backend && !remote) {
+    Xapian::Database db = get_database("apitest_simpledata");
+    Xapian::Query query{"it"};
+    Xapian::Enquire enq{db};
+    enq.set_query(query);
+    MDecider_estimateop2 mdecider;
+    try {
+	auto mset = enq.get_mset(0, 10, nullptr, &mdecider);
+	FAIL_TEST("Expected exception Exception_estimateop2 not thrown");
+    } catch (const Exception_estimateop2&) {
+    }
+}
