@@ -2,7 +2,7 @@
  * @brief tests of replication functionality
  */
 /* Copyright 2008 Lemur Consulting Ltd
- * Copyright 2009,2010,2011,2012,2013,2014,2015,2016,2017,2020 Olly Betts
+ * Copyright 2009-2022 Olly Betts
  * Copyright 2010 Richard Boulton
  * Copyright 2011 Dan Colish
  *
@@ -64,14 +64,6 @@ static void mktmpdir(const string & path) {
     }
 }
 
-static off_t get_file_size(const string & path) {
-    off_t size = file_size(path);
-    if (errno) {
-	FAIL_TEST("Can't stat '" << path << "'");
-    }
-    return size;
-}
-
 static size_t do_read(int fd, char * p, size_t desired)
 {
     size_t total = 0;
@@ -103,8 +95,10 @@ static void do_write(int fd, const char * p, size_t n)
 }
 
 // Make a truncated copy of a file.
-static off_t
-truncated_copy(const string & srcpath, const string & destpath, off_t tocopy)
+static file_size_type
+truncated_copy(const string& srcpath,
+	       const string& destpath,
+	       file_size_type tocopy)
 {
     FD fdin(open(srcpath.c_str(), O_RDONLY | O_BINARY));
     if (fdin == -1) {
@@ -449,9 +443,12 @@ replicate_with_brokenness(Xapian::DatabaseMaster & master,
 
     // Try applying truncated changesets of various different lengths.
     string brokenchangesetpath = tempdir + "/changeset_broken";
-    off_t filesize = get_file_size(changesetpath);
-    off_t len = 10;
-    off_t copylen;
+    auto filesize = file_size(changesetpath);
+    if (errno) {
+	FAIL_TEST("Can't stat '" << changesetpath << "'");
+    }
+    file_size_type len = 10;
+    file_size_type copylen;
     while (len < filesize) {
 	copylen = truncated_copy(changesetpath, brokenchangesetpath, len);
 	TEST_EQUAL(copylen, len);
@@ -578,8 +575,8 @@ DEFINE_TESTCASE(replicate4, replicas) {
 	doc2.set_data(string("doc2"));
 	doc2.add_term("nopos");
 	orig.add_document(doc2);
-	if (get_dbtype() != "chert") {
-	    // FIXME: Needs to be pre-commit for new-glass
+	if (get_dbtype() == "glass") {
+	    // FIXME: Needs to be pre-commit for glass
 	    set_max_changesets(0);
 	}
 	orig.commit();
@@ -595,7 +592,7 @@ DEFINE_TESTCASE(replicate4, replicas) {
 	TEST(!file_exists(masterpath + "/changes1"));
 
 	// Turn off replication, make sure we don't write anything.
-	if (get_dbtype() == "chert") {
+	if (get_dbtype() != "glass") {
 	    set_max_changesets(0);
 	}
 
@@ -628,7 +625,6 @@ DEFINE_TESTCASE(replicate4, replicas) {
 // Tests for max_changesets
 DEFINE_TESTCASE(replicate5, replicas) {
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
-    SKIP_TEST_FOR_BACKEND("chert");
     UNSET_MAX_CHANGESETS_AFTERWARDS;
     string tempdir = ".replicatmp";
     mktmpdir(tempdir);
