@@ -2,7 +2,7 @@
  * @brief Tests of serialisation functionality.
  */
 /* Copyright 2009 Lemur Consulting Ltd
- * Copyright 2009,2011,2012,2013 Olly Betts
+ * Copyright 2009,2011,2012,2013,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -339,7 +339,7 @@ class ExceptionalPostingSource : public Xapian::PostingSource {
 	return new ExceptionalPostingSource(fail);
     }
 
-    void init(const Xapian::Database&) override { }
+    void reset(const Xapian::Database&, Xapian::doccount) override { }
 
     Xapian::doccount get_termfreq_min() const override { return 0; }
     Xapian::doccount get_termfreq_est() const override { return 1; }
@@ -388,7 +388,7 @@ class ExceptionalWeight : public Xapian::Weight {
     ExceptionalWeight(failmode fail_) : fail(fail_) { }
 
     string name() const override {
-	return "ExceptionalWeight";
+	return "exceptional";
     }
 
     Weight* clone() const override {
@@ -400,17 +400,12 @@ class ExceptionalWeight : public Xapian::Weight {
     void init(double) override { }
 
     double get_sumpart(Xapian::termcount,
+		       Xapian::termcount,
 		       Xapian::termcount,
 		       Xapian::termcount) const override {
 	return 0;
     }
     double get_maxpart() const override { return 0; }
-
-    double get_sumextra(Xapian::termcount,
-			Xapian::termcount) const override {
-	return 0;
-    }
-    double get_maxextra() const override { return 0; }
 };
 
 /// Check that exceptions when registering are handled well.
