@@ -2,7 +2,7 @@
  * @brief tests snippets
  */
 /* Copyright 2012 Mihai Bivol
- * Copyright 2015,2016,2017,2019,2020 Olly Betts
+ * Copyright 2015,2016,2017,2019,2020,2026 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -68,6 +68,16 @@ DEFINE_TESTCASE(snippet1, backend) {
 	// Should prefer "interesting" words for context.
 	{ "And of the rubbish document to this", 18, "...<b>rubbish</b> document..." },
 	{ "And if they document rubbish to be this", 18, "...document <b>rubbish</b>..." },
+
+	// Test handling of soft hyphen (added in Xapian 2.0.0).
+#define SHY "\xc2\xad"
+	{ "rub" SHY "bish ment" SHY "ion", 20,
+	  "<b>rub" SHY "bish</b> <b>ment" SHY "ion</b>" },
+
+	// Test handling of zero-width space (changed in Xapian 2.0.0).
+#define ZWSP "\xe2\x80\x8b"
+	{ "mention" ZWSP "rubbish" ZWSP "dolor", 30,
+	  "<b>mention</b>" ZWSP "<b>rubbish</b>" ZWSP "dolor" },
     };
 
     for (auto i : testcases) {
@@ -518,6 +528,40 @@ DEFINE_TESTCASE(snippet_ngrams, backend) {
     TEST_STRINGS_EQUAL(s, "...<b>已</b><b>經</b>有香港地...");
 }
 
+/// Test word break finding.
+DEFINE_TESTCASE(snippet_wordbreaks, backend) {
+    Xapian::Enquire enquire(get_database("apitest_simpledata"));
+    enquire.set_query(Xapian::Query("已經"));
+
+    Xapian::MSet mset = enquire.get_mset(0, 0);
+
+    Xapian::Stem stem;
+    const char *input = "明末時已經有香港地方的概念";
+    const char *input2 = "明末時已經有香港地方的概念. Hello!";
+    size_t len = strlen(input);
+
+    unsigned flags = Xapian::MSet::SNIPPET_WORD_BREAKS;
+
+#ifdef USE_ICU
+# define DO_TEST(CODE, RESULT) TEST_STRINGS_EQUAL(CODE, RESULT)
+#else
+# define DO_TEST(CODE, RESULT) \
+    try { \
+	CODE; \
+	FAIL_TEST("No exception thrown, expected FeatureUnavailableError"); \
+    } catch (const Xapian::FeatureUnavailableError& e) { \
+	TEST_STRINGS_EQUAL( \
+	    e.get_msg(), \
+	    "SNIPPET_WORD_BREAKS requires building Xapian to use ICU"); \
+    }
+#endif
+    DO_TEST(mset.snippet(input, len, stem, flags, "<b>", "</b>", "..."),
+	    "明末時<b>已經</b>有香港地方的概念");
+    DO_TEST(mset.snippet(input2, len / 2, stem, flags, "[", "]", "~"),
+	    "~時[已經]有香港~");
+#undef DO_TEST
+}
+
 DEFINE_TESTCASE(snippet_empty_mset, backend) {
     Xapian::Enquire enquire(get_database("apitest_simpledata"));
     enquire.set_query(Xapian::Query());
