@@ -317,7 +317,17 @@ DEFINE_TESTCASE(spell8, spelling) {
     TEST_EQUAL(db.get_spelling_suggestion("scimkin", 3), "skinking");
 }
 
-/// spell9 only present on git master.
+/// Regression test for honey spelling encoding bug affecting "tail" entries.
+DEFINE_TESTCASE(spell9, spelling) {
+    Xapian::Database db = get_database("spell9",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&) {
+					   wdb.add_spelling("ruru", 1);
+					   wdb.add_spelling("eel", 1);
+				       });
+    TEST_EQUAL(db.get_spelling_suggestion("gel", 1), "eel");
+    TEST_EQUAL(db.get_spelling_suggestion("thru", 2), "ruru");
+}
 
 /// Regression test for bug due to rejecting based on matching n-gram count.
 DEFINE_TESTCASE(spell10, spelling) {
