@@ -31,6 +31,7 @@
 #include <xapian.h>
 
 #include "filetests.h"
+#include "negate_unsigned.h"
 #include "omassert.h"
 #include "str.h"
 #include "stringutils.h"
@@ -763,7 +764,7 @@ DEFINE_TESTCASE(deldoc4, writable) {
 	did = db.add_document(doc3);
 	TEST_EQUAL(did, i * 3 + 3);
 
-	bool is_power_of_two = ((i & (i - 1)) == 0);
+	bool is_power_of_two = ((i & negate_unsigned(i)) == 0);
 	if (is_power_of_two) {
 	    db.commit();
 	    // reopen() on a writable database shouldn't do anything.
@@ -1356,45 +1357,6 @@ DEFINE_TESTCASE(emptyterm2, writable) {
     }
 }
 
-// Check that PHRASE/NEAR becomes AND if there's no positional info in the
-// database.
-DEFINE_TESTCASE(phraseorneartoand1, backend) {
-    Xapian::Database db = get_database("phraseorneartoand1",
-				       [](Xapian::WritableDatabase& wdb,
-					  const string&)
-				       {
-					   for (int n = 1; n <= 20; ++n) {
-					       Xapian::Document doc;
-					       doc.add_term(str(n));
-					       doc.add_term(str(n ^ 9));
-					       doc.add_term("all");
-					       doc.set_data("pass1");
-					       wdb.add_document(doc);
-					   }
-				       });
-
-    Xapian::Enquire enquire(db);
-    Xapian::MSet mymset;
-
-    static const char * const q1[] = { "all", "1" };
-    enquire.set_query(Xapian::Query(Xapian::Query::OP_PHRASE, q1, q1 + 2));
-    mymset = enquire.get_mset(0, 10);
-    TEST_EQUAL(2, mymset.size());
-
-    enquire.set_query(Xapian::Query(Xapian::Query::OP_NEAR, q1, q1 + 2));
-    mymset = enquire.get_mset(0, 10);
-    TEST_EQUAL(2, mymset.size());
-
-    static const char * const q2[] = { "1", "2" };
-    enquire.set_query(Xapian::Query(Xapian::Query::OP_PHRASE, q2, q2 + 2));
-    mymset = enquire.get_mset(0, 10);
-    TEST_EQUAL(0, mymset.size());
-
-    enquire.set_query(Xapian::Query(Xapian::Query::OP_NEAR, q2, q2 + 2));
-    mymset = enquire.get_mset(0, 10);
-    TEST_EQUAL(0, mymset.size());
-}
-
 static void
 gen_longpositionlist1_db(Xapian::WritableDatabase& db, const string&)
 {
@@ -1406,7 +1368,7 @@ gen_longpositionlist1_db(Xapian::WritableDatabase& db, const string&)
 	// Exercise positions up to 4 billion.
 	Xapian::termpos half_cube = n * n / 2 * n;
 	doc.add_posting("chopsticks", half_cube);
-	if (sizeof(Xapian::termpos) >= 8) {
+	if constexpr(sizeof(Xapian::termpos) >= 8) {
 	    // Exercise 64-bit positions.
 	    doc.add_posting("spork", half_cube * half_cube);
 	}
@@ -1477,7 +1439,7 @@ DEFINE_TESTCASE(longpositionlist1, backend) {
     }
     TEST(p == pend);
 
-    if (sizeof(Xapian::termpos) >= 8) {
+    if constexpr(sizeof(Xapian::termpos) >= 8) {
 	++t;
 	TEST(t != tend);
 	TEST_EQUAL(*t, "spork");
@@ -1549,94 +1511,6 @@ DEFINE_TESTCASE(consistency2, backend) {
     }
 }
 
-DEFINE_TESTCASE(crashrecovery1, chert) {
-    // Glass has a single version file per revision, rather than multiple base
-    // files, so it simply can't get into the situations we are testing
-    // recovery from.
-    const string & dbtype = get_dbtype();
-    string path = ".";
-    path += dbtype;
-    path += "/dbw";
-    const char * base_ext = ".baseB";
-
-    Xapian::Document doc;
-    {
-	Xapian::WritableDatabase db = get_writable_database();
-	Xapian::Database dbr(get_writable_database_as_database());
-	TEST_EQUAL(dbr.get_doccount(), 0);
-
-	// Xapian::Database has full set of baseA, no baseB
-	TEST(file_exists(path + "/postlist.baseA"));
-	TEST(file_exists(path + "/record.baseA"));
-	TEST(file_exists(path + "/termlist.baseA"));
-	TEST(!file_exists(path + "/postlist.baseB"));
-	TEST(!file_exists(path + "/record.baseB"));
-	TEST(!file_exists(path + "/termlist.baseB"));
-
-	db.add_document(doc);
-	db.commit();
-	TEST(dbr.reopen());
-	TEST_EQUAL(dbr.get_doccount(), 1);
-
-	// Xapian::Database has full set of baseB, old baseA
-	TEST(file_exists(path + "/postlist.baseA"));
-	TEST(file_exists(path + "/record.baseA"));
-	TEST(file_exists(path + "/termlist.baseA"));
-	TEST(file_exists(path + "/postlist.baseB"));
-	TEST(file_exists(path + "/record.baseB"));
-	TEST(file_exists(path + "/termlist.baseB"));
-
-	db.add_document(doc);
-	db.commit();
-	TEST(dbr.reopen());
-	TEST_EQUAL(dbr.get_doccount(), 2);
-
-	// Xapian::Database has full set of baseA, old baseB
-	TEST(file_exists(path + "/postlist.baseA"));
-	TEST(file_exists(path + "/record.baseA"));
-	TEST(file_exists(path + "/termlist.baseA"));
-	TEST(file_exists(path + "/postlist.baseB"));
-	TEST(file_exists(path + "/record.baseB"));
-	TEST(file_exists(path + "/termlist.baseB"));
-
-	// Simulate a transaction starting, some of the baseB getting removed,
-	// but then the transaction fails.
-	unlink((path + "/record" + base_ext).c_str());
-	unlink((path + "/termlist" + base_ext).c_str());
-
-	TEST(!dbr.reopen());
-	TEST_EQUAL(dbr.get_doccount(), 2);
-    }
-
-    Xapian::WritableDatabase db(path, Xapian::DB_OPEN);
-    // Xapian::Database has full set of baseA, some old baseB
-    TEST(file_exists(path + "/postlist.baseA"));
-    TEST(file_exists(path + "/record.baseA"));
-    TEST(file_exists(path + "/termlist.baseA"));
-    TEST(file_exists(path + "/postlist.baseB"));
-    TEST(!file_exists(path + "/record.baseB"));
-    TEST(!file_exists(path + "/termlist.baseB"));
-    Xapian::Database dbr = Xapian::Database(path);
-
-    db.add_document(doc);
-    db.commit();
-    TEST(dbr.reopen());
-    TEST_EQUAL(dbr.get_doccount(), 3);
-
-    // Xapian::Database has full set of baseB, old baseA
-    TEST(file_exists(path + "/postlist.baseA"));
-    TEST(file_exists(path + "/record.baseA"));
-    TEST(file_exists(path + "/termlist.baseA"));
-    TEST(file_exists(path + "/postlist.baseB"));
-    TEST(file_exists(path + "/record.baseB"));
-    TEST(file_exists(path + "/termlist.baseB"));
-
-    db.add_document(doc);
-    db.commit();
-    TEST(dbr.reopen());
-    TEST_EQUAL(dbr.get_doccount(), 4);
-}
-
 // Check that DatabaseError is thrown if the docid counter would wrap.
 // Regression test for bug#152.
 DEFINE_TESTCASE(nomoredocids1, writable) {
@@ -1661,6 +1535,46 @@ DEFINE_TESTCASE(nomoredocids1, writable) {
     TEST_EXCEPTION(Xapian::DatabaseError, db.replace_document("Q42", doc));
 }
 
+static void
+gen_synonym_merge1a_db(Xapian::WritableDatabase& db, const string&)
+{
+    db.add_synonym("abc", "asd");
+    db.add_synonym("abc", "asd2");
+    db.add_synonym("abc", "asd4");
+    db.add_synonym("abc", "asd6");
+}
+
+static void
+gen_synonym_merge1b_db(Xapian::WritableDatabase& db, const string&)
+{
+    db.add_synonym("abc", "asd1");
+    db.add_synonym("abc", "asd2");
+    db.add_synonym("abc", "asd3");
+    db.add_synonym("abc", "asd5");
+    db.add_synonym("abc", "asd10");
+}
+
+// Test synonyms from subdbs are appropriately merged.
+DEFINE_TESTCASE(synonym_merge1, synonyms) {
+    Xapian::Database db_multi;
+    db_multi.add_database(get_database("synonym_merge1a",
+				       gen_synonym_merge1a_db));
+    db_multi.add_database(get_database("synonym_merge1b",
+				       gen_synonym_merge1b_db));
+
+    Xapian::TermIterator t;
+    string s, e;
+    s = "|";
+    t = db_multi.synonyms_begin("abc");
+    while (t != db_multi.synonyms_end("abc")) {
+	s += *t++;
+	s += '|';
+    }
+    e = "|asd|asd1|asd10|asd2|asd3|asd4|asd5|asd6|";
+    TEST_STRINGS_EQUAL(s, e);
+}
+
+
 // Test synonym iterators.
 DEFINE_TESTCASE(synonymitor1, writable && synonyms) {
     Xapian::WritableDatabase db = get_writable_database();
@@ -1810,11 +1724,10 @@ DEFINE_TESTCASE(termtoolong1, writable) {
 
     db.commit();
 
-    size_t limit = endswith(get_dbtype(), "glass") ? 255 : 252;
+    size_t limit = 255;
     {
-	// Currently chert and glass escape zero bytes from terms in keys for
-	// some tables, so a term with 127 zero bytes won't work for chert, and
-	// with 128 zero bytes won't work for glass.
+	// Currently glass escapes zero bytes from terms in keys for
+	// some tables, so a term with 128 zero bytes won't work for glass.
 	Xapian::Document doc;
 	doc.add_term(string(limit / 2 + 1, '\0'));
 	db.add_document(doc);
@@ -1833,10 +1746,6 @@ DEFINE_TESTCASE(termtoolong1, writable) {
 	}
     }
 
-    if (get_dbtype().find("chert") != string::npos) {
-	XFAIL("Chert fails to clear pending changes after "
-	      "InvalidArgumentError - fix too invasive");
-    }
     // Try adding a document.  Regression test for a bug fixed in 1.4.15 - in
     // earlier versions the pending changes which caused the
     // InvalidArgumentError were left around and a subsequent commit() on the
@@ -1914,11 +1823,7 @@ DEFINE_TESTCASE(lazytablebug1, synonyms) {
     }
 }
 
-/** Regression test for bug #287 for flint.
- *
- *  Chert also has the same duff code but this testcase doesn't actually
- *  tickle the bug there.
- */
+/// Regression test for bug #287 for flint.
 DEFINE_TESTCASE(cursordelbug1, writable && path) {
     static const int terms[] = { 219, 221, 222, 223, 224, 225, 226 };
     static const int copies[] = { 74, 116, 199, 21, 45, 155, 189 };
@@ -1933,8 +1838,8 @@ DEFINE_TESTCASE(cursordelbug1, writable && path) {
 	doc.add_term("XAdef");
 	doc.add_term("XRghi");
 	doc.add_term("XYabc");
-	size_t c = copies[i];
-	while (c--) db.add_document(doc);
+	for (size_t c = copies[i]; c; --c)
+	    db.add_document(doc);
     }
 
     db.commit();
