@@ -51,7 +51,16 @@ std::string get_named_writable_database_path(const std::string &name);
 
 std::string get_compaction_output_path(const std::string& name);
 
-Xapian::Database get_remote_database(const std::string &db, unsigned timeout);
+Xapian::Database get_remote_database(const std::string& db,
+				     unsigned timeout,
+				     int* port_ptr = nullptr);
+
+/** Kill the server associated with remote database @a db.
+ *
+ *  Currently only supported for remotetcp and only for a database with a
+ *  single shard.
+ */
+void kill_remote(const Xapian::Database& db);
 
 Xapian::Database get_writable_database_as_database();
 
@@ -61,14 +70,14 @@ Xapian::WritableDatabase get_writable_database_again();
 //
 // More precisely, this skips the test for any backend for which the
 // get_dbtype() function does not return a string starting with backend_prefix.
-// This allows backends like "multi_chert" to be covered by specifying "multi".
+// This allows backends like "multi_glass" to be covered by specifying "multi".
 void skip_test_unless_backend(const std::string & backend_prefix);
 
 // Skip the test for any backend of the specified type.
 //
 // More precisely, this skips the test for any backend for which the
 // get_dbtype() function returns a string starting with backend_prefix.  This
-// allows backends like "multi_chert" to be covered by specifying "multi".
+// allows backends like "multi_glass" to be covered by specifying "multi".
 void skip_test_for_backend(const std::string & backend_prefix);
 
 #define SKIP_TEST_UNLESS_BACKEND(B) skip_test_unless_backend(B)
