@@ -45,6 +45,13 @@
 
 using namespace std;
 
+[[noreturn]]
+static void
+invalid_operation(const char* msg)
+{
+    throw Xapian::InvalidOperationError(msg);
+}
+
 void
 BackendManager::index_files_to_database(Xapian::WritableDatabase & database,
 					const vector<string> & files)
@@ -80,7 +87,7 @@ BackendManager::~BackendManager() { }
 string
 BackendManager::do_get_database_path(const vector<string> &)
 {
-    throw Xapian::InvalidArgumentError("Path isn't meaningful for this database type");
+    invalid_operation("Path isn't meaningful for this database type");
 }
 
 Xapian::Database
@@ -201,7 +208,7 @@ BackendManager::get_database_path(const string & file)
 Xapian::WritableDatabase
 BackendManager::get_writable_database(const string &, const string &)
 {
-    throw Xapian::InvalidArgumentError("Attempted to open a disabled database");
+    invalid_operation("Attempted to open a disabled database");
 }
 
 Xapian::WritableDatabase
@@ -217,7 +224,13 @@ BackendManager::get_remote_writable_database(string)
 string
 BackendManager::get_writable_database_path(const std::string &)
 {
-    throw Xapian::InvalidArgumentError("Path isn't meaningful for this database type");
+    invalid_operation("Path isn't meaningful for this database type");
+}
+
+string
+BackendManager::get_compaction_output_path(const std::string&)
+{
+    invalid_operation("Compaction not supported for this database type");
 }
 
 Xapian::WritableDatabase
@@ -226,16 +239,11 @@ BackendManager::get_generated_database(const std::string& name)
     return get_writable_database(name, string());
 }
 
-string
-BackendManager::get_compaction_output_path(const std::string&)
-{
-    throw Xapian::InvalidArgumentError("Compaction now supported for this database type");
-}
-
 string
 BackendManager::get_generated_database_path(const std::string &)
 {
-    throw Xapian::InvalidArgumentError("Generated databases aren't supported for this database type");
+    invalid_operation("Generated databases aren't supported for this database "
+		      "type");
 }
 
 void
@@ -243,9 +251,12 @@ BackendManager::finalise_generated_database(const std::string&)
 { }
 
 Xapian::Database
-BackendManager::get_remote_database(const vector<string> &, unsigned int)
+BackendManager::get_remote_database(const vector<string>&,
+				    unsigned int,
+				    int*)
 {
-    string msg = "BackendManager::get_remote_database() called for non-remote database (type is ";
+    string msg = "BackendManager::get_remote_database() called for non-remote "
+		 "database (type is ";
     msg += get_dbtype();
     msg += ')';
     throw Xapian::InvalidOperationError(msg);
@@ -291,6 +302,13 @@ BackendManager::clean_up()
 {
 }
 
+void
+BackendManager::kill_remote(const Xapian::Database&)
+{
+    throw Xapian::InvalidOperationError("kill_remote() only supported for "
+					"remotetcp databases");
+}
+
 const char *
 BackendManager::get_xapian_progsrv_command()
 {
