@@ -150,11 +150,6 @@ retry_unlink:
 	    dbbase += str(n);
 	    dbs.add_database(Xapian::WritableDatabase(dbbase, flags));
 	    out << subtype << ' ' << dbname << "___" << n << '\n';
-	} else if (subtype == "chert") {
-	    flags |= Xapian::DB_BACKEND_CHERT;
-	    dbbase += str(n);
-	    dbs.add_database(Xapian::WritableDatabase(dbbase, flags));
-	    out << subtype << ' ' << dbname << "___" << n << '\n';
 	} else if (subtype == "remoteprog_glass") {
 	    flags |= Xapian::DB_BACKEND_GLASS;
 	    dbbase += str(n);
@@ -236,7 +231,8 @@ BackendManagerMulti::get_writable_database_path(const std::string& name)
 
 Xapian::Database
 BackendManagerMulti::get_remote_database(const std::vector<std::string>& files,
-					 unsigned int timeout)
+					 unsigned int timeout,
+					 int* port_ptr)
 {
     Xapian::Database db;
     size_t remotes = 0;
@@ -247,7 +243,10 @@ BackendManagerMulti::get_remote_database(const std::vector<std::string>& files,
 	}
 
 	++remotes;
-	db.add_database(sub_manager->get_remote_database(files, timeout));
+	// If there are multiple remote shards, we'll set *port_ptr to the port
+	// used by the last one opened.
+	db.add_database(sub_manager->get_remote_database(files, timeout,
+							 port_ptr));
     }
 
     if (remotes == 0) {
