@@ -26,8 +26,8 @@
 
 #include "testsuite.h"
 #include "backendmanager.h"
-#include "backendmanager_chert.h"
 #include "backendmanager_glass.h"
+#include "backendmanager_honey.h"
 #include "backendmanager_inmemory.h"
 #include "backendmanager_multi.h"
 #include "backendmanager_remoteprog.h"
@@ -69,13 +69,6 @@ TestRunner::set_properties_for_backend(const string & backend_name)
 	{ "none", 0 },
 	{ "inmemory", INMEMORY|
 	    BACKEND|POSITIONAL|WRITABLE|METADATA|VALUESTATS },
-	{ "chert", CHERT|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|SPELLING|METADATA|
-	    SYNONYMS|VALUESTATS|CHECK|COMPACT|PATH
-#ifdef XAPIAN_HAS_REMOTE_BACKEND
-	    |REPLICAS
-#endif
-	},
 	{ "glass", GLASS|
 	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|SPELLING|METADATA|
 	    SYNONYMS|VALUESTATS|CHECK|COMPACT|PATH
@@ -83,30 +76,33 @@ TestRunner::set_properties_for_backend(const string & backend_name)
 	    |REPLICAS
 #endif
 	},
-	{ "multi_chert", MULTI|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS|
-	    PATH },
 	{ "multi_glass", MULTI|
 	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS|
-	    SPELLING|CHECK|COMPACT|PATH },
-	{ "remoteprog_chert", REMOTE|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS },
-	{ "remotetcp_chert", REMOTE|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS },
+	    SPELLING|SYNONYMS|CHECK|COMPACT|PATH },
 	{ "multi_glass_remoteprog_glass", MULTI|REMOTE|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS },
-	{ "multi_remoteprog_glass", MULTI|REMOTE|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS },
-	{ "remoteprog_glass", REMOTE|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS
+	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS|
+	    SYNONYMS
 	},
-	{ "remotetcp_glass", REMOTE|
-	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS
+	{ "multi_remoteprog_glass", MULTI|REMOTE|
+	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS|
+	    SYNONYMS
+	},
+	{ "remoteprog_glass", REMOTE|
+	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS|
+	    SYNONYMS
+	},
+	{ "remotetcp_glass", REMOTE|REMOTETCP|
+	    BACKEND|TRANSACTIONS|POSITIONAL|WRITABLE|METADATA|VALUESTATS|
+	    SYNONYMS
 	},
 	{ "singlefile_glass", SINGLEFILE|
 	    BACKEND|POSITIONAL|METADATA|SPELLING|SYNONYMS|VALUESTATS|
 	    CHECK|COMPACT|PATH
 	},
+	{ "honey", HONEY|
+	    BACKEND|POSITIONAL|METADATA|SPELLING|SYNONYMS|VALUESTATS|
+	    CHECK|COMPACT|PATH
+	},
 	{ NULL, 0 }
     };
 
@@ -142,6 +138,17 @@ TestRunner::run_tests(int argc, char ** argv)
 	srcdir = test_driver::get_srcdir();
 	string datadir = srcdir + "/testdata/";
 
+#ifdef XAPIAN_HAS_HONEY_BACKEND
+# ifdef XAPIAN_HAS_GLASS_BACKEND
+	{
+	    BackendManagerGlass glass_man(datadir);
+	    do_tests_for_backend(BackendManagerHoney(datadir, &glass_man));
+	}
+# else
+	do_tests_for_backend(BackendManagerHoney(datadir));
+# endif
+#endif
+
 	do_tests_for_backend(BackendManager(string(), "none"));
 
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
@@ -176,23 +183,6 @@ TestRunner::run_tests(int argc, char ** argv)
 # endif
 	}
 #endif
-
-#ifdef XAPIAN_HAS_CHERT_BACKEND
-	{
-	    BackendManagerChert chert_man(datadir);
-
-	    // Vector for multi backendmanagers.
-	    vector<BackendManager*> multi_mans;
-	    multi_mans = {&chert_man, &chert_man};
-
-	    do_tests_for_backend(chert_man);
-	    do_tests_for_backend(BackendManagerMulti(datadir, multi_mans));
-# ifdef XAPIAN_HAS_REMOTE_BACKEND
-	    do_tests_for_backend(BackendManagerRemoteProg(&chert_man));
-	    do_tests_for_backend(BackendManagerRemoteTcp(&chert_man));
-# endif
-	}
-#endif
     } catch (const std::exception& e) {
 	cerr << "\nTest harness failed with std::exception: " << e.what()
 	     << '\n';
