@@ -85,7 +85,7 @@ class TestRunner {
 	INMEMORY	= 0x00001000,
 	GLASS		= 0x00002000,
 	COMPACT		= 0x00004000,
-	CHERT		= 0x00008000,
+	HONEY		= 0x00008000,
 	/// Requires get_database_path() or similar.
 	PATH		= 0x00010000,
 	/// TCP variant of remote.
