@@ -40,6 +40,7 @@
 #endif
 
 #include <algorithm>
+#include <chrono>
 #include <ios>
 #include <iostream>
 #include <set>
@@ -53,6 +54,7 @@
 
 #include "gnu_getopt.h"
 
+#include "parseint.h"
 #include <setjmp.h>
 #include <signal.h>
 
@@ -69,7 +71,6 @@
 #endif
 #include "errno_to_string.h"
 #include "filetests.h"
-#include "noreturn.h"
 #include "str.h"
 #include "stringutils.h"
 
@@ -115,6 +116,12 @@ string test_driver::col_red, test_driver::col_green;
 string test_driver::col_yellow, test_driver::col_reset;
 bool test_driver::use_cr = false;
 
+// time constant in seconds to mark tests as slow or not
+static const double SLOW_TEST_THRESHOLD = 10.00;
+
+// vector to store the slow tests
+static vector<pair<const char*, double>> slow_tests;
+
 void
 test_driver::write_and_clear_tout()
 {
@@ -181,7 +188,7 @@ static void * sigaddr = NULL;
 extern "C" {
 
 #if defined HAVE_SIGACTION && defined SA_SIGINFO
-XAPIAN_NORETURN(static void handle_sig(int signum_, siginfo_t *si, void *));
+[[noreturn]]
 static void handle_sig(int signum_, siginfo_t *si, void *)
 {
     // Disable all our signal handlers to avoid problems if the signal
@@ -211,7 +218,7 @@ static void handle_sig(int signum_, siginfo_t *si, void *)
 
 #else
 
-XAPIAN_NORETURN(static void handle_sig(int signum_));
+[[noreturn]]
 static void handle_sig(int signum_)
 {
     // Disable all our signal handlers to avoid problems if the signal
@@ -334,7 +341,18 @@ test_driver::runtest(const test_desc *test)
 
     while (true) {
 	tout.str(string());
+#ifdef _MSC_VER
+// Suppress warning about _setjmp and C++ object destruction.  It's not ideal
+// that some destructors may not get called, but this is in the test harness
+// rather than production code, and overall it's more helpful for the test
+// harness to be able to report clearly which testcase triggered a signal.
+# pragma warning(push)
+# pragma warning(disable:4611)
+#endif
 	if (SIGSETJMP(jb, 1) == 0) {
+#ifdef _MSC_VER
+# pragma warning(pop)
+#endif
 	    SignalRedirector sig; // use object so signal handlers are reset
 	    static bool catch_signals =
 		(getenv("XAPIAN_TESTSUITE_SIG_DFL") == NULL);
@@ -482,10 +500,10 @@ test_driver::runtest(const test_desc *test)
 		    }
 		    if (vg_reachable > 0) {
 			// C++ STL implementations often "horde" released
-			// memory - for GCC 3.4 and newer the runtest script
-			// sets GLIBCXX_FORCE_NEW=1 which will disable this
-			// behaviour so we avoid this issue, but for older
-			// GCC and other compilers this may be an issue.
+			// memory - the runtest script sets GLIBCXX_FORCE_NEW=1
+			// which under GCC will disable this behaviour and so
+			// we avoid this issue, but for other compilers this
+			// may be an issue.
 			//
 			// See also:
 			// https://valgrind.org/docs/manual/faq.html#faq.reports
@@ -717,7 +735,11 @@ test_driver::do_run_tests(vector<string>::const_iterator b,
 	if (do_this_test) {
 	    out << "Running test: " << test->name << "...";
 	    out.flush();
+	    auto starttime = chrono::high_resolution_clock::now();
 	    test_driver::test_result test_res = runtest(test);
+	    auto endtime = chrono::high_resolution_clock::now();
+	    auto test_duration = chrono::duration_cast<chrono::duration<double>>
+				 (endtime - starttime);
 #ifndef NO_LIBXAPIAN
 	    if (backendmanager)
 		backendmanager->clean_up();
@@ -725,10 +747,17 @@ test_driver::do_run_tests(vector<string>::const_iterator b,
 	    switch (test_res) {
 		case PASS:
 		    ++res.succeeded;
+
+		    if (test_duration.count() >= SLOW_TEST_THRESHOLD) {
+			slow_tests.emplace_back(test->name,
+						test_duration.count());
+		    }
+
 		    if (verbose || !use_cr) {
 			out << col_green << " ok" << col_reset << '\n';
 		    } else {
-			out << "\r                                                                               \r";
+			out << "\r                                        "
+			       "                                       \r";
 		    }
 		    break;
 		case XFAIL:
@@ -810,6 +839,16 @@ test_driver::report(const test_driver::result &r, const string &desc)
 	} else {
 	    cout << ".\n";
 	}
+
+	if (!slow_tests.empty()) {
+	    const char* sep = "Slow tests: ";
+	    for (auto& test : slow_tests) {
+		cout << sep << test.first << " (" << test.second << " s)";
+		sep = ", ";
+	    }
+	    cout << ".\n";
+	    slow_tests.clear();
+	}
     }
 }
 
@@ -898,8 +937,12 @@ test_driver::parse_command_line(int argc, char **argv)
 
     if (verbose == 0) {
 	const char *p = getenv("VERBOSE");
-	if (p != NULL) {
-	    verbose = atoi(p);
+	if (p && *p) {
+	    unsigned int temp;
+	    if (!parse_unsigned(p, temp)) {
+		throw "VERBOSE must be a non-negative integer";
+	    }
+	    verbose = temp;
 	}
     }
 
