@@ -21,10 +21,8 @@
  * USA
  */
 
-#ifndef OM_HGUARD_TESTSUITE_H
-#define OM_HGUARD_TESTSUITE_H
-
-#include "noreturn.h"
+#ifndef XAPIAN_INCLUDED_TESTSUITE_H
+#define XAPIAN_INCLUDED_TESTSUITE_H
 
 #ifndef XAPIAN_UNITTEST
 # include "output.h"
@@ -171,7 +169,8 @@ class test_driver {
      */
     static void parse_command_line(int argc, char **argv);
 
-    XAPIAN_NORETURN(static void usage());
+    [[noreturn]]
+    static void usage();
 
     static int run(const test_desc *tests);
 
@@ -331,4 +330,4 @@ inline void XFAIL(const char* msg) {
     expected_failure = msg;
 }
 
-#endif // OM_HGUARD_TESTSUITE_H
+#endif // XAPIAN_INCLUDED_TESTSUITE_H
