@@ -31,7 +31,6 @@
 
 using namespace std;
 
-#include "autoptr.h"
 #include "testsuite.h"
 #include "testutils.h"
 
@@ -141,70 +140,6 @@ static void test_refcnt2()
     TEST_AND_EXPLAIN(!deleted, "Object deleted by self-assignment");
 }
 
-// Class for testing AutoPtr<>.
-class test_autoptr {
-    bool &deleted;
-  public:
-    test_autoptr(bool &deleted_) : deleted(deleted_) {
-	tout << "test_autoptr constructor\n";
-    }
-    ~test_autoptr() {
-	deleted = true;
-	tout << "test_autoptr destructor\n";
-    }
-};
-
-// Test autoptr self-assignment.
-static void test_autoptr1()
-{
-    bool deleted = false;
-
-    test_autoptr * raw_ptr = new test_autoptr(deleted);
-    {
-	AutoPtr<test_autoptr> ptr(raw_ptr);
-
-	TEST_EQUAL(ptr.get(), raw_ptr);
-	TEST(!deleted);
-
-	ptr.reset(ptr.release());
-
-	TEST_EQUAL(ptr.get(), raw_ptr);
-	TEST(!deleted);
-
-	ptr.swap(ptr);
-
-	TEST_EQUAL(ptr.get(), raw_ptr);
-	TEST(!deleted);
-
-	swap(ptr, ptr);
-
-	TEST_EQUAL(ptr.get(), raw_ptr);
-	TEST(!deleted);
-    }
-
-    TEST(deleted);
-
-    deleted = false;
-    raw_ptr = new test_autoptr(deleted);
-
-    bool deleted2 = false;
-    test_autoptr * raw_ptr2 = new test_autoptr(deleted2);
-    AutoPtr<test_autoptr> ptr(raw_ptr2);
-
-    TEST_EQUAL(ptr.get(), raw_ptr2);
-    TEST(!deleted);
-    TEST(!deleted2);
-
-    ptr.reset(raw_ptr);
-    TEST_EQUAL(ptr.get(), raw_ptr);
-    TEST(!deleted);
-    TEST(deleted2);
-
-    ptr.reset();
-    TEST_EQUAL(ptr.get(), static_cast<test_autoptr*>(0));
-    TEST(deleted);
-}
-
 // test string comparisons
 static void test_stringcomp1()
 {
@@ -317,63 +252,6 @@ static void test_pack_uint_preserving_sort1()
     TEST(ptr == end);
 }
 
-/// Test C_pack_uint_preserving_sort()
-static void test_pack_uint_preserving_sort2()
-{
-    string prev_packed;
-    for (unsigned int i = 0; i != 1000; ++i) {
-	string packed;
-	C_pack_uint_preserving_sort(packed, i);
-	const char * ptr = packed.data();
-	const char * end = ptr + packed.size();
-	unsigned int result;
-	TEST(C_unpack_uint_preserving_sort(&ptr, end, &result));
-	TEST_EQUAL(result, i);
-	TEST(ptr == end);
-	TEST_REL(prev_packed, <, packed);
-	swap(prev_packed, packed);
-    }
-    for (unsigned int i = 2345; i < 65000; i += 113) {
-	string packed;
-	C_pack_uint_preserving_sort(packed, i);
-	const char * ptr = packed.data();
-	const char * end = ptr + packed.size();
-	unsigned int result;
-	TEST(C_unpack_uint_preserving_sort(&ptr, end, &result));
-	TEST_EQUAL(result, i);
-	TEST(ptr == end);
-	TEST_REL(prev_packed, <, packed);
-	swap(prev_packed, packed);
-    }
-    unsigned int prev = 64999;
-    for (unsigned int i = 65000; i > prev; prev = i, i = (i << 1) ^ 1337) {
-	string packed;
-	C_pack_uint_preserving_sort(packed, i);
-	const char * ptr = packed.data();
-	const char * end = ptr + packed.size();
-	unsigned int result;
-	TEST(C_unpack_uint_preserving_sort(&ptr, end, &result));
-	TEST_EQUAL(result, i);
-	TEST(ptr == end);
-	TEST_REL(prev_packed, <, packed);
-	swap(prev_packed, packed);
-    }
-
-    /* Test packing multiple numbers to one string. */
-    string packed;
-    for (unsigned int i = 23456; i < 765432; i += 1131) {
-	C_pack_uint_preserving_sort(packed, i);
-    }
-    const char * ptr = packed.data();
-    const char * end = ptr + packed.size();
-    for (unsigned int i = 23456; i < 765432; i += 1131) {
-	unsigned int result;
-	TEST(C_unpack_uint_preserving_sort(&ptr, end, &result));
-	TEST_EQUAL(result, i);
-    }
-    TEST(ptr == end);
-}
-
 /// Test C_isupper() etc.
 static void test_chartype1()
 {
@@ -568,11 +446,9 @@ static const test_desc tests[] = {
     TESTCASE(exception1),
     TESTCASE(refcnt1),
     TESTCASE(refcnt2),
-    TESTCASE(autoptr1),
     TESTCASE(stringcomp1),
     TESTCASE(temporarydtor1),
     TESTCASE(pack_uint_preserving_sort1),
-    TESTCASE(pack_uint_preserving_sort2),
     TESTCASE(chartype1),
     {0, 0}
 };
