@@ -31,6 +31,7 @@
 #include <zlib.h>
 
 #include <xapian.h>
+#include "parseint.h"
 #include "testsuite.h"
 
 using namespace std;
@@ -103,11 +104,11 @@ test_stemjunk()
 	 << '\n';
 
     if (stemmed_size > JUNKSIZE * 101 / 100) {
-	FAIL_TEST("Stemmed data is significantly bigger than input ("
+	FAIL_TEST("Stemmed data is significantly bigger than input: "
 		  << stemmed_size << " vs. " << JUNKSIZE);
     }
-    if (stemmed_size < JUNKSIZE / 2) {
-	FAIL_TEST("Stemmed data is significantly smaller than input ("
+    if (stemmed_size < JUNKSIZE * 2 / 5) {
+	FAIL_TEST("Stemmed data is significantly smaller than input: "
 		  << stemmed_size << " vs. " << JUNKSIZE);
     }
 }
@@ -201,7 +202,11 @@ try {
     srcdir = test_driver::get_srcdir();
     int result = 0;
 
-    if (!seed_str.empty()) seed = atoi(seed_str.c_str());
+    if (!seed_str.empty()) {
+	if (!parse_signed(seed_str.c_str(), seed)) {
+	    throw "seed must be an integer";
+	}
+    }
     cout << "The random seed is " << seed << '\n';
     cout << "Please report the seed when reporting a test failure.\n";
 
