@@ -1,7 +1,7 @@
 /** @file
  * @brief Unit tests of non-Xapian-specific internal code.
  */
-/* Copyright (C) 2006-2024 Olly Betts
+/* Copyright (C) 2006-2025 Olly Betts
  * Copyright (C) 2007 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -30,6 +30,8 @@
 #include <cstring>
 #include <iostream>
 #include <limits>
+#include <memory>
+#include <string_view>
 #include <utility>
 
 #include "safeunistd.h"
@@ -57,8 +59,8 @@ static const char * unittest_assertion_failed = NULL;
     } while (false)
 
 // Utility code we use:
+#include "../backends/multi.h"
 #include "../common/stringutils.h"
-#include "../common/log2.h"
 
 // Simpler version of TEST_EXCEPTION macro.
 #define TEST_EXCEPTION(TYPE, CODE) \
@@ -78,14 +80,16 @@ static const char * unittest_assertion_failed = NULL;
 #include "../common/io_utils.cc"
 #include "../common/fileutils.cc"
 #include "../common/overflow.h"
+#include "../common/pack.cc"
 #include "../common/parseint.h"
 #include "../common/posixy_wrapper.cc"
 #include "../common/serialise-double.cc"
 #include "../common/str.cc"
-#include "../net/length.cc"
 #include "../net/serialise-error.cc"
 #include "../api/error.cc"
+#include "../api/smallvector.cc"
 #include "../api/sortable-serialise.cc"
+#include "../include/xapian/intrusive_ptr.h"
 
 // fileutils.cc uses opendir(), etc though not in a function we currently test.
 #include "../common/msvc_dirent.cc"
@@ -99,7 +103,7 @@ using namespace std;
 // The full implementation needs Xapian::Utf8Iterator and
 // Xapian::Unicode::append_utf8().
 void
-description_append(std::string & desc, const std::string &s)
+description_append(std::string& desc, std::string_view s)
 {
     desc += s;
 }
@@ -248,9 +252,7 @@ DEFINE_TESTCASE(serialisedouble1) {
     check_double_serialisation(DBL_MIN);
     check_double_serialisation(-DBL_MIN);
 
-    const double *p;
-    for (p = test_values; p < test_values + sizeof(test_values) / sizeof(double); ++p) {
-	double val = *p;
+    for (double val : test_values) {
 	check_double_serialisation(val);
 	check_double_serialisation(-val);
 	check_double_serialisation(1.0 / val);
@@ -258,17 +260,17 @@ DEFINE_TESTCASE(serialisedouble1) {
     }
 }
 
-#ifdef XAPIAN_HAS_REMOTE_BACKEND
-// Check serialisation of lengths.
-static void test_serialiselength1()
+// Test pack_uint() and unpack_uint().
+static void test_packuint1()
 {
     size_t n = 0;
     while (n < 0xff000000) {
-	string s = encode_length(n);
-	const char *p = s.data();
-	const char *p_end = p + s.size();
+	string s;
+	pack_uint(s, n);
+	const char* p = s.data();
+	const char* p_end = p + s.size();
 	size_t decoded_n;
-	decode_length(&p, p_end, decoded_n);
+	TEST(unpack_uint(&p, p_end, &decoded_n));
 	if (n != decoded_n || p != p_end) tout << "[" << s << "]\n";
 	TEST_EQUAL(n, decoded_n);
 	TEST_EQUAL(p_end - p, 0);
@@ -280,98 +282,66 @@ static void test_serialiselength1()
     }
 }
 
-// Regression test: vetting the remaining buffer length
-static void test_serialiselength2()
+static void
+packstring1_helper(size_t len)
 {
-    // Special case tests for 0
+    string s;
+    pack_string(s, string(len, 'x'));
     {
-	string s = encode_length(0);
-	{
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    TEST(r == 0);
-	    TEST(p == p_end);
-	}
-	s += 'x';
-	{
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    TEST(r == 0);
-	    TEST_EQUAL(p_end - p, 1);
-	}
+	const char* p = s.data();
+	const char* p_end = p + s.size();
+	// unpack_string() should overwrite any existing value.
+	string r = "dummy";
+	TEST(unpack_string(&p, p_end, r));
+	TEST_EQUAL(r.size(), len);
+	TEST_EQUAL(r.find_first_not_of('x'), r.npos);
+	TEST(p == p_end);
     }
-    // Special case tests for 1
+    s += 'x';
     {
-	string s = encode_length(1);
-	TEST_EXCEPTION(Xapian_NetworkError,
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    (void)r;
-	);
-	s += 'x';
-	{
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    TEST(r == 1);
-	    TEST_EQUAL(p_end - p, 1);
-	}
-	s += 'x';
-	{
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    TEST(r == 1);
-	    TEST_EQUAL(p_end - p, 2);
-	}
+	const char* p = s.data();
+	const char* p_end = p + s.size();
+	// unpack_string() should overwrite any existing value.
+	string r = "dummy";
+	TEST(unpack_string(&p, p_end, r));
+	TEST_EQUAL(r.size(), len);
+	TEST_EQUAL(r.find_first_not_of('x'), r.npos);
+	TEST_EQUAL(p_end - p, 1);
     }
+    // Test truncated encodings fail to unpack.
+    size_t trunc_len = s.size() - 2;
+    do {
+	const char* p = s.data();
+	const char* p_end = p + trunc_len;
+	string r;
+	TEST(!unpack_string(&p, p_end, r));
+	TEST(!p);
+	trunc_len >>= 1;
+    } while (trunc_len);
+}
+
+// Test pack_string() and unpack_string().
+static void test_packstring1()
+{
+    packstring1_helper(0);
+    packstring1_helper(1);
     // Nothing magic here, just test a range of odd and even values.
     for (size_t n = 2; n < 1000; n = (n + 1) * 2 + (n >> 1)) {
-	string s = encode_length(n);
-	TEST_EXCEPTION(Xapian_NetworkError,
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    (void)r;
-	);
-	s.append(n - 1, 'x');
-	TEST_EXCEPTION(Xapian_NetworkError,
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    (void)r;
-	);
-	s += 'x';
-	{
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    TEST(r == n);
-	    TEST_EQUAL(size_t(p_end - p), n);
-	}
-	s += 'x';
-	{
-	    const char *p = s.data();
-	    const char *p_end = p + s.size();
-	    size_t r;
-	    decode_length_and_check(&p, p_end, r);
-	    TEST(r == n);
-	    TEST_EQUAL(size_t(p_end - p), n + 1);
-	}
+	packstring1_helper(n);
     }
 }
 
+// Test pack_string_empty()
+static void test_packstring2()
+{
+    string s;
+    pack_string(s, {});
+    string s_empty;
+    pack_string_empty(s_empty);
+    TEST_EQUAL(s, s_empty);
+}
+
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
 // Check serialisation of Xapian::Error.
 static void test_serialiseerror1()
 {
@@ -408,15 +378,6 @@ static void test_serialiseerror1()
 }
 #endif
 
-// Test log2() (which might be our replacement version).
-static void test_log2()
-{
-    TEST_EQUAL(log2(1.0), 0.0);
-    TEST_EQUAL(log2(2.0), 1.0);
-    TEST_EQUAL(log2(1024.0), 10.0);
-    TEST_EQUAL(log2(0.5), -1.0);
-}
-
 static const double test_sortableserialise_numbers[] = {
     -HUGE_VAL,
     -DBL_MAX,
@@ -592,6 +553,28 @@ static void test_closefrom1()
 #endif
 }
 
+static void test_shard1()
+{
+    for (Xapian::docid did = 1; did != 10; ++did) {
+	for (Xapian::doccount n = 1; n != 10; ++n) {
+	    Xapian::docid s_did = shard_docid(did, n);
+	    Xapian::doccount shard = shard_number(did, n);
+	    TEST_EQUAL(s_did, (did - 1) / n + 1);
+	    TEST_EQUAL(shard, (did - 1) % n);
+	    if (n == 1)
+		TEST_EQUAL(did, s_did);
+	    if (did == 1)
+		TEST_EQUAL(s_did, 1);
+	    if (s_did == 1)
+		TEST(did <= n);
+	    TEST(s_did != 0);
+	    TEST(s_did <= did);
+	    TEST(shard < n);
+	    TEST_EQUAL(did, unshard(s_did, shard, n));
+	}
+    }
+}
+
 static void test_uuid1()
 {
     Uuid uuid, uuid2;
@@ -706,13 +689,33 @@ static void test_movesupport1()
 
 	// Test move constructor
 	Xapian::Internal::intrusive_ptr<A> p2(std::move(p1));
+	TEST(p2);
 	TEST_EQUAL(p2->get_x(), 5);
 	TEST_EQUAL(p1.get(), 0);
+	TEST(!p1);
 
 	// Test move assignment
 	p3 = std::move(p2);
+	TEST(p3);
 	TEST_EQUAL(p3->get_x(), 5);
 	TEST_EQUAL(p2.get(), 0);
+	TEST(!p2);
+    }
+
+    {
+	// Same test for intrusive_ptr_nonnull class
+	Xapian::Internal::intrusive_ptr_nonnull<A> p1(new A{5});
+	Xapian::Internal::intrusive_ptr_nonnull<A> p3(new A{6});
+
+	// Test move constructor
+	Xapian::Internal::intrusive_ptr_nonnull<A> p2(std::move(p1));
+	TEST(p2.get());
+	TEST_EQUAL(p2->get_x(), 5);
+
+	// Test move assignment
+	p3 = std::move(p2);
+	TEST(p3.get());
+	TEST_EQUAL(p3->get_x(), 5);
     }
 
     bool alive = false;
@@ -725,14 +728,18 @@ static void test_movesupport1()
 
 	// Test move constructor
 	Xapian::Internal::opt_intrusive_ptr<B> p2(std::move(p1));
+	TEST(p2);
 	TEST_EQUAL(p2->get_x(), 5);
 	TEST_EQUAL(p1.get(), 0);
+	TEST(!p1);
 	TEST_EQUAL(alive, true);
 
 	// Test move assignment
 	p3 = std::move(p2);
+	TEST(p3);
 	TEST_EQUAL(p3->get_x(), 5);
 	TEST_EQUAL(p2.get(), 0);
+	TEST(!p2);
 	TEST_EQUAL(alive, true);
     }
     // Test that object b1 has been deleted.
@@ -838,7 +845,7 @@ static void test_muloverflows1()
 template<typename U>
 inline static void parseunsigned_helper() {
     U val;
-    const U max_val = numeric_limits<U>::max();
+    constexpr U max_val = numeric_limits<U>::max();
     tout << "Testing with parseunsigned_helper\n";
     TEST(parse_unsigned("0", val));
     TEST_EQUAL(val, 0);
@@ -851,7 +858,7 @@ inline static void parseunsigned_helper() {
     TEST(!parse_unsigned("abc", val));
     TEST(!parse_unsigned("0a", val));
     // Only test if we can construct a value one larger easily.
-    if (max_val + 1ull != 0)
+    if constexpr(max_val + 1ull != 0)
 	TEST(!parse_unsigned(str(max_val + 1ull).c_str(), val));
 }
 
@@ -890,7 +897,15 @@ inline static void parsesigned_helper() {
     unsigned long long one_too_large = max_val + 1ull;
     TEST(!parse_signed(str(one_too_large).c_str(), val));
 
-    unsigned long long one_too_small_negated = 1ull - min_val;
+    // We need to use an unsigned long long here so this works when S is
+    // long long.  The somewhat contorted way we calculate this is to
+    // avoid unsigned overflow and value-changing implicit conversions.
+    // These aren't undefined behaviour, but we want to limit where they
+    // happen in the library to a few low-level places and ubsan's extra
+    // checks provide a way to check that, so keeping the testsuite clean
+    // of such warnings is useful.
+    unsigned long long one_too_small_negated =
+	static_cast<unsigned long long>(-(min_val + 1)) + 2ull;
     TEST(!parse_signed(("-" + str(one_too_small_negated)).c_str(), val));
 }
 
@@ -939,7 +954,7 @@ try {
 	// has any effect is much harder to do.
 	TEST(io_full_sync(fd));
 
-	if (sizeof(off_t) <= 4) {
+	if constexpr(sizeof(off_t) <= 4) {
 	    SKIP_TEST("Skipping rest of testcase - no Large File Support");
 	}
 
@@ -1001,21 +1016,45 @@ try {
     throw e.get_description();
 }
 
+DEFINE_TESTCASE(vec1) {
+    Xapian::Vec<int> v_int;
+    Xapian::Vec<double> v_double;
+    Xapian::Vec<char> v_char;
+    Xapian::Vec<const char*> v_cstring;
+    for (int i = 0; i < 100; ++i) {
+	v_int.push_back(i);
+	v_double.push_back(double(i));
+	v_char.push_back(char(i & 0xff));
+	v_cstring.push_back("string");
+    }
+}
+
+DEFINE_TESTCASE(vecdeleter1) {
+    Xapian::VecUniquePtr<int> v_unique_int;
+    struct U { int x; };
+    Xapian::VecUniquePtr<U> v_unique_u;
+    for (int i = 0; i < 100; ++i) {
+	v_unique_int.push_back(new int(42));
+	v_unique_u.push_back(new U());
+    }
+}
+
 static const test_desc tests[] = {
     TESTCASE(simple_exceptions_work1),
     TESTCASE(class_exceptions_work1),
     TESTCASE(resolverelativepath1),
     TESTCASE(serialisedouble1),
+    TESTCASE(packuint1),
+    TESTCASE(packstring1),
+    TESTCASE(packstring2),
 #ifdef XAPIAN_HAS_REMOTE_BACKEND
-    TESTCASE(serialiselength1),
-    TESTCASE(serialiselength2),
     TESTCASE(serialiseerror1),
 #endif
-    TESTCASE(log2),
     TESTCASE(sortableserialise1),
     TESTCASE(tostring1),
     TESTCASE(strbool1),
     TESTCASE(closefrom1),
+    TESTCASE(shard1),
     TESTCASE(uuid1),
     TESTCASE(movesupport1),
     TESTCASE(addoverflows1),
@@ -1024,6 +1063,8 @@ static const test_desc tests[] = {
     TESTCASE(parseunsigned1),
     TESTCASE(parsesigned1),
     TESTCASE(ioblock1),
+    TESTCASE(vec1),
+    TESTCASE(vecdeleter1),
     END_OF_TESTCASES
 };
 
