@@ -2,7 +2,7 @@
  * @brief Xapian::BB2Weight class - the BB2 weighting scheme of the DFR framework.
  */
 /* Copyright (C) 2013,2014 Aarsh Shah
- * Copyright (C) 2014,2015,2016,2017 Olly Betts
+ * Copyright (C) 2014,2015,2016,2017,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,12 +22,15 @@
 #include <config.h>
 
 #include "xapian/weight.h"
-#include "common/log2.h"
+
+#include "weightinternal.h"
 
 #include "serialise-double.h"
 
 #include "xapian/error.h"
 
+#include <cmath>
+
 using namespace std;
 
 namespace Xapian {
@@ -119,7 +122,7 @@ BB2Weight::init(double factor)
 string
 BB2Weight::name() const
 {
-    return "Xapian::BB2Weight";
+    return "bb2";
 }
 
 string
@@ -141,7 +144,7 @@ BB2Weight::unserialise(const string & s) const
 
 double
 BB2Weight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
-		       Xapian::termcount) const
+		       Xapian::termcount, Xapian::termcount) const
 {
     if (wdf == 0) return 0.0;
 
@@ -177,16 +180,25 @@ BB2Weight::get_maxpart() const
     return upper_bound;
 }
 
-double
-BB2Weight::get_sumextra(Xapian::termcount, Xapian::termcount) const
+[[noreturn]]
+static inline void
+parameter_error(const char* message, const char* params)
 {
-    return 0;
+    Xapian::Weight::Internal::parameter_error(message, "bb2", params);
 }
 
-double
-BB2Weight::get_maxextra() const
+BB2Weight*
+BB2Weight::create_from_parameters(const char* params) const
 {
-    return 0;
+    const char* p = params;
+    if (*p == '\0')
+	return new Xapian::BB2Weight();
+    double c = 1.0;
+    if (!Xapian::Weight::Internal::double_param(&p, &c))
+	parameter_error("Parameter is invalid", params);
+    if (*p)
+	parameter_error("Extra data after parameter", params);
+    return new Xapian::BB2Weight(c);
 }
 
 }
