@@ -1,7 +1,7 @@
 /** @file
  * @brief Xapian::BM25Weight class - the BM25 probabilistic formula
  */
-/* Copyright (C) 2009,2010,2011,2012,2014,2015 Olly Betts
+/* Copyright (C) 2009,2010,2011,2012,2014,2015,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 #include <config.h>
 
 #include "xapian/weight.h"
+#include "weightinternal.h"
 
 #include "debuglog.h"
 #include "omassert.h"
@@ -131,7 +132,7 @@ BM25Weight::init(double factor)
 string
 BM25Weight::name() const
 {
-    return "Xapian::BM25Weight";
+    return "bm25";
 }
 
 string
@@ -162,7 +163,7 @@ BM25Weight::unserialise(const string & s) const
 
 double
 BM25Weight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
-			Xapian::termcount) const
+			Xapian::termcount, Xapian::termcount) const
 {
     LOGCALL(WTCALC, double, "BM25Weight::get_sumpart", wdf | len);
     Xapian::doclength normlen = max(len * len_factor, param_min_normlen);
@@ -210,7 +211,9 @@ BM25Weight::get_maxpart() const
  * 2 * param_k2 * query_length / (1 + normlen)
  */
 double
-BM25Weight::get_sumextra(Xapian::termcount len, Xapian::termcount) const
+BM25Weight::get_sumextra(Xapian::termcount len,
+			 Xapian::termcount,
+			 Xapian::termcount) const
 {
     LOGCALL(WTCALC, double, "BM25Weight::get_sumextra", len);
     double num = (2.0 * param_k2 * get_query_length());
@@ -228,4 +231,37 @@ BM25Weight::get_maxextra() const
 			    param_min_normlen)));
 }
 
+[[noreturn]]
+static inline void
+parameter_error(const char* message, const char* params)
+{
+    Xapian::Weight::Internal::parameter_error(message, "bm25", params);
+}
+
+BM25Weight*
+BM25Weight::create_from_parameters(const char* params) const
+{
+    const char* p = params;
+    if (*p == '\0')
+	return new Xapian::BM25Weight();
+    double k1 = 1;
+    double k2 = 0;
+    double k3 = 1;
+    double b = 0.5;
+    double min_normlen = 0.5;
+    if (!Xapian::Weight::Internal::double_param(&p, &k1))
+	parameter_error("Parameter 1 (k1) is invalid", params);
+    if (*p && !Xapian::Weight::Internal::double_param(&p, &k2))
+	parameter_error("Parameter 2 (k2) is invalid", params);
+    if (*p && !Xapian::Weight::Internal::double_param(&p, &k3))
+	parameter_error("Parameter 3 (k3) is invalid", params);
+    if (*p && !Xapian::Weight::Internal::double_param(&p, &b))
+	parameter_error("Parameter 4 (b) is invalid", params);
+    if (*p && !Xapian::Weight::Internal::double_param(&p, &min_normlen))
+	parameter_error("Parameter 5 (min_normlen) is invalid", params);
+    if (*p)
+	parameter_error("Extra data after parameter 5", params);
+    return new Xapian::BM25Weight(k1, k2, k3, b, min_normlen);
+}
+
 }
