@@ -24,7 +24,7 @@
 #include "xapian/weight.h"
 
 #include "xapian/error.h"
-#include "common/log2.h"
+
 #include <algorithm>
 #include <cmath>
 
@@ -41,9 +41,6 @@ DPHWeight::clone() const
 void
 DPHWeight::init(double factor)
 {
-    // Avoid warnings about unused private member.
-    (void)lower_bound;
-
     if (factor == 0.0) {
 	// This object is for the term-independent contribution, and that's
 	// always zero for this scheme.
@@ -116,7 +113,7 @@ DPHWeight::init(double factor)
 string
 DPHWeight::name() const
 {
-    return "Xapian::DPHWeight";
+    return "dph";
 }
 
 string
@@ -135,7 +132,7 @@ DPHWeight::unserialise(const string& s) const
 
 double
 DPHWeight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
-		       Xapian::termcount) const
+		       Xapian::termcount, Xapian::termcount) const
 {
     if (wdf == 0 || wdf == len) return 0.0;
 
@@ -158,16 +155,12 @@ DPHWeight::get_maxpart() const
     return upper_bound;
 }
 
-double
-DPHWeight::get_sumextra(Xapian::termcount, Xapian::termcount) const
+DPHWeight *
+DPHWeight::create_from_parameters(const char * p) const
 {
-    return 0;
-}
-
-double
-DPHWeight::get_maxextra() const
-{
-    return 0;
+    if (*p != '\0')
+	throw InvalidArgumentError("No parameters are required for DPHWeight");
+    return new Xapian::DPHWeight();
 }
 
 }
