@@ -2,6 +2,7 @@
  * @brief Xapian::IfB2Weight class - the IfB2 weighting scheme of the DFR framework.
  */
 /* Copyright (C) 2013,2014 Aarsh Shah
+ * Copyright (C) 2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,12 +22,15 @@
 #include <config.h>
 
 #include "xapian/weight.h"
-#include "common/log2.h"
+
+#include "weightinternal.h"
 
 #include "serialise-double.h"
 
 #include "xapian/error.h"
 
+#include <cmath>
+
 using namespace std;
 
 namespace Xapian {
@@ -93,7 +97,7 @@ IfB2Weight::init(double factor)
 string
 IfB2Weight::name() const
 {
-    return "Xapian::IfB2Weight";
+    return "ifb2";
 }
 
 string
@@ -115,7 +119,7 @@ IfB2Weight::unserialise(const string & s) const
 
 double
 IfB2Weight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
-			Xapian::termcount) const
+			Xapian::termcount, Xapian::termcount) const
 {
     if (wdf == 0) return 0.0;
     double wdfn = wdf;
@@ -132,16 +136,25 @@ IfB2Weight::get_maxpart() const
     return upper_bound;
 }
 
-double
-IfB2Weight::get_sumextra(Xapian::termcount, Xapian::termcount) const
+[[noreturn]]
+static inline void
+parameter_error(const char* message, const char* params)
 {
-    return 0;
+    Xapian::Weight::Internal::parameter_error(message, "ifb2", params);
 }
 
-double
-IfB2Weight::get_maxextra() const
+IfB2Weight*
+IfB2Weight::create_from_parameters(const char* params) const
 {
-    return 0;
+    const char* p = params;
+    if (*p == '\0')
+	return new Xapian::IfB2Weight();
+    double c = 1.0;
+    if (!Xapian::Weight::Internal::double_param(&p, &c))
+	parameter_error("Parameter is invalid", params);
+    if (*p)
+	parameter_error("Extra data after parameter", params);
+    return new Xapian::IfB2Weight(c);
 }
 
 }
