@@ -23,13 +23,15 @@
 #include <config.h>
 
 #include "xapian/weight.h"
-#include "common/log2.h"
+
+#include "weightinternal.h"
 
 #include "serialise-double.h"
 
 #include "xapian/error.h"
 
 #include <algorithm>
+#include <cmath>
 
 using namespace std;
 
@@ -132,7 +134,7 @@ PL2PlusWeight::init(double factor_)
 string
 PL2PlusWeight::name() const
 {
-    return "Xapian::PL2PlusWeight";
+    return "pl2+";
 }
 
 string
@@ -157,7 +159,7 @@ PL2PlusWeight::unserialise(const string & s) const
 
 double
 PL2PlusWeight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
-			   Xapian::termcount) const
+			   Xapian::termcount, Xapian::termcount) const
 {
     // Note: lambda_t in the paper is 1/mean.
     if (wdf == 0 || mean > 1) {
@@ -184,16 +186,28 @@ PL2PlusWeight::get_maxpart() const
     return upper_bound;
 }
 
-double
-PL2PlusWeight::get_sumextra(Xapian::termcount, Xapian::termcount) const
+[[noreturn]]
+static inline void
+parameter_error(const char* message, const char* params)
 {
-    return 0;
+    Xapian::Weight::Internal::parameter_error(message, "pl2+", params);
 }
 
-double
-PL2PlusWeight::get_maxextra() const
+PL2PlusWeight *
+PL2PlusWeight::create_from_parameters(const char* params) const
 {
-    return 0;
+    const char* p = params;
+    if (*p == '\0')
+	return new Xapian::PL2PlusWeight();
+    double c = 1.0;
+    double delta = 0.8;
+    if (!Xapian::Weight::Internal::double_param(&p, &c))
+	parameter_error("Parameter 1 (c) is invalid", params);
+    if (!Xapian::Weight::Internal::double_param(&p, &delta))
+	parameter_error("Parameter 2 (delta) is invalid", params);
+    if (*p)
+	parameter_error("Extra data after parameter 2", params);
+    return new Xapian::PL2PlusWeight(c, delta);
 }
 
 }
