@@ -2,7 +2,7 @@
  * @brief Xapian::PL2Weight class - the PL2 weighting scheme of the DFR framework.
  */
 /* Copyright (C) 2013 Aarsh Shah
- * Copyright (C) 2013,2014,2016 Olly Betts
+ * Copyright (C) 2013,2014,2016,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,13 +22,15 @@
 #include <config.h>
 
 #include "xapian/weight.h"
-#include "common/log2.h"
+
+#include "weightinternal.h"
 
 #include "serialise-double.h"
 
 #include "xapian/error.h"
 
 #include <algorithm>
+#include <cmath>
 
 using namespace std;
 
@@ -64,8 +66,7 @@ PL2Weight::init(double factor_)
 	return;
     }
 
-    // lower_bound is really factor.
-    lower_bound = factor_;
+    factor = factor_;
 
     if (get_wdf_upper_bound() == 0) {
 	// The "extra" weight object is cloned, init() called and then
@@ -76,8 +77,7 @@ PL2Weight::init(double factor_)
 	return;
     }
 
-    // lower_bound is really factor.
-    lower_bound *= get_wqf();
+    factor *= get_wqf();
 
     cl = param_c * get_average_length();
 
@@ -122,8 +122,7 @@ PL2Weight::init(double factor_)
     // giving us a bound that can't be bettered if wdfn_upper is tight.
     double wdfn_optb = P1 + P2 > 0 ? wdfn_upper : wdfn_lower;
     double P_max2b = (P1 - P2 * wdfn_optb) / (wdfn_optb + 1.0);
-    // lower_bound is really factor.
-    upper_bound = lower_bound * (P_max2a + P_max2b);
+    upper_bound = factor * (P_max2a + P_max2b);
 
     if (rare(upper_bound <= 0)) upper_bound = 0;
 }
@@ -131,7 +130,7 @@ PL2Weight::init(double factor_)
 string
 PL2Weight::name() const
 {
-    return "Xapian::PL2Weight";
+    return "pl2";
 }
 
 string
@@ -153,7 +152,7 @@ PL2Weight::unserialise(const string & s) const
 
 double
 PL2Weight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
-		       Xapian::termcount) const
+		       Xapian::termcount, Xapian::termcount) const
 {
     if (wdf == 0) return 0.0;
 
@@ -162,8 +161,7 @@ PL2Weight::get_sumpart(Xapian::termcount wdf, Xapian::termcount len,
     double P = P1 + (wdfn + 0.5) * log2(wdfn) - P2 * wdfn;
     if (rare(P <= 0)) return 0.0;
 
-    // lower_bound is really factor.
-    return lower_bound * P / (wdfn + 1.0);
+    return factor * P / (wdfn + 1.0);
 }
 
 double
@@ -172,16 +170,25 @@ PL2Weight::get_maxpart() const
     return upper_bound;
 }
 
-double
-PL2Weight::get_sumextra(Xapian::termcount, Xapian::termcount) const
+[[noreturn]]
+static inline void
+parameter_error(const char* message, const char* params)
 {
-    return 0;
+    Xapian::Weight::Internal::parameter_error(message, "pl2", params);
 }
 
-double
-PL2Weight::get_maxextra() const
+PL2Weight*
+PL2Weight::create_from_parameters(const char* params) const
 {
-    return 0;
+    const char* p = params;
+    if (*p == '\0')
+	return new Xapian::PL2Weight();
+    double c = 1.0;
+    if (!Xapian::Weight::Internal::double_param(&p, &c))
+	parameter_error("Parameter is invalid", params);
+    if (*p)
+	parameter_error("Extra data after parameter", params);
+    return new Xapian::PL2Weight(c);
 }
 
 }
