@@ -2,7 +2,7 @@
  * @brief Xapian::Weight::Internal class, holding database and term statistics.
  */
 /* Copyright (C) 2007 Lemur Consulting Ltd
- * Copyright (C) 2009,2010,2011,2012,2013,2014,2015,2017,2020 Olly Betts
+ * Copyright (C) 2009,2010,2011,2012,2013,2014,2015,2017,2020,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,13 +25,13 @@
 
 #include "xapian/enquire.h"
 
+#include "min_non_zero.h"
 #include "omassert.h"
-#include "api/omenquireinternal.h"
+#include "api/rsetinternal.h"
 #include "str.h"
 #include "api/termlist.h"
 
-#include "autoptr.h"
-#include <set>
+#include <memory>
 
 using namespace std;
 
@@ -62,10 +62,19 @@ Weight::Internal::operator+=(const Weight::Internal & inc)
     collection_size += inc.collection_size;
     rset_size += inc.rset_size;
 
+    db_doclength_lower_bound = min_non_zero(db_doclength_lower_bound,
+					    inc.db_doclength_lower_bound);
+    db_doclength_upper_bound = std::max(db_doclength_upper_bound,
+					inc.db_doclength_upper_bound);
+
+    db_unique_terms_lower_bound = min_non_zero(db_unique_terms_lower_bound,
+					       inc.db_unique_terms_lower_bound);
+    db_unique_terms_upper_bound = std::max(db_unique_terms_upper_bound,
+					   inc.db_unique_terms_upper_bound);
+
     // Add termfreqs and reltermfreqs
-    map<string, TermFreqs>::const_iterator i;
-    for (i = inc.termfreqs.begin(); i != inc.termfreqs.end(); ++i) {
-	termfreqs[i->first] += i->second;
+    for (auto&& i : inc.termfreqs) {
+	termfreqs[i.first] += i.second;
     }
     return *this;
 }
@@ -82,6 +91,17 @@ Weight::Internal::accumulate_stats(const Xapian::Database::Internal &subdb,
     collection_size += subdb.get_doccount();
     rset_size += rset.size();
 
+    db_doclength_lower_bound = min_non_zero(db_doclength_lower_bound,
+					    subdb.get_doclength_lower_bound());
+    db_doclength_upper_bound = std::max(db_doclength_upper_bound,
+					subdb.get_doclength_upper_bound());
+    db_unique_terms_lower_bound =
+	min_non_zero(db_unique_terms_lower_bound,
+		     subdb.get_unique_terms_lower_bound());
+    db_unique_terms_upper_bound =
+	std::max(db_unique_terms_upper_bound,
+		 subdb.get_unique_terms_upper_bound());
+
     Xapian::TermIterator t;
     for (t = query.get_unique_terms_begin(); t != Xapian::TermIterator(); ++t) {
 	const string & term = *t;
@@ -94,25 +114,25 @@ Weight::Internal::accumulate_stats(const Xapian::Database::Internal &subdb,
 	tf.collfreq += sub_cf;
     }
 
-    const set<Xapian::docid> & items(rset.internal->get_items());
-    set<Xapian::docid>::const_iterator d;
-    for (d = items.begin(); d != items.end(); ++d) {
-	Xapian::docid did = *d;
+    if (!rset.internal)
+	return;
+
+    for (Xapian::docid did : rset.internal->docs) {
 	Assert(did);
 	// The query is likely to contain far fewer terms than the documents,
 	// and we can skip the document's termlist, so look for each query term
 	// in the document.
-	AutoPtr<TermList> tl(subdb.open_term_list(did));
-	map<string, TermFreqs>::iterator i;
-	for (i = termfreqs.begin(); i != termfreqs.end(); ++i) {
-	    const string & term = i->first;
+	unique_ptr<TermList> tl(subdb.open_term_list(did));
+	for (auto&& i : termfreqs) {
+	    const string& term = i.first;
 	    TermList * ret = tl->skip_to(term);
-	    Assert(ret == NULL);
-	    (void)ret;
-	    if (tl->at_end())
+	    if (ret != NULL) {
+		// No more entries prune shouldn't happen).
+		Assert(ret == tl.get());
 		break;
+	    }
 	    if (term == tl->get_termname())
-		++i->second.reltermfreq;
+		++i.second.reltermfreq;
 	}
     }
 }
@@ -143,8 +163,7 @@ Weight::Internal::get_description() const
     desc += str(finalised);
 #endif
     desc += ", termfreqs={";
-    map<string, TermFreqs>::const_iterator i;
-    for (i = termfreqs.begin(); i != termfreqs.end(); ++i) {
+    for (auto i = termfreqs.begin(); i != termfreqs.end(); ++i) {
 	if (i != termfreqs.begin())
 	    desc += ", ";
 	desc += i->first;
