@@ -2,7 +2,7 @@
  * @brief Xapian::Weight::Internal class, holding database and term statistics.
  */
 /* Copyright (C) 2007 Lemur Consulting Ltd
- * Copyright (C) 2009,2010,2011,2013,2014,2015,2020 Olly Betts
+ * Copyright (C) 2009,2010,2011,2013,2014,2015,2020,2024 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,15 +25,29 @@
 #include "xapian/weight.h"
 
 #include "xapian/database.h"
+#include "xapian/error.h"
 #include "xapian/query.h"
 
-#include "backends/database.h"
+#include "backends/databaseinternal.h"
 #include "internaltypes.h"
 #include "omassert.h"
+#include "stringutils.h"
 
 #include <algorithm>
+#include <cerrno>
+#include <cstdlib>
+#include <functional>
 #include <map>
 #include <string>
+#include <string_view>
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+# include <cstring>
+# include <charconv>
+#endif
+
+namespace Xapian {
+
+namespace Internal {
 
 /// The frequencies for a term.
 struct TermFreqs {
@@ -62,10 +76,28 @@ struct TermFreqs {
 	Assert(other.max_part == 0.0);
     }
 
+    void operator*=(double factor) {
+	termfreq = Xapian::doccount(termfreq * factor + 0.5);
+	reltermfreq = Xapian::doccount(reltermfreq * factor + 0.5);
+	collfreq = Xapian::termcount(collfreq * factor + 0.5);
+    }
+
+    void operator/=(unsigned x) {
+	termfreq /= x;
+	reltermfreq /= x;
+	collfreq /= x;
+    }
+
     /// Return a std::string describing this object.
     std::string get_description() const;
 };
 
+}
+
+}
+
+using Xapian::Internal::TermFreqs;
+
 namespace Xapian {
 
 class RSet;
@@ -93,21 +125,30 @@ class Weight::Internal {
     /** Number of relevant documents in the collection. */
     Xapian::doccount rset_size = 0;
 
+    /// A lower bound on the minimum length of any document in the database.
+    Xapian::termcount db_doclength_lower_bound = 0;
+
+    /// An upper bound on the maximum length of any document in the database.
+    Xapian::termcount db_doclength_upper_bound = 0;
+
+    /// A lower bound on the number of unique terms in any document.
+    Xapian::termcount db_unique_terms_lower_bound = 0;
+
+    /// An upper bound on the number of unique terms in any document.
+    Xapian::termcount db_unique_terms_upper_bound = 0;
+
     /** Has max_part been set for any term?
      *
      *  If not, we can avoid having to serialise max_part.
      */
     bool have_max_part = false;
 
-    /** Database to get the bounds on doclength and wdf from. */
-    Xapian::Database db;
-
     /** The query. */
     Xapian::Query query;
 
     /** Map of term frequencies and relevant term frequencies for the
      *  collection. */
-    std::map<std::string, TermFreqs> termfreqs;
+    std::map<std::string, TermFreqs, std::less<>> termfreqs;
 
     Internal() { }
 
@@ -140,7 +181,7 @@ class Weight::Internal {
      *  collfreq is the total number of occurrences of the term in all
      *  documents.
      */
-    bool get_stats(const std::string & term,
+    bool get_stats(std::string_view term,
 		   Xapian::doccount & termfreq,
 		   Xapian::doccount & reltermfreq,
 		   Xapian::termcount & collfreq) const {
@@ -169,7 +210,7 @@ class Weight::Internal {
     }
 
     /// Get just the termfreq.
-    bool get_stats(const std::string & term,
+    bool get_stats(std::string_view term,
 		   Xapian::doccount & termfreq) const {
 	Xapian::doccount dummy1;
 	Xapian::termcount dummy2;
@@ -177,7 +218,7 @@ class Weight::Internal {
     }
 
     /// Get the termweight.
-    bool get_termweight(const std::string & term, double & termweight) const {
+    bool get_termweight(std::string_view term, double& termweight) const {
 #ifdef XAPIAN_ASSERTIONS
 	finalised = true;
 #endif
@@ -232,18 +273,65 @@ class Weight::Internal {
 #ifdef XAPIAN_ASSERTIONS
 	finalised = true;
 #endif
-	if (rare(collection_size == 0)) return 0;
+	// We shortcut an empty shard and avoid creating a postlist tree for
+	// it, and all shards must be empty for collection_size to be zero.
+	Assert(collection_size);
 	return Xapian::doclength(total_length) / collection_size;
     }
 
-    /** Set the "bounds" stats from Database @a db. */
-    void set_bounds_from_db(const Xapian::Database &db_) {
-	Assert(!finalised);
-	db = db_;
-    }
-
     /// Return a std::string describing this object.
     std::string get_description() const;
+
+    static bool double_param(const char ** p, double * ptr_val) {
+#ifdef HAVE_STD_FROM_CHARS_DOUBLE
+	const char* startptr = *p;
+	// Unlike strtod(), std::from_chars() doesn't skip leading whitespace.
+	while (C_isspace(*startptr)) ++startptr;
+	const char* endptr = startptr + std::strlen(startptr);
+	double v;
+	const auto& r = std::from_chars(startptr, endptr, v);
+	if (r.ec != std::errc()) {
+	    return false;
+	}
+	*p = r.ptr;
+	*ptr_val = v;
+#else
+	char *end;
+	errno = 0;
+	double v = strtod(*p, &end);
+	if (*p == end || errno) return false;
+	*p = end;
+	*ptr_val = v;
+#endif
+	return true;
+    }
+
+    static bool param_name(const char** p, std::string& name) {
+	const char* q = *p;
+	while (*q != ' ') {
+	    if (*q == '\0') break;
+	    name += *(q)++;
+	}
+	if (q == *p) return false;
+	if (*q == ' ') q++;
+	*p = q;
+	return true;
+    }
+
+    [[noreturn]]
+    static void parameter_error(const char* msg,
+				const std::string& scheme,
+				const char* params) {
+	std::string m(msg);
+	m += ": '";
+	m += scheme;
+	if (*params) {
+	    m += ' ';
+	    m += params;
+	}
+	m += "'";
+	throw InvalidArgumentError(m);
+    }
 };
 
 }
